/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;

namespace CodeMetricsGui
{
    /// <summary>
    /// GUICNX
    /// </summary>
    public partial class CDMtrGui : Form
    {
        public enum EXEC_MODE
        {
            ROW = 0,
            DIFF,
            STRUCT,
            CK
        };

        public enum RESTYLE_MODE
        {
            NONE = 0,
            STD,
            BOTH
        };

        public const string FILENAME_PROP = "ncmeter.properties";

        public const string FILENAME_OUTPUT_DEFAULT = "ncmeter";

        public const string FILENAME_OUTPUT = "ncmeter.csv";

        public const string FILENAME_XML_OUTPUT = "ncmeter.xml";

        public const string FILENAME_FORMATTOOL = "FormatTool.xls";

        public const string ENVIRONMENT_PATH = @"..\etc\ncmeter";

        public const string FORMATTOOL_PATH = @"..\etc\FormatTool";

        private const string FILENAME_LOG = "log.txt";

        private const string FILENAME_INI = "gui.ini";

        private const string FILENAME_HELP = @"..\etc\ncmeter\doc\guihelp.html";

        private const string FILENAME_DEBUGLOG = @"log_debug.txt";

        private string strStartupPath;

        private CDMtrGuiSetInfo guiSetInfo = null;

        private CDMtrGuiCount guiCount = null;

        private bool bExecutingFlg = false;

        private int iExecuteMode = 0;

        private bool bButtonClicked = false;

        private string strExecType;

        /// <summary>
        /// RXgN^
        /// </summary>
        public CDMtrGui()
        {
            InitializeComponent();

            this.strStartupPath = Application.StartupPath;

            guiSetInfo = new CDMtrGuiSetInfo(this.strStartupPath);

            guiCount = new CDMtrGuiCount(this, FILENAME_LOG);

            cdmtrGetCommandParameter();


            return;
        }

        /// <summary>
        /// \̏
        /// </summary>
        private void CDMtrGui_Actived(object sender, EventArgs e)
        {
            cdmtrGuiInit();

            this.Activated -= new EventHandler(CDMtrGui_Actived);

            return;
        }

        /// <summary>
        /// N\𔻒f
        /// SecurityExceptionȂǂ邽߁A֐Ocatch邱ƁB
        /// </summary>
        static public bool CDMtrIsEnableExecute()
        {
            string strProcess;
            int iProcessNum;

            strProcess = System.Diagnostics.Process.GetCurrentProcess().ProcessName;
            iProcessNum = System.Diagnostics.Process.GetProcessesByName(strProcess).Length;

            if (iProcessNum > 1)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// ʕ\񏉊
        /// </summary>
        private void cdmtrGuiInit()
        {
            string strIniFilePath;

            strIniFilePath = Path.Combine(strStartupPath, FILENAME_INI);

            if (CDMtrGuiPath.CDMtrPathIsExistFile(strIniFilePath))
            {
                guiSetInfo.CDMtrGuiSetInfoRead(strIniFilePath);

                for (int i = 0; i < guiSetInfo.iRowInputFileNum; i++)
                {
                    RowListBox.Items.Add(guiSetInfo.strRowInputFileArray[i]);
                }
                DiffOldTextBox.Text = guiSetInfo.strDiffOldInputFile;
                DiffNewTextBox.Text = guiSetInfo.strDiffNewInputFile;

                OutFolderPathTextBox.Text = guiSetInfo.strOutFolderPath;
                OutFileNameTextBox.Text = guiSetInfo.strOutFileName;
                AddSuffixChkBox.Checked = guiSetInfo.bAddSuffixCheck;
            }
            else
            {
                OutFolderPathTextBox.Text = strStartupPath;
                OutFileNameTextBox.Text = FILENAME_OUTPUT_DEFAULT;
            }

            cdmtrCheckRadioButton(guiSetInfo.iExecMode);

            cdmtrChangeCountStartButtonText();
            return;
        }

        /// <summary>
        /// R}hp[^擾
        /// </summary>
        private void cdmtrGetCommandParameter()
        {
            try
            {
                string[] args = Environment.GetCommandLineArgs();
                if (args != null)
                {
                    for (int i = 0; i < args.Length; i++)
                    {
                        if (args[i].CompareTo("-debug") == 0)
                        {
                            CDMtrGuiDebug.open(FILENAME_DEBUGLOG);
                            CDMtrGuiDebug.write(CodeMetricsGui.Properties.Resources.Gui_01 + DateTime.Now.ToString());
                        }
                    }
                }
            }
            catch(Exception ex)
            {
                CDMtrGuiDebug.write(ex.ToString());
            }

            return;
        }

        /// <summary>
        /// R}hp[^p̓̓t@C񐶐
        /// </summary>
        private string cdmtrCreateInputSrc()
        {
            int iInputSrcNum;
            string strInputSrc;
            string strMsg;

            iInputSrcNum = 0;
            strInputSrc = null;
            strMsg = null;

            StringBuilder sbInputSrc = new StringBuilder();
            if (sbInputSrc == null)
            {
                goto FUNC_END;
            }

            if (this.iExecuteMode == (int)EXEC_MODE.DIFF)
            {
                if (DiffOldTextBox.Text == "")
                {
                    strMsg = CodeMetricsGui.Properties.Resources.Gui_02;
                    goto FUNC_END;
                }
                if (DiffNewTextBox.Text == "")
                {
                    strMsg = CodeMetricsGui.Properties.Resources.Gui_03;
                    goto FUNC_END;
                }
                iInputSrcNum = 2;

                sbInputSrc.Append(" -diff ");

                sbInputSrc.Append("\"");
                sbInputSrc.Append(DiffOldTextBox.Text);
                sbInputSrc.Append("\" ");
                sbInputSrc.Append("\"");
                sbInputSrc.Append(DiffNewTextBox.Text);
                sbInputSrc.Append("\" ");
            }
            else
            {
                if (this.iExecuteMode == (int)EXEC_MODE.STRUCT)
                {
                    sbInputSrc.Append(" -struct ");
                    strMsg = CodeMetricsGui.Properties.Resources.Gui_04;
                }
                else if (this.iExecuteMode == (int)EXEC_MODE.CK)
                {
                    sbInputSrc.Append(" -ck ");
                    strMsg = CodeMetricsGui.Properties.Resources.Gui_05;
                }
                else
                {
                    sbInputSrc.Append(" -row ");
                    strMsg = CodeMetricsGui.Properties.Resources.Gui_06;
                }

                iInputSrcNum = RowListBox.Items.Count;
                if (iInputSrcNum == 0)
                {
                    goto FUNC_END;
                }

                for (int i = 0; i < iInputSrcNum; i++)
                {
                    sbInputSrc.Append("\"");
                    sbInputSrc.Append(RowListBox.Items[i].ToString());
                    sbInputSrc.Append("\" ");
                }
            }

            strInputSrc = sbInputSrc.ToString();
        FUNC_END:
            if (strInputSrc == null)
            {
                CDMtrGuiDialog.CDMtrDialogWarn(strMsg);
            }
            return strInputSrc;
        }

        /// <summary>
        /// R}hp[^񐶐
        /// </summary>
        private string cdmtrCreateParam()
        {
            string strParam;
            string strMsg;
            string strOutFilePath;


            strParam = null;
            strMsg = "";
            strOutFilePath = "";

            StringBuilder sbParam = new StringBuilder();
            if (sbParam == null)
            {
                goto FUNC_END;
            }

            if (this.OutFolderPathTextBox.Text == "")
            {
                strMsg = CodeMetricsGui.Properties.Resources.Gui_25;
                this.OutFolderPathTextBox.Focus();
                goto FUNC_END;
            }

            if (this.OutFileNameTextBox.Text == "")
            {
                strMsg = CodeMetricsGui.Properties.Resources.Gui_26;
                this.OutFileNameTextBox.Focus();
                goto FUNC_END;
            }


            if (guiSetInfo.strOutPutFormat != "")
            {
                sbParam.Append(" -F " + guiSetInfo.strOutPutFormat);
            }
            else
            {
                sbParam.Append(" -F csv");
            }

            if (guiSetInfo.strPropFile != "")
            {
                sbParam.Append(" -Tfp \"" + guiSetInfo.strPropFile + "\"");
            }
            guiSetInfo.strOutFolderPath = this.OutFolderPathTextBox.Text;

            if (AddSuffixChkBox.Checked == true)
            {
                switch (this.iExecuteMode)
                {
                    case (int)EXEC_MODE.ROW:
                        strExecType = "row";
                        break;
                    case (int)EXEC_MODE.DIFF:
                        strExecType = "diff";
                        break;
                    case (int)EXEC_MODE.STRUCT:
                        strExecType = "struct";
                        break;
                    case (int)EXEC_MODE.CK:
                        strExecType = "ck";
                        break;
                    default:
                        break;
                }

                guiSetInfo.strExecOutFileName = OutFileNameTextBox.Text
                                                + "_"
                                                + strExecType
                                                + "_"
                                                + System.DateTime.Now.ToString("yyyyMMddHHmmss")
                                                + "."
                                                + guiSetInfo.strOutPutFormat;
            }
            else
            {
                guiSetInfo.strExecOutFileName = OutFileNameTextBox.Text
                                                + "."
                                                + guiSetInfo.strOutPutFormat;
            }



            strOutFilePath = CDMtrGuiPath.CDMtrPathCombine(guiSetInfo.strOutFolderPath, guiSetInfo.strExecOutFileName);

            sbParam.Append(" -Tfo \"" + strOutFilePath + "\"");

            switch (guiSetInfo.iRestyleCheck)
            {
                case (int)CDMtrGui.RESTYLE_MODE.BOTH:
                    sbParam.Append(" -RESTYLE BOTH ");
                    break;
                case (int)CDMtrGui.RESTYLE_MODE.STD:
                    sbParam.Append(" -RESTYLE STD ");
                    break;
                case (int)CDMtrGui.RESTYLE_MODE.NONE:
                default:
                    sbParam.Append(" -RESTYLE NONE ");
                    break;
            }

            if (guiSetInfo.bDetailCheck)
            {
                sbParam.Append(" -v ");
            }

            if (guiSetInfo.bDebugCheck)
            {
                sbParam.Append(" -debug ");
            }

            for (int i = 0; i < guiSetInfo.iDefNum; i++)
            {
                if (guiSetInfo.strDefArray[i] != "")
                {
                    sbParam.Append(" -D ");
                    sbParam.Append(guiSetInfo.strDefArray[i]);
                }
            }

            for (int i = 0; i < guiSetInfo.iIncNum; i++)
            {
                if (guiSetInfo.strIncArray[i] != "")
                {
                    sbParam.Append(" -I \"");
                    sbParam.Append(guiSetInfo.strIncArray[i]);
                    sbParam.Append("\"");
                }
            }

            for (int i = 0; i < guiSetInfo.iSysIncNum; i++)
            {
                if (guiSetInfo.strSysIncArray[i] != "")
                {
                    sbParam.Append(" -I \"");
                    sbParam.Append(guiSetInfo.strSysIncArray[i]);
                    sbParam.Append("\"");
                }
            }

            for (int i = 0; i < guiSetInfo.iOptionClassNum; i++)
            {
                if (guiSetInfo.strOptionClassArray[i] != "")
                {
                    sbParam.Append(" -optclass \"");
                    sbParam.Append(guiSetInfo.strOptionClassArray[i]);
                    sbParam.Append("\"");
                }
            }

            if (guiSetInfo.strCompVer != "")
            {
                sbParam.Append(" -Ttc ");
                sbParam.Append(guiSetInfo.strCompVer);
            }

            strParam = sbParam.ToString();
        FUNC_END:
            if (strParam == null)
            {
                CDMtrGuiDialog.CDMtrDialogWarn(strMsg);
            }

            return strParam;
        }

        /// <summary>
        /// e{^̃bN/bNs
        /// </summary>
        private void cdmtrLockButton(bool bVal)
        {
            this.ExecuteGroupBox.Enabled = bVal;

            this.MenuToolBar.Enabled = bVal;

            this.DetailSetDialogButton.Enabled = bVal;
            this.OutFileShowButton.Enabled = bVal;
            this.LogShowButton.Enabled = bVal;

            return;
        }

        /// <summary>
        /// o͌ʂOo̓{bNXɕ\
        /// </summary>
        public void CDMtrOutputTextBoxAdd(string strTxt)
        {
            if (strTxt != null)
            {
                LogTextBox.AppendText(strTxt + "\n");
            }

            return;
        }

        /// <summary>
        /// JEg̃tH[bZ[WύX
        /// </summary>
        public void CDMtrControlChange(bool bExitFlg)
        {
            if (!bExecutingFlg)
            {
                cdmtrLockButton(false);

                bExecutingFlg = true;
                this.CountStartButton.Text = CodeMetricsGui.Properties.Resources.Gui_07;
            }
            else
            {
                bExecutingFlg = false;
                cdmtrChangeCountStartButtonText();

                cdmtrLockButton(true);

                if (!bExitFlg)
                {
                    this.OutFileShowButton.Enabled = false;
                }
            }
            return;
        }

        /// <summary>
        /// WI{^`FbN̏
        /// </summary>
        private void cdmtrCheckRadioButton(int iExecuteMode)
        {
            switch (iExecuteMode)
            {
                case (int)EXEC_MODE.STRUCT:
                    RowRadioButton.Checked = false;
                    DiffRadioButton.Checked = false;
                    StructRadioButton.Checked = true;
                    CkRadioButton.Checked = false;
                    StructRadioButton.Focus();
                    break;
                case (int)EXEC_MODE.DIFF:
                    RowRadioButton.Checked = false;
                    DiffRadioButton.Checked = true;
                    StructRadioButton.Checked = false;
                    CkRadioButton.Checked = false;
                    DiffRadioButton.Focus();
                    break;
                case (int)EXEC_MODE.CK:
                    RowRadioButton.Checked = false;
                    DiffRadioButton.Checked = false;
                    StructRadioButton.Checked = false;
                    CkRadioButton.Checked = true;
                    CkRadioButton.Focus();
                    break;
                case (int)EXEC_MODE.ROW:
                default:
                    RowRadioButton.Checked = true;
                    DiffRadioButton.Checked = false;
                    StructRadioButton.Checked = false;
                    CkRadioButton.Checked = false;
                    RowRadioButton.Focus();
                    break;
            }
            this.iExecuteMode = iExecuteMode;

            return;
        }


        /// <summary>
        /// CJEg̓̓t@CQ
        /// </summary>
        private void RowFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(RowTextBox.Text, RowListBox);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, null);
            if (strFilePath != null)
            {
                RowTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// fBNgQƃ_CAO\({^NbN)
        /// </summary>
        private void RowDirDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(RowTextBox.Text, RowListBox);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                RowTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// ̓\[Xǉ{^
        /// </summary>
        private void RowSrcAddButton_Click(object sender, EventArgs e)
        {
            if (RowTextBox.Text != "")
            {
                RowListBox.Items.Add(RowTextBox.Text.Trim());
                RowTextBox.Clear();
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// ̓\[X폜{^
        /// </summary>
        private void RowSrcDelButton_Click(object sender, EventArgs e)
        {
            if (RowListBox.SelectedItem != null)
            {
                RowListBox.Items.RemoveAt(RowListBox.SelectedIndex);
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// ̓Xg{bNXł̃{^Cxg
        /// </summary>
        private void RowListBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                if (RowListBox.SelectedItem != null)
                {
                    RowListBox.Items.RemoveAt(RowListBox.SelectedIndex);
                }
            }
            return;
        }

        /// <summary>
        /// {CXg{bNXS폜
        /// </summary>
        private void RowSrcClearButton_Click(object sender, EventArgs e)
        {
            RowListBox.Items.Clear();
            return;
        }


        /// <summary>
        /// COfBNgI_CAO
        /// </summary>
        private void DiffOldDirDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(DiffOldTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                DiffOldTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// C㍷fBNgI_CAO
        /// </summary>
        private void DiffNewDirDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(DiffNewTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                DiffNewTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// COt@CI_CAO
        /// </summary>
        private void DiffOldFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(DiffOldTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, null);
            if (strFilePath != null)
            {
                DiffOldTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// Ct@CI_CAO
        /// </summary>
        private void DiffNewFileDialogButton_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetSelectDialog(DiffNewTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenFile(strSelectPath, null);
            if (strFilePath != null)
            {
                DiffNewTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;
        }

        /// <summary>
        /// ڍאݒ_CAO\
        /// </summary>
        private void cdmtrDispSetDetail()
        {
            CDMtrGuiSetInfo tmpSetInfo;
            CDMtrGuiSetting guiSetting;

            if ((tmpSetInfo = new CDMtrGuiSetInfo(guiSetInfo)) != null)
            {
                if ((guiSetting = new CDMtrGuiSetting(tmpSetInfo)) != null)
                {

                    guiSetting.CDMtrSettingLoad(guiSetInfo);

                    guiSetting.StartPosition = FormStartPosition.CenterParent;
                    guiSetting.ShowDialog();

                    if (guiSetting.DialogResult == DialogResult.OK)
                    {
                        guiSetInfo = tmpSetInfo;
                    }
                    else if (guiSetting.DialogResult == DialogResult.Cancel)
                    {
                        
                    }
                    else
                    {
                        
                    }
                }
            }
            return;
        }

        /// <summary>
        /// ڍאݒ{^
        /// </summary>
        private void DetailSetButton_Click(object sender, EventArgs e)
        {
            cdmtrDispSetDetail();
            return;
        }

        /// <summary>
        /// vʊJn{^
        /// </summary>
        private void CountStartButton_Click(object sender, EventArgs e)
        {
            string strInputSrc;
            string strParam;

            if (!this.bButtonClicked)
            {
                this.bButtonClicked = true;

                try
                {
                    if (bExecutingFlg)
                    {
                        guiCount.CDMtrProcessKill();
                    }
                    else
                    {
                        strInputSrc = cdmtrCreateInputSrc();
                        if (strInputSrc == null)
                        {
                            goto FUNC_END;
                        }
                        strParam = cdmtrCreateParam();
                        if (strParam == null)
                        {
                            goto FUNC_END;
                        }

                        LogTextBox.Clear();

                        guiCount.CDMtrCountStart(strParam + strInputSrc);
                    }
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
                finally
                {
                    this.bButtonClicked = false;
                }
            }
            
        FUNC_END:
            return;
        }

        /// <summary>
        /// o̓t@C\
        /// [쐬xc[𗘗pꍇA[쐬xc[NCSV\B
        /// </summary>
        private bool cdmtrShowOutFile()
        {
            bool bRet;
            string strOutPath;

            bRet = false;
            strOutPath = null;

            strOutPath = CDMtrGuiPath.CDMtrPathCombine(guiSetInfo.strOutFolderPath, guiSetInfo.strExecOutFileName);

            if (CDMtrGuiPath.CDMtrPathIsExistFile(strOutPath))
            {
              
                string stExtension;
                try
                {
                    stExtension = System.IO.Path.GetExtension(strOutPath);
                }
                catch(Exception)
                {
                    stExtension ="";
                    goto FUNC_END;
                }
                
                if ((stExtension.CompareTo(".csv") == 0) && guiSetInfo.bFormatCheck)
                {
                    if (this.iExecuteMode != (int)EXEC_MODE.CK)
                    {
                        if (guiSetInfo.strFormatTool.CompareTo("") == 0)
                        {
                            CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_08);
                            goto FUNC_END;
                        }

                        if (CDMtrGuiPath.CDMtrPathIsExistFile(guiSetInfo.strFormatTool))
                        {
                            System.Diagnostics.Process process;

                            try
                            {
                                process = System.Diagnostics.Process.Start(guiSetInfo.strFormatTool);
                            }
                            catch (Exception ex)
                            {
                                CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_09 + guiSetInfo.strFormatTool);
                                CDMtrGuiDebug.write(ex.ToString());
                                goto FUNC_END;
                            }

                            try
                            {
                                process = System.Diagnostics.Process.Start(strOutPath);
                            }
                            catch (Exception ex)
                            {
                                CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_10 + strOutPath);
                                CDMtrGuiDebug.write(ex.ToString());
                                goto FUNC_END;
                            }
                        }
                        else
                        {
                            CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_11 + guiSetInfo.strFormatTool);
                            goto FUNC_END;
                        }
                    }
                    else
                    {
                        CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_12 + System.Environment.NewLine + CodeMetricsGui.Properties.Resources.Gui_13);
                        try
                        {
                            System.Diagnostics.Process.Start(strOutPath);
                        }
                        catch (Exception ex)
                        {
                            CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_14 + strOutPath);
                            CDMtrGuiDebug.write(ex.ToString());
                            goto FUNC_END;
                        }
                    }
                }
                else
                {
                    try
                    {
                        System.Diagnostics.Process.Start(strOutPath);
                    }
                    catch(Exception ex)
                    {
                        CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_15 + strOutPath);
                        CDMtrGuiDebug.write(ex.ToString());
                        goto FUNC_END;
                    }
                }
            }
            else
            {
                CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_16 + strOutPath);
                goto FUNC_END;
            }
            bRet = true;
        FUNC_END:
            return bRet;
        }

        /// <summary>
        /// o̓t@C\B
        /// gqɊ֘AtĂAvP[VŋNB
        /// [쐬xc[gpꍇlB
        /// CSVt@CEXCELt@C̊gqႤ߁A
        /// ɂĂ͈ႤAvP[VNꍇB
        /// </summary>
        private void OutFileShowButton_Click(object sender, EventArgs e)
        {
            if (!this.bButtonClicked)
            {
                this.bButtonClicked = true;

                try
                {
                    cdmtrShowOutFile();
                }
                catch(Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
                finally
                {
                    this.bButtonClicked = false;
                }
            }
            return;
        }



        /// <summary>
        /// O\{^
        /// </summary>
        private void LogShowButton_Click(object sender, EventArgs e)
        {
            if (!this.bButtonClicked)
            {
                this.bButtonClicked = true;
                try
                {
                    System.Diagnostics.Process.Start(FILENAME_LOG);
                }
                catch (Exception ex)
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_17 + FILENAME_LOG);
                    CDMtrGuiDebug.write(ex.ToString());
                }
                finally
                {
                    this.bButtonClicked = false;
                }
            }
            return;
        }

        /// <summary>
        /// {CXg{bNXICxg
        /// </summary>
        private void RowListBox_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            if (RowListBox.SelectedItem != null)
            {
                RowTextBox.Text = RowListBox.SelectedItem.ToString();
            }
            return;
        }

        /// <summary>
        /// N[hɂJn{^bZ[W؂ւ
        /// </summary>
        private void cdmtrChangeCountStartButtonText()
        {
            switch (this.iExecuteMode)
            {
                case (int)EXEC_MODE.STRUCT:
                    CountStartButton.Text = CodeMetricsGui.Properties.Resources.Gui_19 
                                            + System.Environment.NewLine 
                                            + CodeMetricsGui.Properties.Resources.Gui_24;
                    break;
                case (int)EXEC_MODE.DIFF:
                    CountStartButton.Text = CodeMetricsGui.Properties.Resources.Gui_21 
                                            + System.Environment.NewLine 
                                            + CodeMetricsGui.Properties.Resources.Gui_24;
                    break;
                case (int)EXEC_MODE.CK:
                    CountStartButton.Text = CodeMetricsGui.Properties.Resources.Gui_22
                                            + System.Environment.NewLine
                                            + CodeMetricsGui.Properties.Resources.Gui_24;
                    break;
                case (int)EXEC_MODE.ROW:
                default:
                    CountStartButton.Text = CodeMetricsGui.Properties.Resources.Gui_23
                                            + System.Environment.NewLine
                                            + CodeMetricsGui.Properties.Resources.Gui_24; 
                    break;
            }
            return;
        }

        /// <summary>
        /// ݒ蒆̐ݒۑ
        /// </summary>
        private void cdmtrSaveSetMainInfo()
        {
            guiSetInfo.iExecMode = this.iExecuteMode;

            guiSetInfo.iRowInputFileNum = RowListBox.Items.Count;
            guiSetInfo.strRowInputFileArray = new string[guiSetInfo.iRowInputFileNum];
            if (guiSetInfo.strRowInputFileArray != null)
            {
                for (int i = 0; i < guiSetInfo.iRowInputFileNum; i++)
                {
                    guiSetInfo.strRowInputFileArray[i] = RowListBox.Items[i].ToString();
                }
            }

            guiSetInfo.strDiffOldInputFile = DiffOldTextBox.Text;
            guiSetInfo.strDiffNewInputFile = DiffNewTextBox.Text;

            guiSetInfo.strOutFolderPath = OutFolderPathTextBox.Text;
            guiSetInfo.strOutFileName = OutFileNameTextBox.Text;
            guiSetInfo.bAddSuffixCheck = AddSuffixChkBox.Checked;

            return;
        }


        /// <summary>
        /// tH[IOɌ݂̋Nۑ
        /// </summary>
        private void CDMtrGui_FormClosed(object sender, FormClosingEventArgs e)
        {
            cdmtrSaveSetMainInfo();
            guiSetInfo.CDMtrGuiSetInfoSave(CDMtrGuiPath.CDMtrPathCombine(strStartupPath, FILENAME_INI));

            guiCount.CDMtrCloseThread();

            CDMtrGuiDebug.close();

            return;
        }

        /// <summary>
        /// j[c[o[(I)
        /// </summary>
        private void ExitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
            return;
        }

        /// <summary>
        /// j[c[o[(wv\)
        /// </summary>
        private void HelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!this.bButtonClicked)
            {
                this.bButtonClicked = true;

                try
                {
                    System.Diagnostics.Process.Start(FILENAME_HELP);
                }
                catch (Exception ex)
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.Gui_18 + FILENAME_HELP);
                    CDMtrGuiDebug.write(ex.ToString());
                }
                finally
                {
                    this.bButtonClicked = false;
                }
            }
            return;
        }

        /// <summary>
        /// j[c[o[(o[W\)
        /// </summary>
        private void ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CDMtrGuiDispVer guiDispVer;

            using (guiDispVer = new CDMtrGuiDispVer())
            {
                guiDispVer.ShowDialog();
            }
            return;
        }

        /// <summary>
        /// j[c[o[(ڍאݒ)
        /// </summary>
        private void MenuDetailSet_Click(object sender, EventArgs e)
        {
            cdmtrDispSetDetail();
            return;
        }

        /// <summary>
        /// {CWI{^
        /// </summary>
        private void RowRadioButton_Click(object sender, EventArgs e)
        {
            cdmtrCheckRadioButton((int)EXEC_MODE.ROW);
            cdmtrChangeCountStartButtonText();
            return;
        }

        /// <summary>
        /// R[h\gNXWI{^
        /// </summary>
        private void StructRadioButton_Click(object sender, EventArgs e)
        {
            cdmtrCheckRadioButton((int)EXEC_MODE.STRUCT);
            cdmtrChangeCountStartButtonText();
            return;
        }

        /// <summary>
        /// CWI{^
        /// </summary>
        private void DiffRadioButton_Click(object sender, EventArgs e)
        {
            cdmtrCheckRadioButton((int)EXEC_MODE.DIFF);
            cdmtrChangeCountStartButtonText();
            return;
        }

        /// <summary>
        /// CKgNXWI{^
        /// </summary>
        private void CkRadioButton_Click(object sender, EventArgs e)
        {
            cdmtrCheckRadioButton((int)EXEC_MODE.CK);
            cdmtrChangeCountStartButtonText();
            return;
        }

        /// <summary>
        /// o͐tH_pXI_CAO
        /// </summary>
        private void OutputFileDirDialog_Click(object sender, EventArgs e)
        {
            string strFilePath;
            string strSelectPath;

            strSelectPath = CDMtrGuiPath.CDMtrPathGetDirName(OutFolderPathTextBox.Text);

            strFilePath = CDMtrGuiDialog.CDMtrDialogOpenDir(strSelectPath);
            if (strFilePath != null)
            {
                OutFolderPathTextBox.Text = strFilePath;
            }
            else
            {
                
            }
            return;

        }

        /// <summary>
        /// hbOCxgŋʂɃR[郁\bh
        /// (LɂɂFormAllowDropvpeBTrueɂ)
        /// </summary>
        private void GuiMainBoxDragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;
        }

        /// <summary>
        /// hbvCxgŋʂɃR[郁\bh
        /// (LɂɂFormAllowDropvpeBTrueɂ)
        /// </summary>
        private void GuiMainBoxDragDrop(object sender, DragEventArgs e)
        {
            string[] s = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (sender.GetType().Equals(typeof(TextBox)))
            {
                if (s.Length > 0) ((TextBox)sender).Text = s[0].Trim();
            }
            else if (sender.GetType().Equals(typeof(ListBox)))
            {
                for (int i = 0; i < s.Length; i++)
                    ((ListBox)sender).Items.Add(s[i].Trim());
            }
        }

        /// <summary>
        /// tH[[h
        /// </summary>
        private void CDMtrGui_Load(object sender, EventArgs e)
        {

#if OSS
            this.CkRadioButton.Visible = false;
#else
            
#endif
        }

    }
}