﻿/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Threading;

namespace CodeMetricsGui
{
    /// <summary>
    /// カウント用クラス
    /// </summary>
    class CDMtrGuiCount
    {
        private CDMtrGui guiMain = null;

        private const string strToolName = @"ncmeter.exe";

        private string strArg = "";

        private string strLogName = null;

        private StreamWriter srLogStream = null;

        private Thread tThread = null;

        private Process pProcess = null;

        private const int DELETE_RETRY_MAX = 5;

        delegate void TextBoxChangeDeligate(string strText);

        delegate void ControlChangeDeligate(bool bExitFlg);

        TextBoxChangeDeligate tStdOutDeligate = null;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public CDMtrGuiCount(CDMtrGui guiMain, string strLogName)
        {
            this.guiMain = guiMain;
            this.strLogName = strLogName;
            this.tStdOutDeligate = new TextBoxChangeDeligate(guiMain.CDMtrOutputTextBoxAdd);
            return;
        }

        /// <summary>
        /// スレッドを生成しラインカウントを開始
        /// </summary>
        public void CDMtrCountStart(string strArg)
        {
            ThreadStart tThreadStart = null;

            if(this.guiMain == null ||
                this.tStdOutDeligate == null){
                    cdmtrWriteLog(CodeMetricsGui.Properties.Resources.GuiCount_01);
                    return;
            }

            this.strArg = strArg;

            tThreadStart = new ThreadStart(cdmtrProcessStart);
            if (tThreadStart != null)
            {
                this.tThread = new Thread(tThreadStart);
                if (this.tThread != null)
                {
                    this.tThread.IsBackground = true;
                    try
                    {
                        this.tThread.Start();
                    }
                    catch (Exception ex)
                    {
                        CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_02 + ex.Message);
                        CDMtrGuiDebug.write(ex.ToString());
                    }
                }
                else
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_03);
                }
            }
            else
            {
                CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_04);
            }
            return;
        }

        /// <summary>
        /// プロセス終了
        /// </summary>
        public void CDMtrProcessKill()
        {
            if (this.pProcess != null)
            {
                try
                {
                    if (!this.pProcess.HasExited)
                    {
                        using (Process p = new Process())
                        {
                            try
                            {
                                p.StartInfo.FileName = Environment.GetEnvironmentVariable("ComSpec");
                                p.StartInfo.CreateNoWindow = true;
                                p.StartInfo.UseShellExecute = false;
                                p.StartInfo.RedirectStandardInput = false;
                                p.StartInfo.RedirectStandardOutput = false;
                                p.StartInfo.Arguments = @"/C TASKKILL /T /F /PID " + this.pProcess.Id.ToString();
                                CDMtrGuiDebug.write("kill chiled process(" + p.StartInfo.FileName + " " + p.StartInfo.Arguments +")");
                                p.Start();
                                p.WaitForExit();
                            }
                            catch (Exception ex)
                            {
                                CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_05 + ex.Message);
                                CDMtrGuiDebug.write(ex.ToString());
                                return;
                            }
                            finally
                            {
                                p.Close();
                            }
                            this.pProcess.WaitForExit();
                            this.cdmtrDeleteTmpDir(this.pProcess.Id);
                        }
                    }
                }
                catch (Exception ex)
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_06 + ex.Message);
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }
            return;
        }

        /// <summary>
        /// ncmeterが使用するテンポラリディレクトリを削除する
        /// </summary>
        private void cdmtrDeleteTmpDir(int iProcessID)
        {
            string strTmpPath = CDMtrGuiPath.CDMtrPathGetTempDirectory();
            if (strTmpPath != "")
            {
                string strTmpDir = CDMtrGuiPath.CDMtrPathCombine(strTmpPath, "ncmeter" + iProcessID);
                if (!this.cdmtrDeleteDir(strTmpDir))
                {
                    CDMtrGuiDialog.CDMtrDialogErr(CodeMetricsGui.Properties.Resources.GuiCount_07 + strTmpDir);
                    CDMtrGuiDebug.write(CodeMetricsGui.Properties.Resources.GuiCount_08 + strTmpDir);
                }
            }

            return;
        }

        private bool cdmtrDeleteDir(string strDirName)
        {
            bool bRet = false;
            int iRetry = DELETE_RETRY_MAX;

            while (iRetry > 0)
            {
                try
                {
                    if (CDMtrGuiPath.CDMtrPathIsExistDir(strDirName))
                    {
                        System.IO.Directory.Delete(strDirName, true);
                        CDMtrGuiDebug.write(CodeMetricsGui.Properties.Resources.GuiCount_09 + strDirName );
                        bRet = true;
                        break;
                    }
                    else
                    {
                        bRet = true;
                        break;
                    }
                }
                catch (IOException ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                    Thread.Sleep(100);
                    iRetry--;
                    CDMtrGuiDebug.write(CodeMetricsGui.Properties.Resources.GuiCount_10 + (DELETE_RETRY_MAX - iRetry));
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                    break;
                }
            }
            return bRet;
        }


        /// <summary>
        /// プロセス起動
        /// </summary>
        private void cdmtrProcessStart()
        {
            ControlChangeDeligate cCtrlDeligate;
            bool bExitFlg = false;

            if ((cCtrlDeligate = new ControlChangeDeligate(guiMain.CDMtrControlChange)) != null)
            {
                guiMain.Invoke(cCtrlDeligate, new object[] { true });

                try
                {
                    bExitFlg = cdmtrProcessStartCount();
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
                finally
                {
                    guiMain.Invoke(cCtrlDeligate, new object[] { bExitFlg });
                }
            }

            return;
        }

        /// <summary>
        /// カウント開始
        /// </summary>
        private bool cdmtrProcessStartCount()
        {
            bool bRet;
            int iExitCode;
            string strMsg;

            bRet = false;
            iExitCode = -1;

            CDMtrOpenLogfile();

            strMsg = CodeMetricsGui.Properties.Resources.GuiCount_11;
            cdmtrWriteLog(strMsg);

            this.pProcess = new Process();
            if (this.pProcess != null)
            {
                this.pProcess.StartInfo.FileName = strToolName;
                this.pProcess.StartInfo.CreateNoWindow = true;
                this.pProcess.StartInfo.UseShellExecute = false;
                this.pProcess.StartInfo.RedirectStandardInput = false;
                this.pProcess.StartInfo.RedirectStandardOutput = true;
                this.pProcess.StartInfo.Arguments = strArg;

                cdmtrWriteLog(this.pProcess.StartInfo.FileName + " " + this.pProcess.StartInfo.Arguments);
				cdmtrWriteLog("");

                try
                {
                    this.pProcess.Start();

                    do
                    {
                        try
                        {
                            strMsg = this.pProcess.StandardOutput.ReadLine();
                            cdmtrWriteLog(strMsg);
                        }
                        catch
                        {
                            cdmtrWriteLog(CodeMetricsGui.Properties.Resources.GuiCount_12);
                        }
                    } while (!this.pProcess.HasExited);

                    this.pProcess.WaitForExit();

                    strMsg = this.pProcess.StandardOutput.ReadToEnd();
                    cdmtrWriteLog(strMsg);
                }
                catch (Exception ex)
                {
                    cdmtrWriteLog(CodeMetricsGui.Properties.Resources.GuiCount_13 + ex.Message);
                }

                iExitCode = this.pProcess.ExitCode;
                this.pProcess.Close();
                this.pProcess.Dispose();
                this.pProcess = null;
            }

            if (iExitCode == 0)
            {
                strMsg = CodeMetricsGui.Properties.Resources.GuiCount_14;
                bRet = true;
            }
            else
            {
                strMsg = CodeMetricsGui.Properties.Resources.GuiCount_15;
                bRet = false;
            }
            cdmtrWriteLog("");
            cdmtrWriteLog(strMsg);



            CDMtrCloseLogFile();

            return bRet;
        }

        /// <summary>
        /// スレッド/プロセス終了
        /// </summary>
        public void CDMtrCloseThread()
        {
            if (this.tThread != null && this.tThread.IsAlive == true)
            {
                this.CDMtrProcessKill();

                try
                {
                    this.tThread.Abort();
                }
                catch (System.Security.SecurityException ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }
            CDMtrCloseLogFile();
            return;
        }

        /// <summary>
        /// ログファイルオープン
        /// </summary>
        public void CDMtrOpenLogfile()
        {
            if (this.srLogStream == null && this.strLogName != null)
            {
                try
                {
                    this.srLogStream = new StreamWriter((new FileStream(strLogName, FileMode.Create)), Encoding.Default);
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }

            return;
        }

        /// <summary>
        /// ログファイルクローズ
        /// </summary>
        public void CDMtrCloseLogFile()
        {
            if (srLogStream != null)
            {
                srLogStream.Close();
                srLogStream.Dispose();
                srLogStream = null;
            }
            return;
        }

        /// <summary>
        /// テキストボックスとログファイルにログ出力
        /// </summary>
        private void cdmtrWriteLog(string strTxt)
        {
            if (this.guiMain != null)
            {
                try
                {
                    guiMain.Invoke(this.tStdOutDeligate, new object[] { strTxt });
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }

            if (srLogStream != null)
            {
                try
                {
                    srLogStream.Write(strTxt + "\r\n");
                }
                catch (Exception ex)
                {
                    CDMtrGuiDebug.write(ex.ToString());
                }
            }
            return;
        }
    }
}
