﻿// Copyright (c) 2015 NEC Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
// to whom the Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Text;

using System.Diagnostics;

using ICSharpCode.NRefactory.CSharp;
using ICSharpCode.NRefactory.CSharp.TypeSystem;
using ICSharpCode.NRefactory.Editor;
using ICSharpCode.NCMCSParser.Properties;

namespace ICSharpCode.NCMCSParser
{
    #region C#言語対応のメインクラス

    public class NCMCSParser
    {
        private const int CST_NCMCS_OK = (0);
        private const int CST_NCMCS_NG = (-1);
        private static readonly String CST_NCMCS_CRLF = Environment.NewLine;

        private const String CST_CSHARP_LANGTYPE = "CS";

        private enum LocType { NON, ELOC, CLOC, BLOC, MLOC, SKIP };

        private static NCMStructData structdata;

        #region メイン関数
        /**
         * CSパーサメイン関数
         */
        [STAThread]
        static int Main(string[] args)
        {

            int csharp_exit = CST_NCMCS_OK;

            List<string> errors = null;

            try
            {

                errors = new List<string>();
                structdata = new NCMStructData();
            }
            catch (OutOfMemoryException ex)
            {

                Console.WriteLine(Resources.MSG_LOG_ID_CS_00000001_01 + ex.Message); //* MSG_LOG_ID_CS_00000001
                return CST_NCMCS_NG;
            }

            #region parameter check

            if (!parseOptions(args, ref structdata))
            {

                outputErrorFile(ref structdata);
                return CST_NCMCS_NG;
            }



            int lastSeparate = structdata.target_filepath.LastIndexOf('\\');
            if (lastSeparate != -1)
            {
                structdata.DirPath = structdata.target_filepath.Substring(0, lastSeparate);
                structdata.FileName = structdata.target_filepath.Substring(lastSeparate + 1);
            }
            else
            {
                structdata.DirPath = structdata.target_filepath;
            }

            structdata.LangType = CST_CSHARP_LANGTYPE;
            #endregion


            int parseres = ParseFile(ref errors, ref structdata);
            if (errors.Count != 0 || parseres != CST_NCMCS_OK)
            {

                outputErrorFile(ref structdata);
                csharp_exit = CST_NCMCS_NG;
            }
            else
            {

                lineCount(structdata);


                if (!outputMetricFile(ref structdata))
                {

                    csharp_exit = CST_NCMCS_NG;
                }
                outputErrorFile(ref structdata);
            }

#if DEBUG
            
#endif
            return csharp_exit;
        }
        #endregion

        #region オプション解析
        /**
         * オプションを解析し、測定情報に設定.
         */
        private static bool parseOptions(string[] args, ref NCMStructData structdata)
        {
            const String CNT_STR_OLD = "old";
            const String CNT_STR_NEW = "new";
            const String CNT_STR_PARAM = "param";
            const String CNT_STR_NONE = "none";
            bool res = false;

            for (int i = 0; i < args.Length; i++)
            {
                String arg = args[i];
                if (arg == "--metric_file" || arg == "-m")
                {

                    if (i + 1 >= args.Length || structdata.metric_filepath != null)
                    {
                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000002_01); //* MSG_LOG_ID_CS_00000002
                        return res;
                    }
                    structdata.metric_filepath = args[++i];
                }
                else if (arg == "--target_file_path" || arg == "-t")
                {

                    if (i + 1 >= args.Length || structdata.target_filepath != null)
                    {
                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000003_01); //* MSG_LOG_ID_CS_00000003
                        return res;
                    }
                    structdata.target_filepath = args[++i];
                }
                else if (arg == "--error_output" || arg == "-e")
                {

                    if (i + 1 >= args.Length || structdata.error_filepath != null)
                    {
                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000004_01); //* MSG_LOG_ID_CS_00000004
                        return res;
                    }
                    structdata.error_filepath = args[++i];
                }
                else if (arg == "--debug")
                {

                    if (structdata.debugLevel == 0)
                        structdata.debugLevel = 1;
                }
                else if (arg == "--debug2")
                {

                    if (structdata.debugLevel <= 1)
                        structdata.debugLevel = 2;
                }
                else if (arg == "--charset" || arg == "-c")
                {

                    if (i + 1 >= args.Length || structdata.charset_name != null)
                    {
                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000005_01); //* MSG_LOG_ID_CS_00000005
                        return res;
                    }
                    structdata.charset_name = args[++i];
                }
                else if (arg == "--skip" || arg == "-s")
                {

                    if (i + 1 >= args.Length)
                    {
                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000006_01); //* MSG_LOG_ID_CS_00000006
                        return res;
                    }
                    String wkarg = args[++i].Trim().ToLower();
                    if (wkarg.Equals(CNT_STR_OLD))
                    {

                        structdata.outofskip = false;
                    }
                    else if (!wkarg.ToLower().Equals(CNT_STR_NEW))
                    {

                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000007_01); //* MSG_LOG_ID_CS_00000007
                        return res;
                    }
                }
                else if (arg == "--func_opt" || arg == "-f")
                {

                    if (i + 1 >= args.Length)
                    {

                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000008_01); //* MSG_LOG_ID_CS_00000008
                        return res;
                    }
                    String wkarg = args[++i].Trim().ToLower();
                    if (wkarg.Equals(CNT_STR_PARAM))
                    {

                        structdata.funcopt = "param";
                    }
                    else if (wkarg.ToLower().Equals(CNT_STR_NONE))
                    {

                        structdata.funcopt = "none";
                    }
                    else
                    {

                        structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000009_01); //* MSG_LOG_ID_CS_00000009
                        return res;
                    }
                }
                else if (File.Exists(arg))
                {
                    structdata.parse_filepath = arg;
                }
                else
                {

                    structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000010_01 + arg + ""); //* MSG_LOG_ID_CS_00000010
                    return res;
                }
            }


            if (structdata.parse_filepath == null)
            {

                structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000011_01); //* MSG_LOG_ID_CS_00000011
                return res;
            }
            else if (structdata.error_filepath == null)
            {

                structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000012_01); //* MSG_LOG_ID_CS_00000012
                return res;
            }
            else if (structdata.metric_filepath == null)
            {

                structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000013_01); //* MSG_LOG_ID_CS_00000013
                return res;
            }
            else if (structdata.target_filepath == null)
            {

                structdata.dbugbufPrintLn(0, Resources.MSG_LOG_ID_CS_00000014_01); //* MSG_LOG_ID_CS_00000014
                return res;
            }

            if (1 > structdata.funcopt.Length)
            {

                structdata.funcopt = "param";
            }

            return true;

        }
        #endregion

        #region 解析エラーのログ出力
        /**
         * Parseで発生したエラーからメッセージを作成し、測定情報のログバッファに設定.
         */
        private static void PrintErrors(CSharpParser csp, NCMStructData structdata)
        {
            if (csp.HasErrors)
            {

                foreach (var error in csp.ErrorPrinter.Errors)
                {
                    if (error.Region != null && error.Region.BeginLine > 0)
                    {

                        structdata.dbugbufPrintLn(1, String.Format(Resources.MSG_LOG_ID_CS_00000015_01 +
                                                               "{0}" + Resources.MSG_LOG_ID_CS_00000015_02 +
                                                               "{1}" + Resources.MSG_LOG_ID_CS_00000015_03 +
                                                               "{2}" + Resources.MSG_LOG_ID_CS_00000015_04 + "{3}"
                            , error.ErrorType.ToString(), error.Message, error.Region.BeginLine, error.Region.BeginColumn)); //* MSG_LOG_ID_CS_00000015
                    }
                    else
                    {

                        structdata.dbugbufPrintLn(1, String.Format(Resources.MSG_LOG_ID_CS_00000016_01 + "{0}" + Resources.MSG_LOG_ID_CS_00000016_02 + "{1}", error.ErrorType.ToString(), error.Message)); //* MSG_LOG_ID_CS_00000016
                    }
                }
            }
        }
        #endregion

        #region Ｃ＃パーサ処理
        /**
         * パーサ処理関数。字句解析、構文解析、visitorによる意味解析、ラインカウンタ処理を行う.
         */
        private static int ParseFile(ref List<string> errors, ref NCMStructData structdata)
        {
            int res = CST_NCMCS_OK;

            if (structdata == null)
            {

                return CST_NCMCS_NG;
            }
            if (structdata.parse_filepath == null)
            {

                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000017_01); //* MSG_LOG_ID_CS_00000017
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000018_01); //* MSG_LOG_ID_CS_00000018
                return CST_NCMCS_NG;
            }

            ITextSource Content = null;
            try
            {
                Content = new StringTextSource(File.ReadAllText(structdata.parse_filepath));
            }
            catch (Exception ex)
            {
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000019_01); //* MSG_LOG_ID_CS_00000019
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000020_01 + ex.Message); //* MSG_LOG_ID_CS_00000020
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000021_01 + ex.GetType().ToString()); //* MSG_LOG_ID_CS_00000021
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000022_01 + ex.StackTrace.ToString()); //* MSG_LOG_ID_CS_00000022
                res = CST_NCMCS_NG;
            }

            List<string> args = new List<string>();


            CSharpParser p = new CSharpParser(args.ToArray());


            CompilationUnit compilationUnit = null;
            try
            {
                compilationUnit = p.Parse(Content.CreateReader(), structdata.parse_filepath);
            }
            catch (Exception e)
            {
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000023_01); //* MSG_LOG_ID_CS_00000023
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000024_01); //* MSG_LOG_ID_CS_00000024
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000025_01 + e.StackTrace); //* MSG_LOG_ID_CS_00000025
                return CST_NCMCS_NG;
            }


            PrintErrors(p, structdata);

            if (compilationUnit == null)
            {

                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000026_01); //* MSG_LOG_ID_CS_00000026
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000027_01); //* MSG_LOG_ID_CS_00000027
                return CST_NCMCS_NG;
            }

            try
            {

                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000028_01); //* MSG_LOG_ID_CS_00000028

                NCMCSVisitor visitor = new NCMCSVisitor();
                if (visitor == null)
                {
                    structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000029_01); //* MSG_LOG_ID_CS_00000029
                    structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000030_01); //* MSG_LOG_ID_CS_00000030
                    return CST_NCMCS_NG;
                }
                else
                {
                    compilationUnit.AcceptVisitor(visitor, (object)structdata);
                    structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000031_01); //* MSG_LOG_ID_CS_00000031
                }
            }
            catch (Exception ex)
            {

                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000032_01); //* MSG_LOG_ID_CS_00000032
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000033_01 + ex.Message); //* MSG_LOG_ID_CS_00000033
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000034_01 + ex.Message + Resources.MSG_LOG_ID_CS_00000034_02 +
                                              ex.GetType().ToString() + Resources.MSG_LOG_ID_CS_00000034_03); //* MSG_LOG_ID_CS_00000034
                structdata.dbugbufPrintLn(2, Resources.MSG_LOG_ID_CS_00000035_01 + ex.StackTrace.ToString()); //* MSG_LOG_ID_CS_00000035
                return CST_NCMCS_NG;
            }

            return res;
        }
        #endregion

        #region ラインカウンタ関数
        /**
         * ラインカウンタ関数.TokenリストやPrePro情報から関数内のFLOC,ELOC,CLOC,BLOCをカウントする.
         */
        private static void lineCount(NCMStructData structdata)
        {

            foreach (var funcdata in structdata.FuncList)
            {

                funcdata.Value.cntFloc =
                    funcdata.Value.BodyEndToken.Line - funcdata.Value.FuncStartToken.Line + 1;


                funcdata.Value.cntBloc =
                    funcdata.Value.cntFloc - funcdata.Value.cntEloc - funcdata.Value.cntCloc - funcdata.Value.cntSkip;

                if (!structdata.outofskip)
                {

                    funcdata.Value.cntEloc += funcdata.Value.cntSkip;
                }
            }
        }
        #endregion

        #region 結果CSVファイルへの出力関数
        /**
         * 結果CSVファイルへの出力関数.
         */
        private static bool outputMetricFile(ref NCMStructData structdata)
        {

            System.Text.StringBuilder csvbuff = null;
            SortedDictionary<ulong, NCMStructData.NCMFuncData> funclist = null;


            String outpath = null;
            bool res = true;

            if (structdata == null)
            {
                return false;
            }

            if (structdata.metric_filepath == null)
            {
                return false;
            }
            outpath = structdata.metric_filepath;





            try
            {
                csvbuff = new System.Text.StringBuilder();
            }
            catch (OutOfMemoryException ex)
            {

                Console.WriteLine(Resources.MSG_LOG_ID_CS_00000036_01 + ex.Message); //* MSG_LOG_ID_CS_00000036
                return false;
            }

            int functionCount = structdata.functionCount();
            if (functionCount <= 0)
            {

                csvbuff.Append("\"" + structdata.DirPath + "\",\"" + structdata.FileName + "\",-," + structdata.LangType);
                csvbuff.Append(",-,-,-,-,-,-,-,-,-,-,-,-,-" + CST_NCMCS_CRLF);
            }
            else
            {


                try
                {
                    funclist = new SortedDictionary<ulong, NCMStructData.NCMFuncData>(structdata.FuncList);
                }
                catch (OutOfMemoryException ex)
                {

                    Console.WriteLine(Resources.MSG_LOG_ID_CS_00000037_01 + ex.Message); //* MSG_LOG_ID_CS_00000037
                    return false;
                }

                foreach (KeyValuePair<ulong, NCMStructData.NCMFuncData> fncdata in funclist)
                {

                    csvbuff.Append("\"" + structdata.DirPath + "\",\"" + structdata.FileName + "\"");


                    if (structdata.funcopt.CompareTo("param") == 0)
                    {
                        csvbuff.Append(",\"" + fncdata.Value.nameFunc + fncdata.Value.funcParams + "\"," + structdata.LangType);
                    }
                    else
                    {
                        csvbuff.Append(",\"" + fncdata.Value.nameFunc + "\"," + structdata.LangType);
                    }

                    csvbuff.Append("," + ((fncdata.Value.cntFloc < 0) ? "0" : fncdata.Value.cntFloc.ToString()) +
                                   "," + ((fncdata.Value.cntEloc < 0) ? "0" : fncdata.Value.cntEloc.ToString()) +
                                   "," + ((fncdata.Value.cntCloc < 0) ? "0" : fncdata.Value.cntCloc.ToString()) +
                                   "," + ((fncdata.Value.cntBloc < 0) ? "0" : fncdata.Value.cntBloc.ToString()));
                    csvbuff.Append("," + ((fncdata.Value.cntCyclo < 0) ? "0" : fncdata.Value.cntCyclo.ToString()) +
                                   "," + ((fncdata.Value.cntCycloWC < 0) ? "0" : fncdata.Value.cntCycloWC.ToString()) +
                                   "," + ((fncdata.Value.cntBranchs < 0) ? "0" : fncdata.Value.cntBranchs.ToString()) +
                                   "," + ((fncdata.Value.cntCndOps < 0) ? "0" : fncdata.Value.cntCndOps.ToString()) +
                                   "," + ((fncdata.Value.cntNests < 0) ? "0" : fncdata.Value.cntNests.ToString()) +
                                   ",-,-,-,-" + CST_NCMCS_CRLF);
                }
            }


            try
            {

                Encoding typeEncode = Encoding.GetEncoding("UTF-8");
                File.WriteAllText(outpath, csvbuff.ToString(), typeEncode);
            }
            catch (IOException ex)
            {
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000038_01 + outpath); //* MSG_LOG_ID_CS_00000038
                structdata.dbugbufPrintLn(1, Resources.MSG_LOG_ID_CS_00000039_01 + ex.Message); //* MSG_LOG_ID_CS_00000039
                res = false;
            }
            return (res);
        }
        #endregion

        #region エラーログファイルへの出力関数
        /**
         * エラーログファイルへの出力関数.
         */
        private static bool outputErrorFile(ref NCMStructData structdata)
        {

            String outpath = null;
            bool res = false;

            if (structdata == null)
            {
                return res;
            }

            if (structdata.error_filepath == null)
            {
                return res;
            }
            outpath = structdata.error_filepath;


            try
            {

                Encoding typeEncode = Encoding.GetEncoding("UTF-8");
                File.WriteAllText(outpath, structdata.dbugbufread(), typeEncode);
            }
            catch (IOException ex)
            {
                Console.WriteLine(Resources.MSG_LOG_ID_CS_00000040_01 + ex.Message);  //* MSG_LOG_ID_CS_00000040
                return res;
            }
            return (true);
        }
        #endregion

    }

    #endregion

}
