﻿// Copyright (c) 2015 NEC Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
// to whom the Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.Text;
using ICSharpCode.NRefactory;
using System.Diagnostics;



namespace ICSharpCode.NCMCSParser
{
    public class NCMStructData
    {
        protected static readonly String CST_NCMCS_CRLF = Environment.NewLine;

        public String target_filepath = null;
        public String parse_filepath = null;
        public String error_filepath = null;
        public String metric_filepath = null;
        public String charset_name = null;

        public bool outofskip = true;
        public String funcopt = "";
        public int curfuncline = 0;
        public int curfunccol = 0;
        public ulong curfunckey = 0;

        public String DirPath = "";
        public String FileName = "";
        public String FuncName = "";
        public String LangType = "";

        private DebugLog DebugLogInfo = new DebugLog();
        private Dictionary<ulong, NCMFuncData> funclist = new Dictionary<ulong, NCMFuncData>();


        #region ログ出力用パラメータ＆メソッド



        public int debugLevel
        {
            set { this.DebugLogInfo.DebugLevel = value; }
            get { return this.DebugLogInfo.DebugLevel; }
        }
        public void dbugbufPrint(int lvl, String arg)
        {
            this.DebugLogInfo.dbugbufPrint(lvl, arg);
        }
        public void dbugbufPrintLn(int lvl)
        {
            this.DebugLogInfo.dbugbufPrintLn(lvl);
        }
        public void dbugbufPrintLn(int lvl, String arg)
        {
            this.DebugLogInfo.dbugbufPrintLn(lvl, arg);
        }
        public String dbugbufread()
        {
            return this.DebugLogInfo.ToString();
        }

        #endregion

        #region ログ情報クラス

        sealed class DebugLog
        {

            StringBuilder _dbugbuf = new StringBuilder();
            int _stpoint_dbugbuf = 0;
            int _debugLevel = 0;

            public DebugLog()
            {
            }
            public DebugLog(int debugLevel)
            {
                this._debugLevel = debugLevel;
            }

            public int DebugLevel
            {
                get { return this._debugLevel; }
                set { this._debugLevel = value; }
            }
            /**
             * "DEBUG"ビルド用デバッグログに文字列を追加.
             */
            [Conditional("DEBUG")]
            public void dbugbufPrintLevel3(int lvl, String arg)
            {
                if (lvl == 3) _dbugbuf.Append(arg);
            }

            /**
             * デバッグログに文字列を追加.
             */
            public void dbugbufPrint(int lvl, String arg)
            {
                if (_debugLevel >= lvl) _dbugbuf.Append(arg);
                dbugbufPrintLevel3(lvl, arg);
            }

            /**
             * デバッグログに改行コードを追加.
             */
            public void dbugbufPrintLn(int lvl)
            {
                if (_debugLevel >= lvl) _dbugbuf.Append(CST_NCMCS_CRLF);
                dbugbufPrintLevel3(lvl, CST_NCMCS_CRLF);
            }

            /**
             * デバッグログに文字列と改行コードを追加.
             */
            public void dbugbufPrintLn(int lvl, String arg)
            {
                if (_debugLevel >= lvl) _dbugbuf.Append(arg + CST_NCMCS_CRLF);
                dbugbufPrintLevel3(lvl, arg + CST_NCMCS_CRLF);
            }

            /**
             * デバッグログの内容を文字列で返却.
             */
            public override string ToString()
            {
                return _dbugbuf.ToString();
            }

            /**
             * デバッグログの内容を前回コールした内容から保存されている所まで返却.
             */
            public String dbugbufOutput(int lvl)
            {

                String strRes = "";
                if (_debugLevel >= lvl)
                {
                    String strbuf = _dbugbuf.ToString();
                    if (strbuf.Length > _stpoint_dbugbuf)
                    {
                        strRes = strbuf.Substring(_stpoint_dbugbuf);
                        _stpoint_dbugbuf = strbuf.Length;
                    }
                }
                return strRes;
            }
        }

        #endregion

        #region 関数計測情報クラス


        public class NCMFuncData
        {

            public int cntFloc = 0;
            public int cntEloc = 0;
            public int cntCloc = 0;
            public int cntBloc = 0;
            public int cntSkip = 0;
            public int cntCyclo = 0;
            public int cntCycloWC = 0;
            public int cntBranchs = 0;
            public int cntCndOps = 0;
            public int cntNests = 0;
            public int cntNestsLevel = 0;





            public int cntFunctionBase = 0;
            public int cntLoopsDoWhile = 0;
            public int cntLoopsFor = 0;
            public int cntConditionalIf = 0;
            public int cntSelectionSwitch = 0;
            public int cntSelectionCase = 0;
            public int cntSelectionDefault = 0;
            public int cntInlinedIf = 0;
            public int cntLogicalAnd = 0;

            public String nameFunc = "";

            public String funcParams = "";


            public TextLocation FuncStartToken;
            public TextLocation BodyStartToken;
            public TextLocation BodyEndToken;

            public Dictionary<int, NCMLineData> counterList = null;

            /**
             * 関数データを初期化する(未使用)
             */
            public void CounterClear()
            {

                cntFloc = 0;
                cntEloc = 0;
                cntCloc = 0;
                cntBloc = 0;
                cntCyclo = 0;
                cntCycloWC = 0;
                cntBranchs = 0;
                cntCndOps = 0;
                cntNests = 0;
                cntNestsLevel = 0;
                cntFunctionBase = 0;
                cntLoopsDoWhile = 0;
                cntLoopsFor = 0;
                cntConditionalIf = 0;
                cntSelectionSwitch = 0;
                cntSelectionCase = 0;
                cntSelectionDefault = 0;
                cntInlinedIf = 0;
                cntLogicalAnd = 0;
                nameFunc = "";

                funcParams = "";

            }

            /**
             * 関数データを返却する(未使用)
             */
            public String getCounter()
            {
                String txt = "[" + nameFunc + "]"
                            + "FLOC:" + cntFloc
                            + ",ELOC:" + cntEloc
                            + ",CLOC:" + cntCloc
                            + ",BLOC:" + cntBloc
                            + ",CYCLO:" + cntCyclo
                            + ",CYCLOWC:" + cntCycloWC
                            + ",BRANCHS:" + cntBranchs
                            + ",CNDOPS:" + cntCndOps
                            + ",NEST:" + cntNests
                            + ",FuncBase:" + cntFunctionBase
                            + ",DoWhile:" + cntLoopsDoWhile
                            + ",For:" + cntLoopsFor
                            + ",If:" + cntConditionalIf
                            + ",Switch:" + cntSelectionSwitch
                            + ",Case:" + cntSelectionCase
                            + ",Default:" + cntSelectionDefault
                            + ",InlinedIf:" + cntInlinedIf;
                return txt;
            }

            #region ラインカウント情報クラス
            public class NCMLineData
            {

                public int cntEloc = 0;
                public int cntCloc = 0;

                public bool isInactiveCode = false;


                public ushort cntCndOps = 0;
                public ushort cntLoopsDoWhile = 0;
                public ushort cntLoopsFor = 0;
                public ushort cntConditionalIf = 0;
                public ushort cntSelectionSwitch = 0;
                public ushort cntSelectionCase = 0;
                public ushort cntSelectionDefault = 0;
                public ushort cntLogicalAnd = 0;
                public ushort cntNests = 0;

                public override string ToString()
                {
                    return string.Format("{0,4},{1,4},{2,4},{3,4},{4,4},{5,4},{6,4},{7,4},{8,4},{9,4},{10,4},{11,4}",
                        (cntEloc == 0) ? "" : cntEloc.ToString(),
                        (cntCloc == 0) ? "" : cntCloc.ToString(),
                        (!isInactiveCode) ? "" : "SKIP",
                        (cntCndOps == 0) ? "" : cntCndOps.ToString(),
                        (cntLoopsDoWhile == 0) ? "" : cntLoopsDoWhile.ToString(),
                        (cntLoopsFor == 0) ? "" : cntLoopsFor.ToString(),
                        (cntConditionalIf == 0) ? "" : cntConditionalIf.ToString(),
                        (cntSelectionSwitch == 0) ? "" : cntSelectionSwitch.ToString(),
                        (cntSelectionCase == 0) ? "" : cntSelectionCase.ToString(),
                        (cntSelectionDefault == 0) ? "" : cntSelectionDefault.ToString(),
                        (cntLogicalAnd == 0) ? "" : cntLogicalAnd.ToString(),
                        (cntNests == 0) ? "" : cntNests.ToString()
                        );
                }
            }
            #endregion
        }

        #endregion

        #region 関数計測情報用メソッド


        /**
         * 関数単位のデータリストのサイズを返却.
         */
        public int functionCount()
        {
            return this.funclist.Count;
        }

        /**
         * 関数単位のデータリストのインデクサ.
         */
        public Dictionary<ulong, NCMFuncData> FuncList
        {
            get { return this.funclist; }
            set { this.funclist = value; }
        }

        /**
         * 関数単位のデータリストに新たに関数データ領域を追加.
         */
        public NCMFuncData functionNewData(TextLocation tkn)
        {
            NCMFuncData obj = new NCMFuncData();
            this.curfuncline = tkn.Line;
            this.curfunccol = tkn.Column;
            this.curfunckey = ((ulong)this.curfuncline * 10000) + (ulong)this.curfunccol;

            while (funclist.ContainsKey(this.curfunckey))
            {
                this.curfunckey++;
            }
            funclist.Add(this.curfunckey, obj);
            return obj;
        }

        /**
         * 関数単位のデータリストから最後に登録した関数の関数データを取得.
         */
        public NCMFuncData functionGetData()
        {
            NCMFuncData obj = null;
            if (this.curfuncline > 0)
            {
                obj = funclist[this.curfunckey];
            }
            return (obj);
        }

        #endregion

    }
}
