// Copyright (c) 2015 NEC Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this
// software and associated documentation files (the "Software"), to deal in the Software
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
// to whom the Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
using System.Diagnostics;
using ICSharpCode.NRefactory;
using ICSharpCode.NRefactory.CSharp;

namespace ICSharpCode.NCMCSParser
{
    [DebuggerDisplay("ID = {Token}, Role = {RoleName}, Start = {StartLocation}, End = {EndLocation}")]
    public class TokenData
    {
        private TextLocation startLocation;
        private TextLocation endLocation;
        private string token;
        private string nodeType;
        private string roleName;
        private string parentRole;

        public string ItemType = string.Empty;
        public int Data = -1;

        public TokenData(TextLocation sl, TextLocation ln)
        {
            this.token = string.Empty;
            this.nodeType = string.Empty;
            this.roleName = string.Empty;
            this.parentRole = string.Empty;
            this.startLocation = sl;
            this.endLocation = ln;
        }
        public TokenData(string tkn, string nt, string rn, TextLocation sl, TextLocation ln)
        {
            this.token = tkn;
            this.nodeType = nt;
            this.roleName = rn;
            this.parentRole = string.Empty;
            this.startLocation = sl;
            this.endLocation = ln;
        }
        public TokenData(string tkn, string nt, string rn, string pr, TextLocation sl, TextLocation ln)
        {
            this.token = tkn;
            this.nodeType = nt;
            this.roleName = rn;
            this.parentRole = pr;
            this.startLocation = sl;
            this.endLocation = ln;
        }

        public string Token
        {
            get { return token; }
            set { token = value; }
        }
        public string NodeType
        {
            get { return nodeType; }
            set { nodeType = value; }
        }
        public string RoleName
        {
            get { return roleName; }
            set { roleName = value; }
        }
        public string ParentRole
        {
            get { return parentRole; }
            set { parentRole = value; }
        }
        public TextLocation StartLocation
        {
            get { return startLocation; }
            set { startLocation = value; }
        }
        public TextLocation EndLocation
        {
            get { return endLocation; }
            set { endLocation = value; }
        }

        public PreProcessorDirectiveType PpType { get; set; }
        public bool PpTake { get; set; }

        public string ToLongString()
        {
            return "Token:'" + token + "',NodeType:'" + nodeType + "',RoleName:'" + roleName + "',ParentRole:'" + parentRole
                + "',StartLine:" + StartLocation.Line + "(" + StartLocation.Column
                + "), EndLine: " + EndLocation.Line + "(" + EndLocation.Column + ")";
        }

        public override string ToString()
        {
            return string.Format(
                "{0,3},{1,3}:{2,3},{3,3}, Tkn={4}, NdType={5}, Role={6}",
                startLocation.Line, startLocation.Column,
                endLocation.Line, endLocation.Column,
                (!string.IsNullOrEmpty(token)) ? token.Replace("\r", "\\r").Replace("\n", "\\n") : "",
                nodeType, roleName);
        }
    }
}
