/*
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Igor Bukanov, igor@fastmail.fm
 *   Norris Boyd,
 *   NEC Corporation.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */

package com.nec.jp.ncmeter.struct;

import java.util.ArrayList;
import java.util.ResourceBundle;

import com.nec.jp.ncmeter.cmn.file.NCMCmnFile;
import com.nec.jp.ncmeter.cmn.log.NCMCmnLog;

/**
 * コード構造メトリクス計測
 */
public class NCMStructData {

    private static final int initialFuncNum = 100;

    private static final int initialLineNum = 2000;

    private static final int initialStringBuff = 1024;

    private int cachedELOCLine = 0;

    private int cachedCLOCLine = 0;

    private int lastLine = 0;

    private NCMMetricsTable cachedMdata = null;

    private int iFunctionIndex = 0;

    private int nestOutFuncLevel = 0;

    private int nestInFuncLevel = 0;

    private boolean isInlineFunc = false;

    private boolean isCondition = false;

    private ArrayList<NCMFunctionTable> funcList;

    private ArrayList<NCMMetricsTable> metricsList;

    private static NCMStructData structData = new NCMStructData();

    private ResourceBundle myResources;

    /**
     * インスタンス作成
     */
    private NCMStructData() {

        funcList = new ArrayList<NCMFunctionTable>(initialFuncNum);
        metricsList = new ArrayList<NCMMetricsTable>(initialLineNum);

        NCMMetricsTable mdata = new NCMMetricsTable();
        mdata.line_number = 1;
        mdata.iStartOffset = 1;
        mdata.iEndOffset = 1;
        this.cachedMdata = mdata;
        metricsList.add(mdata);
    }

    /**
     * コード構造のデータ（インスタンス）を取得
     */
    public static NCMStructData getStructData() {
        return structData;
    }

    /**
     * コード構造のデータ（インスタンス）を取得
     */
    public boolean getCondition() {
        return this.isCondition;
    }

    /**
     * デバッグレベルの設定
     */
    public void setDebugLevel(NCMCmnLog.LEVEL level) {
        NCMCmnLog.setLogger(level);
    }

    /**
     * 関数情報保持リストへの登録関数
     */
    public void addFuncList(int index, String name, String param,
            int startLine, int endLine, int startColum, int nested) {

        NCMFunctionTable fdata = new NCMFunctionTable();
        fdata.func_id = index;
        fdata.func_name = name;
        fdata.func_startline = startLine;
        fdata.func_endline = endLine;
        fdata.param_name = param;
        fdata.internalfunc = nested;

        if (nested == 0) {

            funcList.add(fdata);
        } else {

        }

    }

    /**
     * 構造メトリクス情報保持リストへの登録関数
     */
    public void addMetricsList(int line, int offset) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }

        lastLine = this.cachedMdata.line_number;
        try {
            if (myResources != null) {
                String msg06301 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000063_01");
                String msg06302 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000063_02");
                NCMCmnLog.outputLogDebug(msg06301 + line + msg06302 + offset); // MSG_LOG_ID_JAVA_00000063
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        if (this.cachedMdata.iStartOffset < offset) {
            this.cachedMdata.iEndOffset = offset - 1;
        }

        if (this.cachedMdata.effective_line == NCMMetricsTable.LINEDTL_EFFECTIVE_NONE
                && this.cachedMdata.comment_line == NCMMetricsTable.LINEDTL_COMMENT_NONE
                && this.cachedMdata.blank_line == NCMMetricsTable.LINEDTL_BLANK_NONE) {

            this.cachedMdata.line_number = line;
            this.cachedMdata.iStartOffset = offset;
            this.cachedMdata.iEndOffset = offset;
        } else {

            NCMMetricsTable mdata = new NCMMetricsTable();
            mdata.line_number = line;
            mdata.iStartOffset = offset;
            mdata.iEndOffset = offset;
            if (isInlineFunc) {

                mdata.func_id = iFunctionIndex;
                mdata.func_line = NCMMetricsTable.LINEDTL_FUNC_LINE_IN;
            } else {

                mdata.func_id = 0;
                mdata.func_line = NCMMetricsTable.LINEDTL_FUNC_LINE_OUT;
            }
            mdata.line_number = line;

            this.cachedMdata = mdata;

            this.metricsList.add(mdata);
        }

        this.cachedELOCLine = 0;
        this.cachedCLOCLine = 0;

    }

    /**
     * 関数開始
     */
    public void setStartFunction(int line) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg06401 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000064_01");
                NCMCmnLog.outputLogDebug(msg06401 + line); // MSG_LOG_ID_JAVA_00000064
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }

        this.iFunctionIndex++;
        this.isInlineFunc = true;

        this.cachedMdata.func_delim |= NCMMetricsTable.LINEDTL_FUNC_DELIM_START;
        this.cachedMdata.func_id = iFunctionIndex;
        this.cachedMdata.func_line = NCMMetricsTable.LINEDTL_FUNC_LINE_IN;

    }

    /**
     * 関数終了
     */
    public void setEndFunction(int line) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg06501 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000065_01");
                NCMCmnLog.outputLogDebug(msg06501 + line); // MSG_LOG_ID_JAVA_00000065
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }

        this.isInlineFunc = false;

        this.cachedMdata.func_delim |= NCMMetricsTable.LINEDTL_FUNC_DELIM_END;

    }

    /**
     * 条件式開始
     */
    public void setStartCondition() {
        this.isCondition = true;
    }

    /**
     * 条件式終了
     */
    public void setEndCondition() {
        this.isCondition = false;
    }

    /**
     * 有効行更新
     */
    public void updateELOC(int line, int kind) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        if (this.cachedELOCLine != line) {

            this.cachedELOCLine = line;
            this.cachedMdata.effective_line = NCMMetricsTable.LINEDTL_EFFECTIVE_LINE;

            this.cachedMdata.eloc_kind = kind;
            try {
                if (myResources != null) {
                    String msg06601 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000066_01");
                    NCMCmnLog.outputLogDebug(msg06601 + line); // MSG_LOG_ID_JAVA_00000066
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }

        } else {

        }
    }

    /**
     * コメント行更新
     */
    public void updateCLOC(int line, int kind) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        if (this.cachedCLOCLine != line) {

            this.cachedCLOCLine = line;
            this.cachedMdata.comment_line = NCMMetricsTable.LINEDTL_COMMENT_LINE;
            this.cachedMdata.cloc_kind = kind;

            try {
                if (myResources != null) {
                    String msg06701 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000067_01");
                    NCMCmnLog.outputLogDebug(msg06701 + line); // MSG_LOG_ID_JAVA_00000067
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }

        } else {

        }
    }

    /**
     * 空白行更新
     */
    public void updateBLOC(int line) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }

        if (this.cachedELOCLine != line && this.cachedCLOCLine != line) {
            if (this.lastLine != line) {

                this.cachedMdata.blank_line = NCMMetricsTable.LINEDTL_BLANK_LINE;
                try {
                    if (myResources != null) {
                        String msg06801 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000068_01");
                        NCMCmnLog.outputLogDebug(msg06801 + line); // MSG_LOG_ID_JAVA_00000068
                    }
                } catch (Exception ex) {
                    NCMCmnLog.outputLogDebug(ex.toString());
                }
            }
        }
    }

    /**
     * CYCLO計測
     */
    public int addCycro(int iCycro) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg06901 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000069_01");
                NCMCmnLog.outputLogDebug(msg06901 + iCycro); // MSG_LOG_ID_JAVA_00000069
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        this.cachedMdata.control_statements += iCycro;
        return this.cachedMdata.control_statements;
    }

    /**
     * CYCLOWC計測
     */
    public int addCycroWC(int iCycroWC) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07001 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000070_01");
                NCMCmnLog.outputLogDebug(msg07001 + iCycroWC); // MSG_LOG_ID_JAVA_00000070
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        this.cachedMdata.cyclowc += iCycroWC;
        return this.cachedMdata.cyclowc;
    }

    /**
     * BRANCHS計測
     */
    public int addBranch(int iBranch) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07101 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000071_01");
                NCMCmnLog.outputLogDebug(msg07101 + iBranch); // MSG_LOG_ID_JAVA_00000071
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        this.cachedMdata.branches += iBranch;
        return this.cachedMdata.branches;
    }

    /**
     * CNDOPS計測
     */
    public int addCndOps(int iCndOps) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07201 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000072_01");
                NCMCmnLog.outputLogDebug(msg07201 + iCndOps); // MSG_LOG_ID_JAVA_00000072
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        this.cachedMdata.cndops += iCndOps;
        return this.cachedMdata.cndops;
    }

    /**
     * ネスト数計測
     */
    public int addNest(int iNest) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07301 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000073_01");
                NCMCmnLog.outputLogDebug(msg07301 + iNest); // MSG_LOG_ID_JAVA_00000073
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        if (isInlineFunc) {

            this.nestInFuncLevel += iNest;
            if (this.nestInFuncLevel > this.cachedMdata.nest_level) {

                this.cachedMdata.nest_level = this.nestInFuncLevel;
            }
        } else {

            this.nestOutFuncLevel += iNest;
            if (this.nestOutFuncLevel > this.cachedMdata.nest_level) {

                this.cachedMdata.nest_level = this.nestOutFuncLevel;
            }
        }
        return this.cachedMdata.nest_level;
    }

    /**
     * 継続行更新
     */
    public void setLineContinue() {
        this.cachedMdata.line_continuation = NCMMetricsTable.LINEDTL_CONTINUATION_LINE;
    }

    /**
     * コメント継続行更新
     */
    public void setCommentContinue() {
        this.cachedMdata.comment_continuation = NCMMetricsTable.LINEDTL_COMMENT_CONTINUATION_TRUE;
    }

    /**
     * ファイル全体の総行数取得
     */
    public int getTLOC() {

        int tloc = this.cachedMdata.line_number;
        return tloc;
    }

    /**
     * 関数リスト出力
     */
    public boolean outputFunctionList(String dir, String name) {
        String data = createFunctionList();
        String head = createFunctionHead();
        NCMCmnFile file = new NCMCmnFile();
        return file.outputFile(dir, name, head, data);
    }

    /**
     * 関数リストのヘッダ情報出力
     */
    private String createFunctionHead() {
        StringBuilder sb = new StringBuilder(initialStringBuff);
        sb.append("\"Func ID\",");
        sb.append("\"Func Name\",");
        sb.append("\"Func_StartLine\",");
        sb.append("\"Func EndLine\",");
        sb.append("\"PARMS\",");
        sb.append("\"Param Name\",");
        sb.append("\"Internal\",");
        sb.append("\"Status\"");
        sb.append("\n");
        return sb.toString();
    }

    /**
     * 関数一覧の情報を取得
     */
    private String createFunctionList() {
        StringBuilder sb = new StringBuilder(initialStringBuff);
        for (NCMFunctionTable fdata : funcList) {
            sb.append(fdata.func_id);
            sb.append(",\"");
            sb.append(fdata.func_name);
            sb.append("\",");
            sb.append(fdata.func_startline);
            sb.append(",");
            sb.append(fdata.func_endline);
            sb.append(",\"");
            sb.append("-");
            sb.append("\",\"");
            sb.append(fdata.param_name);
            sb.append("\",");
            sb.append(fdata.internalfunc);
            sb.append(",");
            sb.append(fdata.status);
            sb.append("\n");
        }
        return sb.toString();
    }

    /**
     * 行情報メトリクスリスト出力
     */
    public boolean outputMetricsList(String dir, String name) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07401 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000074_01");
                NCMCmnLog.outputLogDebug(msg07401 + this.getTLOC()); // MSG_LOG_ID_JAVA_00000074
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        String head = createMetricsHead();
        String data = createMetricsList(null);
        NCMCmnFile file = new NCMCmnFile();
        return file.outputFile(dir, name, head, data);
    }

    /**
     * 行情報メトリクスリスト出力(デバッグ用)
     */
    public boolean outputMetricsList(String dir, String name, String source) {
        String head = createMetricsHead();
        String data = createMetricsList(source);
        NCMCmnFile file = new NCMCmnFile();
        return file.outputFile(dir, name, head, data);
    }

    /**
     * 行情報メトリクスのヘッダ情報出力
     */
    private String createMetricsHead() {
        StringBuilder sb = new StringBuilder(initialStringBuff);

        sb.append("\"Line Number\",");
        sb.append("\"Status\",");
        sb.append("\"namespace Delim\",");
        sb.append("\"Class Delim\",");
        sb.append("\"Func Delim\",");
        sb.append("\"Func ID\",");
        sb.append("\"Func Line\",");
        sb.append("\"Effective Line\",");
        sb.append("\"Comment Line\",");
        sb.append("\"Blank Line\",");
        sb.append("\"ELOC Kind\",");
        sb.append("\"PREP Kind\",");
        sb.append("\"CLOC Kind\",");
        sb.append("\"Statements\",");
        sb.append("\"Control Statement\",");
        sb.append("\"CLOCWC\",");
        sb.append("\"Control Kind\",");
        sb.append("\"BRANCHS\",");
        sb.append("\"CNDOPS\",");
        sb.append("\"Nest Level\",");
        sb.append("\"Line Continuation\",");
        sb.append("\"Comment Continuation\",");
        sb.append("\"Attribute Line\",");
        sb.append("\"EVALS\",");
        sb.append("\"EVALR\",");
        sb.append("\"AVALS\"");
        sb.append("\n");
        return sb.toString();
    }

    /**
     * 行情報の一覧を取得
     */
    private String createMetricsList(String source) {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        StringBuilder sb = new StringBuilder(initialStringBuff);
        StringBuilder sbSource = null;
        int iSourceLength = 0;
        if (source != null) {
            sbSource = new StringBuilder(source);
            iSourceLength = source.length();
        }

        for (NCMMetricsTable mdata : metricsList) {
            sb.append(mdata.line_number);
            sb.append(",");
            sb.append(mdata.status);
            sb.append(",");
            sb.append(mdata.namespace_delim);
            sb.append(",");
            sb.append(mdata.class_delim);
            sb.append(",");
            sb.append(mdata.func_delim);
            sb.append(",");
            sb.append(mdata.func_id);
            sb.append(",");
            sb.append(mdata.func_line);
            sb.append(",");
            sb.append(mdata.effective_line);
            sb.append(",");
            sb.append(mdata.comment_line);
            sb.append(",");
            sb.append(mdata.blank_line);
            sb.append(",");
            sb.append(mdata.eloc_kind);
            sb.append(",");
            sb.append(mdata.prep_kind);
            sb.append(",");
            sb.append(mdata.cloc_kind);
            sb.append(",");
            sb.append(mdata.statements);
            sb.append(",");
            sb.append(mdata.control_statements);
            sb.append(",");
            sb.append(mdata.cyclowc);
            sb.append(",");
            sb.append(mdata.control_kind);
            sb.append(",");
            sb.append(mdata.branches);
            sb.append(",");
            sb.append(mdata.cndops);
            sb.append(",");
            sb.append(mdata.nest_level);
            sb.append(",");
            sb.append(mdata.line_continuation);
            sb.append(",");
            sb.append(mdata.comment_continuation);
            sb.append(",");
            sb.append(mdata.attibute_line);
            sb.append(",");
            sb.append(mdata.evals);
            sb.append(",");
            sb.append(mdata.evalr);
            sb.append(",");
            sb.append(mdata.avals);

            if (sbSource != null) {

                if (iSourceLength > mdata.iEndOffset && mdata.iStartOffset > 0) {
                    String line = sbSource.substring(mdata.iStartOffset - 1,
                            mdata.iEndOffset);
                    line = line.replaceAll("[\r\n]", "@");
                    line = line.replaceAll("\\\\", "@");
                    line = line.replace("\"", "@");
                    sb.append(",");
                    sb.append("\"" + line + "\"");
                } else {
                    try {
                        if (myResources != null) {
                            String msg07501 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000075_01");
                            String msg07502 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000075_02");
                            NCMCmnLog.outputLogWarn(msg07501
                                    + mdata.iStartOffset + msg07502
                                    + mdata.iEndOffset); // MSG_LOG_ID_JAVA_00000075
                        }
                    } catch (Exception ex) {
                        NCMCmnLog.outputLogDebug(ex.toString());
                    }
                }
            }

            sb.append("\n");
        }
        return sb.toString();
    }

}
