/*
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Igor Bukanov, igor@fastmail.fm
 *   Norris Boyd,
 *   NEC Corporation.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */

package com.nec.jp.ncmeter.struct.js;

import java.util.ResourceBundle;

import org.mozilla.javascript.Token;

import com.nec.jp.ncmeter.cmn.log.NCMCmnLog;
import com.nec.jp.ncmeter.struct.NCMMetricsTable;
import com.nec.jp.ncmeter.struct.NCMStructData;

/**
 * JavaScriptパーサから利用されるコード構造メトリクスのインタフェース
 */
public final class NCMJSIface {

    /**
     * トークン種別単位での計測処理
     */
    public static void checkToken(int line, int kind, String s, int offset) {
        switch (kind) {
        case Token.EOF:
        case Token.EOL:

            break;
        default:

            if (kind == Token.LC) {

                NCMJSIface.visitBlockStart();
            } else if (kind == Token.RC) {

                NCMJSIface.visitBlockEnd();
            } else if (kind == Token.FUNCTION
                    || ((kind == Token.NAME) && (s.equals("get") || s
                            .equals("set")))) {

                addMetricsList(line, offset);

            } else {

            }

            NCMJSIface.updateELOC(line);

            break;
        }
    }

    /**
     * 関数情報保持リストへの登録関数
     */
    public static void addFuncList(int index, String name, String param,
            int startLine, int endLine, int startColum, int nested) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addFuncList(index, name, param, startLine, endLine, startColum,
                nested);
    }

    /**
     * 構造メトリクス情報保持リストへの登録関数
     */
    public static void addMetricsList(int line, int offset) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addMetricsList(line, offset);
    }

    /**
     * 関数開始
     */
    public static void setStartFunction(int line) {
        NCMStructData mdata = NCMStructData.getStructData();

        mdata.setStartFunction(line);

        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * 関数終了
     */
    public static void setEndFunction(int line, int offset) {
        NCMStructData mdata = NCMStructData.getStructData();

        mdata.setEndFunction(line);

        mdata.addMetricsList(line, offset + 1);
    }

    /**
     * ファイル終了
     */
    public static void setEOF(int line, int offset) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addMetricsList(line, offset);
        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * 条件式開始
     */
    public static void setStartCondition() {
        NCMStructData mdata = NCMStructData.getStructData();

        mdata.setStartCondition();
    }

    /**
     * 条件式終了
     */
    public static void setEndCondition() {
        NCMStructData mdata = NCMStructData.getStructData();

        mdata.setEndCondition();
    }

    /**
     * 有効行更新
     */
    public static void updateELOC(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.updateELOC(line, NCMMetricsTable.LINEDTL_ELOC_KIND_CODE);
    }

    /**
     * ラインコメント行更新
     */
    public static void updateSingleCLOC(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.updateCLOC(line, NCMMetricsTable.LINEDTL_COMMENT_LINE);
    }

    public static void updateMultiCLOC(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.updateCLOC(line, NCMMetricsTable.LINEDTL_COMMENT_BLOCK);
    }

    public static void updateBLOC(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.updateBLOC(line);
    }

    public static void setLineContinue(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.setLineContinue();
    }

    public static void setCommentContinue(int line) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.setCommentContinue();
    }

    /**
     * 関数リスト出力
     */
    public static boolean outputFunctionList(String dir, String name) {
        NCMStructData mdata = NCMStructData.getStructData();
        return mdata.outputFunctionList(dir, name);
    }

    /**
     * 行情報メトリクスリスト出力
     */
    public static boolean outputMetricsList(String dir, String name) {
        NCMStructData mdata = NCMStructData.getStructData();
        return mdata.outputMetricsList(dir, name);
    }

    /**
     * 行情報メトリクスリスト出力
     */
    public static boolean outputMetricsListDebug(String dir, String name,
            String source) {
        NCMStructData mdata = NCMStructData.getStructData();
        return mdata.outputMetricsList(dir, name, source);
    }

    /**
     * if文計測
     */
    public static void visitIf() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07601 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000076_01");
                NCMCmnLog.outputLogDebug(msg07601); // MSG_LOG_ID_JAVA_00000076
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * switch文計測
     */
    public static void visitSwitch() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07701 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000077_01");
                NCMCmnLog.outputLogDebug(msg07701); // MSG_LOG_ID_JAVA_00000077
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycro(1);
    }

    /**
     * case文計測
     */
    public static void visitCase() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07801 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000078_01");
                NCMCmnLog.outputLogDebug(msg07801); // MSG_LOG_ID_JAVA_00000078
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * default文計測
     */
    public static void visitDefault() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg07901 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000079_01");
                NCMCmnLog.outputLogDebug(msg07901); // MSG_LOG_ID_JAVA_00000079
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }

    }

    /**
     * while文計測
     */
    public static void visitWhile() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08001 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000080_01");
                NCMCmnLog.outputLogDebug(msg08001); // MSG_LOG_ID_JAVA_00000080
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }

        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * do-while文計測
     */
    public static void visitDo() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08101 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000081_01");
                NCMCmnLog.outputLogDebug(msg08101); // MSG_LOG_ID_JAVA_00000081
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * for文計測
     */
    public static void visitFor() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08201 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000082_01");
                NCMCmnLog.outputLogDebug(msg08201); // MSG_LOG_ID_JAVA_00000082
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCycro(1);
        mdata.addCycroWC(1);
        mdata.addBranch(1);
    }

    /**
     * try文計測
     */
    public static void visitTry() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08301 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000083_01");
                NCMCmnLog.outputLogDebug(msg08301); // MSG_LOG_ID_JAVA_00000083
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
    }

    /**
     * with文計測
     */
    public static void visitWith() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08401 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000084_01");
                NCMCmnLog.outputLogDebug(msg08401); // MSG_LOG_ID_JAVA_00000084
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
    }

    /**
     * let文計測
     */
    public static void visitLet() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08501 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000085_01");
                NCMCmnLog.outputLogDebug(msg08501); // MSG_LOG_ID_JAVA_00000085
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
    }

    /**
     * トークン "{" 計測
     */
    public static void visitBlockStart() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08601 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000086_01");
                NCMCmnLog.outputLogDebug(msg08601); // MSG_LOG_ID_JAVA_00000086
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addNest(1);
    }

    /**
     * トークン "}" 計測
     */
    public static void visitBlockEnd() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08701 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000087_01");
                NCMCmnLog.outputLogDebug(msg08701); // MSG_LOG_ID_JAVA_00000087
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addNest(-1);
    }

    /**
     * 関数計測
     */
    public static void visitFunction() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08801 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000088_01");
                NCMCmnLog.outputLogDebug(msg08801); // MSG_LOG_ID_JAVA_00000088
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
    }

    /**
     * トークン";"計測
     */
    public static void visitSemi() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg08901 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000089_01");
                NCMCmnLog.outputLogDebug(msg08901); // MSG_LOG_ID_JAVA_00000089
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
    }

    /**
     * 三項演算子計測
     */
    public static void visitConditionalExpr() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg09001 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000090_01");
                NCMCmnLog.outputLogDebug(msg09001); // MSG_LOG_ID_JAVA_00000090
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.addCndOps(1);
    }

    /**
     * 条件演算子（&&, ||）計測
     */
    public static void visitBinaryExpr() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg09101 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000091_01");
                NCMCmnLog.outputLogDebug(msg09101); // MSG_LOG_ID_JAVA_00000091
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        NCMStructData mdata = NCMStructData.getStructData();
        if (mdata.getCondition()) {

            mdata.addBranch(1);
        }
    }

    /**
     * 無名関数の関数識別子を作成する。
     */
    public static String createAnonymousFuncName(int line, int colum) {
        String name = "anonymous_" + line + "_" + colum;
        return name;
    }

    /**
     * デバッグレベルの設定
     */
    public static void setDebugLevel(NCMCmnLog.LEVEL level) {
        NCMStructData mdata = NCMStructData.getStructData();
        mdata.setDebugLevel(level);
    }
}
