/*
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Igor Bukanov, igor@fastmail.fm
 *   Norris Boyd,
 *   NEC Corporation.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */

package com.nec.jp.ncmeter.struct.js;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ResourceBundle;

import org.mozilla.javascript.CompilerEnvirons;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Parser;
import org.mozilla.javascript.tools.SourceReader;
import org.mozilla.javascript.tools.ToolErrorReporter;

import com.nec.jp.ncmeter.cmn.log.NCMCmnLog;

/**
 * JavaScriptパーサメイン
 */
public class NCMJSMain {

    private String outputDir = "";

    private String sourceID = "";

    private final String outputLineFile = "_LineInfo.csv";

    private final String outputFuncFile = "_FuncInfo.csv";

    private final String outputLineFileDebug = "_LineInfo_dbg.csv";

    private final static int exitSuccess = 0;

    private final static int exitError = -1;

    private boolean bDebug = false;

    /**
     * Main entry point.
     */
    public static void main(String args[]) {
        int exitCode;
        NCMJSMain main = new NCMJSMain();
        exitCode = main.run(args);
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        try {
            if (myResources != null) {
                String msg09201 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000092_01");
                NCMCmnLog.outputLogDebug(msg09201 + exitCode); // MSG_LOG_ID_JAVA_00000092
            }
        } catch (Exception ex) {
            NCMCmnLog.outputLogDebug(ex.toString());
        }
        System.exit(exitCode);
    }

    /**
     * JavaScriptパーサ開始
     */
    public int run(String args[]) {
        int exitCode = exitError;
        boolean success = false;
        String[] files = processOptions(args);
        if (files == null) {

        } else {
            if (!checkRequireOptions()) {

            } else {
                if (!reporter.hasReportedError()) {

                    success = processSource(files);
                }
            }
        }

        if (success) {

            exitCode = exitSuccess;
        }
        return exitCode;
    }

    /**
     * JavaScriptパーサ コンストラクタ
     */
    public NCMJSMain() {
        reporter = new ToolErrorReporter(true);
        compilerEnv = new CompilerEnvirons();
        compilerEnv.setErrorReporter(reporter);

    }

    /**
     * Parse arguments.
     */
    private String[] processOptions(String args[]) {

        compilerEnv.setGenerateDebugInfo(false);
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (!arg.startsWith("-")) {
                int tail = args.length - i;
                if (targetName != null && tail > 1) {
                    addError("msg.multiple.js.to.file", targetName);
                    return null;
                }
                String[] result = new String[tail];
                for (int j = 0; j != tail; ++j) {
                    result[j] = args[i + j];
                }
                return result;
            }

            try {
                if (arg.equals("-version") && ++i < args.length) {
                    int version = Integer.parseInt(args[i]);
                    compilerEnv.setLanguageVersion(version);
                    continue;
                }

            } catch (NumberFormatException e) {
                badUsage(args[i]);
                return null;
            }
            if (arg.equals("--m") && ++i < args.length) {

                this.outputDir = args[i];
                continue;
            }

            if (arg.equals("--n") && ++i < args.length) {

                this.sourceID = args[i];
                continue;
            }
            if (arg.equals("--debug")) {

                NCMJSIface.setDebugLevel(NCMCmnLog.LEVEL.ERROR);
                continue;
            }
            if (arg.equals("--debug2")) {

                NCMJSIface.setDebugLevel(NCMCmnLog.LEVEL.DEBUG);
                bDebug = true;
                continue;
            }
            if (arg.equals("--charset") && ++i < args.length) {

                characterEncoding = args[i];
                continue;
            }

            badUsage(arg);
            return null;
        }

        NCMCmnLog.outputLogError(ToolErrorReporter.getMessage("msg.no.file")); // MSG_LOG_ID_JAVA_00000093
        return null;
    }

    /**
     * Print a usage message.
     */
    private static void badUsage(String s) {
        System.err.println(ToolErrorReporter.getMessage("msg.jsc.bad.usage",
                NCMJSMain.class.getName(), s)); // MSG_LOG_ID_JAVA_00000100
    }

    /**
     * 必須オプションが指定されているかをチェック
     */
    private boolean checkRequireOptions() {
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/jp/ncmeter/struct/js/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        if (this.outputDir.length() == 0) {
            try {
                if (myResources != null) {
                    String msg09401 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000094_01");
                    NCMCmnLog.outputLogError(msg09401); // MSG_LOG_ID_JAVA_00000094
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }
            return false;
        } else {
            try {
                if (myResources != null) {
                    String msg09501 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000095_01");
                    NCMCmnLog.outputLogInfo(msg09501 + this.outputDir); // MSG_LOG_ID_JAVA_00000095
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }
        }

        if (this.sourceID.length() == 0) {
            try {
                if (myResources != null) {
                    String msg09601 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000096_01");
                    NCMCmnLog.outputLogError(msg09601); // MSG_LOG_ID_JAVA_00000096
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }
            return false;
        } else {
            try {
                if (myResources != null) {
                    String msg09701 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000097_01");
                    NCMCmnLog.outputLogInfo(msg09701 + this.sourceID); // MSG_LOG_ID_JAVA_00000097
                }
            } catch (Exception ex) {
                NCMCmnLog.outputLogDebug(ex.toString());
            }
        }
        return true;
    }

    private boolean outputResultFile() {
        boolean bRet = false;

        bRet = NCMJSIface.outputFunctionList(this.outputDir, this.sourceID
                + this.outputFuncFile);
        if (!bRet) {

            return bRet;
        }

        bRet = NCMJSIface.outputMetricsList(this.outputDir, this.sourceID
                + this.outputLineFile);
        return bRet;
    }

    private boolean outputResultFileDebug(String source) {
        boolean bRet = false;
        if (source != null) {

            bRet = NCMJSIface.outputMetricsListDebug(this.outputDir,
                    this.sourceID + this.outputLineFileDebug, source);
        }

        return bRet;
    }

    /**
     * Compile JavaScript source.
     */
    private boolean processSource(String[] filenames) {
        boolean bRet = false;
        for (int i = 0; i != filenames.length; ++i) {
            String filename = filenames[i];

            File f = new File(filename);
            String source = readSource(f);
            if (source == null) {
                bRet = false;
                break;
            }

            try {
                Parser p = new Parser(compilerEnv,
                        compilerEnv.getErrorReporter());

                p.parse(source, filename, 1);

                bRet = outputResultFile();

                if (bDebug) {
                    outputResultFileDebug(source);
                }
            } catch (EvaluatorException e) {

                NCMCmnLog.outputLogError(e.getMessage()); // MSG_LOG_ID_JAVA_00000098
                bRet = false;
                break;
            } catch (Exception e) {

                NCMCmnLog.outputLogError(e.getMessage()); // MSG_LOG_ID_JAVA_00000099
                e.printStackTrace(); // MSG_LOG_ID_JAVA_00000101
                bRet = false;
                break;
            }

            break;
        }
        return bRet;
    }

    private String readSource(File f) {
        String absPath = f.getAbsolutePath();
        if (!f.isFile()) {
            addError("msg.jsfile.not.found", absPath);
            return null;
        }
        try {
            return (String) SourceReader.readFileOrUrl(absPath, true,
                    characterEncoding);
        } catch (FileNotFoundException ex) {
            addError("msg.couldnt.open", absPath);
        } catch (IOException ioe) {
            addFormatedError(ioe.toString());
        }
        return null;
    }

    private void addError(String messageId, String arg) {
        String msg;
        if (arg == null) {
            msg = ToolErrorReporter.getMessage(messageId);
        } else {
            msg = ToolErrorReporter.getMessage(messageId, arg);
        }
        addFormatedError(msg);
    }

    private void addFormatedError(String message) {
        reporter.error(message, null, -1, null, -1);
    }

    private ToolErrorReporter reporter;
    private CompilerEnvirons compilerEnv;

    private String targetName;

    private String characterEncoding;
}
