/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package com.nec.ncmeter;

import gnu.getopt.Getopt;
import gnu.getopt.LongOpt;
import japa.parser.JavaParser;
import japa.parser.ParseException;
import japa.parser.TokenMgrError;
import japa.parser.ast.CompilationUnit;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.ResourceBundle;

public final class NCMJParser {

    static String target_filepath;

    static String error_filepath;

    static String metric_filepath;

    static String parse_filepath;

    static String charset_name;

    static int debug_level = 0;

    static int java_exit = 0;

    static String crlf;

    static String func_opt;

    static String comp_opt;

    private static NCMStructData structdata;

    private static NCMNodeParser nodeParse;

    /**
     * メイン処理
     */
    public static void main(String[] args) throws Exception {
        crlf = System.getProperty("line.separator");
        StringBuffer errorbuff = new StringBuffer();
        ResourceBundle myResources;
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/ncmeter/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
        if (!parseOptions(args, errorbuff)) {
            try {
                if (myResources != null) {
                    String msg00101 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000001_01");

                    outputLogFile(1, msg00101 + errorbuff.toString() + crlf); // MSG_LOG_ID_JAVA_00000001
                }
            } catch (Exception ex) {
                outputLogFile(1, ex + crlf);
            }
            errorbuff = null;
            System.exit(1);
        }

        CompilationUnit cu = null;
        FileInputStream in = null;

        try {

            in = new FileInputStream(parse_filepath);

            if (charset_name == null)
                cu = JavaParser.parse(in);
            else
                cu = JavaParser.parse(in, charset_name);

        } catch (FileNotFoundException ex) {

            errorbuff.append(ex.getMessage() + crlf);
            outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000002
            cu = null;
        } catch (ParseException ex) {
            try {
                if (myResources != null) {
                    String msg00301 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_01");
                    String msg00302 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_02");
                    String msg00303 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_03");
                    String msg00304 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_04");
                    String msg00305 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_05");
                    String msg00306 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_06");
                    String msg00307 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_07");
                    String msg00308 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_08");
                    String msg00309 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000003_09");

                    errorbuff.append(msg00301 + crlf + msg00302
                            + ex.currentToken.beginLine + msg00303
                            + ex.currentToken.beginColumn);
                    errorbuff.append(msg00304 + ex.currentToken.image
                            + msg00305 + crlf);
                    if (ex.currentToken.next != null) {
                        errorbuff.append(msg00306
                                + ex.currentToken.next.beginLine + msg00307
                                + ex.currentToken.next.beginColumn);
                        errorbuff.append(msg00308 + ex.currentToken.next.image
                                + msg00309 + crlf);
                    }
                    errorbuff.append(ex.getMessage() + crlf);
                    outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000003
                }
            } catch (Exception ex2) {
                outputLogFile(1, ex2 + crlf);
            }
            cu = null;
        } catch (TokenMgrError ex) {

            errorbuff.append(ex.getMessage() + crlf);
            outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000004
            cu = null;
        } finally {

            try {
                if (in != null)
                    in.close();
                in = null;
            } catch (Exception e) {
                errorbuff.append(e.getMessage() + crlf);
                outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000005
            }
        }

        structdata = new NCMStructData();
        structdata.DirPath = getPathName(target_filepath);
        structdata.FileName = getFileName(target_filepath);
        structdata.LangType = "JAVA";
        structdata.debugLevel = debug_level;
        structdata.funcOpt = func_opt;

        if (cu != null) {

            nodeParse = new NCMNodeParser();
            try {
                nodeParse.visit(cu, structdata);
            } catch (StackOverflowError ex) {
                try {
                    if (myResources != null) {
                        String msg00601 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000006_01");

                        errorbuff.append(msg00601 + crlf);
                        outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000006

                    }
                } catch (Exception ex3) {
                    outputLogFile(1, ex3 + crlf);
                }
                structdata.errorLevel = 2;
                java_exit = 1;
            }
            outputLogFile(2, structdata.dbugbufread()); // MSG_LOG_ID_JAVA_00000007
        } else {
            structdata.errorLevel = 1;
            java_exit = 1;
        }

        if (!outputMetricFile(structdata, errorbuff)) {

            errorbuff.append(errorbuff);
            outputLogFile(1, errorbuff.toString()); // MSG_LOG_ID_JAVA_00000008
            java_exit = 1;
        }
        System.exit(java_exit);
    }

    /**
     * ファイルのフルパス名からファイル名を取得
     */
    private static String getFileName(String filename) {

        int index = filename.lastIndexOf(File.separator);
        if (index > -1)
            return filename.substring(index + 1);
        else
            return filename;
    }

    /**
     * パースを行う際のJavaバージョン情報を取得
     */
    public static String getCompOpt() {
        return comp_opt;
    }

    /**
     * ファイルのフルパス名からパス名を取得
     */
    public static String getPathName(String filename) {
        int index = filename.lastIndexOf(File.separator);
        if (index > 0)
            return filename.substring(0, index);
        else
            return "";
    }

    /**
     * オプションを解析します。 引数を受け取りクラス変数にオプションに指定された情報を設定します。 パラメータ解析処理はgnu.getoptを利用。
     */
    private static boolean parseOptions(final String[] args,
            StringBuffer errorbuf) {
        boolean res = true;
        int c;
        LongOpt[] longopts = new LongOpt[9];
        longopts[0] = new LongOpt("metric_file", LongOpt.REQUIRED_ARGUMENT,
                null, 'm');
        longopts[1] = new LongOpt("target_file_path",
                LongOpt.REQUIRED_ARGUMENT, null, 't');
        longopts[2] = new LongOpt("charset", LongOpt.REQUIRED_ARGUMENT, null,
                'c');
        longopts[3] = new LongOpt("error_output", LongOpt.REQUIRED_ARGUMENT,
                null, 'e');
        longopts[4] = new LongOpt("debug", LongOpt.NO_ARGUMENT, null, 2);
        longopts[5] = new LongOpt("debug2", LongOpt.NO_ARGUMENT, null, 3);

        longopts[6] = new LongOpt("func_opt", LongOpt.REQUIRED_ARGUMENT, null,
                'f');
        longopts[7] = new LongOpt("comp_opt", LongOpt.REQUIRED_ARGUMENT, null,
                'p');

        longopts[8] = new LongOpt("hex_file_path", LongOpt.REQUIRED_ARGUMENT,
                null, 'x');

        Getopt g = new Getopt("ncmeter", args, "-:e:m:t:c:f:p:x", longopts);
        g.setOpterr(false);

        while ((c = g.getopt()) != -1) {
            switch (c) {
            case 1:
                if (parse_filepath == null) {
                    parse_filepath = g.getOptarg();
                } else {

                    errorbuf.append("file1:" + parse_filepath + crlf
                            + ",file2:" + g.getOptarg());
                    res = false;
                }
                break;
            case 2:
                if (debug_level == 0)
                    debug_level = 1;
                break;
            case 3:
                if (debug_level <= 1)
                    debug_level = 2;
                break;
            case 'e':
                if (error_filepath == null) {
                    error_filepath = g.getOptarg();
                } else {
                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;
            case 'm':
                if (metric_filepath == null) {
                    metric_filepath = g.getOptarg();
                } else {
                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;
            case 't':

                if (target_filepath == null) {
                    File tFile = new File(g.getOptarg());
                    try {

                        target_filepath = tFile.getCanonicalPath();
                    } catch (IOException ioe) {
                        errorbuf.append("-" + (char) c);
                        res = false;
                    } catch (SecurityException se) {

                        errorbuf.append("-" + (char) c);
                        res = false;
                    }
                } else {

                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;
            case 'x':

                if (target_filepath == null) {
                    byte[] bytes = StringToByteArray(g.getOptarg());
                    if (bytes == null) {
                        errorbuf.append("-" + (char) c);
                        res = false;
                    } else {
                        try {
                            String str = new String(bytes, "UTF-16");
                            target_filepath = str;
                        } catch (UnsupportedEncodingException e) {
                            errorbuf.append("-" + (char) c);
                            res = false;
                        }
                    }
                } else {

                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;
            case 'c':
                if (charset_name == null) {
                    charset_name = g.getOptarg();
                } else {
                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;

            case 'f':
                if (g.getOptarg().compareTo("param") == 0) {
                    func_opt = "param";
                } else if (g.getOptarg().compareTo("none") == 0) {
                    func_opt = "none";
                } else {
                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;
            case 'p':
                if (comp_opt == null) {
                    comp_opt = g.getOptarg();
                } else {
                    errorbuf.append("-" + (char) c);
                    res = false;
                }
                break;

            case '?':
            default:
                errorbuf.append(args[g.getOptind() - 1]);
                res = false;
                break;
            }
            if (!res)
                break;
        }

        longopts = null;
        g = null;
        if (res) {
            if (parse_filepath == null) {
                errorbuf.append("error is parse filepath");
                res = false;
            } else if (error_filepath == null) {
                errorbuf.append("error is --error_output");
                res = false;
            } else if (metric_filepath == null) {
                errorbuf.append("error is --metric_filepath");
                res = false;
            } else if (target_filepath == null) {
                errorbuf.append("error is --target_filepath");
                res = false;
            } else if (func_opt == null) {

                func_opt = "param";
            } else {
            }
        }
        return (res);
    }

    /**
     * 文字列をバイト配列に変換する。
     */
    public static byte[] StringToByteArray(String hex) {

        byte[] bytes = new byte[hex.length() / 2];

        for (int index = 0; index < bytes.length; index++) {

            try {
                bytes[index] = (byte) Integer.parseInt(
                        hex.substring(index * 2, (index + 1) * 2), 16);
            } catch (Exception ex) {
                outputLogFile(1, ex.getMessage()); // MSG_LOG_ID_JAVA_00000009
                bytes = null;
                break;
            }
        }
        return bytes;
    }

    /**
     * ファイルにログを出力
     */
    private static void outputLogFile(final int idebug, final String buf) {
        FileOutputStream fo = null;
        OutputStreamWriter fw = null;
        if (debug_level >= idebug) {
            try {

                fo = new FileOutputStream(error_filepath, false);
                fw = new OutputStreamWriter(fo);
                fw.write(buf);
            } catch (IOException ex) {
                System.out.println(ex); // MSG_LOG_ID_JAVA_00000010

            } finally {
                try {
                    if (fw != null) {
                        fw.close();
                        fw = null;
                    }
                } catch (IOException ex) {

                    System.out.println(ex); // MSG_LOG_ID_JAVA_00000011
                } finally {
                    try {
                        if (fo != null) {
                            fo.close();
                            fo = null;
                        }
                    } catch (IOException ex) {

                        System.out.println(ex); // MSG_LOG_ID_JAVA_00000012
                    }
                }
            }
        }
        return;
    }

    /**
     * メトリクス情報出力
     */
    private static boolean outputMetricFile(final NCMStructData structdata,
            StringBuffer errbuf) {
        FileOutputStream fo = null;
        OutputStreamWriter fw = null;
        NCMStructData.NCMSFuncData fncdata;
        StringBuffer csvbuff = new StringBuffer();
        boolean res = true;

        if (structdata.errorLevel > 0) {

        } else {

            int functionCount = structdata.functionCount();
            if (functionCount == 0) {

                csvbuff.append("\"" + structdata.DirPath + "\",\""
                        + structdata.FileName + "\"");
                csvbuff.append(",-,JAVA");
                csvbuff.append(",-,-,-,-,-,-,-,-,-,-,-,-,-" + crlf);
            } else {
                for (int ii = 0; ii < functionCount; ii++) {
                    fncdata = structdata.functionGetData(ii);
                    csvbuff.append("\"" + structdata.DirPath + "\",\""
                            + structdata.FileName + "\",");
                    if (func_opt.compareTo("param") == 0) {

                        String str_func_name;
                        str_func_name = fncdata.nameFunc;
                        int length = fncdata.param_string.length();
                        if (length > 0) {
                            str_func_name = str_func_name
                                    + fncdata.param_string;
                        } else {
                            str_func_name = str_func_name + "(" + ")";
                        }
                        csvbuff.append("\"" + str_func_name + "\",JAVA");
                    } else {

                        csvbuff.append("\"" + fncdata.nameFunc + "\",JAVA");
                    }
                    csvbuff.append("," + fncdata.cntFloc + ","
                            + fncdata.cntEloc + "," + fncdata.cntCloc + ","
                            + fncdata.cntBloc);
                    csvbuff.append("," + fncdata.cntCyclo + ","
                            + fncdata.cntCycloWC + "," + fncdata.cntBranchs
                            + "," + fncdata.cntCndOps + "," + fncdata.cntNests
                            + ",-,-,-,-" + crlf);
                }
            }
        }
        fncdata = null;
        try {

            fo = new FileOutputStream(metric_filepath, false);
            fw = new OutputStreamWriter(fo, "UTF-8");

            fw.write(csvbuff.toString());
        } catch (IOException ex) {
            errbuf.append(ex.getMessage());
            res = false;
        } finally {
            csvbuff = null;
            try {
                if (fw != null) {
                    fw.close();
                    fw = null;
                }
            } catch (IOException ex) {
                errbuf.append(ex.getMessage());
                res = false;
            } finally {
                try {
                    if (fo != null) {
                        fo.close();
                        fo = null;
                    }
                } catch (IOException ex) {
                    errbuf.append(ex.getMessage());
                    res = false;
                }
            }
        }
        return (res);
    }

}
