/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package com.nec.ncmeter;

import japa.parser.ast.CompilationUnit;
import japa.parser.ast.ImportDeclaration;
import japa.parser.ast.TypeParameter;
import japa.parser.ast.body.AnnotationDeclaration;
import japa.parser.ast.body.BodyDeclaration;
import japa.parser.ast.body.ClassOrInterfaceDeclaration;
import japa.parser.ast.body.ConstructorDeclaration;
import japa.parser.ast.body.EnumConstantDeclaration;
import japa.parser.ast.body.EnumDeclaration;
import japa.parser.ast.body.MethodDeclaration;
import japa.parser.ast.body.Parameter;
import japa.parser.ast.body.TypeDeclaration;
import japa.parser.ast.expr.AnnotationExpr;
import japa.parser.ast.expr.ArrayInitializerExpr;
import japa.parser.ast.expr.BinaryExpr;
import japa.parser.ast.expr.ConditionalExpr;
import japa.parser.ast.expr.DimExpr;
import japa.parser.ast.expr.Expression;
import japa.parser.ast.expr.ObjectCreationExpr;
import japa.parser.ast.stmt.BlockStmt;
import japa.parser.ast.stmt.DoStmt;
import japa.parser.ast.stmt.ForStmt;
import japa.parser.ast.stmt.ForeachStmt;
import japa.parser.ast.stmt.IfStmt;
import japa.parser.ast.stmt.Statement;
import japa.parser.ast.stmt.SwitchEntryStmt;
import japa.parser.ast.stmt.SwitchStmt;
import japa.parser.ast.stmt.WhileStmt;
import japa.parser.ast.type.ClassOrInterfaceType;
import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.VoidVisitorAdapter;

import java.util.ArrayList;
import java.util.ResourceBundle;

/**
 * Parse後の AST の node を visit する
 */
public final class NCMNodeParser extends VoidVisitorAdapter<NCMStructData> {

    boolean flgOutsideFunction = false;

    boolean flgBranchCondition = false;

    private int nestlevel = 0;

    private int scopelevel = 0;

    private ArrayList<String> scopenames = new ArrayList<String>();

    private ResourceBundle myResources = null;

    /**
     * コンストラクタ
     */
    public NCMNodeParser() {
        try {
            myResources = ResourceBundle
                    .getBundle("com/nec/ncmeter/CDMtrMessage");
        } catch (Exception ex) {
            myResources = null;
        }
    }

    /**
     * スコープ名取得
     */
    public String getScopeNames() {
        String txt = "";
        int sz = scopelevel;
        if (scopenames.size() - 1 < sz)
            sz = scopenames.size() - 1;
        for (int ii = 0; ii <= sz; ii++) {
            if (ii > 0) {
                txt += "." + scopenames.get(ii);
            } else {
                txt += scopenames.get(ii);
            }
        }
        return txt;
    }

    /**
     * ブロック開始
     */
    public void indent(int iBeginLine, NCMStructData arg) {
        nestlevel++;

        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            int curNestLevel = fncdata.addNests(1);
            try {
                if (myResources != null) {
                    String msg01301 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000013_01");
                    String msg01302 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000013_02");
                    String msg01303 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000013_03");
                    arg.dbugbufPrintLn(2, msg01301 + iBeginLine + msg01302
                            + curNestLevel + msg01303); // MSG_LOG_ID_JAVA_00000013
                }

            } catch (Exception ex) {
                arg.dbugbufPrintLn(1, ex.toString());

            }
        }
    }

    /**
     * ブロック終了
     */
    public void unindent(int iEndLine, NCMStructData arg) {
        nestlevel--;
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            int curNestLevel = fncdata.addNests(-1);
            try {
                if (myResources != null) {
                    String msg01401 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000014_01");
                    String msg01402 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000014_02");
                    String msg01403 = myResources
                            .getString("MSG_LOG_ID_JAVA_00000014_03");
                    arg.dbugbufPrintLn(2, msg01401 + iEndLine + msg01402
                            + curNestLevel + msg01403); // MSG_LOG_ID_JAVA_00000014
                }

            } catch (Exception ex) {
                arg.dbugbufPrintLn(1, ex.toString());

            }
        }
    }

    /**
     * ブランチ行数計測
     */
    private void addBranch(int iBeginLine, NCMStructData arg,
            NCMStructData.NCMSFuncData fncdata) {
        int iCurBranch = fncdata.addBranch(1);
        try {
            if (myResources != null) {
                String msg01501 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000015_01");
                String msg01502 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000015_02");
                String msg01503 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000015_03");
                arg.dbugbufPrintLn(2, msg01501 + iBeginLine + msg01502
                        + iCurBranch + msg01503); // MSG_LOG_ID_JAVA_00000015
            }

        } catch (Exception ex) {
            arg.dbugbufPrintLn(1, ex.toString());

        }
    }

    /**
     * 条件式のブランチ計測
     */
    private void addBranchBinaryExpr(BinaryExpr n, NCMStructData arg) {
        if (flgOutsideFunction && flgBranchCondition) {
            switch (n.getOperator()) {
            case or:
            case and:

                NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
                addBranch(n.getBeginLine(), arg, fncdata);
                break;
            default:
                break;
            }
        }
    }

    /**
     * スコープ名を追加する
     */
    private void addScopeList(String name) {
        if (scopelevel >= 0) {
            scopenames.add(scopelevel, name);
        } else {
            scopenames.add(0, name);
        }
        scopelevel++;
    }

    /**
     * スコープ名を削除する
     */
    private void removeScopeList() {
        scopelevel--;
        if (scopenames.size() > scopelevel && scopelevel >= 0) {
            scopenames.remove(scopelevel);
        }
    }

    /**
     * コンパイルユニット
     */
    @Override
    public void visit(CompilationUnit n, NCMStructData arg) {
        if (n.getPackage() != null) {
            arg.PackageName = n.getPackage().getName().toString();
            n.getPackage().accept(this, arg);
        }
        if (n.getImports() != null) {
            arg.cntImports = n.getImports().size();
            for (ImportDeclaration i : n.getImports()) {
                i.accept(this, arg);
            }
        }

        try {
            if (myResources != null) {
                String msg01601 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000016_01");
                String msg01602 = myResources
                        .getString("MSG_LOG_ID_JAVA_00000016_02");
                arg.dbugbufPrintLn(2, msg01601 + arg.PackageName + msg01602
                        + arg.cntImports); // MSG_LOG_ID_JAVA_00000016
            }

        } catch (Exception ex) {
            arg.dbugbufPrintLn(1, ex.toString());

        }

        if (n.getTypes() != null) {
            for (TypeDeclaration typeDeclaration : n.getTypes()) {
                typeDeclaration.accept(this, arg);
            }
        }
    }

    /**
     * クラス宣言 クラス宣言時に呼ばれる。
     */
    @Override
    public void visit(ClassOrInterfaceDeclaration n, NCMStructData arg) {
        indent(n.getBeginLine(), arg);
        if (n.getMembers() != null) {

            addScopeList(n.getName());
            if (!flgOutsideFunction) {

                try {
                    if (myResources != null) {
                        String msg01701 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000017_01");
                        String msg01702 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000017_02");
                        String msg01703 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000017_03");
                        String msg01704 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000017_04");

                        arg.dbugbufPrint(2, msg01701 + scopelevel + msg01702
                                + n.getBeginLine() + msg01703 + n.getEndLine()
                                + msg01704); // MSG_LOG_ID_JAVA_00000017
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                if (n.isInterface()) {
                    try {
                        if (myResources != null) {
                            String msg01801 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000018_01");
                            String msg01802 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000018_02");
                            arg.dbugbufPrint(2, msg01801 + getScopeNames()
                                    + msg01802); // MSG_LOG_ID_JAVA_00000018
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                } else {
                    try {
                        if (myResources != null) {
                            String msg01901 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000019_01");
                            String msg01902 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000019_02");
                            arg.dbugbufPrint(2, msg01901 + getScopeNames()
                                    + msg01902); // MSG_LOG_ID_JAVA_00000019
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                }

                if (n.getJavaDoc() != null) {
                    n.getJavaDoc().accept(this, arg);
                }
                if (n.getAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg02001 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000020_01");
                            arg.dbugbufPrint(2, msg02001
                                    + n.getAnnotations().toString()); // MSG_LOG_ID_JAVA_00000020
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getAnnotations()) {
                        a.accept(this, arg);
                    }
                }
                if (n.getTypeParameters() != null) {
                    try {
                        if (myResources != null) {
                            String msg02101 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000021_01");
                            arg.dbugbufPrint(2, msg02101
                                    + n.getTypeParameters().size()); // MSG_LOG_ID_JAVA_00000021
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (TypeParameter t : n.getTypeParameters()) {
                        t.accept(this, arg);
                    }
                }
                if (n.getExtends() != null) {
                    try {
                        if (myResources != null) {
                            String msg02201 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000022_01");
                            arg.dbugbufPrint(2, msg02201
                                    + n.getExtends().size()); // MSG_LOG_ID_JAVA_00000022
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (ClassOrInterfaceType c : n.getExtends()) {
                        c.accept(this, arg);
                    }
                }

                if (n.getImplements() != null) {
                    try {
                        if (myResources != null) {
                            String msg02301 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000023_01");
                            arg.dbugbufPrint(2, msg02301
                                    + n.getImplements().size()); // MSG_LOG_ID_JAVA_00000023
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (ClassOrInterfaceType c : n.getImplements()) {
                        c.accept(this, arg);
                    }
                }

                arg.dbugbufPrintLn(2); // MSG_LOG_ID_JAVA_00000024
                arg.dbugbufOutput(2);
            }

            for (BodyDeclaration member : n.getMembers()) {
                member.accept(this, arg);
            }

            removeScopeList();
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * enum宣言
     */
    @Override
    public void visit(EnumDeclaration n, NCMStructData arg) {
        indent(n.getBeginLine(), arg);
        if (n.getEntries() != null || n.getMembers() != null) {

            addScopeList(n.getName());
            if (!flgOutsideFunction) {

                try {
                    if (myResources != null) {
                        String msg02501 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000025_01");
                        String msg02502 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000025_02");
                        String msg02503 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000025_03");
                        String msg02504 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000025_04");
                        arg.dbugbufPrint(2, msg02501 + scopelevel + msg02502
                                + n.getBeginLine() + msg02503 + n.getEndLine()
                                + msg02504); // MSG_LOG_ID_JAVA_00000025
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                try {
                    if (myResources != null) {
                        String msg02601 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000026_01");
                        String msg02602 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000026_02");
                        arg.dbugbufPrint(2, msg02601 + getScopeNames()
                                + msg02602); // MSG_LOG_ID_JAVA_00000026
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                if (n.getJavaDoc() != null) {
                    n.getJavaDoc().accept(this, arg);
                }
                if (n.getAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg02701 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000027_01");
                            arg.dbugbufPrint(2, msg02701
                                    + n.getAnnotations().toString()); // MSG_LOG_ID_JAVA_00000027
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getAnnotations()) {
                        a.accept(this, arg);
                    }

                }
                try {
                    if (myResources != null) {
                        String msg02801 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000028_01");
                        arg.dbugbufPrint(2, msg02801 + n.getModifiers()); // MSG_LOG_ID_JAVA_00000028
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                if (n.getImplements() != null) {
                    try {
                        if (myResources != null) {
                            String msg02901 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000029_01");
                            arg.dbugbufPrint(2, msg02901
                                    + n.getImplements().size()); // MSG_LOG_ID_JAVA_00000029
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (ClassOrInterfaceType c : n.getImplements()) {
                        c.accept(this, arg);
                    }
                }

                if (n.getEntries() != null) {
                    try {
                        if (myResources != null) {
                            String msg03001 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000030_01");
                            arg.dbugbufPrintLn(2, msg03001
                                    + n.getEntries().size()); // MSG_LOG_ID_JAVA_00000030
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    arg.dbugbufOutput(2);
                    for (EnumConstantDeclaration e : n.getEntries()) {
                        e.accept(this, arg);
                    }
                } else {
                    arg.dbugbufPrintLn(2); // MSG_LOG_ID_JAVA_00000031
                    arg.dbugbufOutput(2);
                }

                if (n.getMembers() != null) {
                    for (BodyDeclaration member : n.getMembers()) {
                        member.accept(this, arg);
                    }
                }
            } else {
                if (n.getEntries() != null) {
                    for (EnumConstantDeclaration e : n.getEntries()) {
                        e.accept(this, arg);
                    }
                }
                if (n.getMembers() != null) {
                    for (BodyDeclaration member : n.getMembers()) {
                        member.accept(this, arg);
                    }
                }
            }

            removeScopeList();
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * enum内の識別子 。enum内に識別子のみの定義や、関数が定義される場合もある。
     */
    @Override
    public void visit(EnumConstantDeclaration n, NCMStructData arg) {

        if (n.getClassBody() != null) {
            indent(n.getBeginLine(), arg);

            addScopeList(n.getName());
            if (!flgOutsideFunction) {

                try {
                    if (myResources != null) {
                        String msg03201 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000032_01");
                        String msg03202 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000032_02");
                        String msg03203 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000032_03");
                        String msg03204 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000032_04");

                        arg.dbugbufPrint(2, msg03201 + scopelevel + msg03202
                                + n.getBeginLine() + msg03203 + n.getEndLine()
                                + msg03204); // MSG_LOG_ID_JAVA_00000032
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                try {
                    if (myResources != null) {
                        String msg03301 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000033_01");
                        String msg03302 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000033_02");
                        arg.dbugbufPrint(2, msg03301 + getScopeNames()
                                + msg03302); // MSG_LOG_ID_JAVA_00000033
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                if (n.getJavaDoc() != null) {
                    n.getJavaDoc().accept(this, arg);
                }
                if (n.getAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg03401 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000034_01");
                            arg.dbugbufPrint(2, msg03401
                                    + n.getAnnotations().toString()); // MSG_LOG_ID_JAVA_00000034
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getAnnotations()) {
                        a.accept(this, arg);
                    }
                }
                if (n.getArgs() != null) {
                    try {
                        if (myResources != null) {
                            String msg03501 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000035_01");
                            arg.dbugbufPrint(2, msg03501
                                    + n.getArgs().toString()); // MSG_LOG_ID_JAVA_00000035
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (Expression e : n.getArgs()) {
                        e.accept(this, arg);
                    }
                }
                arg.dbugbufPrintLn(2); // MSG_LOG_ID_JAVA_00000036
                arg.dbugbufOutput(2);
            }

            for (BodyDeclaration member : n.getClassBody()) {
                member.accept(this, arg);
            }

            removeScopeList();
            unindent(n.getEndLine(), arg);
        }
    }

    /**
     * コンストラクタ宣言/定義
     */
    @Override
    public void visit(ConstructorDeclaration n, NCMStructData arg) {
        if (n.getBlock() != null) {

            addScopeList(n.getName());
            if (!flgOutsideFunction) {

                flgOutsideFunction = true;

                NCMStructData.NCMSFuncData fncdata = arg.functionNewData();
                fncdata.cntFunctionBase++;

                addBranch(n.getBeginLine(), arg, fncdata);

                try {
                    if (myResources != null) {
                        String msg03701 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000037_01");
                        String msg03702 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000037_02");
                        String msg03703 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000037_03");
                        String msg03704 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000037_04");

                        arg.dbugbufPrint(1, msg03701 + scopelevel + msg03702
                                + n.getBeginLine() + msg03703 + n.getEndLine()
                                + msg03704); // MSG_LOG_ID_JAVA_00000037
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                fncdata.nameFunc = getScopeNames();

                try {
                    if (myResources != null) {
                        String msg03801 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000038_01");
                        String msg03802 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000038_02");
                        arg.dbugbufPrint(1, msg03801 + fncdata.nameFunc
                                + msg03802); // MSG_LOG_ID_JAVA_00000038
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                if (n.getJavaDoc() != null) {
                    n.getJavaDoc().accept(this, arg);
                }
                if (n.getAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg03901 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000039_01");
                            arg.dbugbufPrint(1, msg03901
                                    + n.getAnnotations().toString()); // MSG_LOG_ID_JAVA_00000039
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getAnnotations()) {
                        a.accept(this, arg);
                    }
                }
                try {
                    if (myResources != null) {
                        String msg04001 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000040_01");
                        arg.dbugbufPrint(1, msg04001 + n.getModifiers()); // MSG_LOG_ID_JAVA_00000040
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                if (n.getTypeParameters() != null) {
                    try {
                        if (myResources != null) {
                            String msg04101 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000041_01");
                            arg.dbugbufPrint(1, msg04101
                                    + n.getTypeParameters().size()); // MSG_LOG_ID_JAVA_00000041
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (TypeParameter t : n.getTypeParameters()) {
                        t.accept(this, arg);
                    }
                }

                String param_str = "( ";
                if (n.getParameters() != null && n.getParameters().size() > 0) {
                    try {
                        if (myResources != null) {
                            String msg04201 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000042_01");
                            arg.dbugbufPrint(1, msg04201
                                    + n.getParameters().size()); // MSG_LOG_ID_JAVA_00000042
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (Parameter p : n.getParameters()) {
                        p.accept(this, arg);
                        param_str = param_str + (p.getType()).toString()
                                + " , ";
                    }
                    param_str = param_str.substring(0, param_str.length() - 2);
                }
                param_str = param_str + ")";
                fncdata.param_string = param_str;

                if (n.getThrows() != null) {
                    try {
                        if (myResources != null) {
                            String msg04301 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000043_01");
                            arg.dbugbufPrint(1, msg04301 + n.getThrows().size()); // MSG_LOG_ID_JAVA_00000043
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (ClassOrInterfaceType coit : n.getThrows()) {
                        coit.accept(this, arg);
                    }
                }
                arg.dbugbufPrintLn(2); // MSG_LOG_ID_JAVA_00000044
                arg.dbugbufOutput(2);

                n.getBlock().accept(this, arg);

                try {
                    if (myResources != null) {
                        String msg04501 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000045_01");
                        arg.functionEndCalc(n.getLOC());
                        arg.dbugbufPrintLn(1, msg04501 + fncdata.getCounter()); // MSG_LOG_ID_JAVA_00000045
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                arg.dbugbufOutput(1);
                flgOutsideFunction = false;

            } else {

                n.getBlock().accept(this, arg);
            }

            removeScopeList();
        }
    }

    /**
     * メソッド宣言/定義 メソッド宣言のみの場合、n.getBody() が null で返却されるため、 計測対象とはならない。
     */
    @Override
    public void visit(MethodDeclaration n, NCMStructData arg) {
        if (n.getBody() != null) {

            addScopeList(n.getName());
            if (!flgOutsideFunction) {

                flgOutsideFunction = true;

                NCMStructData.NCMSFuncData fncdata = arg.functionNewData();
                fncdata.cntFunctionBase++;

                addBranch(n.getBeginLine(), arg, fncdata);

                try {
                    if (myResources != null) {
                        String msg04601 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000046_01");
                        String msg04602 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000046_02");
                        String msg04603 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000046_03");
                        String msg04604 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000046_04");

                        arg.dbugbufPrint(1, msg04601 + scopelevel + msg04602
                                + n.getBeginLine() + msg04603 + n.getEndLine()
                                + msg04604); // MSG_LOG_ID_JAVA_00000046
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                fncdata.nameFunc = getScopeNames();

                try {
                    if (myResources != null) {
                        String msg04701 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000047_01");
                        String msg04702 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000047_02");
                        arg.dbugbufPrint(1, msg04701 + fncdata.nameFunc
                                + msg04702); // MSG_LOG_ID_JAVA_00000047
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                if (n.getJavaDoc() != null) {
                    n.getJavaDoc().accept(this, arg);
                }
                if (n.getAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg04801 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000048_01");
                            arg.dbugbufPrint(1, msg04801
                                    + n.getAnnotations().toString()); // MSG_LOG_ID_JAVA_00000048
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getAnnotations()) {
                        a.accept(this, arg);
                    }
                }
                try {
                    if (myResources != null) {
                        String msg04901 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000049_01");
                        arg.dbugbufPrint(1, msg04901 + n.getModifiers()); // MSG_LOG_ID_JAVA_00000049
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }
                if (n.getTypeParameters() != null) {
                    try {
                        if (myResources != null) {
                            String msg05001 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000050_01");
                            arg.dbugbufPrint(1, msg05001
                                    + n.getTypeParameters().size()); // MSG_LOG_ID_JAVA_00000050
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (TypeParameter t : n.getTypeParameters()) {
                        t.accept(this, arg);
                    }
                }
                if (n.getMHAnnotations() != null) {
                    try {
                        if (myResources != null) {
                            String msg03401 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000034_01");
                            arg.dbugbufPrint(1, msg03401
                                    + n.getMHAnnotations().toString()); // MSG_LOG_ID_JAVA_00000034
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (AnnotationExpr a : n.getMHAnnotations()) {
                        a.accept(this, arg);
                    }
                }
                if (n.getType() != null) {
                    try {
                        if (myResources != null) {
                            String msg05101 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000051_01");
                            arg.dbugbufPrint(1, msg05101
                                    + n.getType().toString()); // MSG_LOG_ID_JAVA_00000051
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    n.getType().accept(this, arg);
                }

                String param_str = "( ";
                if (n.getParameters() != null && n.getParameters().size() > 0) {
                    try {
                        if (myResources != null) {
                            String msg05201 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000052_01");
                            arg.dbugbufPrint(1, msg05201
                                    + n.getParameters().size()); // MSG_LOG_ID_JAVA_00000052
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (Parameter p : n.getParameters()) {
                        p.accept(this, arg);
                        param_str = param_str + (p.getType()).toString()
                                + " , ";
                    }
                    param_str = param_str.substring(0, param_str.length() - 2);
                }
                param_str = param_str + ")";
                fncdata.param_string = param_str;

                if (n.getDims() != null) {
                    try {
                        if (myResources != null) {
                            String msg05601 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000056_01");
                            arg.dbugbufPrint(1, msg05601 + n.getDims().size()); // MSG_LOG_ID_JAVA_00000056
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (DimExpr a : n.getDims()) {
                        a.accept(this, arg);
                    }
                }

                if (n.getThrows() != null) {
                    try {
                        if (myResources != null) {
                            String msg05301 = myResources
                                    .getString("MSG_LOG_ID_JAVA_00000053_01");
                            arg.dbugbufPrint(1, msg05301 + n.getThrows().size()); // MSG_LOG_ID_JAVA_00000053
                        }

                    } catch (Exception ex) {
                        arg.dbugbufPrintLn(1, ex.toString());

                    }
                    for (ClassOrInterfaceType coit : n.getThrows()) {
                        coit.accept(this, arg);
                    }
                }
                arg.dbugbufPrintLn(2); // MSG_LOG_ID_JAVA_00000054
                arg.dbugbufOutput(2);

                n.getBody().accept(this, arg);
                try {
                    if (myResources != null) {
                        String msg05501 = myResources
                                .getString("MSG_LOG_ID_JAVA_00000055_01");
                        arg.functionEndCalc(n.getLOC());
                        arg.dbugbufPrintLn(1, msg05501 + fncdata.getCounter()); // MSG_LOG_ID_JAVA_00000055
                    }

                } catch (Exception ex) {
                    arg.dbugbufPrintLn(1, ex.toString());

                }

                arg.dbugbufOutput(1);

                flgOutsideFunction = false;
            } else {

                n.getBody().accept(this, arg);
            }

            removeScopeList();
        }
    }

    /**
     * 三項演算子 三項演算子の条件式は BRANCHS にカウントしない。
     */
    @Override
    public void visit(ConditionalExpr n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntInlinedIf++;

        }

        n.getCondition().accept(this, arg);
        n.getThenExpr().accept(this, arg);
        n.getElseExpr().accept(this, arg);
    }

    /**
     * switch ステートメント switch は必ずBLOCKをもつため、BlockStmtのvisit関数をとおらない。
     */
    @Override
    public void visit(SwitchStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntSelectionSwitch++;

        }
        flgBranchCondition = true;
        n.getSelector().accept(this, arg);
        flgBranchCondition = false;
        indent(n.getBeginLine(), arg);

        if (n.getEntries() != null) {
            for (SwitchEntryStmt e : n.getEntries()) {
                e.accept(this, arg);
            }
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * switch内の case, default
     */
    @Override
    public void visit(SwitchEntryStmt n, NCMStructData arg) {
        NCMStructData.NCMSFuncData fncdata = null;
        if (flgOutsideFunction) {
            fncdata = arg.functionGetData();
        }
        if (n.getLabel() != null) {
            if (fncdata != null) {
                fncdata.cntSelectionCase++;

                addBranch(n.getBeginLine(), arg, fncdata);
            }
            n.getLabel().accept(this, arg);
        } else {
            if (fncdata != null) {
                fncdata.cntSelectionDefault++;

            }
        }
        if (n.getStmts() != null) {
            for (Statement s : n.getStmts()) {
                s.accept(this, arg);
            }
        }
    }

    /**
     * if ステートメント
     */
    @Override
    public void visit(IfStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntConditionalIf++;

            addBranch(n.getBeginLine(), arg, fncdata);
        }
        flgBranchCondition = true;
        n.getCondition().accept(this, arg);
        flgBranchCondition = false;
        n.getThenStmt().accept(this, arg);
        if (n.getElseStmt() != null) {
            n.getElseStmt().accept(this, arg);
        }
    }

    /**
     * while ステートメント
     */
    @Override
    public void visit(WhileStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntLoopsDoWhile++;

            addBranch(n.getBeginLine(), arg, fncdata);
        }
        flgBranchCondition = true;
        n.getCondition().accept(this, arg);
        flgBranchCondition = false;
        n.getBody().accept(this, arg);
    }

    /**
     * do-while ステートメント
     */
    @Override
    public void visit(DoStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntLoopsDoWhile++;

            addBranch(n.getBeginLine(), arg, fncdata);
        }
        n.getBody().accept(this, arg);
        flgBranchCondition = true;
        n.getCondition().accept(this, arg);
        flgBranchCondition = false;
    }

    /**
     * for-each ステートメント
     */
    @Override
    public void visit(ForeachStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntLoopsFor++;
            addBranch(n.getBeginLine(), arg, fncdata);
        }
        if (n.getAnnotations() != null) {
            for (AnnotationExpr a : n.getAnnotations()) {
                a.accept(this, arg);
            }
        }
        if (n.getType() != null) {
            n.getType().accept(this, arg);
        }
        if (n.getId() != null) {
            n.getId().accept(this, arg);
        }
        if (n.getIterable() != null) {
            n.getIterable().accept(this, arg);
        }
        if (n.getBody() != null) {
            n.getBody().accept(this, arg);
        }
    }

    /**
     * for ステートメント
     */
    @Override
    public void visit(ForStmt n, NCMStructData arg) {
        if (flgOutsideFunction) {
            NCMStructData.NCMSFuncData fncdata = arg.functionGetData();
            fncdata.cntLoopsFor++;
            addBranch(n.getBeginLine(), arg, fncdata);
        }

        if (n.getInit() != null) {
            for (Expression e : n.getInit()) {
                e.accept(this, arg);
            }
        }
        if (n.getCompare() != null) {
            flgBranchCondition = true;
            n.getCompare().accept(this, arg);
            flgBranchCondition = false;
        }
        if (n.getUpdate() != null) {
            for (Expression e : n.getUpdate()) {
                e.accept(this, arg);
            }
        }
        if (n.getBody() != null) {
            n.getBody().accept(this, arg);
        }
    }

    /**
     * オブジェクト作成ステートメント 匿名メソッド利用時にも呼び出される。
     */
    @Override
    public void visit(ObjectCreationExpr n, NCMStructData arg) {
        if (n.getScope() != null) {
            n.getScope().accept(this, arg);
        }
        if (n.getTypeArgs() != null) {
            for (Type t : n.getTypeArgs()) {
                t.accept(this, arg);
            }
        }
        n.getType().accept(this, arg);
        if (n.getArgs() != null) {
            for (Expression e : n.getArgs()) {
                e.accept(this, arg);
            }
        }
        if (n.getAnonymousClassBody() != null) {

            addScopeList("");
            indent(n.getBeginLine(), arg);
            for (BodyDeclaration member : n.getAnonymousClassBody()) {
                member.accept(this, arg);
            }
            unindent(n.getEndLine(), arg);
            removeScopeList();
        }
    }

    /**
     * ブロックステートメント
     */
    @Override
    public void visit(BlockStmt n, NCMStructData arg) {
        indent(n.getBeginLine(), arg);
        if (n.getStmts() != null) {
            for (Statement s : n.getStmts()) {
                s.accept(this, arg);
            }
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * アノテーション宣言/定義
     */
    @Override
    public void visit(AnnotationDeclaration n, NCMStructData arg) {
        if (n.getJavaDoc() != null) {
            n.getJavaDoc().accept(this, arg);
        }
        if (n.getAnnotations() != null) {
            for (AnnotationExpr a : n.getAnnotations()) {
                a.accept(this, arg);
            }
        }
        indent(n.getBeginLine(), arg);
        if (n.getMembers() != null) {
            for (BodyDeclaration member : n.getMembers()) {
                member.accept(this, arg);
            }
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * 配列初期化子
     */
    @Override
    public void visit(ArrayInitializerExpr n, NCMStructData arg) {
        indent(n.getBeginLine(), arg);
        if (n.getValues() != null) {
            for (Expression expr : n.getValues()) {
                expr.accept(this, arg);
            }
        }
        unindent(n.getEndLine(), arg);
    }

    /**
     * 二項演算子
     */
    @Override
    public void visit(BinaryExpr n, NCMStructData arg) {

        addBranchBinaryExpr(n, arg);
        n.getLeft().accept(this, arg);
        n.getRight().accept(this, arg);
    }
}
