/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package com.nec.ncmeter;

import java.util.ArrayList;

public class NCMStructData {

    public int debugLevel = 0;

    public int errorLevel = 0;

    public int cntfunc = 0;

    public String DirPath;

    public String FileName;

    public String FuncName = "";

    public String LangType = "";

    public String PackageName = "";

    public int cntImports = 0;

    public String param_string = "";

    public String funcOpt = "";

    private ArrayList<NCMSFuncData> funclist = new ArrayList<NCMSFuncData>();

    /**
     * 関数単位のデータリストのサイズを返却
     */
    public int functionCount() {
        return funclist.size();
    }

    /**
     * 関数単位のデータリストに新たに関数データ領域を追加
     */
    public NCMSFuncData functionNewData() {
        funclist.add(new NCMSFuncData());
        this.cntfunc++;
        return functionGetData();
    }

    /**
     * 関数単位のデータリストから指定した番号の関数データを取得
     */
    public NCMSFuncData functionGetData(int idx) {
        return funclist.get(idx);
    }

    /**
     * 関数単位のデータリストから最後に登録した関数の関数データを取得
     */
    public NCMSFuncData functionGetData() {
        return funclist.get(funclist.size() - 1);
    }

    /**
     * 関数解析後の計量処理 必ず関数スコープの解析終了後に呼び出すこと
     */
    public void functionEndCalc(NCMMethodLOCData loc) {

        NCMSFuncData fncd = functionGetData();

        fncd.cntFloc = loc.floc;
        fncd.cntEloc = loc.eloc;
        fncd.cntCloc = loc.cloc;
        fncd.cntBloc = loc.bloc;

        fncd.calcCyclo();
        fncd.calcCycloCW();
        fncd.calcCNDOPS();
    }

    StringBuilder dbugbuf = new StringBuilder();

    int stpoint_dbugbuf = 0;

    String crlf = System.getProperty("line.separator");

    /**
     * デバッグ情報クリア
     */
    public void dbugbufClear() {
        dbugbuf.setLength(0);
    }

    /**
     * デバッグ情報を追加(改行なし)
     */
    public void dbugbufPrint(int lvl, String arg) {
        if (debugLevel >= lvl)
            dbugbuf.append(arg);
    }

    /**
     * デバッグ情報に改行を追加
     */
    public void dbugbufPrintLn(int lvl) {
        if (debugLevel >= lvl)
            dbugbuf.append(crlf);
    }

    /**
     * デバッグ情報を追加(改行あり)
     */
    public void dbugbufPrintLn(int lvl, String arg) {
        if (debugLevel >= lvl)
            dbugbuf.append(arg + crlf);
    }

    /**
     * デバッグ情報文字列取得
     */
    public String dbugbufread() {
        return dbugbuf.toString();

    }

    /**
     * デバッグ情報文字列を標準出力に出力
     */
    public void dbugbufOutput(int lvl) {
        if (debugLevel >= lvl) {
            if (dbugbuf.length() > stpoint_dbugbuf) {

                System.out.println(dbugbuf.substring(stpoint_dbugbuf));
                stpoint_dbugbuf = dbugbuf.length();
            }
        }
    }

    /**
     * 関数単位のカウントデータ
     */
    class NCMSFuncData {

        public int cntFloc = 0;

        public int cntEloc = 0;

        public int cntCloc = 0;

        public int cntBloc = 0;

        public int cntCyclo = 0;

        public int cntCycloWC = 0;

        public int cntBranchs = 0;

        public int cntCndOps = 0;

        public int cntNests = 0;

        public int cntFunctionBase = 0;

        public int cntLoopsDoWhile = 0;

        public int cntLoopsFor = 0;

        public int cntConditionalIf = 0;

        public int cntSelectionSwitch = 0;

        public int cntSelectionCase = 0;

        public int cntSelectionDefault = 0;

        public int cntInlinedIf = 0;

        public int cntLogicalAnd = 0;

        public String nameFunc = "";

        public String param_string = "";

        private int curNestingDepth = 0;

        /**
         * サイクロマティック数を計算し結果を返却する. 必ず関数スコープの解析終了後に呼び出すこと
         */
        private int calcCyclo() {

            this.cntCyclo = cntFunctionBase + cntLoopsDoWhile + cntLoopsFor;
            this.cntCyclo += cntConditionalIf + cntSelectionSwitch;

            return this.cntCyclo;
        }

        /**
         * サイクロマティック数(case文)を計算し結果を返却する. 必ず関数スコープの解析終了後に呼び出すこと
         */
        private int calcCycloCW() {

            this.cntCycloWC = cntFunctionBase + cntLoopsDoWhile + cntLoopsFor
                    + cntConditionalIf + cntSelectionCase;

            return this.cntCycloWC;
        }

        /**
         * 三項演算子数を計算し結果を返却する. 必ず関数スコープの解析終了後に呼び出すこと
         */
        private int calcCNDOPS() {

            this.cntCndOps = cntInlinedIf;

            return this.cntCndOps;
        }

        /**
         * 現在のbranch数を返却する。
         */
        public int addBranch(int iBranchs) {
            cntBranchs += iBranchs;
            return cntBranchs;
        }

        /**
         * 現在のnest数を返却する。
         */
        public int addNests(int iNests) {
            curNestingDepth += iNests;
            if (curNestingDepth > cntNests) {

                cntNests = curNestingDepth;
            }
            return curNestingDepth;
        }

        /**
         * 関数データを初期化する
         */
        public void CounterClear() {

            cntFloc = 0;
            cntEloc = 0;
            cntCloc = 0;
            cntBloc = 0;
            cntCyclo = 0;
            cntCycloWC = 0;
            cntBranchs = 0;
            cntCndOps = 0;
            cntNests = 0;
            cntFunctionBase = 0;
            cntLoopsDoWhile = 0;
            cntLoopsFor = 0;
            cntConditionalIf = 0;
            cntSelectionSwitch = 0;
            cntSelectionCase = 0;
            cntSelectionDefault = 0;
            cntInlinedIf = 0;
            cntLogicalAnd = 0;
            nameFunc = "";

            param_string = "";

        }

        /**
         * 関数データを返却する
         */
        public String getCounter() {
            String txt = "[" + nameFunc + "]"

            + "PARAM:[" + param_string + "]"

            + "FLOC:" + cntFloc + ",ELOC:" + cntEloc + ",CLOC:" + cntCloc
                    + ",BLOC:" + cntBloc + ",CYCLO:" + cntCyclo + ",CYCLOWC:"
                    + cntCycloWC + ",BRANCH:" + cntBranchs + ",CNDOPS:"
                    + cntCndOps + ",NEST:" + cntNests + ",FuncBase:"
                    + cntFunctionBase + ",DoWhile:" + cntLoopsDoWhile + ",For:"
                    + cntLoopsFor + ",If:" + cntConditionalIf + ",Switch:"
                    + cntSelectionSwitch + ",Case:" + cntSelectionCase
                    + ",Default:" + cntSelectionDefault + ",InlinedIf:"
                    + cntInlinedIf;
            return txt;
        }
    }
}
