/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package japa.parser.ast.body;

import japa.parser.Token;
import japa.parser.ast.Node;
import japa.parser.ast.expr.Expression;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

public class Resource extends Node {
    Parameter parameter;
    Token eqtoken;
    Expression expression;

    public Resource(int beginLine, int beginColumn, int endLine, int endColumn,
            Parameter param, Token eq, Expression exp) {
        super(beginLine, beginColumn, endLine, endColumn);
        this.parameter = param;
        this.eqtoken = eq;
        this.expression = exp;
    }

    @Override
    public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
        return v.visit(this, arg);
    }

    @Override
    public <A> void accept(VoidVisitor<A> v, A arg) {
        v.visit(this, arg);
    }

    public Parameter getParameter() {
        return parameter;
    }

    public Expression getExpression() {
        return expression;
    }

    public Token getEqualToken() {
        return eqtoken;
    }

    public void setParameter(Parameter param) {
        this.parameter = param;
    }

    public void setExpression(Expression exp) {
        this.expression = exp;
    }

    public void setToken(Token eq) {
        this.eqtoken = eq;
    }
}
