/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package japa.parser.ast.expr;

import japa.parser.ast.stmt.BlockStmt;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

public final class LambdaBodyExpr extends Expression {

	private BlockStmt body = null;
	private Expression expr = null;

	public LambdaBodyExpr() {
	}

	public LambdaBodyExpr(BlockStmt body, Expression expr) {
		this.body = body;
		this.expr = expr;
	}

	public LambdaBodyExpr(int beginLine, int beginColumn, int endLine, int endColumn, BlockStmt body, Expression expr) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.body = body;
		this.expr = expr;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public Expression getExpr() {
		return expr;
	}

	public BlockStmt getBody() {
		return body;
	}

	public void setExpr(Expression expr) {
		this.expr = expr;
	}

	public void setBody(BlockStmt body) {
		this.body = body;
	}
}
