/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package japa.parser.ast.expr;

import japa.parser.ast.type.Type;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

public final class MethodReferenceExpr extends Expression {

	private Expression scope = null;

	private List<Type> typeArgs = null;

	private boolean isNew = false;

	private String name = null;

	public MethodReferenceExpr() {
	}

	public MethodReferenceExpr(Expression scope) {
		this.scope = scope;
	}

	public MethodReferenceExpr(Expression scope, List<Type> typeArgs, boolean isNew, String name) {
		this.scope = scope;
		this.typeArgs = typeArgs;
		this.isNew = isNew;
		this.name = name;
	}

	public MethodReferenceExpr(int beginLine, int beginColumn, int endLine, int endColumn, Expression scope,
			List<Type> typeArgs, boolean isNew, String name) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.scope = scope;
		this.typeArgs = typeArgs;
		this.isNew = isNew;
		this.name = name;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<Type> getTypeArgs() {
		return typeArgs;
	}

	public String getName() {
		return name;
	}

	public boolean isNew() {
		return isNew;
	}

	public Expression getScope() {
		return scope;
	}

	public void setTypeArgs(List<Type> typeArgs) {
		this.typeArgs = typeArgs;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setNew(boolean isNew) {
		this.isNew = isNew;
	}

	public void setScope(Expression scope) {
		this.scope = scope;
	}
}
