/*
 * Copyright (C) 2007 J�lio Vilmar Gesser.
 *
 * This file is part of Java 1.5 parser and Abstract Syntax Tree.
 *
 * Java 1.5 parser and Abstract Syntax Tree is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Java 1.5 parser and Abstract Syntax Tree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Java 1.5 parser and Abstract Syntax Tree.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 05/10/2006
 */
/* Modified by NEC Corporation in 2015. */

package japa.parser.ast.type;

import japa.parser.ast.expr.DimExpr;
import japa.parser.ast.visitor.GenericVisitor;
import japa.parser.ast.visitor.VoidVisitor;

import java.util.List;

/**
 * @author Julio Vilmar Gesser
 */
public final class ReferenceType extends Type {

	private Type type;

	private List<DimExpr> dims = null;

	public ReferenceType() {
	}

	public ReferenceType(Type type) {
		this.type = type;
	}

	public ReferenceType(Type type, List<DimExpr> dims) {
		this.type = type;
		this.dims = dims;
	}

	public ReferenceType(int beginLine, int beginColumn, int endLine, int endColumn, Type type, List<DimExpr> dims) {
		super(beginLine, beginColumn, endLine, endColumn);
		this.type = type;
		this.dims = dims;
	}

	@Override
	public <R, A> R accept(GenericVisitor<R, A> v, A arg) {
		return v.visit(this, arg);
	}

	@Override
	public <A> void accept(VoidVisitor<A> v, A arg) {
		v.visit(this, arg);
	}

	public List<DimExpr> getDims() {
		return dims;
	}

	public Type getType() {
		return type;
	}

	public void setDims(List<DimExpr> dims) {
		this.dims = dims;
	}

	public void setType(Type type) {
		this.type = type;
	}

}
