/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <tchar.h>

#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibSystemW.h"
#include "CDMtrLibOptW.h"
#include "CDMtrCntMain.h"
#include "CDMtrCntFunc.h"
#include "CDMtrCntOutputXML.h"
#include "CDMtrCntLog.h"
#include "CDMtrCntOutputCSV.h"
#include "CDMtrMessage.h"

/*----------------------------------------------------------------------------*/
/* static variable declaration                                                */
/*----------------------------------------------------------------------------*/
static _TCHAR *stXMLTagTableOutLine[] = {
    _T("execute_properties"), _T("execute_mode"),       _T("mode_detail"),      _T("tool_path"),       _T("tool_version"),
    _T("command_parameter"),  _T("property_path"),      _T("out_style"),        _T("doc_count"),       _T("macro_def"),
    _T("skip_type"),          _T("diff_blank"),         _T("diff_count_mode"), _T("result_summary"),
    _T("start_date"),         _T("end_date"),         _T("total_number_of_files"), _T("success_number_of_files"),
    _T("error_number_of_files"), _T("ncmeter"),      _T("report"),  _T("src_file"), _T("function") , _T("class")
    , _T("ncm_debug"), _T("ncm_debug_info")
};

static _TCHAR *stXMLTagTableCom[] = {
    _T("metrics_result"),  _T("restyle_type"), _T("dir"), _T("file"), _T("lang"), _T("func")
};

static _TCHAR *stXMLOutputDiffType[] = {
    _T("diff_none"), _T("diff_std"), _T("diff_ext")
};

static _TCHAR *stXMLTagTableRow[][4] = {
    { _T("stmt"), _T("stmt"),  _T("stmt"),  _T("stmt")   },
    { _T("tloc"), _T("tloc"),  _T("rtloc"), _T("rtloc")  },
    { _T("eloc"), _T("eloc"),  _T("reloc"), _T("reloc")  },
    { _T("cloc"), _T("cloc"),  _T("rcloc"), _T("rcloc")  },
    { _T("bloc"), _T("bloc"),  _T("rbloc"), _T("rbloc")  },
    { _T("skip"), _T("mskip"), _T("rskip"), _T("rmskip") },
    { _T("icloc"), _T("icloc"), _T("ricloc"), _T("ricloc") },
    { _T("code"), _T("code"),  _T("rcode"), _T("rcode")  },
    { _T("prep"), _T("prep"),  _T("rprep"), _T("rprep")  },
    { _T("doc") , _T("doc") ,  _T("rdoc") , _T("rdoc")   },
    { _T("rem") , _T("rem"),   _T("rrem"),  _T("rrem")   }
};

static _TCHAR *stXMLTagTableDiffStd[][4] = {
    { _T("state"), _T("state"), _T("state"), _T("state")               },
    { _T("equ_eloc"), _T("equ_eloc"), _T("equ_reloc"), _T("equ_reloc") },
    { _T("add_eloc"), _T("add_eloc"), _T("add_reloc"), _T("add_reloc") },
    { _T("del_eloc"), _T("del_eloc"), _T("del_reloc"), _T("del_reloc") },
    { _T("equ_cloc"), _T("equ_cloc"), _T("equ_rcloc"), _T("equ_rcloc") },
    { _T("add_cloc"), _T("add_cloc"), _T("add_rcloc"), _T("add_rcloc") },
    { _T("del_cloc"), _T("del_cloc"), _T("del_rcloc"), _T("del_rcloc") },
    { _T("olddir"),   _T("olddir"),   _T("olddir"),    _T("olddir")    },
    { _T("oldfile"),  _T("oldfile"),  _T("oldfile"),   _T("oldfile")   }
};

static _TCHAR *stXMLTagTableDiffExt[][4] = {
    { _T("state"),        _T("state"),        _T("state"),        _T("state")        },
    { _T("equ_total"),    _T("equ_total"),    _T("equ_total"),    _T("equ_total")    },
    { _T("add_total"),    _T("add_total"),    _T("add_total"),    _T("add_total")    },
    { _T("del_total"),    _T("del_total"),    _T("del_total"),    _T("del_total")    },
    { _T("modadd_total"), _T("modadd_total"), _T("modadd_total"), _T("modadd_total") },
    { _T("moddel_total"), _T("moddel_total"), _T("moddel_total"), _T("moddel_total") },
    { _T("equ_eloc"),     _T("equ_eloc"),     _T("equ_reloc"),    _T("equ_reloc")    },
    { _T("add_eloc"),     _T("add_eloc"),     _T("add_reloc"),    _T("add_reloc")    },
    { _T("del_eloc"),     _T("del_eloc"),     _T("del_reloc"),    _T("del_reloc")    },
    { _T("modadd_eloc"),  _T("modadd_eloc"),  _T("modadd_reloc"), _T("modadd_reloc") },
    { _T("moddel_eloc"),  _T("moddel_eloc"),  _T("moddel_reloc"), _T("moddel_reloc") },
    { _T("equ_cloc"),     _T("equ_cloc"),     _T("equ_rcloc"),    _T("equ_rcloc")    },
    { _T("add_cloc"),     _T("add_cloc"),     _T("add_rcloc"),    _T("add_rcloc")    },
    { _T("del_cloc"),     _T("del_cloc"),     _T("del_rcloc"),    _T("del_rcloc")    },
    { _T("modadd_cloc"),  _T("modadd_cloc"),  _T("modadd_rcloc"), _T("modadd_rcloc") },
    { _T("moddel_cloc"),  _T("moddel_cloc"),  _T("moddel_rcloc"), _T("moddel_rcloc") },
    { _T("equ_bloc"),     _T("equ_bloc"),     _T("equ_rbloc"),    _T("equ_rbloc")    },
    { _T("add_bloc"),     _T("add_bloc"),     _T("add_rbloc"),    _T("add_rbloc")    },
    { _T("del_bloc"),     _T("del_bloc"),     _T("del_rbloc"),    _T("del_rbloc")    },
    { _T("modadd_bloc"),  _T("modadd_bloc"),  _T("modadd_rbloc"), _T("modadd_rbloc") },
    { _T("moddel_bloc"),  _T("moddel_bloc"),  _T("moddel_rbloc"), _T("moddel_rbloc") },
    { _T("equ_skip"),     _T("equ_mskip"),    _T("equ_rskip"),    _T("equ_rmskip")   },
    { _T("add_skip"),     _T("add_mskip"),    _T("add_rskip"),    _T("add_rmskip")   },
    { _T("del_skip"),     _T("del_mskip"),    _T("del_rskip"),    _T("del_rmskip")   },
    { _T("modadd_skip"),  _T("modadd_mskip"), _T("modadd_rskip"), _T("modadd_rmskip")},
    { _T("moddel_skip"),  _T("moddel_mskip"), _T("moddel_rskip"), _T("moddel_rmskip")},
    { _T("olddir"),       _T("olddir"),       _T("olddir"),       _T("olddir")       },
    { _T("oldfile"),      _T("oldfile"),      _T("oldfile"),      _T("oldfile")      },
};

static _TCHAR *stXMLOutputTable_Struct[][2] = {
    { _T("floc"),    _T("rfloc")   },
    { _T("eloc"),    _T("reloc")   },
    { _T("cloc"),    _T("rcloc")   },
    { _T("bloc"),    _T("rbloc")   },
    { _T("cyclo"),   _T("cyclo")   },
    { _T("cyclowc"), _T("cyclowc") },
    { _T("branchs"), _T("branchs") },
    { _T("cndops"),  _T("cndops")  },
    { _T("nests"),   _T("nests")   },
    { _T("evalr"),   _T("evalr")   },
    { _T("evals"),   _T("evals")   },
    { _T("avals"),   _T("avals")   },
    { _T("parms"),   _T("parms")   },
    { _T("fanin"),   _T("fanin")   },
    { _T("fanout"),  _T("fanout")  },
    { _T("fanin_func"),  _T("fanin_func")  },
    { _T("fanin_param"), _T("fanin_param") },
    { _T("fanin_gval"),  _T("fanin_gval")  },
    { _T("fanout_func"), _T("fanout_func") },
    { _T("fanout_param"), _T("fanout_param")},
    { _T("fanout_gval"), _T("fanout_gval") }

};

static _TCHAR *stXMLTagTableCkCom[] = {
    _T("metrics_result"), _T("dir"), _T("file"), _T("package_name"), _T("class_name"), _T("lang")
};

static _TCHAR *stXMLOutputTable_Ck[][2] = {
    { _T("wmc"),    _T("wmc")   },
    { _T("rfc"),    _T("rfc")   },
    { _T("lcom"),   _T("lcom")  },
    { _T("dit"),    _T("dit")   },
    { _T("noc"),    _T("noc")   },
    { _T("cbo"),    _T("cbo")   }
};

/*----------------------------------------------------------------------------*/
/* static function declaration                                                */
/*----------------------------------------------------------------------------*/
static int cdmtrCntOutputXMLCharEscape(_TCHAR **pChangeStr, _TCHAR **pStr);

/*----------------------------------------------------------------------------*/
/* global function                                                            */
/*----------------------------------------------------------------------------*/
/*!
    @brief      茋ʂɃwb_[o͂(XML)
*/
int CDMtrCntOutputXMLWriteXMLHeader(void)
{
    int iRet = RET_NG;
    _TCHAR *buff = NULL;
    int iFuncRet = RET_NG;
    if (spcOutFile == NULL) {
        iRet = RET_NG;
        goto FUNC_END;
    }
    if (giOutHeaderFlg) {
        if (giBothHeadFlg == CDMTR_FALSE) {
            _fputts(OUTPUT_XML_INIT_DECLARATION, spcOutFile);
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, ROOT_TAG_NAME, NULL);
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, EXE_PROP_TAG_NAME, NULL);
            switch (iExecuteMode) {
            case ROW_MODE:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, EXE_MODE_TAG_NAME, _T("ROW"));
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_OUTLINE, MODE_DETL_TAG_NAME, NULL);
                break;
            case DIFF_MODE:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, EXE_MODE_TAG_NAME, _T("DIFF"));
                if (iDiffModeDetailOut != CDMTR_TRUE) {
                    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_OUTLINE, MODE_DETL_TAG_NAME, NULL);
                } else {
                    if (iDiffMode == OPTION_DIFF_FUNC) {
                        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MODE_DETL_TAG_NAME, _T("FUNC"));
                    } else {
                        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MODE_DETL_TAG_NAME, _T("FILE"));
                    }
                }
                break;
            case STRUCT_MODE:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, EXE_MODE_TAG_NAME, _T("STRUCT"));
                if (iStructMode == OPTION_STRUCT_STD) {
                    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MODE_DETL_TAG_NAME, _T("STANDARD"));
                } else {
                    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MODE_DETL_TAG_NAME, _T("SIMPLE"));
                }
                break;
            case CK_MODE:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, EXE_MODE_TAG_NAME, _T("CK"));
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_OUTLINE, MODE_DETL_TAG_NAME, NULL);
                break;
            }
            iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_OUTLINE, TOOL_PATH_TAG_NAME, pcObjectName);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
            iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_OUTLINE, TOOL_VRSN_TAG_NAME, TOOL_VER);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
            if (pcCommandOptions_MODE != NULL) {
                iFuncRet = CDMtrLibUtilCreateStringW(&buff, pcCommandOptions, pcCommandOptions_MODE, pcCommandOptions_OTHER, NULL);
            } else {
                iFuncRet = CDMtrLibUtilCreateStringW(&buff, pcCommandOptions, pcCommandOptions_OTHER, NULL);
            }
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            }

            iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_OUTLINE,
                                                    CMD_PARAM_TAG_NAME, buff);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
            CDMtrLibUtilFreeW((void **)&buff);
            if (iProfileFlag == CDMTR_TRUE && pcProfFileNameLogOut != NULL) {
                iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_OUTLINE,
                                                        PRP_FILE_TAG_NAME, (void *)pcProfFileNameLogOut);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
            } else {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_OUTLINE, PRP_FILE_TAG_NAME, NULL);
            }
            switch (giRestyleMode) {
            case RESTYLE_BOTH:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, STYL_TYPE_TAG_NAME, OUTPUT_STYLE_BOTH);
                break;
            case RESTYLE_STD:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, STYL_TYPE_TAG_NAME, OUTPUT_STYLE_STD);
                break;
            case RESTYLE_NONE:
            default:
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, STYL_TYPE_TAG_NAME, OUTPUT_STYLE_NONE);
                break;
            }
            if (pcDoc != NULL) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DOC_CNT_TAG_NAME, pcDoc);
            } else {
                if (giDocFlag == CDMTR_FALSE) {
                    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DOC_CNT_TAG_NAME, CNT_STR_FALSE);
                } else {
                    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DOC_CNT_TAG_NAME, CNT_STR_TRUE);
                }
            }
            if (pcDef != NULL) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MCR_DEF_TAG_NAME, pcDef);
            } else {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, MCR_DEF_TAG_NAME, CNT_STR_FALSE);
            }
            if (pcSkip != NULL) {
                CDMtrLibUtilToupperW(pcSkip);
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, SKIP_TYPE_TAG_NAME, pcSkip);
                CDMtrLibUtilTolowerW(pcSkip);
            } else {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, SKIP_TYPE_TAG_NAME, CNT_STR_OLD);
            }
            if (giDiffBlankFlg) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DIFF_BLNK_TAG_NAME, CNT_STR_TRUE);
            } else {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DIFF_BLNK_TAG_NAME, CNT_STR_FALSE);
            }
            if (giDiffCountMode == DIFF_COUNT_EXT) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DIFF_CNTM_TAG_NAME, CNT_STR_EXT);
            } else {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, DIFF_CNTM_TAG_NAME, CNT_STR_STD);
            }
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, EXE_PROP_TAG_NAME, NULL);
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, REPT_TAG_NAME, NULL);
        }
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief      茋ʂɋNƏIo͂(XML)
*/
int CDMtrCntOutputXMLWriteXMLInfomation(_SysLocale *ptSysLocal)
{
    int iRet = RET_NG;
    int iFuncRet;
    _TCHAR *cNum = NULL;
    int iBufLength = 0;
    if (spcOutFile == NULL) {
        goto FUNC_END;
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, REPT_TAG_NAME, NULL);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, RSLT_SMM_TAG_NAME, NULL);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, STRT_DATE_TAG_NAME, pcStartDate);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_OUTLINE, END_DATE_TAG_NAME, pcEndDate);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START, TAG_OUTLINE, TOTL_FLS_TAG_NAME, NULL);
    if (giOutFileAdd) {
        _fputts(OUTPUT_NOT_TARGET, spcOutFile);
    } else {
        iBufLength = (CDMtrLibUtilGetNumSizeW(iNumOfFiles) + 1);
        iFuncRet = CDMtrLibUtilMallocW((void **)&cNum, iBufLength * sizeof(_TCHAR));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        swprintf_s(cNum, iBufLength, _T("%d"), iNumOfFiles);
        _fputts(cNum, spcOutFile);
        CDMtrLibUtilFreeW((void **)&cNum);
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, TOTL_FLS_TAG_NAME, NULL);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START, TAG_OUTLINE, SCS_FLS_TAG_NAME, NULL);
    if (giOutFileAdd) {
        _fputts(OUTPUT_NOT_TARGET, spcOutFile);
    } else {
        iBufLength = (CDMtrLibUtilGetNumSizeW(iNumOfSuccess) + 1);
        iFuncRet = CDMtrLibUtilMallocW((void **)&cNum, iBufLength * sizeof(_TCHAR));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        swprintf_s(cNum, iBufLength, _T("%d"), iNumOfSuccess);
        _fputts(cNum, spcOutFile);
        CDMtrLibUtilFreeW((void **)&cNum);
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, SCS_FLS_TAG_NAME, NULL);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START, TAG_OUTLINE, ERR_FLS_TAG_NAME, NULL);
    if (giOutFileAdd) {
        _fputts(OUTPUT_NOT_TARGET, spcOutFile);
    } else {
        iBufLength = (CDMtrLibUtilGetNumSizeW(iNumOfFiles - iNumOfSuccess) + 1);
        iFuncRet = CDMtrLibUtilMallocW((void **)&cNum, iBufLength * sizeof(_TCHAR));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        swprintf_s(cNum, iBufLength, _T("%d"), (iNumOfFiles - iNumOfSuccess));
        _fputts(cNum, spcOutFile);
        CDMtrLibUtilFreeW((void **)&cNum);
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, ERR_FLS_TAG_NAME, NULL);
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, RSLT_SMM_TAG_NAME, NULL);
    if (pcNcmDebugProp) {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, NCM_DEBUG_TAG_NAME, NULL);
        CDMtrCntLogPrintDebugInfos(DEBUG_PRINT_XML, ptSysLocal);
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, NCM_DEBUG_TAG_NAME, NULL);
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, ROOT_TAG_NAME, NULL);
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          茋ʂɉ͎so͂(XML)
*/
int CDMtrCntOutputXMLStructFileToFanOutXMLFile(_TCHAR *pcDirName, _TCHAR *pcFileName, _TCHAR *pcLangName)
{
    int iRet = RET_NG;
    int iFuncRet;
    int i;
    if (spcOutFile == NULL) {
        goto FUNC_END;
    }
    if (pcDirName == NULL) {
        goto FUNC_END;
    }
    if (pcFileName == NULL) {
        goto FUNC_END;
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
    iFuncRet = CDMtrCntOutputXMLComOutPrint(spcOutFile, 6, pcBeautifier, METRICS_NG_STRING, pcDirName, pcFileName,
                                            pcLangName, NULL);
    if (iFuncRet) goto FUNC_END;
    for (i = FLOC_TAG_STRUCT; i <= FOUT_GVAL_TAG_STRUCT; i++) {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_STRUCT, i,  NULL);
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          \gNX̑茋ʂo͂(XML)
*/
int CDMtrCntOutputXMLStructFileToOutXMLFile(_TCHAR *pBuffer)
{
    int iRet = RET_NG;
    int iLength = 0;
    int icnt = 0;
    int quotFlag = 0;
    int iTagCnt = 0;
    int comma_pos = 0;
    int i_start = 0;
    int bFlocState = METRICS_NG;
    int i = 0;
    _TCHAR *pMetricsRslt;
    int iFuncRet;
    _TCHAR **functions = NULL;
    iFuncRet = CDMtrLibUtilMallocW((void **)&functions, sizeof(_TCHAR *) * (FOUT_GVAL_VALUE + 1));
    if (iFuncRet == RET_NG) goto FUNC_END;
    for (i = 0; i < FOUT_GVAL_VALUE + 1; i++) {
        functions[i] = NULL;
    }
    if (spcOutFile == NULL) {
        goto FUNC_END;
    }
    if (pBuffer ==  NULL) {
        goto FUNC_END;
    }
    for (icnt = 0; icnt < (int)_tcslen(pBuffer); icnt++) {
        switch (pBuffer[icnt]) {
        case _T('\"'):
            quotFlag = ~quotFlag;
            break;
        case _T(','):
        case _T('\r'):
        case _T('\n'):
            if (!quotFlag) {
                if (iTagCnt == 0) {
                    i_start = comma_pos + 1;
                    iLength = &pBuffer[icnt - 1] - &pBuffer[i_start];
                } else if (iTagCnt == 1 || iTagCnt == 2) {
                    if ( (icnt - comma_pos) >= 4) {
                        i_start = comma_pos + 2;
                        iLength = &pBuffer[icnt - 1] - &pBuffer[i_start];
                    } else {
                        i_start = comma_pos + 1;
                        iLength = &pBuffer[icnt] - &pBuffer[i_start];
                    }
                } else {
                    i_start = comma_pos + 1;
                    iLength = &pBuffer[icnt - 1] - &pBuffer[comma_pos];
                }
                if (iTagCnt > FOUT_GVAL_VALUE) goto FUNC_END;
                iFuncRet = CDMtrLibUtilMallocW((void **) & (functions[iTagCnt]), (iLength + 1) * sizeof(_TCHAR));
                if (iFuncRet == RET_NG) goto FUNC_END;
                memset((functions[iTagCnt]), 0x00, (iLength + 1)*sizeof(_TCHAR));
                _tcsncpy((functions[iTagCnt]), pBuffer + i_start, iLength);
                functions[iTagCnt][iLength] = _T('\0');
                iTagCnt++;
                comma_pos = icnt;
            }
        }
    }
    if (functions[FLOC_VALUE] == NULL) {
        goto FUNC_END;
    }
    pMetricsRslt = METRICS_OK_STRING;
    if (_tcscmp(functions[FLOC_VALUE], _T("x")) == 0) {
        pMetricsRslt = METRICS_NG_STRING;
    } else if (_tcscmp(functions[FLOC_VALUE], _T("-")) == 0) {
        bFlocState = METRICS_NO_FUNC;
    } else {
        bFlocState = METRICS_OK;
    }
    for (i = 0; i < FOUT_GVAL_VALUE + 1; i++) {
        if (i == FUNC_VALUE) {
            if (bFlocState == METRICS_NG) {
                continue;
            }
        } else if (i == ELOC_VALUE) {
            if (bFlocState == METRICS_NG) {
                break;
            }
        }
        if (functions[i] == NULL) {
            goto FUNC_END;
        }
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
    iFuncRet = CDMtrCntOutputXMLComOutPrint(spcOutFile, 5, pcBeautifier,
                                            pMetricsRslt, functions[DIR_VALUE], functions[FILE_VALUE], functions[LANG_VALUE]);
    if (iFuncRet == RET_NG) {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
        goto FUNC_END;
    }
    if (bFlocState != METRICS_NG) {
        if (FOUT_GVAL_VALUE + 1 < iTagCnt) {
            CDMtrCntOutputCSVDelQuot(functions[FUNC_VALUE]);
            iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_COM_TYPE, FUNC_TAG_NAME, functions[FUNC_VALUE]);
            if (iFuncRet == RET_NG) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
                goto FUNC_END;
            }
            for (i = FLOC_TAG_STRUCT; i <= FOUT_GVAL_TAG_STRUCT; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_STRUCT, i, NULL);
            }
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
            goto FUNC_END;
        }
        if (bFlocState == METRICS_NO_FUNC) {
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_COM_TYPE, FUNC_TAG_NAME, NULL);
            for (i = FLOC_TAG_STRUCT; i <= FOUT_GVAL_TAG_STRUCT; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_STRUCT, i, NULL);
            }
        } else {
            CDMtrCntOutputCSVDelQuot(functions[FUNC_VALUE]);
            iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_COM_TYPE, FUNC_TAG_NAME, functions[FUNC_VALUE]);
            if (iFuncRet == RET_NG) {
                goto FUNC_END;
            }
            for (i = FLOC_TAG_STRUCT; i <= FOUT_GVAL_TAG_STRUCT; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_STRUCT, i, functions[i + FLOC_VALUE]);
            }
        }
    } else {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_COM_TYPE, FUNC_TAG_NAME, NULL);
        for (i = FLOC_TAG_STRUCT; i <= FOUT_GVAL_TAG_STRUCT; i++) {
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_STRUCT, i, NULL);
        }
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FUNC_UNIT_TAG_NAME, NULL);
    iRet = RET_OK;
FUNC_END:
    for (i = 0; i < FOUT_GVAL_VALUE + 1; i++) {
        if (functions[i] != NULL) {
            CDMtrLibUtilFreeW((void **) & (functions[i]));
        }
    }
    if (functions != NULL) {
        CDMtrLibUtilFreeW((void **)&functions);
    }
    return iRet;
}

/*!
    @brief          茋ʂ̋ʏo͂(XML)
*/
int CDMtrCntOutputXMLComOutPrint(FILE *pfile, int inum, ...)
{
    int iRet = RET_NG;
    _TCHAR *pcString;
    int   iCnt;
    int iFuncRet;
    va_list list;
    enum OUTPUT_COM {
        NUM_RESTYLE = 0, NUM_RSLT, NUM_DIR, NUM_FILE, NUM_LANG, NUM_FUNC
    };
    va_start(list, inum);
    for (iCnt = 0; iCnt < inum; iCnt++) {
        pcString = va_arg(list, _TCHAR *);
        switch (iCnt) {
        case NUM_RESTYLE:
            if (iExecuteMode != CK_MODE) {
                if (pcString) {
                    if (_tcscmp(pcString, _T("true")) == 0) {
                        CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, RESTYLE_TAG_NAME, _T("RESTYLE"));
                    } else {
                        CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, RESTYLE_TAG_NAME, _T("NO_RESTYLE"));
                    }
                } else {
                    CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, RESTYLE_TAG_NAME, _T("NO_RESTYLE"));
                }
            }
            break;
        case NUM_RSLT:
            if (pcString) {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, METRICS_TAG_NAME, pcString);
            } else {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, METRICS_TAG_NAME, METRICS_NG_STRING);
            }
            break;
        case NUM_DIR:
            if (pcString) {
                iFuncRet = CDMtrCntOutputXMLOutPrintUTF(pfile, TAG_PEAR_STRING_UTF, TAG_COM_TYPE, DIR_TAG_NAME, pcString);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
            } else {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_EMPTY, TAG_COM_TYPE, DIR_TAG_NAME, NULL);
            }
            break;
        case NUM_FILE:
            if (pcString) {
                iFuncRet = CDMtrCntOutputXMLOutPrintUTF(pfile, TAG_PEAR_STRING_UTF, TAG_COM_TYPE, FILE_TAG_NAME, pcString);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
            } else {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_EMPTY, TAG_COM_TYPE, FILE_TAG_NAME, NULL);
            }
            break;
        case NUM_LANG:
            if (pcString != NULL) {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, TAG_COM_TYPE, LANG_TAG_NAME, pcString);
            } else {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_NO_TARGET, TAG_COM_TYPE, LANG_TAG_NAME, NULL);
            }
            break;
        case NUM_FUNC:
            if (pcString) {
                iFuncRet = CDMtrCntOutputXMLOutPrintUTF(pfile, TAG_PEAR_STRING_UTF, TAG_COM_TYPE, FUNC_TAG_NAME, pcString);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
            } else {
                CDMtrCntOutputXMLOutPrint(pfile, TAG_EMPTY, TAG_COM_TYPE, FUNC_TAG_NAME, NULL);
            }
            break;
        }
    }
    va_end(list);
    iRet = RET_OK;
FUNC_END:
    va_end(list);
    return iRet;
}


/*!
    @brief          {s̃gNXo͂(XML)
*/
int CDMtrCntOutputXMLOutputBasicLineCount(RESULTINFO_FILEITEMS *ptFileItems, RESULTINFO_ITEMS *ptItems, _LINECOUNT *ptCountLines, int iInvalidFlg)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int i = 0;

    /**  */
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, FILE_UNIT_TAG_NAME, NULL);

    /** Ώۂ̃t@Co͂ */
    iFuncRet = CDMtrCntOutputXMLComOutPrint(spcOutFile, 6, pcBeautifier, METRICS_OK_STRING, ptFileItems->pcDIR,
                                            ptFileItems->pcFILE, ptItems->pcLANG, ptItems->pcFUNC);
    if (iFuncRet == RET_NG) goto FUNC_END;

    /** {sgNXo͂ */
    if (ptItems->pcSTATE != NULL && _tcscmp(ptItems->pcSTATE, DIFF_STATUS_DEL) == 0) {
        /* sŁAA폜t@C(DEL_FILE)̏ꍇ */
        /* {sgNX͏o͂Ȃ(taĝݏo) */
        for (i = STMT_TAG_NAME; i <= REM_TAG_NAME; i++) {
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_ROW_TYPE, i, NULL);
        }

    } else {
        /* s̍폜t@C(DEL_FILE)ȊȌꍇ */
        if (ptItems->plSTMT != NULL) {
            /* STMTڂ̏o͂ */
            if (iInvalidFlg == CDMTR_FALSE) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_NUMBER, TAG_ROW_TYPE, STMT_TAG_NAME, ptItems->plSTMT);
            } else {
                /* s(֐P)ŁAAΏۂ̊֐폜֐܂͖֐̏ꍇ */
                /* STMTڂ͏o͂Ȃ(taĝݏo) */
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_ROW_TYPE, STMT_TAG_NAME, NULL);
            }
        } else {
            /* Xe[gg茋ʂȂꍇASTMTڂ͏o͂Ȃ(taĝݏo) */
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_ROW_TYPE, STMT_TAG_NAME, NULL);
        }

        /* {sgNX̏o͂ */
        switch (ptItems->iLangType) {
        case LANG_TYPE_TXT:
        case LANG_TYPE_BIN:
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_NUMBER, TAG_ROW_TYPE, TLOC_TAG_NAME, &(ptCountLines->ulTLOC));
            for (i = TLOC_TAG_NAME + 1; i <= REM_TAG_NAME; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_ROW_TYPE, i, NULL);
            }
            break;

        case LANG_TYPE_CPP:
        case LANG_TYPE_CPPH:
        case LANG_TYPE_C:
        case LANG_TYPE_CH:
        case LANG_TYPE_JAVA:
        case LANG_TYPE_CS:
        case LANG_TYPE_VB:
        case LANG_TYPE_JS:
        default:
            if (iInvalidFlg == CDMTR_TRUE) {
                /* o͑Ώۂ֐̏ꍇA{sgNX͏o͂Ȃ */
                for (i = TLOC_TAG_NAME; i <= REM_TAG_NAME; i++) {
                    iFuncRet = CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_ROW_TYPE, i, NULL);
                    if (iFuncRet == RET_NG) goto FUNC_END;
                }
            } else {
                /* {sgNXo͂ */
                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, TLOC_TAG_NAME, REM_TAG_NAME, TAG_ROW_TYPE,
                           TAG_TYPE_VALUE, ptCountLines->ulTLOC,
                           TAG_TYPE_VALUE, ptCountLines->ulELOC,
                           TAG_TYPE_VALUE, ptCountLines->ulCLOC,
                           TAG_TYPE_VALUE, ptCountLines->ulBlank,
                           TAG_TYPE_VALUE, ptCountLines->ulUndef,
                           TAG_TYPE_VALUE, ptCountLines->ulICLOC,
                           TAG_TYPE_VALUE, ptCountLines->ulLine,
                           TAG_TYPE_VALUE, ptCountLines->ulPre,
                           TAG_TYPE_VALUE, ptCountLines->ulDoc,
                           TAG_TYPE_VALUE, ptCountLines->ulCmnt);
            }
            if (iFuncRet == RET_NG) goto FUNC_END;
            break;
        }
    }

    iRet = RET_OK;
FUNC_END:
    return iRet;

}

/*!
    @brief          s̃gNXo͂(XML)
*/
int CDMtrCntOutputXMLOutputLineChangeCount(RESULTINFO_FILEITEMS *ptFileItems, RESULTINFO_ITEMS *ptItems, _DIFFLINES *ptDiffLines, int iInvalidFlg)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    int i = 0;

    /** @mF */
    if (ptItems->pcSTATE == NULL) {
        /** {s̑茋ʏo͂̏ꍇAsgNX͏o͂Ȃ */
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_DIFF_TYPE, TAG_TYPE_DIFF_NONE, NULL);
        iRet = RET_OK;
        goto FUNC_END;

    }

    /** s̑茋ʏo͂̏ꍇAsgNXo͂ */
    if (giDiffCountMode == DIFF_COUNT_EXT) {
        /* vpeB̐ݒ肪DIFF_COUNT_MODE=EXT̏ꍇ */
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_DIFF_TYPE, TAG_TYPE_DIFF_EXT, NULL);

        switch (ptItems->iLangType) {
        case LANG_TYPE_BIN:
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_DIFF_EXT, STATE_EXT, (_TCHAR *)ptItems->pcSTATE);
            for (i = STATE_EXT + 1; i < OLD_DIR_EXT; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_DIFF_EXT, i, NULL);
            }
            iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, OLD_DIR_EXT, OLD_FILE_EXT, TAG_DIFF_EXT,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
            if (iFuncRet == RET_NG) goto FUNC_END;
            break;

        case LANG_TYPE_TXT:
            iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, STATE_EXT, MODDEL_TOTAL_EXT, TAG_DIFF_EXT,
                       TAG_TYPE_STRING, ptItems->pcSTATE,
                       TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulEqu,
                       TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulAdd,
                       TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulDel,
                       TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulModAdd,
                       TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulModDel);
            if (iFuncRet == RET_NG) goto FUNC_END;
            for (i = MODDEL_TOTAL_EXT + 1; i < OLD_DIR_EXT; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_DIFF_EXT, i, NULL);
            }

            iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, OLD_DIR_EXT, OLD_FILE_EXT, TAG_DIFF_EXT,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
            if (iFuncRet == RET_NG) goto FUNC_END;
            break;

        case LANG_TYPE_CPP:
        case LANG_TYPE_CPPH:
        case LANG_TYPE_C:
        case LANG_TYPE_CH:
        case LANG_TYPE_JAVA:
        case LANG_TYPE_CS:
        case LANG_TYPE_VB:
        case LANG_TYPE_JS:
        default:
            /* STATEڂo͂ */
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_DIFF_EXT, STATE_EXT, (_TCHAR *)ptItems->pcSTATE);

            if (iInvalidFlg == CDMTR_TRUE) {
                /* o͑Ώۂ֐̏ꍇAsgNX͏o͂Ȃ */
                for (i = STATE_EXT + 1; i < OLD_DIR_EXT; i++) {
                    iFuncRet = CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_DIFF_EXT, i, NULL);
                    if (iFuncRet == RET_NG) goto FUNC_END;
                }

                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, OLD_DIR_EXT, OLD_FILE_EXT, TAG_DIFF_EXT,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);

                if (iFuncRet == RET_NG) goto FUNC_END;

            } else {
                if (ptDiffLines == NULL) {
                    goto FUNC_END;
                }

                /* sgNXo͂ */
                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_TOTAL_EXT, MODDEL_TOTAL_EXT, TAG_DIFF_EXT,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulEqu,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulDel,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulModAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffTotal.ulModDel);
                if (iFuncRet == RET_NG) goto FUNC_END;

                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_ELOC_EXT, MODDEL_ELOC_EXT, TAG_DIFF_EXT,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffEloc.ulEqu,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffEloc.ulAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffEloc.ulDel,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffEloc.ulModAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffEloc.ulModDel);

                if (iFuncRet == RET_NG) goto FUNC_END;

                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_CLOC_EXT, MODDEL_CLOC_EXT, TAG_DIFF_EXT,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffCloc.ulEqu,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffCloc.ulAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffCloc.ulDel,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffCloc.ulModAdd,
                           TAG_TYPE_VALUE, ptDiffLines->tDiffCloc.ulModDel);
                if (iFuncRet == RET_NG) goto FUNC_END;

                if (giDiffBlankFlg) {
                    iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_BLOC_EXT, MODDEL_BLOC_EXT, TAG_DIFF_EXT,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffBloc.ulEqu,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffBloc.ulAdd,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffBloc.ulDel,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffBloc.ulModAdd,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffBloc.ulModDel);
                    if (iFuncRet == RET_NG) goto FUNC_END;

                } else {

                    iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_BLOC_EXT, MODDEL_BLOC_EXT, TAG_DIFF_EXT,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET);
                    if (iFuncRet == RET_NG) goto FUNC_END;

                }
                if (iSkipChangeFlg) {
                    iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_SKIP_EXT, OLD_FILE_EXT, TAG_DIFF_EXT,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffSkip.ulEqu,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffSkip.ulAdd,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffSkip.ulDel,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffSkip.ulModAdd,
                               TAG_TYPE_VALUE, ptDiffLines->tDiffSkip.ulModDel,
                               TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                               TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
                    if (iFuncRet == RET_NG) goto FUNC_END;

                } else {
                    iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, EQU_SKIP_EXT, OLD_FILE_EXT, TAG_DIFF_EXT,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING, OUTPUT_NOT_TARGET,
                               TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                               TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
                    if (iFuncRet == RET_NG) goto FUNC_END;
                }
            }
            break;

        }
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_DIFF_TYPE, TAG_TYPE_DIFF_EXT, NULL);

    } else {
        /* vpeB̐ݒ肪DIFF_COUNT_MODE=STD̏ꍇ */
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_DIFF_TYPE, TAG_TYPE_DIFF_STD, NULL);

        switch (ptItems->iLangType) {
        case LANG_TYPE_BIN:
        case LANG_TYPE_TXT:
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_DIFF_STD, STATE_STD, (_TCHAR *)ptItems->pcSTATE);
            for (i = STATE_STD + 1; i < OLD_DIR_STD; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_DIFF_STD, i, NULL);
            }
            iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, OLD_DIR_STD, OLD_FILE_STD, TAG_DIFF_STD,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                       TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
            if (iFuncRet == RET_NG) goto FUNC_END;
            break;

        case LANG_TYPE_CPP:
        case LANG_TYPE_CPPH:
        case LANG_TYPE_C:
        case LANG_TYPE_CH:
        case LANG_TYPE_JAVA:
        case LANG_TYPE_CS:
        case LANG_TYPE_VB:
        case LANG_TYPE_JS:
        default:
            if (iInvalidFlg == CDMTR_TRUE) {
                /* o͑Ώۂ֐̏ꍇAsgNX͏o͂Ȃ */
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_DIFF_STD, STATE_STD, (_TCHAR *)ptItems->pcSTATE);
                for (i = STATE_STD + 1; i < OLD_DIR_STD; i++) {
                    iFuncRet = CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_NO_TARGET, TAG_DIFF_STD, i, NULL);
                    if (iFuncRet == RET_NG) goto FUNC_END;
                }

                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, OLD_DIR_STD, OLD_FILE_STD, TAG_DIFF_STD,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
                if (iFuncRet == RET_NG) goto FUNC_END;
            } else {
                if (ptDiffLines == NULL) {
                    goto FUNC_END;
                }
                /* sgNXo͂ */
                iFuncRet = CDMtrCntOutputXMLTagCountinue(spcOutFile, STATE_STD, OLD_FILE_STD, TAG_DIFF_STD,
                           TAG_TYPE_STRING, ptItems->pcSTATE,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffEloc.ulEqu,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffEloc.ulAdd,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffEloc.ulDel,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffCloc.ulEqu,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffCloc.ulAdd,
                           TAG_TYPE_VALUE,  ptDiffLines->tDiffCloc.ulDel,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDDIR,
                           TAG_TYPE_STRING_UTF, ptFileItems->pcOLDFILE);
                if (iFuncRet == RET_NG) goto FUNC_END;
            }
            break;
        }
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_DIFF_TYPE, TAG_TYPE_DIFF_STD, NULL);
    }
    iRet = RET_OK;
FUNC_END:
    if (iRet == RET_OK) {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, FILE_UNIT_TAG_NAME, NULL);
    }
    return iRet;
}

/*!
    @brief          CKgNX̑茋ʂo͂(XML)
*/
int CDMtrCntOutputXMLCkFileToOutFile(_TCHAR *pBuffer)
{
    int iRet = RET_NG;
    int iLength = 0;
    int icnt = 0;
    int quotFlag = 0;
    int iTagCnt = 0;
    int comma_pos = 0;
    int i_start = 0;
    int bFlocState = METRICS_NG;
    int i = 0;
    _TCHAR *pMetricsRslt;
    int iFuncRet;
    int iInClassTag = CDMTR_FALSE;
    _TCHAR *classes[CK_CBO_VALUE + 1];
    for (i = 0; i < CK_CBO_VALUE + 1; i++) {
        classes[i] = NULL;
    }
    if (spcOutFile == NULL) {
        goto FUNC_END;;
    }
    if (pBuffer ==  NULL) {
        goto FUNC_END;
    }
    for (icnt = 0; icnt < (int)_tcslen(pBuffer); icnt++) {
        switch (pBuffer[icnt]) {
        case _T('\"'):
            quotFlag = ~quotFlag;
            break;
        case _T(','):
        case _T('\r'):
        case _T('\n'):
            if (!quotFlag) {
                if (iTagCnt == 0) {
                    i_start = comma_pos + 1;
                    iLength = &pBuffer[icnt - 1] - &pBuffer[i_start];
                } else if (CK_CLASS_VALUE >= iTagCnt) {
                    if ( (icnt - comma_pos) >= 4) {
                        i_start = comma_pos + 2;
                        iLength = &pBuffer[icnt - 1] - &pBuffer[i_start];
                    } else {
                        i_start = comma_pos + 1;
                        iLength = &pBuffer[icnt] - &pBuffer[i_start];
                    }
                } else {
                    i_start = comma_pos + 1;
                    iLength = &pBuffer[icnt - 1] - &pBuffer[comma_pos];
                }
                if (iTagCnt > CK_CBO_VALUE) goto FUNC_END;
                iFuncRet = CDMtrLibUtilMallocW((void **) & (classes[iTagCnt]), (iLength + 1) * sizeof(_TCHAR));
                if (iFuncRet == RET_NG) goto FUNC_END;
                memset((classes[iTagCnt]), 0x00, (iLength + 1)*sizeof(_TCHAR));
                _tcsncpy((classes[iTagCnt]), pBuffer + i_start, iLength);
                classes[iTagCnt][iLength] = _T('\0');
                iTagCnt++;
                comma_pos = icnt;
            }
        }
    }
    CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_START_CRL, TAG_OUTLINE, CLASS_UNIT_TAG_NAME, NULL);
    iInClassTag = CDMTR_TRUE;
    pMetricsRslt = METRICS_OK_STRING;
    if (_tcscmp(classes[CK_WMC_VALUE], _T("x")) == 0) {
        pMetricsRslt = METRICS_NG_STRING;
    } else {
        bFlocState = METRICS_OK;
    }
    iFuncRet = CDMtrCntOutputXMLComOutPrint(spcOutFile, 5, pcBeautifier,
                                            pMetricsRslt, classes[CK_DIR_VALUE], classes[CK_FILE_VALUE], classes[CK_LANG_VALUE]);
    if (iFuncRet == RET_NG) {
        goto FUNC_END;
    }
    if (bFlocState != METRICS_NG) {
        CDMtrCntOutputCSVDelQuot(classes[CK_PACKAGE_VALUE]);
        iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_CK_TYPE, PACKAGE_CK_TAG_NAME, classes[CK_PACKAGE_VALUE]);
        if (iFuncRet == RET_NG) {
            goto FUNC_END;
        }
        CDMtrCntOutputCSVDelQuot(classes[CK_CLASS_VALUE]);
        iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_PEAR_STRING_UTF, TAG_CK_TYPE, CLASS_CK_TAG_NAME, classes[CK_CLASS_VALUE]);
        if (iFuncRet == RET_NG) {
            goto FUNC_END;
        }
        if (CK_CBO_VALUE + 1 < iTagCnt) {
            for (i = WMC_TAG_CK; i <= CBO_TAG_CK; i++) {
                CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_CK, i, NULL);
            }
            goto FUNC_END;
        }
        for (i = WMC_TAG_CK; i <= CBO_TAG_CK; i++) {
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_PEAR_STRING, TAG_CK, i, classes[i + CK_WMC_VALUE]);
        }
    } else {
        iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_EMPTY, TAG_CK_TYPE, PACKAGE_CK_TAG_NAME, classes[CK_PACKAGE_VALUE]);
        if (iFuncRet == RET_NG) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrCntOutputXMLOutPrintUTF(spcOutFile, TAG_EMPTY, TAG_CK_TYPE, CLASS_CK_TAG_NAME, classes[CK_CLASS_VALUE]);
        if (iFuncRet == RET_NG) {
            goto FUNC_END;
        }
        for (i = WMC_TAG_CK; i <= CBO_TAG_CK; i++) {
            CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_EMPTY, TAG_CK, i, NULL);
        }
    }
    iRet = RET_OK;
FUNC_END:
    if (iInClassTag) {
        CDMtrCntOutputXMLOutPrint(spcOutFile, TAG_END, TAG_OUTLINE, CLASS_UNIT_TAG_NAME, NULL);
    }
    for (i = 0; i < CK_CBO_VALUE + 1; i++) {
        if (classes[i] != NULL) {
            CDMtrLibUtilFreeW((void **) & (classes[i]));
        }
    }
    return iRet;
}




/*!
    @brief          茋ʃt@C(XML)ɏo͂
*/
int CDMtrCntOutputXMLTagCountinue(FILE *pfile, int start_tag, int end_tag, int iTagType, ...)
{
    int iRet = RET_NG;
    _TCHAR *pcString;
    unsigned long ulVal = 0;
    int iCnt = 0;
    int iFuncRet;
    int i;
    int iTagSetVal = 0;
    int iSet = 0;
    va_list list;
    va_start(list, iTagType);
    for (i = 0; i < (end_tag - start_tag + 1) * 2; i++) {
        iSet = ~iSet;
        if (iSet == 0) {
            switch (iTagSetVal) {
            case TAG_TYPE_STRING:
                pcString = va_arg(list, _TCHAR *);
                iFuncRet = CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_STRING, iTagType, start_tag + iCnt, pcString);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
                break;
            case TAG_TYPE_STRING_UTF:
                pcString = va_arg(list, _TCHAR *);
                iFuncRet = CDMtrCntOutputXMLOutPrintUTF(pfile, TAG_PEAR_STRING_UTF, iTagType, start_tag + iCnt, pcString);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
                break;
            case TAG_TYPE_VALUE:
                ulVal    = va_arg(list, unsigned long);
                iFuncRet = CDMtrCntOutputXMLOutPrint(pfile, TAG_PEAR_NUMBER, iTagType, start_tag + iCnt, &ulVal);
                if (iFuncRet == RET_NG) {
                    goto FUNC_END;
                }
            }
            iCnt++;
        } else {
            iTagSetVal = va_arg(list, int);
        }
    }
    va_end(list);
    iRet = RET_OK;
FUNC_END:
    va_end(list);
    return iRet;
}

/*!
    @brief          UTF-8ɕϊđ茋ʃt@C(XML)o͂
*/
int CDMtrCntOutputXMLOutPrintUTF(FILE *pcFile, int out_type, int iTagType, int tag_id, void *value)
{
    int iRet = RET_NG;
    iRet = CDMtrCntOutputXMLOutPrint(pcFile, out_type, iTagType, tag_id, value);
    return iRet;
}

/*!
    @brief          茋ʃt@C(XML)o͂
*/
int CDMtrCntOutputXMLOutPrint(FILE *pcFile, int out_type, int iTagType, int tag_id, void *value)
{
    int iRet = RET_NG;
    _TCHAR *tag_name = NULL;
    _TCHAR *str_xml_buff = NULL;
    int iFuncRet = RET_NG;
    unsigned long val_number ;
    int iRSetting = 0;
    int iMSetting = 0;
    char *cBuff = NULL;
    if (pcDef != NULL && _tcscmp(pcDef, _T("true")) == 0) {
        iMSetting = 1;
    }
    if (pcBeautifier != NULL && _tcscmp(pcBeautifier, _T("true")) == 0) {
        iRSetting = 2;
    }
    switch (iTagType) {
    case TAG_COM_TYPE:
        tag_name = stXMLTagTableCom[tag_id];
        break;
    case TAG_ROW_TYPE:
        tag_name = stXMLTagTableRow[tag_id][iMSetting + iRSetting];
        break;
    case TAG_DIFF_STD:
        tag_name = stXMLTagTableDiffStd[tag_id][iMSetting + iRSetting];
        break;
    case TAG_DIFF_EXT:
        tag_name = stXMLTagTableDiffExt[tag_id][iMSetting + iRSetting];
        break;
    case TAG_DIFF_TYPE:
        tag_name = stXMLOutputDiffType[tag_id];
        break;
    case TAG_STRUCT:
        if (iRSetting > 0) {
            tag_name = stXMLOutputTable_Struct[tag_id][iRSetting - 1];
        } else {
            tag_name = stXMLOutputTable_Struct[tag_id][iRSetting];
        }
        break;
    case TAG_CK_TYPE:
        tag_name = stXMLTagTableCkCom[tag_id];
        break;
    case TAG_CK:
        if (iRSetting > 0) {
            tag_name = stXMLOutputTable_Ck[tag_id][iRSetting - 1];
        } else {
            tag_name = stXMLOutputTable_Ck[tag_id][iRSetting];
        }
        break;
    case TAG_OUTLINE:
        tag_name = stXMLTagTableOutLine[tag_id];
        break;
    }
    if (pcFile == NULL) {
        goto FUNC_END;
    }
    switch (out_type) {
    case TAG_PEAR_NUMBER:
        val_number = *(unsigned long *)value;
        _ftprintf(pcFile, OUTPUT_XML_LINES_TARGET_FORM, tag_name, val_number, tag_name);
        break;
    case TAG_PEAR_STRING:
        _ftprintf(pcFile, OUTPUT_XML_STR_TARGET_FORM, tag_name, (_TCHAR *)value, tag_name);
        break;
    case TAG_PEAR_STRING_UTF:
    case TAG_PEAR_STRING_EUCtoUTF:
        iFuncRet = cdmtrCntOutputXMLCharEscape(&str_xml_buff, (_TCHAR **)&value);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        cBuff = CDMtrLibUtilConvertStringW(ENCODING_UTF_8, OUTPUT_XML_STR_TARGET_FORM, tag_name, str_xml_buff, tag_name);
        if (cBuff != NULL) {
            fputs(cBuff, pcFile);
        }
        break;
    case TAG_START_CRL:
        _ftprintf(pcFile, OUTPUT_XML_TAGS_STRT_CRL_FORM, tag_name);
        break;
    case TAG_START:
        _ftprintf(pcFile, OUTPUT_XML_TAGS_STRT_FORM, tag_name);
        break;
    case TAG_END:
        _ftprintf(pcFile, OUTPUT_XML_TAGS_END_FORM, tag_name);
        break;
    case TAG_EMPTY:
        _ftprintf(pcFile, OUTPUT_XML_EMPTY_FORM, tag_name, tag_name);
        break;
    case TAG_NO_TARGET:
        _ftprintf(pcFile, OUTPUT_XML_NOT_TARGETS_FORM, tag_name, tag_name);
        break;
    default:
        goto FUNC_END;
        break;
    }
    iRet = RET_OK;
FUNC_END:
    CDMtrLibUtilFreeW((void **)&cBuff);
    CDMtrLibUtilFreeW((void **)&str_xml_buff);
    return iRet;
}

/*----------------------------------------------------------------------------*/
/* static function                                                            */
/*----------------------------------------------------------------------------*/
/*!
    @brief          XMLpւ̕ϊs
*/
static int cdmtrCntOutputXMLCharEscape(_TCHAR **pChangeStr, _TCHAR **pStr)
{
    int iRet = RET_NG;
    _TCHAR *StrOutput = NULL;
    _TCHAR *orgBuff = NULL;
    _TCHAR subBuff[10];
    _TCHAR *pBuff = NULL;
    int iLength = 0;
    int currentLen = 0;
    int i = 0;
    int iFuncRet = 0;
    if (*pStr != NULL) {
        orgBuff = *pStr;
        iLength = _tcslen(orgBuff);
    } else {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilMallocW((void **)&StrOutput, (iLength * 2 + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    memset(StrOutput, 0x00, (iLength * 2 + 1)*sizeof(_TCHAR));
    if (iLength > 0) {
        for (i = 0; i < iLength; i++) {
            memset(subBuff, 0x00, sizeof(subBuff));
            switch (orgBuff[i]) {
            case _T('\''):
                _tcscpy(subBuff, _T("&apos;"));
                break;
            case _T('\"'):
                _tcscpy(subBuff, _T("&quot;"));
                break;
            case _T('&'):
                _tcscpy(subBuff, _T("&amp;"));
                break;
            case _T('<'):
                _tcscpy(subBuff, _T("&lt;"));
                break;
            case _T('>'):
                _tcscpy(subBuff, _T("&gt;"));
                break;
            default:
                subBuff[0] = orgBuff[i];
            }
            currentLen = _tcslen(StrOutput);
            if (iLength * 2 + (int)_tcslen(subBuff) >= currentLen) {
                iFuncRet = CDMtrLibUtilMallocW((void **)&pBuff, (iLength * 2 + iLength + 1) * sizeof(_TCHAR));
                if (iFuncRet != RET_OK) {
                    CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000298, pBuff); //* MSG_LOG_ID_C_00000298
                    goto FUNC_END;
                }
                _tcscpy(pBuff, StrOutput);
                CDMtrLibUtilFreeW((void **)&StrOutput);
                StrOutput = pBuff;
                pBuff = NULL;
            }
            _tcscat(StrOutput, subBuff);
        }
        iFuncRet = CDMtrLibUtilMallocW((void **)pChangeStr, (_tcslen(StrOutput) + 1) * sizeof(_TCHAR));
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        _tcscpy(*pChangeStr, StrOutput);
        iRet = RET_OK;
    }
FUNC_END:
    if (StrOutput != NULL) {
        CDMtrLibUtilFreeW((void **)&StrOutput);
    }
    if (pBuff != NULL) {
        CDMtrLibUtilFreeW((void **)&pBuff);
    }
    return iRet;
}
