/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "CDMtrCmn.h"
#include "CDMtrExtCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibSystemW.h"
#include "CDMtrCntMain.h"
#include "CDMtrCntXrefDataUtil.h"
#include "CDMtrMessage.h"

#define TIME_BUFFER_SIZE 40

#define ALLOC_BASE_SIZE 10000
#define SNAME_TABLE_SIZE 100
#define NAME_ALLOC_SIZE 10
#define NAME_STACK_SIZE 1000

static nameStack *nameTable = NULL;

static xrefData *spXrefDataList = NULL;
static xrefData **sppStaticList = NULL;
static fanCounter *spFanCounter = NULL;

/*!
    @brief  static֐e[ü̗
*/
void CDMtrCntXrefDataUtilFreeNameTable()
{
    int i;
    if (nameTable == NULL) {
        return;
    }
    for (i = 0; i < SNAME_TABLE_SIZE; i++) {
        if (nameTable[i].pNameStack != NULL) {
            free(nameTable[i].pNameStack);
            nameTable[i].pNameStack = NULL;
        }
        if (nameTable[i].ppXrefData != NULL) {
            free(nameTable[i].ppXrefData);
            nameTable[i].ppXrefData = NULL;
        }
    }
    free(nameTable);
    nameTable = NULL;
}

/*!
    @brief  NXt@Xt@Cf[^̗̈
*/
void CDMtrCntXrefDataUtilFreeXrefData()
{
    if (spXrefDataList != NULL) {
        free(spXrefDataList);
        spXrefDataList = NULL;
    }
}

/*!
    @brief  static֐Xg̗̈
*/
void CDMtrCntXrefDataUtilFreeStaticList()
{
    if (sppStaticList != NULL) {
        free(sppStaticList);
        sppStaticList = NULL;
    }
}

/*!
    @brief  FANIN/FANOUTJEgʊi[̗̈̈
*/
void CDMtrCntXrefDataUtilFreeFanCounter()
{
    if (spFanCounter != NULL) {
        free(spFanCounter);
        spFanCounter = NULL;
    }
}

/*!
    @brief  ُ픭̈
*/
void CDMtrCntXrefDataUtilFreeAllXrefTables()
{
    CDMtrCntXrefDataUtilFreeNameTable();
    CDMtrCntXrefDataUtilFreeXrefData();
    CDMtrCntXrefDataUtilFreeStaticList();
    CDMtrCntXrefDataUtilFreeFanCounter();
    CDMtrCntXrefDataUtilFreeHashTable();
}

/*!
    @brief  Od`FbN
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilIsDuplicateName")
int CDMtrCntXrefDataUtilIsDuplicateName(xrefData *xref)
{
    int i = 0, iDuplicateFlag = 0;
    unsigned int tableIndex = 0;
    if (nameTable == NULL) {
        nameTable = (nameStack *)malloc(sizeof(nameStack) * SNAME_TABLE_SIZE);
        if (nameTable == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000450)); //* MSG_LOG_ID_C_00000450
            return RET_UTIL_ERROR;
        }
        memset(nameTable, 0, sizeof(nameStack)*SNAME_TABLE_SIZE);
    }
    tableIndex = getHashNumber(xref->s_name) % SNAME_TABLE_SIZE;
    for (i = 0; i < nameTable[tableIndex].numOfStack; i++) {
        if (_tcscmp(nameTable[tableIndex].pNameStack + (XREF_NAME_LEN * i), xref->s_name) == 0) {
            if (_tcscmp(nameTable[tableIndex].ppXrefData[i]->s_id, xref->s_id) != 0) {
                if (iDuplicateFlag == 0) {
                    WARNING_LOG((MSG_LOG_ID_C_00000467, xref->s_name, //* MSG_LOG_ID_C_00000467
                                 xref->a_dir, xref->a_file,
                                 nameTable[tableIndex].ppXrefData[i]->a_dir, nameTable[tableIndex].ppXrefData[i]->a_file));
                    iDuplicateFlag = 1;
                }
            } else {
                DEBUG_LOG((MSG_LOG_ID_C_00000470, //* MSG_LOG_ID_C_00000470
                           xref->s_name, xref->a_dir, xref->a_file));
                return RET_UTIL_DUPLICATE_SKIP;
            }
        }
    }
    if (nameTable[tableIndex].numOfAlloced == nameTable[tableIndex].numOfStack) {
        nameTable[tableIndex].pNameStack = (_TCHAR *)realloc(nameTable[tableIndex].pNameStack,
                                           sizeof(_TCHAR) * XREF_NAME_LEN * (nameTable[tableIndex].numOfAlloced + NAME_ALLOC_SIZE) );
        if (nameTable[tableIndex].pNameStack == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000451)); //* MSG_LOG_ID_C_00000451
            return RET_UTIL_ERROR;
        }
        memset(nameTable[tableIndex].pNameStack + (XREF_NAME_LEN * nameTable[tableIndex].numOfAlloced),
               0, sizeof(_TCHAR)*XREF_NAME_LEN * NAME_ALLOC_SIZE);
        nameTable[tableIndex].ppXrefData = (xrefData **)realloc(nameTable[tableIndex].ppXrefData,
                                           sizeof(xrefData *) * (nameTable[tableIndex].numOfAlloced + NAME_ALLOC_SIZE) );
        if (nameTable[tableIndex].ppXrefData == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000452)); //* MSG_LOG_ID_C_00000452
            return RET_UTIL_ERROR;
        }
        memset(nameTable[tableIndex].ppXrefData + nameTable[tableIndex].numOfAlloced,
               0, sizeof(xrefData *)*NAME_ALLOC_SIZE);
        nameTable[tableIndex].numOfAlloced += NAME_ALLOC_SIZE;
    }
    _tcsncpy(nameTable[tableIndex].pNameStack + (XREF_NAME_LEN * nameTable[tableIndex].numOfStack), xref->s_name, XREF_NAME_LEN - 1);
    nameTable[tableIndex].ppXrefData[nameTable[tableIndex].numOfStack] = xref;
    nameTable[tableIndex].numOfStack++;
    return RET_UTIL_NEWREGISTERED;
}

/*!
    @brief  ֐Xgo\bh
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilGetFunctionList")
fanCounter *CDMtrCntXrefDataUtilGetFunctionList(fanXrefDataList *xrefDataList, int *funcListSize)
{
    int i, iRet, dataSize = 0, outDataSize = 0, outDataAllocNum = 0,
                 numOfStaticList = 0;
    int staticAllocNum = 0;
    xrefData *pSrcData = NULL;
    fanCounter *pOutData = NULL;
    if ( xrefDataList == NULL || xrefDataList->pSrcList == NULL
            || xrefDataList->numOfSrcList == 0) {
        return NULL;
    }
    pSrcData = xrefDataList->pSrcList;
    dataSize = xrefDataList->numOfSrcList;
    for (i = 0; i < dataSize; i++) {
        if (pSrcData[i].s_category[0] == _T('f')
                && _tcscmp(XREF_A_CATEGORY_DEFINITION, pSrcData[i].a_category) == 0) {
            if (_tcscmp(XREF_D_DSPEC_STATIC, pSrcData[i].d_dspec) == 0) {
                if (numOfStaticList == staticAllocNum) {
                    sppStaticList = (xrefData **)realloc(sppStaticList, sizeof(xrefData *) * (staticAllocNum + ALLOC_BASE_SIZE));
                    if (sppStaticList == NULL) {
                        ERROR_LOG((MSG_LOG_ID_C_00000453,  sizeof(xrefData *) * (staticAllocNum + ALLOC_BASE_SIZE))); //* MSG_LOG_ID_C_00000453
                        goto FUNC_END;
                    }
                    memset(sppStaticList + staticAllocNum, 0, sizeof(xrefData *)*ALLOC_BASE_SIZE);
                    staticAllocNum += ALLOC_BASE_SIZE;
                }
                sppStaticList[numOfStaticList] = pSrcData + i;
                numOfStaticList++;
            } else {
                iRet = CDMtrCntXrefDataUtilIsDuplicateName(&(pSrcData[i]));
                if (iRet == RET_UTIL_DUPLICATE_SKIP) {
                    continue;
                }
                if (iRet == RET_UTIL_ERROR || iRet == RET_UTIL_DUPLICATE_ABORT) {
                    ERROR_LOG((MSG_LOG_ID_C_00000454, //* MSG_LOG_ID_C_00000454
                               pSrcData[i].s_name, pSrcData[i].s_id));
                    goto FUNC_END;
                }
            }
            if (outDataSize == outDataAllocNum) {
                spFanCounter = (fanCounter *)realloc(spFanCounter, sizeof(fanCounter) * (outDataAllocNum + ALLOC_BASE_SIZE));
                if (spFanCounter == NULL) {
                    ERROR_LOG((MSG_LOG_ID_C_00000455, sizeof(fanCounter) * (outDataAllocNum + ALLOC_BASE_SIZE))); //* MSG_LOG_ID_C_00000455
                    goto FUNC_END;
                }
                memset(spFanCounter + outDataAllocNum, 0, sizeof(fanCounter)*ALLOC_BASE_SIZE);
                outDataAllocNum += ALLOC_BASE_SIZE;
            }
            spFanCounter[outDataSize].pXrefData = pSrcData + i;
            outDataSize ++;
        }
    }
    *funcListSize = outDataSize;
    xrefDataList->ppStaticList = sppStaticList;
    xrefDataList->numOfStaticList = numOfStaticList;
    DEBUG_LOG((MSG_LOG_ID_C_00000471, outDataSize, numOfStaticList)); //* MSG_LOG_ID_C_00000471
    pOutData = spFanCounter;
FUNC_END:
    if (pOutData == NULL) {
        CDMtrCntXrefDataUtilFreeStaticList();
        CDMtrCntXrefDataUtilFreeFanCounter();
    }
    CDMtrCntXrefDataUtilFreeNameTable();
    return pOutData;
};

/*!
    @brief  xrefData\̂S_IDƂ̔rs
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilIsSameSid")
int CDMtrCntXrefDataUtilIsSameSid(xrefData **ppXrefDataList, int dataSize, _TCHAR *id, unsigned int hash_id)
{
    int i;
    int iRet = RET_FALSE;
    if (id == NULL ) {
        ERROR_LOG((MSG_LOG_ID_C_00000456)); //* MSG_LOG_ID_C_00000456
        goto FUNC_END;
    }
    if (ppXrefDataList == NULL) {
        goto FUNC_END;
    }
    for (i = 0; i < dataSize; i ++ ) {
        if (ppXrefDataList[i]->s_id_hash == hash_id
                && _tcscmp(ppXrefDataList[i]->s_id, id) == 0) {
            iRet = RET_TRUE;
            break;
        }
    }
FUNC_END:
    return iRet;
}


/*!
    @brief  OX^bN̈̏֐
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilInitNameStack")
void CDMtrCntXrefDataUtilInitNameStack(nameStack *pStack)
{
    if (pStack == NULL) {
        ;
    } else {
        pStack->numOfStack = 0;
    }
}

/*!
    @brief  OX^bNɒǉ
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilPushNameStack")
int CDMtrCntXrefDataUtilPushNameStack(nameStack *pStack, _TCHAR *name)
{
    int iRet = RET_NG;
    if (pStack == NULL || name == NULL ) {
        goto FUNC_END;
    }
    if (pStack->numOfStack == pStack->numOfAlloced) {
        pStack->pNameStack
            = (_TCHAR *)realloc(pStack->pNameStack, XREF_NAME_LEN * (pStack->numOfAlloced + NAME_STACK_SIZE));
        if (pStack->pNameStack == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000457)); //* MSG_LOG_ID_C_00000457
            goto FUNC_END;
        }
        memset(pStack->pNameStack + pStack->numOfAlloced, 0, XREF_NAME_LEN * NAME_STACK_SIZE);
        pStack->numOfAlloced += NAME_STACK_SIZE;
    }
    _tcscpy(pStack->pNameStack + (XREF_NAME_LEN * pStack->numOfStack), name);
    pStack->numOfStack++;
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief  OB
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilIsStacked")
int CDMtrCntXrefDataUtilIsStacked(nameStack *pStack, _TCHAR *name)
{
    int i = 0;
    int iRet = RET_FALSE;
    if (pStack == NULL || name == NULL ) {
        goto FUNC_END;
    } else {
        for (i = 0; i < pStack->numOfStack; i++) {
            if ( _tcscmp(pStack->pNameStack + (XREF_NAME_LEN * i), name) == 0 ) {
                iRet = RET_TRUE;
                break;
            }
        }
    }
FUNC_END:
    return iRet;
}
/*!
    @brief  OX^bÑf[^̗̈̈B
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilFreeNameStackData")
void CDMtrCntXrefDataUtilFreeNameStackData(nameStack *pStack)
{
    if (pStack != NULL) {
        if (pStack->pNameStack != NULL) {
            free(pStack->pNameStack);
            pStack->pNameStack = NULL;
        }
        if (pStack->ppXrefData != NULL) {
            free(pStack->ppXrefData);
            pStack->ppXrefData = NULL;
        }
        if (pStack->pHashStack != NULL) {
            free(pStack->pHashStack);
            pStack->pHashStack = NULL;
        }
    }
}

/*!
    @brief  NXt@Xt@C(CSV)Token擾
*/
#define CSV_LINE_MAX_LEN XREF_ID_LEN + XREF_NAME_LEN + XREF_CATEGORY_LEN + XREF_DSPEC_LEN + XREF_TYPE_LEN + XREF_CATEGORY_LEN + XREF_DIR_LEN + XREF_FILE_LEN + XREF_ID_LEN + XREF_NAME_LEN + XREF_MODIFIED_LEN + XREF_VARIABLE_LEN + 12

#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilGetNextCsvToken")
_TCHAR *CDMtrCntXrefDataUtilGetNextCsvToken(_TCHAR *strToken, int tokenMax, const _TCHAR *strCsvLineData)
{
    static const _TCHAR *strSourceData = NULL;
    static int readCount = 0;
    int startPos = 0, endPos = 0;
    int doubleQuot = 0;
    if (strToken == NULL || tokenMax == 0) {
        ERROR_LOG((MSG_LOG_ID_C_00000458)); //* MSG_LOG_ID_C_00000458
        return NULL;
    }
    if (strCsvLineData != NULL) {
        strSourceData = strCsvLineData;
        readCount = 0;
    }
    if (strSourceData == NULL) {
        return NULL;
    }
    startPos = readCount;
    if (strSourceData[readCount] == _T('"') ) {
        doubleQuot = 1;
        startPos++;
        readCount++;
    }
    for (; readCount < startPos + tokenMax; readCount++) {
        endPos = readCount;
        if (strSourceData[readCount] == _T('"') && doubleQuot == 1 && strSourceData[readCount + 1] == _T(',')) {
            readCount += 2;
            break;
        } else if ( (strSourceData[readCount] == _T(',')) && (doubleQuot == 0) ) {
            readCount++;
            break;
        } else if ( strSourceData[readCount] == _T('\r') || strSourceData[readCount] == _T('\n') || strSourceData[readCount] == _T('\0') ) {
            if (doubleQuot == 0) {
                readCount++;
                break;
            } else {
                return NULL;
            }
        }
    }
    if (startPos == endPos) {
        strToken = NULL;
    } else {
        _tcsncpy(strToken, strSourceData + startPos, endPos - startPos);
        strToken[endPos - startPos] = _T('\0');
    }
    return strToken;
}

xrefHash *gXrefAfuncHashTable = NULL;
xrefHash *gXrefSnameHashTable = NULL;
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilInitHashTable")
void CDMtrCntXrefDataUtilInitHashTable()
{
    if (gXrefAfuncHashTable == NULL) {
        gXrefAfuncHashTable = (xrefHash *)malloc(sizeof(xrefHash) * HASH_SIZE);
        if (gXrefAfuncHashTable == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000459)); //* MSG_LOG_ID_C_00000459
            return ;
        }
    }
    memset(gXrefAfuncHashTable, 0, sizeof(xrefHash)*HASH_SIZE);
    if (gXrefSnameHashTable == NULL) {
        gXrefSnameHashTable = (xrefHash *)malloc(sizeof(xrefHash) * HASH_SIZE);
        if (gXrefSnameHashTable == NULL) {
            ERROR_LOG((MSG_LOG_ID_C_00000460)); //* MSG_LOG_ID_C_00000460
            return ;
        }
    }
    memset(gXrefSnameHashTable, 0, sizeof(xrefHash)*HASH_SIZE);
}

#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilFreeHashTable")
void CDMtrCntXrefDataUtilFreeHashTable()
{
    int i = 0;
    if (gXrefAfuncHashTable != NULL) {
        for (i = 0; i < HASH_SIZE; i++) {
            if ((gXrefAfuncHashTable + i)->ppXrefDataList != NULL) {
                free((gXrefAfuncHashTable + i)->ppXrefDataList);
            }
        }
        free(gXrefAfuncHashTable);
        gXrefAfuncHashTable = NULL;
    }
    if (gXrefSnameHashTable != NULL) {
        for (i = 0; i < HASH_SIZE; i++) {
            if ((gXrefSnameHashTable + i)->ppXrefDataList != NULL) {
                free((gXrefSnameHashTable + i)->ppXrefDataList);
            }
        }
        free(gXrefSnameHashTable);
        gXrefSnameHashTable = NULL;
    }
}

/*!
    @brief  nbVe[uւ̃f[^o^
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilSetHashTable")
int CDMtrCntXrefDataUtilSetHashTable(xrefData *pXrefData)
{
    int iRet = RET_NG;
    xrefHash *pXrefHash = NULL;
    if (pXrefData == NULL) {
        goto FUNC_END;
    }
    pXrefHash = gXrefAfuncHashTable + (pXrefData->a_func_hash % HASH_SIZE);
    if (pXrefHash->allocNum == pXrefHash->xrefDataNum) {
        pXrefHash->ppXrefDataList
            = (xrefData **)realloc(pXrefHash->ppXrefDataList,
                                   sizeof(xrefData *) * (pXrefHash->allocNum + HASH_LIST_BASE));
        if (pXrefHash->ppXrefDataList == NULL) {
            goto FUNC_END;
        }
        memset(pXrefHash->ppXrefDataList + pXrefHash->allocNum, 0,
               sizeof(xrefData *)*HASH_LIST_BASE);
        pXrefHash->allocNum += HASH_LIST_BASE;
    }
    pXrefHash->ppXrefDataList[pXrefHash->xrefDataNum] = pXrefData;
    pXrefHash->xrefDataNum++;
    pXrefHash = gXrefSnameHashTable + (pXrefData->s_name_hash % HASH_SIZE);
    if (pXrefHash->allocNum == pXrefHash->xrefDataNum) {
        pXrefHash->ppXrefDataList
            = (xrefData **)realloc(pXrefHash->ppXrefDataList,
                                   sizeof(xrefData *) * (pXrefHash->allocNum + HASH_LIST_BASE));
        if (pXrefHash->ppXrefDataList == NULL) {
            goto FUNC_END;
        }
        memset(pXrefHash->ppXrefDataList + pXrefHash->allocNum, 0,
               sizeof(xrefData *)*HASH_LIST_BASE);
        pXrefHash->allocNum += HASH_LIST_BASE;
    }
    pXrefHash->ppXrefDataList[pXrefHash->xrefDataNum] = pXrefData;
    pXrefHash->xrefDataNum++;
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief  NXt@Xt@C̓ǂݍ݁nbVe[u쐬
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefDataUtilLoadXrefByFileAndMakeHashTable")
xrefData *CDMtrCntXrefDataUtilLoadXrefByFileAndMakeHashTable(_TCHAR *filename, int *structSize)
{
#define XREF_DUMMY_LEN 256
    FILE *fp;
    _TCHAR csvLineData[CSV_LINE_MAX_LEN];
    int dataCount = 0, lineCount = 0;
    xrefData *pNewData = NULL;
    int iSkipFlg = CDMTR_FALSE;
    if ((fp = _tfopen(filename, _T("r, ccs=UTF-8"))) == NULL) {
        DEBUG_LOG((MSG_LOG_ID_C_00000472, filename)); //* MSG_LOG_ID_C_00000472
        goto FUNC_END;
    }
    while (_fgetts(csvLineData, CSV_LINE_MAX_LEN, fp) != NULL) {
        lineCount++;
    }
    DEBUG_LOG((MSG_LOG_ID_C_00000473, lineCount, sizeof(xrefData))); //* MSG_LOG_ID_C_00000473
    spXrefDataList = (xrefData *)calloc(lineCount, sizeof(xrefData));
    if (spXrefDataList == NULL) {
        ERROR_LOG((MSG_LOG_ID_C_00000461)); //* MSG_LOG_ID_C_00000461
        goto FUNC_END;
    }
    if (fseek(fp, 0, SEEK_SET) != 0) {
        ERROR_LOG((MSG_LOG_ID_C_00000462)); //* MSG_LOG_ID_C_00000462
        goto FUNC_END;
    }
    CDMtrCntXrefDataUtilInitHashTable();
    if (gXrefAfuncHashTable == NULL || gXrefSnameHashTable == NULL) {
        ERROR_LOG((MSG_LOG_ID_C_00000463)); //* MSG_LOG_ID_C_00000463
        goto FUNC_END;
    }
    while (_fgetts(csvLineData, CSV_LINE_MAX_LEN, fp) != NULL) {
        if (csvLineData[0] == _T('\0')) {
            break;
        }
        if (_tcschr(csvLineData, _T('\n')) == NULL) {
            iSkipFlg = CDMTR_TRUE;
            continue;
        } else {
            if (iSkipFlg == CDMTR_TRUE) {
                iSkipFlg = CDMTR_FALSE;
                continue;
            } else {
            }
        }
        pNewData = spXrefDataList + dataCount;
        pNewData->s_id[0] = _T('\0');
        pNewData->s_name[0] = _T('\0');
        pNewData->s_category[0] = _T('\0');
        pNewData->d_dspec[0] = _T('\0');
        pNewData->d_type[0] = _T('\0');
        pNewData->a_category[0] = _T('\0');
        pNewData->a_dir[0] = _T('\0');
        pNewData->a_file[0] = _T('\0');
        pNewData->a_id[0] = _T('\0');
        pNewData->a_func[0] = _T('\0');
        pNewData->a_modified[0] = _T('\0');
        pNewData->variable_type[0] = _T('\0');
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->s_id, XREF_ID_LEN, csvLineData);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->s_name, XREF_NAME_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->s_category, XREF_CATEGORY_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->d_dspec, XREF_DSPEC_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->d_type, XREF_TYPE_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_category, XREF_CATEGORY_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_dir, XREF_DIR_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_file, XREF_FILE_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_id, XREF_ID_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_func, XREF_NAME_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->a_modified, XREF_MODIFIED_LEN, NULL);
        CDMtrCntXrefDataUtilGetNextCsvToken(pNewData->variable_type, XREF_VARIABLE_LEN, NULL);
        pNewData->s_id_hash = getHashNumber(pNewData->s_id);
        pNewData->s_name_hash = getHashNumber(pNewData->s_name);
        pNewData->a_id_hash = getHashNumber(pNewData->a_id);
        if (pNewData->a_func[0] != _T('\0')) {
            pNewData->a_func_hash = getHashNumber(pNewData->a_func);
        }
        if (pNewData->s_category[0] == XREF_S_CATEGORY_FUNCTION_INITIAL) {
            pNewData->bitFlag |= XREF_S_CATEGORY_FUNCTION_BIT;
        }
        if (pNewData->d_dspec[0] == XREF_D_DSPEC_STATIC_INITIAL) {
            pNewData->bitFlag |= XREF_D_DSPEC_STATIC_BIT;
        }
        if (_tcscmp(pNewData->a_category, XREF_A_CATEGORY_DEFINITION) == 0) {
            pNewData->bitFlag |= XREF_A_CATEGORY_DEFINITION_BIT;
        } else if (_tcscmp(pNewData->a_category, XREF_A_CATEGORY_DECL) == 0 ) {
            memset(pNewData, 0, sizeof(xrefData));
            continue;
        }
        if (pNewData->a_modified[0] == XREF_A_MODIFIED_MODIFIED_INITIAL) {
            pNewData->bitFlag |= XREF_A_MODIFIED_MODIFIED_BIT;
        }
        if (pNewData->variable_type[0] == XREF_VARIABLE_TYPE_P_INITAL) {
            pNewData->bitFlag |= XREF_VARIABLE_TYPE_P_BIT;
        } else if (pNewData->variable_type[0] == XREF_VARIABLE_TYPE_L_INITAL) {
            memset(pNewData, 0, sizeof(xrefData));
            continue;
        } else if (pNewData->variable_type[0] == XREF_VARIABLE_TYPE_G_INITAL) {
            pNewData->bitFlag |= XREF_VARIABLE_TYPE_G_BIT;
        }
        if (_tcscmp(pNewData->a_category, XREF_A_CATEGORY_DEFINITION) != 0) {
            CDMtrCntXrefDataUtilSetHashTable(pNewData);
        }
        dataCount++;
    }
    if (fp != NULL) {
        fclose(fp);
        fp = NULL;
    }
    *structSize = dataCount;
    return spXrefDataList;
FUNC_END:
    if (fp != NULL) {
        fclose(fp);
        fp = NULL;
    }
    CDMtrCntXrefDataUtilFreeHashTable();
    CDMtrCntXrefDataUtilFreeXrefData();
    return NULL;
}


typedef struct csvHash {
    fanCounter **ppFanCounter;
    int hashNum;
    int allocNum;
} csvHash;

/*!
    @brief  nbVe[u쐬
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntMakeFanCountHashTable")
int CDMtrCntMakeFanCountHashTable(fanCounterList *fanCountList, csvHash *hashTable)
{
    xrefData *pXrefData = NULL;
    csvHash *pCsvhash = NULL;
    _TCHAR pFanCounterKey[XREF_DIR_LEN + XREF_FILE_LEN + XREF_NAME_LEN];
    int i;
    int iRet = RET_NG;
    for (i = 0; i < fanCountList->iListSize; i++ ) {
        memset(pFanCounterKey, 0, XREF_DIR_LEN + XREF_FILE_LEN + XREF_NAME_LEN);
        if (hashTable == NULL) {
            goto FUNC_END;
        }
        pXrefData = fanCountList->pFanCounter[i].pXrefData;
        _tcscat(pFanCounterKey, pXrefData->a_dir);
        _tcscat(pFanCounterKey, pXrefData->a_file);
        _tcscat(pFanCounterKey, pXrefData->s_name);
        fanCountList->pFanCounter[i].csvHashKey = getHashNumber(pFanCounterKey);
        pCsvhash = hashTable + (fanCountList->pFanCounter[i].csvHashKey % HASH_SIZE);
        if (pCsvhash->allocNum == pCsvhash->hashNum) {
            pCsvhash->ppFanCounter = (fanCounter **)realloc(pCsvhash->ppFanCounter,
                                     sizeof(fanCounter *) * (pCsvhash->allocNum + HASH_LIST_BASE));
            if (pCsvhash->ppFanCounter == NULL) {
                ERROR_LOG((MSG_LOG_ID_C_00000464)); //* MSG_LOG_ID_C_00000464
                goto FUNC_END;
            }
            pCsvhash->allocNum += HASH_LIST_BASE;
        }
        pCsvhash->ppFanCounter[pCsvhash->hashNum] = fanCountList->pFanCounter + i;
        pCsvhash->hashNum++;
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief  R[h\gNXƃt@CEt@AEg̃}[WsAvʃt@Cɏo
*/
#ifdef _METHOD_LABEL_
#undef _METHOD_LABEL_
#endif
#define _METHOD_LABEL_ _T("CDMtrCntXrefMerge")
int CDMtrCntXrefMerge(_TCHAR *csvfile, fanCounterList *fanCountList, _TCHAR *outfile)
{
    FILE *fpCsvfile = NULL;
    FILE *fpOutfile = NULL;
    _TCHAR *tReadBuffer = NULL;
    _TCHAR csvDir[XREF_DIR_LEN];
    _TCHAR csvFile[XREF_FILE_LEN];
    _TCHAR csvName[XREF_NAME_LEN];
    _TCHAR csvKey[XREF_DIR_LEN + XREF_FILE_LEN + XREF_NAME_LEN + 1];
    _TCHAR csvLang[CSV_STRBUFF_SIZE];
    _TCHAR csvFloc[CSV_STRBUFF_SIZE];
    _TCHAR *buff = NULL;
    _TCHAR finfoutBuff[CSV_STRBUFF_SIZE];
    int lSize = 0, i, hitFlag = 0;
    unsigned int hashKey = 0;
    _TCHAR *p = NULL;
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    csvHash *hashTable = NULL;
    int bomOffset = 0;
    int readLength = 0;
    hashTable = (csvHash *)malloc(sizeof(csvHash) * HASH_SIZE);
    if (hashTable == NULL) {
        ERROR_LOG((MSG_LOG_ID_C_00000465)); //* MSG_LOG_ID_C_00000465
        return -1;
    }
    memset(hashTable, 0, sizeof(csvHash) * HASH_SIZE);
    if (csvfile == NULL || outfile == NULL) {
        ERROR_LOG((MSG_LOG_ID_C_00000466)); //* MSG_LOG_ID_C_00000466
        goto FUNC_END;
    }
    if (CDMtrCntMakeFanCountHashTable(fanCountList, hashTable) != 0) {
        WARNING_LOG((MSG_LOG_ID_C_00000468)); //* MSG_LOG_ID_C_00000468
        goto FUNC_END;
    }
    if ((fpCsvfile = _tfopen(csvfile, _T("r, ccs=UTF-8"))) == NULL) {
        DEBUG_LOG((MSG_LOG_ID_C_00000474, csvfile)); //* MSG_LOG_ID_C_00000474
        goto FUNC_END;
    }
    if ((fpOutfile = _tfopen(outfile, _T("w, ccs=UTF-8"))) == NULL) {
        WARNING_LOG((MSG_LOG_ID_C_00000469, outfile)); //* MSG_LOG_ID_C_00000469
        goto FUNC_END;
    }
    bomOffset = ftell(fpCsvfile);
    if (fseek(fpCsvfile, 0L, SEEK_END) != 0) {
        goto FUNC_END;
    }
    lSize = ftell(fpCsvfile);
    if (lSize <= 0) {
        goto FUNC_END;
    }
    lSize++;
    if (fseek(fpCsvfile, bomOffset, SEEK_SET) != 0) {
        goto FUNC_END;
    }
    lSize += 6 * 8;
    iFuncRet = CDMtrLibUtilMallocW((void **)&tReadBuffer, (lSize + 1) * sizeof(_TCHAR));
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    while (_fgetts(tReadBuffer, lSize, fpCsvfile) != NULL) {
        hitFlag = 0;
        p = _tcschr(tReadBuffer, _T('\n'));
        if (p != NULL) {
            *p = _T('\0');
        }
        readLength = _tcslen(tReadBuffer);
        if (readLength >= 1 && tReadBuffer[readLength - 1] == _T('x')) {
            _tcscat(tReadBuffer, _T("\n"));
        } else {
            csvDir[0] = _T('\0');
            csvFile[0] = _T('\0');
            csvName[0] = _T('\0');
            csvLang[0] = _T('\0');
            csvFloc[0] = _T('\0');
            CDMtrCntXrefDataUtilGetNextCsvToken(csvDir, XREF_DIR_LEN, tReadBuffer);
            CDMtrCntXrefDataUtilGetNextCsvToken(csvFile, XREF_FILE_LEN, NULL);
            CDMtrCntXrefDataUtilGetNextCsvToken(csvName, XREF_NAME_LEN, NULL);
            CDMtrCntXrefDataUtilGetNextCsvToken(csvLang, CSV_STRBUFF_SIZE, NULL);
            CDMtrCntXrefDataUtilGetNextCsvToken(csvFloc, CSV_STRBUFF_SIZE, NULL);
            if (csvLang[0] == _T('C') && csvFloc[0] != _T('x')) {
                buff = _tcschr(csvName, _T('('));
                if (buff != NULL) {
                    *buff = _T('\0');
                }
                memset(csvKey, _T('\0'), sizeof(csvKey));
                _tcscat(csvKey, csvDir);
                _tcscat(csvKey, _T("\\"));
                _tcscat(csvKey, csvFile);
                _tcscat(csvKey, csvName);
                hashKey = getHashNumber(csvKey);
                for (i = 0; i < hashTable[hashKey % HASH_SIZE].hashNum; i++) {
                    fanCounter *fCount = hashTable[hashKey % HASH_SIZE].ppFanCounter[i];
                    if (hashKey == fCount->csvHashKey) {
                        memset(finfoutBuff, 0, (CSV_STRBUFF_SIZE)*sizeof(_TCHAR));
                        swprintf_s(finfoutBuff, CSV_STRBUFF_SIZE, _T(",%d,%d,%d,%d,%d,%d,%d,%d\n"),
                                   fCount->finNumOfCallThisProc + fCount->finNumOfGlobalVariablesRead + fCount->finNumOfParametersRead,
                                   fCount->foutNumOfByRefParamWrittenTo + fCount->foutNumOfGlobalVariablesWrittenTo + fCount->foutNumOfProcCall,
                                   fCount->finNumOfCallThisProc,
                                   fCount->finNumOfParametersRead,
                                   fCount->finNumOfGlobalVariablesRead,
                                   fCount->foutNumOfProcCall,
                                   fCount->foutNumOfByRefParamWrittenTo,
                                   fCount->foutNumOfGlobalVariablesWrittenTo
                                  );
                        hitFlag = 1;
                        break;
                    }
                }
            }
            if (hitFlag) {
                _tcscat(tReadBuffer, finfoutBuff);
            } else {
                _tcscat(tReadBuffer, _T(",-,-,-,-,-,-,-,-\n"));
            }
        }
        _fputts(tReadBuffer, fpOutfile);
    }
    iRet = RET_OK;
FUNC_END:
    if (fpCsvfile != NULL) {
        fclose(fpCsvfile);
        fpCsvfile = NULL;
    }
    if (fpOutfile != NULL) {
        fclose(fpOutfile);
        fpOutfile = NULL;
    }
    if (tReadBuffer != NULL) {
        CDMtrLibUtilFreeW((void **)&tReadBuffer);
    }
    if (hashTable != NULL) {
        for (i = 0; i < HASH_SIZE; i++) {
            if ((hashTable + i)->ppFanCounter != NULL) {
                free((hashTable + i)->ppFanCounter);
                (hashTable + i)->ppFanCounter = NULL;
            }
        }
        free(hashTable);
        hashTable = NULL;
    }
    return iRet;
}

static const unsigned int crc32tab[256] = {
    0x00000000, 0x77073096, 0xee0e612c, 0x990951ba,
    0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
    0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
    0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
    0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de,
    0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
    0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec,
    0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
    0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
    0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
    0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940,
    0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
    0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116,
    0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
    0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
    0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
    0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a,
    0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
    0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818,
    0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
    0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
    0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
    0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c,
    0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
    0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2,
    0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
    0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
    0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
    0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086,
    0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
    0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4,
    0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
    0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
    0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
    0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8,
    0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
    0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe,
    0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
    0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
    0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
    0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252,
    0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
    0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60,
    0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
    0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
    0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
    0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04,
    0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
    0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a,
    0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
    0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
    0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
    0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e,
    0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
    0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c,
    0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
    0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
    0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
    0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0,
    0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
    0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6,
    0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
    0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
    0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d,
};

unsigned int crc32(_TCHAR *p, int len)
{
    unsigned int crcinit = 0;
    unsigned int crc = 0;
    crc = crcinit ^ 0xFFFFFFFF;
    for (; len--; p++) {
        crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*p)) & 0xFF];
    }
    return crc ^ 0xFFFFFFFF;
}

unsigned int getHashNumber(_TCHAR *key)
{
    return crc32(key, _tcslen(key));
}
