/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "CDMtrExtCmn.h"
#include "CDMtrExtMain.h"
#include "CDMtrExtProp.h"
#include "CDMtrExtLine.h"
#include "CDMtrExtTake.h"

#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrMessage.h"

#define COMMENT_DEFAULTNUM 2
#define COMMENT_LINE        _T("//")
#define COMMENT_BLOCKSTART  _T("/*")
#define COMMENT_BLOCKEND    _T("*/")

#define COMMENT_INDEX_NONE (-1)

#define DOC_JAVA     _T("/**")
#define DOC_QT       _T("/*!")
#define DOC_LINE1    _T("///")
#define DOC_LINE2    _T("//!")
#define DOC_BLOCKEND _T("*/")

#define DOC_VB       _T("'''")

#define DOC_JAVA_INDEX 0
#define DOC_QT_INDEX 1

#define PREPRO_IF_FALSE     _T("false")

#define PREPRO_IF_THEN     _T("then")

#define PREPRO_START _T("#")
#define PREPRO_NAME_INCLUDE _T("include")
#define PREPRO_NAME_IF      _T("if")
#define PREPRO_NAME_IFDEF   _T("ifdef")
#define PREPRO_NAME_IFNDEF  _T("ifndef")
#define PREPRO_NAME_ELSE    _T("else")
#define PREPRO_NAME_ELIF    _T("elif")
#define PREPRO_NAME_ENDIF   _T("endif")
#define PREPRO_NAME_REGION  _T("region")
#define PREPRO_NAME_ENDREGION   _T("endregion")
#define PREPRO_NAME_EXTERNAL  _T("externalsource")
#define PREPRO_NAME_ELSEIF  _T("elseif")
#define PREPRO_NAME_END  _T("end")

#define IS_PREPRO(p) (*(p) == '#')

#define IS_IDENTIFIER(p) (is_alpha(*p) || is_num(*p) || (*p == '_'))

#define UNDEFLINE_MARK _T("@CDMTR_SKIP@")
#define DETAIL_MARK_CODE _T("C,")
#define DETAIL_MARK_PREP _T("P,")
#define DETAIL_MARK_SKIP _T("S,")
#define DETAIL_MARK_DOC  _T("D,")
#define DETAIL_MARK_REM  _T("R,")
#define DETAIL_MARK_BLANK  _T("B,")


typedef enum {
    PREPRO_ID_UNKNOW = 0,
    PREPRO_ID_INCLUDE,
    PREPRO_ID_IF,
    PREPRO_ID_IFDEF,
    PREPRO_ID_IFNDEF,
    PREPRO_ID_ELSE,
    PREPRO_ID_ELIF,
    PREPRO_ID_ENDIF,
    PREPRO_ID_REGION,
    PREPRO_ID_ENDREGION,
    PREPRO_ID_EXTERNAL,
    PREPRO_ID_ENDEXTERNAL,
    PREPRO_ID_END,

} PREPRO_NAME_ID;

typedef enum {
    TOKEN_TYPE_UNKNOW = 0,
    TOKEN_TYPE_STRING,
    TOKEN_TYPE_NUM,
} TOKEN_TYPE_ID;

typedef enum {
    FUNC_LC_STATUS_INIT = 1,
    FUNC_LC_STATUS_COUNTING,
    FUNC_LC_STATUS_INVALID,
    FUNC_LC_STATUS_END,
} FUNC_LC_STATUS;

typedef struct tag_LINECOL {
    unsigned long ulCol;
    unsigned long ulPreCol;
    unsigned long ulCmntCol;
    unsigned long ulDocCol;
    unsigned long ulUndefCol;
    unsigned long ulUndefCmntCol;
}
_LINECOL;

typedef struct tag_KEY_NAME {
    int iType;
    _TCHAR *pcName;
}
_KEY_NAME;

extern _LINECOUNT_LIST *gptLCList;

static _LINECOL  stCol;

static _COMMENT  stComment;

static int  siCmntCaseFlg;

static _KEY_NAME *ptTmpDocTable = NULL;

static int siDocumentTableSize = 0;


static int siStatus;

static int siOldStatus;

static int siStrStatus;

static int siCommentIndex;

static int siUndefPreproDepth;

static _TCHAR stInputBuff[BUFF_LINE_LENGTH];

static int iFlgAvailableIdentifier = CDMTR_FALSE;


static int siIsBreakLineCode = CDMTR_FALSE;

static int siIsBlankLine = CDMTR_FALSE;

static int siLineIndex;

static int siCountPREP;
static int siCountCLOC;
static int siCountSKIP;
static int siCountString;

static _KEY_NAME stPreproTable[] = {
    {PREPRO_ID_INCLUDE,     PREPRO_NAME_INCLUDE},
    {PREPRO_ID_IF,          PREPRO_NAME_IF},
    {PREPRO_ID_IFDEF,       PREPRO_NAME_IFDEF},
    {PREPRO_ID_IFNDEF,      PREPRO_NAME_IFNDEF},
    {PREPRO_ID_ELSE,        PREPRO_NAME_ELSE},
    {PREPRO_ID_ELIF,        PREPRO_NAME_ELIF},
    {PREPRO_ID_ENDIF,       PREPRO_NAME_ENDIF},
    {PREPRO_ID_REGION,      PREPRO_NAME_REGION},
    {PREPRO_ID_ENDREGION,   PREPRO_NAME_ENDREGION},
};
static _KEY_NAME stPreproVBTable[] = {
    {PREPRO_ID_IF,          PREPRO_NAME_IF},
    {PREPRO_ID_ELSE,        PREPRO_NAME_ELSE},
    {PREPRO_ID_ELIF,        PREPRO_NAME_ELSEIF},
    {PREPRO_ID_END,         PREPRO_NAME_END},
    {PREPRO_ID_REGION,      PREPRO_NAME_REGION},
    {PREPRO_ID_EXTERNAL,    PREPRO_NAME_EXTERNAL},

};

static _KEY_NAME stPreproVBEndTable[] = {
    {PREPRO_ID_ENDIF,       PREPRO_NAME_IF},
    {PREPRO_ID_ENDREGION,   PREPRO_NAME_REGION},
    {PREPRO_ID_ENDEXTERNAL, PREPRO_NAME_EXTERNAL},
};

static _KEY_NAME stDocTable[] = {
    {COMMENT_TYPE_BLOCK, DOC_JAVA},
    {COMMENT_TYPE_BLOCK, DOC_QT},
    {COMMENT_TYPE_LINE, DOC_LINE1},
    {COMMENT_TYPE_LINE, DOC_LINE2},
};

static _KEY_NAME stDocVBTable[] = {
    {COMMENT_TYPE_LINE, DOC_VB},
};

static void cdmtrExtLineSeachLine(void);
static int cdmtrExtLineGetStatus(_TCHAR **ppcurrent);
static int cdmtrExtLineGetPreproStatus(_TCHAR **ppcurrent);
static int cdmtrExtLineGetPreproID(_TCHAR **ppcurrent);
static int cdmtrExtLineGetPreproToken(_TCHAR **ppcurrent, _TCHAR **ppcPreproToken);
static void cdmtrExtLineCountCols(const _TCHAR *currentp, _TCHAR *oldp);
static void cdmtrExtLineCountLine(void);
static int cdmtrExtLineIsCommentStart(_TCHAR **ppcurrent);
static int cdmtrExtLineSeachCommentEnd(int iCurStatus, _TCHAR **ppcurrent);
static int cdmtrExtLineIsDocStart(_TCHAR **ppcurrent);
static int cdmtrExtLineSeachDocEnd(_TCHAR **ppcurrent);
static int cdmtrExtLineIsUndefLine(_TCHAR **ppcurrent);
static int cdmtrExtLineSeachUndefEnd(_TCHAR **ppcurrent);
static int cdmtrExtLineSeachStringKey(_TCHAR **ppcurrent);
static int cdmtrExtLineIsStringKey(_TCHAR *pcurrent);
static void cdmtrExtLineChkCount(int iLangType);
static int cdmtrExtLineIsContinueLine(int iLineStatus);
static int cdmtrSkipBOM(_TCHAR **ppcurrent);
static int cdmtrExtLineExtractLinesFuncMode(FILE *fin);
static void cdmtrExtLineSetLCList(int iStatus, long lWriteIndex, _LINECOUNT_LIST *ptLCList);
static void cdmtrExtLineChkBlankLine(void);

/*!
    @brief          {CJEg̃Rgݒ肷
*/
int CDMtrExtLineInitSetComment(const _TCHAR *pcPropFile, const _TCHAR *pcLangType)
{
    int iRet, iFuncRet;
    _COMMENT tComment;
    iRet = RET_NG;
    memset(&stComment, 0x00, sizeof(_COMMENT));
    memset(&tComment, 0x00, sizeof(_COMMENT));
    ptTmpDocTable = stDocTable;
    siDocumentTableSize = sizeof(stDocTable) / sizeof(_KEY_NAME);
    siCmntCaseFlg = gtOptInfo_lc.iCmntCaseFlg;
    if (pcLangType == NULL || CDMtrExtPropIsDefaultType(pcLangType)) {
        tComment.uiNum = COMMENT_DEFAULTNUM;
        tComment.tCmnt[0].iType = COMMENT_TYPE_LINE;
        _tcscpy(tComment.tCmnt[0].cCmntS, COMMENT_LINE);
        tComment.tCmnt[1].iType = COMMENT_TYPE_BLOCK;
        _tcscpy(tComment.tCmnt[1].cCmntS, COMMENT_BLOCKSTART);
        _tcscpy(tComment.tCmnt[1].cCmntE, COMMENT_BLOCKEND);
    } else if (CDMtrExtPropIsVBType(pcLangType)) {
        tComment.uiNum = 1;
        tComment.tCmnt[0].iType = COMMENT_TYPE_LINE;
        _tcscpy(tComment.tCmnt[0].cCmntS, _T("'"));
        ptTmpDocTable = stDocVBTable;
        siDocumentTableSize = sizeof(stDocVBTable) / sizeof(_KEY_NAME);
        siCmntCaseFlg = CDMTR_FALSE;
    } else if (CDMtrExtPropIsTxtType(pcLangType)) {
        ptTmpDocTable = NULL;
        siDocumentTableSize = 0;
    } else {
        iFuncRet = CDMtrExtPropGetPropComment(pcPropFile, pcLangType, &tComment);
        if (iFuncRet != RET_OK) {
        }
    }
    memcpy(&stComment, &tComment, sizeof(_COMMENT));
    iRet = RET_OK;
    return iRet;
}


/*!
    {CJEg
*/
int CDMtrExtLineExtractLines(FILE *fin)
{
    int iRet;
    iRet = RET_NG;
    if (fin == NULL) {
        goto FUNC_END;
    }
    siStatus = LINE_STATUS_INIT;
    siOldStatus = LINE_STATUS_INIT;
    siStrStatus = STR_STATUS_INIT;
    siCommentIndex = COMMENT_INDEX_NONE;
    siUndefPreproDepth = 0;
    siLineIndex = 0;
    siCountPREP = CDMTR_FALSE;
    siCountCLOC = CDMTR_FALSE;
    siCountSKIP = CDMTR_FALSE;
    siCountString = CDMTR_FALSE;
    cdmtrExtLineChkCount(gtOptInfo_lc.iLangType);
    if (gtOptInfo_lc.pcDiffFuncPath != NULL) {
        iRet = cdmtrExtLineExtractLinesFuncMode(fin);
    } else {
        while (_fgetts(stInputBuff, BUFF_LINE_LENGTH, fin) != NULL) {
            cdmtrExtLineSeachLine();
            cdmtrExtLineCountLine();
        }
        if (iSkipChangeFlg == CDMTR_TRUE) {
            gtCount.ulELOC = gtCount.ulLine + gtCount.ulPre;
            gtCount.ulCLOC = gtCount.ulDoc + gtCount.ulCmnt;
            gtCount.ulTLOC = gtCount.ulELOC + gtCount.ulCLOC + gtCount.ulBlank + gtCount.ulUndef;
        } else {
            gtCount.ulELOC = gtCount.ulLine + gtCount.ulPre + gtCount.ulUndef;
            gtCount.ulCLOC = gtCount.ulDoc + gtCount.ulCmnt;
            gtCount.ulTLOC = gtCount.ulELOC + gtCount.ulCLOC + gtCount.ulBlank;
        }
        memcpy(&(gptLCList->tLCData), &gtCount, sizeof(_LINECOUNT));
        memset(&gtCount, 0x00, sizeof(_LINECOUNT));
        iRet = RET_OK;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief  ݍs1`FbN
*/
static void cdmtrExtLineSeachLine(void)
{
    _TCHAR *currentp, *oldp;
    int iInpLen;
    memset(&stCol, 0x00, sizeof(_LINECOL));
    iInpLen = _tcslen(stInputBuff);
    while (iInpLen > 0) {
        if (IS_BREAK_LINE(&stInputBuff[iInpLen - 1])) {
            siIsBreakLineCode = CDMTR_TRUE;
            stInputBuff[iInpLen - 1] = '\0';
            iInpLen--;
        } else {
            break;
        }
    }
    currentp = stInputBuff;
    if (siLineIndex == 0) {
        cdmtrSkipBOM(&currentp);
    }
    while (!IS_NULL_CHAR(currentp)) {
        if (IS_WHITE_SPACE(currentp) || IS_BREAK_LINE(currentp)) {
            currentp++;
            continue;
        }
        oldp = currentp;
        if (siStrStatus != STR_STATUS_INIT) {
            cdmtrExtLineSeachStringKey(&currentp);
            cdmtrExtLineCountCols(currentp, oldp);
            continue;
        }
        switch (siStatus) {
        case LINE_STATUS_BLOCKDOC:
            cdmtrExtLineSeachDocEnd(&currentp);
            stCol.ulDocCol += currentp - oldp;
            break;
        case LINE_STATUS_LINEDOC:
            currentp += _tcslen(currentp);
            stCol.ulDocCol += currentp - oldp;
            break;
        case LINE_STATUS_BLOCKCOMMENT:
            cdmtrExtLineSeachCommentEnd(siStatus, &currentp);
            stCol.ulCmntCol += currentp - oldp;
            break;
        case LINE_STATUS_LINECOMMENT:
            currentp += _tcslen(currentp);
            stCol.ulCmntCol += currentp - oldp;
            break;
        case LINE_STATUS_UNDEFCOMMENT:
            cdmtrExtLineSeachCommentEnd(siStatus, &currentp);
            stCol.ulUndefCmntCol += currentp - oldp;
            break;
        case LINE_STATUS_UNDEFLINECOMMENT:
            currentp += _tcslen(currentp);
            stCol.ulUndefCmntCol += currentp - oldp;
            break;
        case LINE_STATUS_UNDEFSKIP:
            cdmtrExtLineSeachUndefEnd(&currentp);
            if (siStatus == LINE_STATUS_PREPRO) {
                stCol.ulPreCol += currentp - oldp;
            } else {
                stCol.ulUndefCol += currentp - oldp;
            }
            break;
        case LINE_STATUS_UNDEFPREPRO:
        case LINE_STATUS_UNDEFLINE:
        case LINE_STATUS_PREPRO:
        case LINE_STATUS_INIT:
        default:
            siOldStatus = siStatus;
            cdmtrExtLineGetStatus(&currentp);
            cdmtrExtLineCountCols(currentp, oldp);
            break;
        }
    }
    return;
}

/*!
    @brief          Rgs,vvs,񂪊JnǂmF
*/
static int cdmtrExtLineGetStatus(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p;
    p = *ppcurrent;
    iRet = RET_FALSE;
    if (cdmtrExtLineIsDocStart(&p)) {
        iRet = RET_TRUE;
        goto FUNC_END;
    } else if (cdmtrExtLineIsCommentStart(&p)) {
        iRet = RET_TRUE;
        goto FUNC_END;
    } else if (p == stInputBuff
               && *p == '@'
               && cdmtrExtLineIsUndefLine(&p)) {
        iRet = RET_TRUE;
        goto FUNC_END;
    } else if (siCountPREP
               && IS_PREPRO(p)
               && (stCol.ulCol + stCol.ulPreCol) == 0) {
        p++;
        SKIP_WHITE_SPACE(&p);
        siStatus = LINE_STATUS_PREPRO;
        cdmtrExtLineGetPreproStatus(&p);
        iRet = RET_TRUE;
        goto FUNC_END;
    } else {
    }
    if (cdmtrExtLineIsStringKey(p)) {
        iRet = RET_TRUE;
    }
    p++;
FUNC_END:
    *ppcurrent = p;
    return iRet;
}


/*!
    @brief          vvZXs̎ʎq擾Aݍs̍sXe[^X擾B
*/
static int cdmtrExtLineGetPreproStatus(_TCHAR **ppcurrent)
{
    int iRet;
    int iPreproId;
    _TCHAR *p, *oldp;
    iRet = RET_FALSE;
    p = *ppcurrent;
    if (IS_NULL_CHAR(p)) {
        goto FUNC_END;
    }
    iPreproId = cdmtrExtLineGetPreproID(&p);
    SKIP_WHITE_SPACE(&p);
    switch (iPreproId) {
    case PREPRO_ID_INCLUDE:
        if ((*p) == '<') {
            siStrStatus = STR_STATUS_INCLUDE;
            iRet = RET_TRUE;
            p++;
        } else if ((*p) == '\"') {
            siStrStatus = STR_STATUS_STRING;
            iRet = RET_TRUE;
            p++;
        } else {
        }
        break;
    case PREPRO_ID_IF:
        if (gtOptInfo_lc.iLangType == LANG_TYPE_CS) {
            if (_tcsncmp(p, PREPRO_IF_FALSE, _tcslen(PREPRO_IF_FALSE)) == 0) {
                p = p + _tcslen(PREPRO_IF_FALSE);
                SKIP_WHITE_SPACE(&p);
                oldp = p;
                if (IS_NULL_CHAR(p)) {
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                } else if (cdmtrExtLineIsCommentStart(&p)) {
                    p = oldp;
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                }
            }
        } else if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
            if (CDMtrLibUtilStrCaseCmp(p, PREPRO_IF_FALSE, _tcslen(PREPRO_IF_FALSE)) == 0) {
                p = p + _tcslen(PREPRO_IF_FALSE);
                SKIP_WHITE_SPACE(&p);
                oldp = p;
                if (IS_NULL_CHAR(p)) {
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                } else if (cdmtrExtLineIsCommentStart(&p)) {
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                } else if (CDMtrLibUtilStrCaseCmp(p, PREPRO_IF_THEN, _tcslen(PREPRO_IF_THEN)) == 0) {
                    p = p + _tcslen(PREPRO_IF_THEN);
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                } else {
                }
            }
        } else {
            if ((*p) == '0') {
                p++;
                SKIP_WHITE_SPACE(&p);
                oldp = p;
                if (IS_NULL_CHAR(p)) {
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                } else if (cdmtrExtLineIsCommentStart(&p)) {
                    p = oldp;
                    siStatus = LINE_STATUS_UNDEFPREPRO;
                    iRet = RET_TRUE;
                }
            }
        }
        break;
    case PREPRO_ID_IFDEF:
    case PREPRO_ID_IFNDEF:
    case PREPRO_ID_ELSE:
    case PREPRO_ID_ELIF:
        break;
    case PREPRO_ID_ENDIF:
        break;
    case PREPRO_ID_REGION:
    case PREPRO_ID_ENDREGION:
    case PREPRO_ID_ENDEXTERNAL:
    case PREPRO_ID_END:
    case PREPRO_ID_UNKNOW:
    default:
        if (gtOptInfo_lc.iLangType == LANG_TYPE_CS) {
            p = p + _tcslen(p);
        }
        break;
    }
FUNC_END:
    *ppcurrent = p;
    return iRet;
}


/*!
    @brief          vvZXs̎ʔԍ擾
*/
static int cdmtrExtLineGetPreproID(_TCHAR **ppcurrent)
{
    int iFuncRet;
    int iPreproId;
    int i;
    _TCHAR *pcPreproToken;
    _TCHAR *pcPreproEndToken;
    _TCHAR *p;
    pcPreproToken = NULL;
    pcPreproEndToken = NULL;
    iPreproId = PREPRO_ID_UNKNOW;
    p = *ppcurrent;
    iFuncRet = cdmtrExtLineGetPreproToken(&p, &pcPreproToken);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
        for (i = 0; i < (sizeof(stPreproVBTable) / sizeof(_KEY_NAME)); i++) {
            if (CDMtrLibUtilStrCaseCmp(pcPreproToken,
                                       stPreproVBTable[i].pcName,
                                       _tcslen(stPreproVBTable[i].pcName)) == 0) {
                iPreproId = stPreproVBTable[i].iType;
                break;
            }
        }
        if (iPreproId == PREPRO_ID_END) {
            SKIP_WHITE_SPACE(&p);
            iFuncRet = cdmtrExtLineGetPreproToken(&p, &pcPreproEndToken);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            }
            for (i = 0; i < (sizeof(stPreproVBEndTable) / sizeof(_KEY_NAME)); i++) {
                if (CDMtrLibUtilStrCaseCmp(pcPreproEndToken,
                                           stPreproVBEndTable[i].pcName,
                                           _tcslen(stPreproVBEndTable[i].pcName)) == 0) {
                    iPreproId = stPreproVBEndTable[i].iType;
                    break;
                }
            }
        }
    } else {
        for (i = 0; i < (sizeof(stPreproTable) / sizeof(_KEY_NAME)); i++) {
            if (_tcscmp(pcPreproToken, stPreproTable[i].pcName) == 0) {
                iPreproId = stPreproTable[i].iType;
                break;
            }
        }
    }
FUNC_END:
    if (pcPreproToken != NULL) {
        free(pcPreproToken);
    }
    if (pcPreproEndToken != NULL) {
        free(pcPreproEndToken);
    }
    *ppcurrent = p;
    return iPreproId;
}


/*!
    @brief          vvZX̃g[N(p)擾
*/
static int cdmtrExtLineGetPreproToken(_TCHAR **ppcurrent, _TCHAR **ppcPreproToken)
{
    int iRet;
    unsigned int uiLen;
    _TCHAR *pcToken;
    _TCHAR *p;
    _TCHAR *pcTop, *pcEnd;
    iRet = RET_NG;
    *ppcPreproToken = NULL;
    pcToken = NULL;
    p = *ppcurrent;
    SKIP_WHITE_SPACE(&p);
    pcTop = p;
    pcEnd = p;
    while (!IS_NULL_CHAR(p)) {
        if (!CDMtrLibUtilIsAlpha(*p)) {
            break;
        }
        p++;
        pcEnd = p;
    }
    uiLen = pcEnd - pcTop;
    if (uiLen <= 0) {
        goto FUNC_END;
    }
    pcToken = (_TCHAR *)malloc((uiLen + 1) * sizeof(_TCHAR));
    if (pcToken == NULL) {
        goto FUNC_END;
    }
    _tcsncpy(pcToken, pcTop, uiLen);
    pcToken[uiLen] = '\0';
    iRet = RET_OK;
    *ppcPreproToken = pcToken;
FUNC_END:
    *ppcurrent = p;
    return iRet;
}


/*!
    @brief          w͈͂̕JEg
*/
static void cdmtrExtLineCountCols(const _TCHAR *currentp, _TCHAR *oldp)
{
    int iLen = 0;
    iLen = currentp - oldp;
    switch (siStatus) {
    case LINE_STATUS_LINEDOC:
    case LINE_STATUS_BLOCKDOC:
        stCol.ulDocCol += iLen;
        break;
    case LINE_STATUS_LINECOMMENT:
    case LINE_STATUS_BLOCKCOMMENT:
        stCol.ulCmntCol += iLen;
        break;
    case LINE_STATUS_PREPRO:
    case LINE_STATUS_UNDEFPREPRO:
        stCol.ulPreCol += iLen;
        break;
    case LINE_STATUS_UNDEFLINE:
    case LINE_STATUS_UNDEFSKIP:
        stCol.ulUndefCol += iLen;
        break;
    case LINE_STATUS_UNDEFCOMMENT:
    case LINE_STATUS_UNDEFLINECOMMENT:
        stCol.ulUndefCmntCol += iLen;
        break;
    case LINE_STATUS_INIT:
    default:
        stCol.ulCol += iLen;
        break;
    }
    return;
}

/*!
    @brief  C̃JEgsB
*/
static void cdmtrExtLineCountLine(void)
{
    _TCHAR *pcMark;
    int iBlankFlg;
    pcMark = NULL;
    iBlankFlg = CDMTR_TRUE;
    siLineIndex++;
    CDMtrLibCmnDebugLogOut(gtOptInfo_lc.giDebug, MSG_LOG_ID_C_00000487, siLineIndex); //* MSG_LOG_ID_C_00000487
    if (stCol.ulCol > 0) {
        gtCount.ulLine++;
        if (stCol.ulDocCol > 0 ||
                stCol.ulCmntCol > 0 ||
                stCol.ulUndefCmntCol > 0 ) {
            gtCount.ulICLOC++;
        }
        pcMark = DETAIL_MARK_CODE;
    } else if (stCol.ulPreCol > 0 ||
               siStatus == LINE_STATUS_UNDEFPREPRO) {
        gtCount.ulPre++;
        if (stCol.ulDocCol > 0 ||
                stCol.ulCmntCol > 0 ||
                stCol.ulUndefCmntCol > 0) {
            gtCount.ulICLOC++;
        }
        pcMark = DETAIL_MARK_PREP;
    } else if (stCol.ulUndefCol > 0 || stCol.ulUndefCmntCol > 0 ||
               siStatus == LINE_STATUS_UNDEFLINE ||
               siStatus == LINE_STATUS_UNDEFSKIP ||
               siStatus == LINE_STATUS_UNDEFLINECOMMENT ||
               siStatus == LINE_STATUS_UNDEFCOMMENT) {
        gtCount.ulUndef++;
        pcMark = DETAIL_MARK_SKIP;
    } else if (stCol.ulDocCol > 0 ||
               siStatus == LINE_STATUS_BLOCKDOC ||
               siStatus == LINE_STATUS_LINEDOC) {
        gtCount.ulDoc++;
        pcMark = DETAIL_MARK_DOC;
    } else if (stCol.ulCmntCol > 0 ||
               siStatus == LINE_STATUS_BLOCKCOMMENT ||
               siStatus == LINE_STATUS_LINECOMMENT) {
        gtCount.ulCmnt++;
        pcMark = DETAIL_MARK_REM;
    } else {
        gtCount.ulBlank++;
        pcMark = DETAIL_MARK_BLANK;
        iBlankFlg = CDMTR_FALSE;
    }
    if (gtOptInfo_lc.giDebug) {
        CDMtrExtTakeStdOutMark(CDMTR_TRUE, pcMark, stInputBuff);
    } else if (gtOptInfo_lc.iDetailFlg) {
        CDMtrExtTakeStdOutMark(iBlankFlg, pcMark, stInputBuff);
    }
    switch (siStatus) {
    case LINE_STATUS_BLOCKCOMMENT:
    case LINE_STATUS_BLOCKDOC:
        if (siOldStatus == LINE_STATUS_UNDEFPREPRO) {
            siStatus = LINE_STATUS_UNDEFCOMMENT;
        }
        break;
    case LINE_STATUS_LINECOMMENT:
    case LINE_STATUS_LINEDOC:
        if (siOldStatus == LINE_STATUS_UNDEFPREPRO) {
            siStatus = LINE_STATUS_UNDEFSKIP;
        } else {
            if (cdmtrExtLineIsContinueLine(siStatus)) {
            } else {
                siStatus = LINE_STATUS_INIT;
            }
        }
        break;
    case LINE_STATUS_UNDEFPREPRO:
        siStatus = LINE_STATUS_UNDEFSKIP;
        break;
    case LINE_STATUS_PREPRO:
        if (cdmtrExtLineIsContinueLine(siStatus)) {
        } else {
            siStatus = LINE_STATUS_INIT;
        }
        break;
    case LINE_STATUS_UNDEFLINECOMMENT:
        if (cdmtrExtLineIsContinueLine(siStatus)) {
        } else {
            siStatus = LINE_STATUS_UNDEFSKIP;
        }
        break;
    case LINE_STATUS_UNDEFLINE:
    case LINE_STATUS_UNDEFCOMMENT:
    case LINE_STATUS_UNDEFSKIP:
    case LINE_STATUS_INIT:
    default:
        if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
            siStrStatus = STR_STATUS_INIT;
        }
        break;
    }
    return;
}

/*!
    @brief          ݈ʒu|C^wς̃RgJn邩
*/
static int cdmtrExtLineIsCommentStart(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p;
    unsigned int iLen, iCommentLen;
    unsigned int i;
    int iResStrcmp;
    int iCommentType = 0;
    iRet = RET_FALSE;
    iResStrcmp = 1;
    if (!siCountCLOC) {
        goto FUNC_END;
    }
    if (ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (*ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (IS_NULL_CHAR(*ppcurrent)) {
        goto FUNC_END;
    }
    siCommentIndex = COMMENT_INDEX_NONE;
    p = *ppcurrent;
    i = 0;
    iLen = 0;
    iCommentLen = 0;
    if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
        if ( *p == 'r' || *p == 'R' ) {
            iCommentLen = _tcslen(_T("rem"));
            iCommentType = COMMENT_TYPE_LINE;
            iResStrcmp = CDMtrLibUtilStrCaseCmp(p, _T("rem"), iCommentLen);
            if (iResStrcmp == 0) {
                if (p > stInputBuff &&
                        (CDMtrLibUtilIsAlpha( *(p - 1) ) ||
                         CDMtrLibUtilIsDigit( *(p - 1) ))) {
                    iResStrcmp = 1;
                } else if (iResStrcmp == 0) {
                    if (CDMtrLibUtilIsAlpha(*(p + iCommentLen)) ||
                            CDMtrLibUtilIsDigit(*(p + iCommentLen))) {
                        iResStrcmp = 1;
                    } else {
                        iResStrcmp = 0;
                    }
                }
                goto CMNT_FOUND;
            }
        }
    }
    for (i = 0; i < stComment.uiNum; i++) {
        if (siCmntCaseFlg) {
            if (*p != stComment.tCmnt[i].cCmntS[0]) {
                continue;
            }
        } else {
            if (tolower(*p) != tolower(stComment.tCmnt[i].cCmntS[0])) {
                continue;
            }
        }
        iLen = _tcslen(p);
        iCommentLen = _tcslen(stComment.tCmnt[i].cCmntS);
        iCommentType = stComment.tCmnt[i].iType;
        if (iLen == 0 || iLen < iCommentLen) {
            continue;
        }
        if (siCmntCaseFlg) {
            iResStrcmp = _tcsncmp(p, stComment.tCmnt[i].cCmntS, iCommentLen);
        } else {
            iResStrcmp = CDMtrLibUtilStrCaseCmp(p, stComment.tCmnt[i].cCmntS, iCommentLen);
        }
        if (iResStrcmp == 0) {
            break;
        }
    }
CMNT_FOUND:
    if (iResStrcmp == 0) {
        switch (iCommentType) {
        case COMMENT_TYPE_BLOCK:
            siStatus = LINE_STATUS_BLOCKCOMMENT;
            break;
        case COMMENT_TYPE_LINE:
        default:
            siStatus = LINE_STATUS_LINECOMMENT;
            break;
        }
        p += iCommentLen;
        *ppcurrent = p;
        siCommentIndex = i;
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          ݈ʒu|C^w肵RgI
*/
static int cdmtrExtLineSeachCommentEnd(int iCurStatus, _TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p, *oldp;
    iRet = RET_FALSE;
    p = *ppcurrent;
    oldp = p;
    if (siCmntCaseFlg) {
        p = _tcsstr(p, stComment.tCmnt[siCommentIndex].cCmntE);
    } else {
        p = CDMtrLibUtilStrCaseStr(p, stComment.tCmnt[siCommentIndex].cCmntE);
    }
    if (p == NULL) {
        *ppcurrent += _tcslen(*ppcurrent);
        goto FUNC_END;
    }
    p += _tcslen(stComment.tCmnt[siCommentIndex].cCmntE);
    *ppcurrent += p - oldp;
    switch (iCurStatus) {
    case LINE_STATUS_UNDEFCOMMENT:
        siStatus = LINE_STATUS_UNDEFSKIP;
        break;
    case LINE_STATUS_BLOCKCOMMENT:
    default:
        siStatus = siOldStatus;
        break;
    }
    iRet = RET_TRUE;
FUNC_END:
    return iRet;
}


/*!
    @brief          ݈ʒu|C^wς݂̃RgJn邩
*/
static int cdmtrExtLineIsDocStart(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p, *tmp_p;
    unsigned int iLen, iDocLen;
    int i;
    iRet = RET_FALSE;
    p = NULL;
    tmp_p = NULL;
    if (!siCountCLOC) {
        goto FUNC_END;
    }
    if (!gtOptInfo_lc.iDocFlg) {
        goto FUNC_END;
    }
    if (ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (*ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (IS_NULL_CHAR(*ppcurrent)) {
        goto FUNC_END;
    }
    p = *ppcurrent;
    iLen = 0;
    iDocLen = 0;
    for (i = 0; i < siDocumentTableSize; i++) {
        if (*p != ptTmpDocTable[i].pcName[0]) {
            continue;
        }
        iLen = _tcslen(p);
        iDocLen = _tcslen(ptTmpDocTable[i].pcName);
        if (iLen == 0 || iLen < iDocLen) {
            continue;
        }
        if (_tcsncmp(p, ptTmpDocTable[i].pcName, iDocLen) == 0) {
            tmp_p = p;
            tmp_p += _tcslen(ptTmpDocTable[i].pcName);
            if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
                if (*tmp_p == '\'') {
                    break;
                }
            } else {
                if (i == DOC_JAVA_INDEX) {
                    while (*tmp_p == '*') {
                        tmp_p++;
                    }
                    if (*tmp_p == '/') {
                        break;
                    }
                } else if (i == DOC_QT_INDEX) {
                    if (*tmp_p == '*' || *tmp_p == '/') {
                        break;
                    }
                }
            }
            switch (ptTmpDocTable[i].iType) {
            case COMMENT_TYPE_BLOCK:
                siStatus = LINE_STATUS_BLOCKDOC;
                break;
            case COMMENT_TYPE_LINE:
            default:
                siStatus = LINE_STATUS_LINEDOC;
                break;
            }
            *ppcurrent += _tcslen(ptTmpDocTable[i].pcName);
            iRet = RET_TRUE;
            break;
        }
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          ݈ʒu|C^w肵dlRgI
*/
static int cdmtrExtLineSeachDocEnd(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p, *oldp;
    iRet = RET_FALSE;
    p = *ppcurrent;
    oldp = p;
    p = _tcsstr(p, DOC_BLOCKEND);
    if (p == NULL) {
        *ppcurrent += _tcslen(*ppcurrent);
        goto FUNC_END;
    }
    p += _tcslen(DOC_BLOCKEND);
    *ppcurrent += p - oldp;
    siStatus = siOldStatus;
    iRet = RET_TRUE;
FUNC_END:
    return iRet;
}


/*!
    @brief          ݈ʒu|C^SKIPs
*/
static int cdmtrExtLineIsUndefLine(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p;
    unsigned int uiLen;
    iRet = RET_FALSE;
    if (!siCountSKIP) {
        goto FUNC_END;
    }
    if (ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (*ppcurrent == NULL) {
        goto FUNC_END;
    }
    if (IS_NULL_CHAR(*ppcurrent)) {
        goto FUNC_END;
    }
    p = *ppcurrent;
    uiLen = _tcslen(UNDEFLINE_MARK);
    if (_tcsncmp(stInputBuff, UNDEFLINE_MARK, uiLen) == 0) {
        uiLen = _tcslen(stInputBuff);
        p += uiLen;
        siStatus = LINE_STATUS_UNDEFLINE;
        iRet = RET_TRUE;
    }
    *ppcurrent = p;
FUNC_END:
    return iRet;
}


/*!
    @brief          sIARgsJnB
*/
static int cdmtrExtLineSeachUndefEnd(_TCHAR **ppcurrent)
{
    int iRet;
    int iPreproId;
    _TCHAR *p, *oldp;
    iRet = RET_FALSE;
    p = *ppcurrent;
    oldp = p;
    if (siStatus != LINE_STATUS_UNDEFSKIP) {
        goto FUNC_END;
    }
    if (IS_NULL_CHAR(p)) {
        goto FUNC_END;
    }
    if (IS_PREPRO(p)
            && (stCol.ulCol + stCol.ulPreCol) == 0) {
        p++;
        SKIP_WHITE_SPACE(&p);
        iPreproId = cdmtrExtLineGetPreproID(&p);
        SKIP_WHITE_SPACE(&p);
        switch (iPreproId) {
        case PREPRO_ID_IF:
        case PREPRO_ID_IFDEF:
        case PREPRO_ID_IFNDEF:
            siUndefPreproDepth++;
            break;
        case PREPRO_ID_ENDIF:
            if (siUndefPreproDepth == 0) {
                siStatus = LINE_STATUS_PREPRO;
                iRet = RET_TRUE;
            } else {
                siUndefPreproDepth--;
            }
            break;
        case PREPRO_ID_ELSE:
        case PREPRO_ID_ELIF:
            if (siUndefPreproDepth == 0) {
                siStatus = LINE_STATUS_PREPRO;
                iRet = RET_TRUE;
            }
            break;
        case PREPRO_ID_INCLUDE:
        case PREPRO_ID_UNKNOW:
        default:
            break;
        }
    }
    if (siStatus == LINE_STATUS_UNDEFSKIP) {
        cdmtrExtLineIsCommentStart(&p);
        switch (siStatus) {
        case LINE_STATUS_BLOCKCOMMENT:
            siStatus = LINE_STATUS_UNDEFCOMMENT;
            break;
        case LINE_STATUS_LINECOMMENT:
        default:
            siStatus = LINE_STATUS_UNDEFLINECOMMENT;
            p += _tcslen(p);
            break;
        }
        iRet = RET_TRUE;
    }
FUNC_END:
    *ppcurrent = p;
    return iRet;
}


/*!
    @brief          ݈ʒu|C^當Iʒu
*/
static int cdmtrExtLineSeachStringKey(_TCHAR **ppcurrent)
{
    int iRet;
    _TCHAR *p;
    iRet = RET_FALSE;
    p = *ppcurrent;
    if (gtOptInfo_lc.iLangType == LANG_TYPE_VB) {
        while (!IS_NULL_CHAR(p)) {
            if (IS_DOUBLEQUOTE(p)) {
                p++;
                break;
            }
            p++;
        }
        siStrStatus = STR_STATUS_INIT;
        iRet = RET_TRUE;
    } else {
        while (!IS_NULL_CHAR(p)) {
            if (iFlgAvailableIdentifier) {
                if (IS_DOUBLEQUOTE(p)) {
                    if (IS_DOUBLEQUOTE(p + 1)) {
                        p = p + 2;
                        continue;
                    } else {
                        siStrStatus = STR_STATUS_INIT;
                        iFlgAvailableIdentifier = CDMTR_FALSE;
                        iRet = RET_TRUE;
                        p++;
                        break;
                    }
                }
            } else {
                if (IS_BACKSLASH(p)) {
                    p++;
                    if (IS_NULL_CHAR(p)) {
                        break;
                    }
                } else if (cdmtrExtLineIsStringKey(p)) {
                    iRet = RET_TRUE;
                    p++;
                    break;
                } else {
                }
            }
            p++;
        }
    }
    *ppcurrent = p;
    return iRet;
}


/*!
    @brief          Jn/Iǂ𔻒f
*/
static int cdmtrExtLineIsStringKey(_TCHAR *pcurrent)
{
    int iRet;
    _TCHAR *p;
    iRet = RET_FALSE;
    if (pcurrent == NULL) {
        goto FUNC_END;
    }
    p = pcurrent;
    if (!siCountString) {
        goto FUNC_END;
    }
    if (siStrStatus == STR_STATUS_INCLUDE) {
        if (*p == '>') {
            siStrStatus = STR_STATUS_INIT;
            iRet = RET_TRUE;
        }
        goto FUNC_END;
    }
    if (siStrStatus != STR_STATUS_CHAR && IS_DOUBLEQUOTE(p)) {
        if (siStrStatus == STR_STATUS_INIT) {
            siStrStatus = STR_STATUS_STRING;
            if (gtOptInfo_lc.iLangType == LANG_TYPE_CS) {
                if (stInputBuff < pcurrent && (*(pcurrent - 1) == '@')) {
                    iFlgAvailableIdentifier = CDMTR_TRUE;
                }
            }
        } else {
            siStrStatus = STR_STATUS_INIT;
            iFlgAvailableIdentifier = CDMTR_FALSE;
        }
        iRet = RET_TRUE;
        goto FUNC_END;
    }
    if (siStrStatus != STR_STATUS_STRING && IS_SINGLEQUOTE(p)) {
        if (siStrStatus == STR_STATUS_INIT) {
            siStrStatus = STR_STATUS_CHAR;
            if (gtOptInfo_lc.iLangType == LANG_TYPE_CS) {
                if (stInputBuff < pcurrent && (*(pcurrent - 1) == '@')) {
                    iFlgAvailableIdentifier = CDMTR_TRUE;
                }
            }
        } else {
            siStrStatus = STR_STATUS_INIT;
            iFlgAvailableIdentifier = CDMTR_FALSE;
        }
        iRet = RET_TRUE;
        goto FUNC_END;
    }
FUNC_END:
    return iRet;
}

/*!
    @brief          ʂJEgΏۂ̍ڂݒ
*/
static void cdmtrExtLineChkCount(int iLangType)
{
    switch (iLangType) {
    case LANG_TYPE_CPP:
    case LANG_TYPE_CPPH:
    case LANG_TYPE_C:
    case LANG_TYPE_CH:
        siCountPREP = CDMTR_TRUE;
        siCountCLOC = CDMTR_TRUE;
        siCountSKIP = CDMTR_TRUE;
        siCountString = CDMTR_TRUE;
        break;
    case LANG_TYPE_JAVA:
        siCountPREP = CDMTR_FALSE;
        siCountCLOC = CDMTR_TRUE;
        siCountSKIP = CDMTR_FALSE;
        siCountString = CDMTR_TRUE;
        break;
    case LANG_TYPE_CS:
        siCountPREP = CDMTR_TRUE;
        siCountCLOC = CDMTR_TRUE;
        siCountSKIP = CDMTR_TRUE;
        siCountString = CDMTR_TRUE;
        break;
    case LANG_TYPE_VB:
        siCountPREP = CDMTR_TRUE;
        siCountCLOC = CDMTR_TRUE;
        siCountSKIP = CDMTR_TRUE;
        siCountString = CDMTR_TRUE;
        break;
    case LANG_TYPE_ETC:
        siCountPREP = CDMTR_FALSE;
        siCountCLOC = CDMTR_TRUE;
        siCountSKIP = CDMTR_FALSE;
        siCountString = CDMTR_FALSE;
        break;
    case LANG_TYPE_TXT:
    case LANG_TYPE_BIN:
    case LANG_TYPE_UNKNOW:
    default:
        siCountPREP = CDMTR_FALSE;
        siCountCLOC = CDMTR_FALSE;
        siCountSKIP = CDMTR_FALSE;
        siCountString = CDMTR_FALSE;
        break;
    }
    return;
}


/*!
    @brief          psǂ𔻒f
*/
static int cdmtrExtLineIsContinueLine(int iLineStatus)
{
    int iRet;
    _TCHAR *p;
    iRet = RET_FALSE;
    p = &stInputBuff[_tcslen(stInputBuff)];
    if (p == stInputBuff) {
        goto FUNC_END;
    }
    p = p - 1;
    switch (gtOptInfo_lc.iLangType) {
    case LANG_TYPE_CPP:
    case LANG_TYPE_CPPH:
    case LANG_TYPE_C:
    case LANG_TYPE_CH:
        if (IS_BACKSLASH(p)) {
            iRet = RET_TRUE;
        }
        break;
    case LANG_TYPE_VB:
        if (iLineStatus == LINE_STATUS_INIT ||
                iLineStatus == LINE_STATUS_LINECOMMENT ||
                iLineStatus == LINE_STATUS_LINEDOC) {
        } else {
            if (IS_WHITE_SPACE(p)) {
                SKIPBACK_WHITE_SPACE(stInputBuff, &p);
                if ( stInputBuff < p ) {
                    p--;
                }
            }
            if ( stInputBuff + 2 <= p ) {
                if (*(p - 1) == ' ' && *(p) == '_') {
                    iRet = RET_TRUE;
                }
            }
        }
        break;
    case LANG_TYPE_JAVA:
    case LANG_TYPE_CS:
    case LANG_TYPE_TXT:
    case LANG_TYPE_BIN:
    case LANG_TYPE_ETC:
    case LANG_TYPE_UNKNOW:
    default:
        break;
    }
FUNC_END:
    return iRet;
}

/*!
    @brief          BOMXLbv
*/
static int cdmtrSkipBOM(_TCHAR **ppcurrent)
{
    int iRet = CDMTR_FALSE;
    _TCHAR *p = *ppcurrent;
    int uiLen = _tcslen(p);
    if (uiLen >= 3) {
        if (*p == (_TCHAR)0xEF &&
                *(p + 1) == (_TCHAR)0xBB &&
                *(p + 2) == (_TCHAR)0xBF ) {
            p += 3;
            iRet = CDMTR_TRUE;
        }
    }
    *ppcurrent = p;
    return iRet;
}

/*!
    @brief          ֐Pʂ̊{CJEgs
*/
static int cdmtrExtLineExtractLinesFuncMode(FILE *fin)
{
    int iRet = RET_NG;
    int iStatus = FUNC_LC_STATUS_INIT;
    long lReadIndex = 1;
    long lWriteIndex = 0;
    _LINECOUNT_LIST *ptCurrent = NULL;
    siIsBreakLineCode = CDMTR_FALSE;
    siIsBlankLine = CDMTR_FALSE;
    if (gptLCList != NULL) {
        ptCurrent = gptLCList;
    } else {
        goto FUNC_END;
    }
    while (_fgetts(stInputBuff, BUFF_LINE_LENGTH, fin) != NULL) {
        lWriteIndex++;
        cdmtrExtLineSeachLine();
        cdmtrExtLineChkBlankLine();
        if (lReadIndex == ptCurrent->tFuncData.lStart) {
            if (iStatus == FUNC_LC_STATUS_INIT) {
                iStatus = FUNC_LC_STATUS_COUNTING;
                if (siIsBreakLineCode == CDMTR_FALSE) {
                    if ( siIsBlankLine != CDMTR_FALSE) {
                        if (gtOptInfo_lc.iDetailBlankFlg != CDMTR_TRUE) {
                            iStatus = FUNC_LC_STATUS_INIT;
                        }
                    }
                }
                if (iStatus == FUNC_LC_STATUS_COUNTING) {
                    ptCurrent->tFuncData.lStart = lWriteIndex;
                    memset(&gtCount, 0x00, sizeof(_LINECOUNT));
                }
            }
        }
        cdmtrExtLineCountLine();
        if (siIsBlankLine != CDMTR_FALSE) {
            if (gtOptInfo_lc.iDetailBlankFlg != CDMTR_TRUE) {
                lWriteIndex--;
            }
        }
        if (lReadIndex == ptCurrent->tFuncData.lEnd) {
            if (siIsBreakLineCode == CDMTR_TRUE) {
                cdmtrExtLineSetLCList(iStatus, lWriteIndex, ptCurrent);
                iStatus = FUNC_LC_STATUS_INIT;
                while (1) {
                    if (ptCurrent->next == NULL) {
                        iRet = RET_OK;
                        goto FUNC_END;
                    }
                    ptCurrent = ptCurrent->next;
                    if (lReadIndex == ptCurrent->tFuncData.lStart) {
                        ptCurrent->iInvalidFlg = CDMTR_TRUE;
                        ptCurrent->tFuncData.lStart = 0;
                        iStatus = FUNC_LC_STATUS_INVALID;
                        if (lReadIndex == ptCurrent->tFuncData.lEnd) {
                            ptCurrent->tFuncData.lEnd = 0;
                            iStatus = FUNC_LC_STATUS_INIT;
                            continue;
                        }
                    }
                    break;
                }
            }
        }
        if (siIsBreakLineCode == CDMTR_TRUE) {
            lReadIndex++;
            siIsBreakLineCode = CDMTR_FALSE;
        }
        siIsBlankLine = CDMTR_FALSE;
    }
    iRet = RET_OK;
FUNC_END:
    if (iRet == RET_OK) {
        cdmtrExtLineSetLCList(iStatus, lWriteIndex, ptCurrent);
        while (ptCurrent->next != NULL) {
            ptCurrent = ptCurrent->next;
            ptCurrent->iInvalidFlg = CDMTR_TRUE;
            ptCurrent->tFuncData.lStart = 0;
            ptCurrent->tFuncData.lEnd = 0;
        }
    }
    return iRet;
}

/*!
    @brief          ֐PʂŊ{CJEgʂXgɓo^
*/
static void cdmtrExtLineSetLCList(int iStatus, long lWriteIndex, _LINECOUNT_LIST *ptLCInfo)
{
    switch (iStatus) {
    case FUNC_LC_STATUS_COUNTING:
        ptLCInfo->tFuncData.lEnd = lWriteIndex;
        if (iSkipChangeFlg == CDMTR_TRUE) {
            gtCount.ulELOC = gtCount.ulLine + gtCount.ulPre;
            gtCount.ulCLOC = gtCount.ulDoc + gtCount.ulCmnt;
            gtCount.ulTLOC = gtCount.ulELOC + gtCount.ulCLOC + gtCount.ulBlank + gtCount.ulUndef;
        } else {
            gtCount.ulELOC = gtCount.ulLine + gtCount.ulPre + gtCount.ulUndef;
            gtCount.ulCLOC = gtCount.ulDoc + gtCount.ulCmnt;
            gtCount.ulTLOC = gtCount.ulELOC + gtCount.ulCLOC + gtCount.ulBlank;
        }
        memcpy(&(ptLCInfo->tLCData), &gtCount, sizeof(_LINECOUNT));
        break;
    case FUNC_LC_STATUS_INVALID:
        ptLCInfo->tFuncData.lEnd = 0;
        break;
    case FUNC_LC_STATUS_INIT:
    default:
        break;
    }
    return;
}

/*!
    @brief  󔒍s`FbN
*/
static void cdmtrExtLineChkBlankLine(void)
{
    if (stCol.ulCol > 0) {
    } else if (stCol.ulPreCol > 0 ||
               siStatus == LINE_STATUS_UNDEFPREPRO) {
    } else if (stCol.ulUndefCol > 0 || stCol.ulUndefCmntCol > 0 ||
               siStatus == LINE_STATUS_UNDEFLINE ||
               siStatus == LINE_STATUS_UNDEFSKIP ||
               siStatus == LINE_STATUS_UNDEFLINECOMMENT ||
               siStatus == LINE_STATUS_UNDEFCOMMENT) {
    } else if (stCol.ulDocCol > 0 ||
               siStatus == LINE_STATUS_BLOCKDOC ||
               siStatus == LINE_STATUS_LINEDOC) {
    } else if (stCol.ulCmntCol > 0 ||
               siStatus == LINE_STATUS_BLOCKCOMMENT ||
               siStatus == LINE_STATUS_LINECOMMENT) {
    } else {
        siIsBlankLine = CDMTR_TRUE;
    }
}
