/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef CDMMTRLIBUTIL_H_
#define CDMMTRLIBUTIL_H_

#include <tchar.h>

#define CHAR_NULL          ('\0')
#define CHAR_CR            ('\r')
#define CHAR_LF            ('\n')
#define CHAR_BACKSLASH     ('\\')
#define CHAR_SINGLEQUOTE   ('\'')
#define CHAR_DOUBLEQUOTE   ('\"')
#define CHAR_COMMA         (',')
#define CHAR_EQUAL         ('=')
#define CHAR_HYPHEN        ('-')
#define CHAR_SPACE         (' ')
#define CHAR_TAB           ('\t')

#define IS_NULL_CHAR(p) (*(p) == '\0')

#define IS_LF(p) (*(p) == 0x0a)

#define IS_CR(p) (*(p) == 0x0d)

#define IS_BREAK_LINE(p) (IS_LF(p) || IS_CR(p))

#define IS_HT(p) (*(p) == 0x09)

#define IS_VT(p) (*(p) == 0x0b)

#define IS_CTRL_TAB(p) (IS_HT(p) || IS_VT(p))

#define IS_EQUAL(p) (*(p) == '=')

#define IS_BACKSLASH(p) (*(p) == '\\')

#define IS_SINGLEQUOTE(p) (*(p) == '\'')

#define IS_DOUBLEQUOTE(p) (*(p) == '"')


#define IS_WHITE_SPACE(p) ((*(p) == ' ') || (*(p) == '\t'))

#define SKIP_WHITE_SPACE(pp) {\
        while(IS_WHITE_SPACE(*(pp))){\
            (*(pp))++;\
        }\
    }

#define SKIPBACK_WHITE_SPACE(pStart, ppEnd) {\
        while(*(ppEnd) > pStart){\
            (*(ppEnd))--;\
            if(!IS_WHITE_SPACE(*(ppEnd))){\
                (*(ppEnd))++;\
                break;\
            }\
        }\
    }

#define REGEX_ASTARISK "*"
#define REGEX_ASTARISK_CHAR '*'
#define REGEX_ESCAPE_CHAR '\\'

typedef enum {
    SEARCH_COMPLETE = 0,
    SEARCH_PREFIX,
    SEARCH_SUFFIX,
    SEARCH_PART
} SEARCH_TYPE;

#define ENCODING_EUC_JP             _T("EUC-JP")
#define ENCODING_SHIFT_JIS          _T("Shift_JIS")
#define ENCODING_UTF_8              _T("UTF-8")
#define ENCODING_SYSTEM             _T("SYSTEM")




int CDMtrLibUtilGetStringNextDelimSP(char **string, char **startp, char **endp, char delimiter);
int CDMtrExtPropGetAttributePos(char **currentp, char **name_top, char **name_end,
                                char **val_top, char **val_end);
int CDMtrLibUtilSetString(char **dst, const char *val_top, const char *val_end);
char *CDMtrLibUtilCopyString(const char *pcStr);
int CDMtrLibUtilCompString(const char *pcDst, const char *pcSrc);
int CDMtrLibUtilStrCaseCmp(const char *s1, const char *s2, int n);
char *CDMtrLibUtilStrCaseStr(const char *string, const char *pattern);
int CDMtrLibUtilMatchPattern(const char *pcStr, const char *pcReg);
int CDMtrLibUtilIsStringFalse(const char *val_top, const char *val_end);
int CDMtrLibUtilSetLowString(char *dst);
int CDMtrLibUtilSetUpperString(char *dst);
int CDMtrLibUtilIsAlpha(char c);
int CDMtrLibUtilIsDigit(char c);
int CDMtrLibUtilMBCStoUNICODE(wchar_t **new_char, const char **org_str, _TCHAR *charSet);
int CDMtrLibUtilUNICODEtoMBCS(char **new_char, const wchar_t **org_str, _TCHAR *charSet);
int CDMtrLibUtilCheckSupportCharset(_TCHAR *cpCharset, unsigned int *uipCodePage);
#endif
