/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "CDMtrCmn.h"
#include "CDMtrLibCmn.h"
#include "CDMtrLibProp.h"
#include "CDMtrLibUtil.h"

#define BUFF_LINE_LENGTH (1024)

#define BUFF_LINE_SIZE (BUFF_LINE_LENGTH*sizeof(char))

#define PROP_GROUP_START           '['

#define PROP_GROUP_END             ']'

#define PROP_COMMENT_CHAR          '#'

typedef struct tag_PropInfoTree {
    struct tag_PropInfoTree *next;
    _PropInfo *ptInfo;
} _PropInfoTree;

static char stBuff[BUFF_LINE_LENGTH];


static int cdmtrPropIsGroupName(const char *pcGroupName, char *pcCurrent);
static int cdmtrPropIsGroupStart(char *pcCurrent);
static int cdmtrLibPropGetTree(FILE *pfProp, _PropInfoTree **pptTree);


/*!
    @brief          O[vɑݒ擾
*/
int CDMtrLibPropGetList(const char *pcFileName, const char *pcGroupName, _PropInfoList **pptPropList)
{
    int iRet;
    int iCount;
    int i;
    int iGroupFlag;
    char *currentp;
    FILE *pfProp;
    _PropInfoTree tree_first;
    _PropInfoTree *ptTree;
    _PropInfoList *ptInfo;
    iRet = RET_NG;
    ptInfo = NULL;
    iCount = 0;
    iGroupFlag = RET_FALSE;
    pfProp = NULL;
    tree_first.next = NULL;
    tree_first.ptInfo = NULL;
    ptTree = &tree_first;
    if (pcFileName == NULL) {
        goto FUNC_END;
    }
    if (pcGroupName == NULL) {
        goto FUNC_END;
    }
    if (pptPropList == NULL) {
        goto FUNC_END;
    }
    *pptPropList = NULL;
    pfProp = fopen(pcFileName, "r");
    if (pfProp == NULL) {
        goto FUNC_END;
    }
    while (fgets(stBuff, BUFF_LINE_LENGTH, pfProp) != NULL) {
        currentp = stBuff;
        SKIP_WHITE_SPACE(&currentp);
        if (IS_NULL_CHAR(currentp) || IS_BREAK_LINE(currentp)) {
            continue;
        }
        if (cdmtrPropIsGroupName(pcGroupName, currentp)) {
            iCount = cdmtrLibPropGetTree(pfProp, &ptTree);
            iGroupFlag = RET_TRUE;
            break;
        }
    }
    if (!iGroupFlag) {
        goto FUNC_END;
    }
    ptInfo = (_PropInfoList *)CDMtrLibCmnMalloc(sizeof(_PropInfoList) + (iCount - 1) * sizeof (_PropInfo *));
    if (ptInfo == NULL) {
        goto FUNC_END;
    }
    ptInfo->iNum = iCount;
    ptTree = tree_first.next;
    for (i = 0; i < iCount; i++) {
        ptInfo->ptPropInfo[i] = ptTree->ptInfo;
        ptTree = ptTree->next;
    }
    iRet = RET_OK;
    *pptPropList = ptInfo;
FUNC_END:
    for (ptTree = tree_first.next; ptTree != NULL; ptTree = tree_first.next) {
        tree_first.next = ptTree->next;
        CDMtrLibCmnFree((void **)&ptTree);
    }
    if (pfProp != NULL) {
        fclose(pfProp);
        pfProp = NULL;
    }
    return iRet;
}


/*!
    @brief          w肳ꂽݒ񃊃Xg̃̈
*/
void CDMtrLibPropFreeList(_PropInfoList **pptPropList)
{
    int iCount;
    if (pptPropList == NULL) {
        goto FUNC_END;
    }
    if (*pptPropList == NULL) {
        goto FUNC_END;
    }
    iCount = (*pptPropList)->iNum - 1;
    for (; iCount >= 0 ; iCount--) {
        if ((*pptPropList)->ptPropInfo[iCount] != NULL) {
            CDMtrLibCmnFree((void **) & ((*pptPropList)->ptPropInfo[iCount]->pcAttr));
            CDMtrLibCmnFree((void **) & ((*pptPropList)->ptPropInfo[iCount]->pcVal));
            CDMtrLibCmnFree((void **) & ((*pptPropList)->ptPropInfo[iCount]));
        }
    }
    CDMtrLibCmnFree((void **)pptPropList);
    *pptPropList = NULL;
FUNC_END:
    return;
}


/*!
    @brief          區ږv邩
*/
static int cdmtrPropIsGroupName(const char *pcGroupName, char *pcCurrent)
{
    int iRet, iFuncRet;
    char *p, *startp, *endp;
    char *pcName;
    iRet = RET_FALSE;
    p = pcCurrent;
    pcName = NULL;
    if (*p != PROP_GROUP_START) {
        goto FUNC_END;
    }
    p++;
    SKIP_WHITE_SPACE(&p);
    startp = p;
    endp = p;
    while (!IS_NULL_CHAR(p)) {
        if (IS_WHITE_SPACE(p) || *p == PROP_GROUP_END) {
            break;
        }
        endp = p;
        p++;
    }
    SKIP_WHITE_SPACE(&p);
    if (*p != PROP_GROUP_END) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilSetString(&pcName, startp, endp);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (CDMtrLibUtilCompString(pcGroupName, pcName)) {
        iRet = RET_TRUE;
    }
FUNC_END:
    CDMtrLibCmnFree((void **)&pcName);
    return iRet;
}


/*!
    @brief          區ڊJns𔻒f
*/
static int cdmtrPropIsGroupStart(char *pcCurrent)
{
    int iRet;
    char *p;
    iRet = RET_FALSE;
    p = pcCurrent;
    if (*p != PROP_GROUP_START) {
        goto FUNC_END;
    }
    p++;
    SKIP_WHITE_SPACE(&p);
    while (!IS_NULL_CHAR(p)) {
        if (IS_WHITE_SPACE(p) || *p == PROP_GROUP_END) {
            break;
        }
        p++;
    }
    SKIP_WHITE_SPACE(&p);
    if (*p == PROP_GROUP_END) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          ƒl̃c[擾
*/
static int cdmtrLibPropGetTree(FILE *pfProp, _PropInfoTree **pptTree)
{
    int iFuncRet;
    int iNum;
    char *currentp;
    char *name_top, *name_end, *val_top, *val_end;
    char *pcAttr, *pcVal;
    _PropInfoTree *ptTree;
    iNum = 0;
    ptTree = *pptTree;
    pcAttr = NULL;
    pcVal = NULL;
    while (fgets(stBuff, BUFF_LINE_LENGTH, pfProp) != NULL) {
        currentp = stBuff;
        SKIP_WHITE_SPACE(&currentp);
        if (IS_NULL_CHAR(currentp) || IS_BREAK_LINE(currentp)) {
            continue;
        }
        if (*currentp == PROP_COMMENT_CHAR) {
            continue;
        }
        if (cdmtrPropIsGroupStart(currentp)) {
            break;
        }
        iFuncRet = CDMtrExtPropGetAttributePos(&currentp, &name_top, &name_end, &val_top, &val_end);
        if (iFuncRet != RET_OK) {
            continue;
        }
        if (name_top == NULL || name_end == NULL) {
            continue;
        }
        iFuncRet = CDMtrLibUtilSetString(&pcAttr, name_top, name_end);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        if (val_top == NULL || val_end == NULL) {
            pcVal = (char *)CDMtrLibCmnMalloc(1);
            if (pcVal == NULL) {
                goto FUNC_END;
            }
            *pcVal = '\0';
        } else {
            iFuncRet = CDMtrLibUtilSetString(&pcVal, val_top, val_end);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            }
        }
        ptTree->next = (_PropInfoTree *)CDMtrLibCmnMalloc(sizeof(_PropInfoTree));
        if (ptTree->next == NULL) {
            goto FUNC_END;
        }
        iNum++;
        ptTree = ptTree->next;
        ptTree->ptInfo = (_PropInfo *)CDMtrLibCmnMalloc(sizeof(_PropInfo));
        if (ptTree->ptInfo == NULL) {
            goto FUNC_END;
        }
        memset(ptTree->ptInfo, 0x00, sizeof(_PropInfo));
        ptTree->ptInfo->pcAttr = pcAttr;
        ptTree->ptInfo->pcVal = pcVal;
        pcAttr = NULL;
        pcVal = NULL;
    }
FUNC_END:
    CDMtrLibCmnFree((void **)&pcAttr);
    CDMtrLibCmnFree((void **)&pcVal);
    return iNum;
}
