/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Windows.h>
#include <shlwapi.h>
#ifdef _MBCS
#include <mbstring.h>
#endif

#include "CDMtrExtCmn.h"
#include "CDMtrLibCmnW.h"
#include "CDMtrLibUtilW.h"
#include "CDMtrLibFileW.h"
#include "CDMtrMessage.h"

#define FILE_TARGET_TYPE_W L"*.*"

#define FOLDER_SEPALATE         _T("\\")

#define FILE_SEPALATE '\\'

#define UNC_PREFIX     _T("\\\\")

#define LONGPATH_PREFIX     _T("\\\\?\\")

#define LONGPATH_PREFIX_SIZE    4

#define LONGPATH_UNC_PREFIX     _T("\\\\?\\UNC\\")

#define LONGPATH_UNC_PREFIX_SIZE    8

#define FILE_SEPALATE_EXT '.'
typedef struct tag_FileTree {
    struct tag_FileTree *next;
    wchar_t *fname;
}
_FileTree;

int iShowDetails ;

int iDebug ;

int iNumOfUnrecognized ;


static int cdmtrLibFileIsAbsolutePathW(const wchar_t *pcpath);
static int cdmtrLibFileGetDirW(const wchar_t *pcDirpath, int *piFilenum, _FileTree **ppftree);
static wchar_t *cdmtrLibFileGetRealPathW(const wchar_t *pcPath);
static wchar_t *cdmtrLibFileAddPathNameW(wchar_t *pcPath, const wchar_t *pcAdd);
static wchar_t *cdmtrLibFileDelPathNameW(wchar_t *pcPath);
static int cdmtrLibFileDeleteSubDirW(const wchar_t *pcDirpath);


/*!
    @brief          pXgq擾
*/
int CDMtrLibFileGetFileExtensionW(const wchar_t *pcFilePath, wchar_t *pcExt)
{
    int iRet, iFuncRet;
    wchar_t *p;
    wchar_t *pcFileName;
    int iLen;
    iRet = RET_NG;
    pcFileName = NULL;
    if (pcFilePath == NULL || pcExt == NULL) {
        goto FUNC_END;
    }
    memset(pcExt, 0x00, EXT_SIZE + 1);
    iLen = wcslen(pcFilePath);
    pcFileName = (wchar_t *)malloc((iLen + 1) * sizeof(wchar_t));
    if (pcFileName == NULL) {
        goto FUNC_END;
    }
    memset(pcFileName, 0x00, (iLen + 1) * sizeof(wchar_t));
    iFuncRet = CDMtrLibFileGetFileNameW(pcFilePath, pcFileName);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    p = wcsrchr(pcFileName, FILE_SEPALATE_EXT);
    if (p != NULL) {
        p++;
        wcsncpy(pcExt, p, EXT_SIZE);
        iRet = RET_OK;
    }
FUNC_END:
    if (pcFileName != NULL) {
        free(pcFileName);
    }
    return iRet;
}



/*!
    @brief          pXfBNg擾 (UNICODE)
*/
int CDMtrLibFileGetDirNameW(const wchar_t *pcpath, wchar_t *pcdir)
{
    int iRet;
    const wchar_t *p;
    int iLenDir;
    iRet = RET_NG;
    if (pcpath == NULL || pcdir == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDirW(pcpath)) {
        iLenDir = wcslen(pcpath);
        wcsncpy(pcdir, pcpath, iLenDir);
        pcdir[iLenDir] = '\0';
    } else {
        p = _tcsrchr(pcpath, FILE_SEPALATE);
        if (p == NULL) {
            *pcdir = '\0';
        } else if (p == pcpath) {
            *pcdir = FILE_SEPALATE;
            *(pcdir + 1) = '\0';
        } else {
            iLenDir = p - pcpath;
            wcsncpy(pcdir, pcpath, iLenDir);
        }
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          pXt@C擾
*/
int CDMtrLibFileGetFileNameW(const wchar_t *pcpath, wchar_t *pcfile)
{
    int iRet;
    const wchar_t *p;
    int iLenFile;
    iRet = RET_NG;
    if (pcpath == NULL || pcfile == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDirW(pcpath)) {
        pcfile = '\0';
    } else {
        p = _tcsrchr(pcpath, FILE_SEPALATE);
        if (p == NULL) {
            iLenFile = wcslen(pcpath);
            p = (wchar_t *)pcpath;
        } else {
            iLenFile = wcslen(pcpath) - (p - pcpath);
            p++;
        }
        wcscpy(pcfile, p);
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNgƃt@C
*/
int CDMtrLibFileSetDirFileNameW(const wchar_t *pcDir, const wchar_t *pcFile, wchar_t **ppcPath)
{
    int iRet;
    unsigned int uiDirPath, uiPathLen;
    wchar_t *p;
    wchar_t *pcFilePath;
    iRet = RET_NG;
    if (pcDir == NULL || pcFile == NULL || ppcPath == NULL) {
        goto FUNC_END;
    }
    *ppcPath = NULL;
    uiDirPath = wcslen(pcDir);
    uiPathLen = uiDirPath;
    uiPathLen++;
    uiPathLen += wcslen(pcFile);
    p = (wchar_t *)malloc((uiPathLen + 1) * sizeof(wchar_t));
    if (p == NULL) {
        goto FUNC_END;
    }
    memset(p, 0x00, (uiPathLen + 1) * sizeof(wchar_t));
    pcFilePath = (wchar_t *)pcFile;
    if (*pcFilePath == FILE_SEPALATE) {
        pcFilePath++;
    }
    if (*(pcDir + (uiDirPath - 1)) == FILE_SEPALATE) {
        _stprintf(p, _T("%s%s"), pcDir, pcFilePath);
    } else {
        _stprintf(p, _T("%s%c%s"), pcDir, FILE_SEPALATE, pcFilePath);
    }
    p[uiPathLen] = '\0';
    *ppcPath = p;
    iRet = RET_OK;
FUNC_END:
    return iRet;
}



/*!
    @brief          w肳ꂽpX΃pXɕϊ
*/
int CDMtrLibFileGetAbsolutePathW(const wchar_t *pcfile, wchar_t **ppcpath)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    wchar_t *pcFullPath = NULL;
    if (pcfile == NULL || ppcpath == NULL) {
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibFileGetFullPathNameW(pcfile, &pcFullPath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (CDMtrLibUtilCheckPrefixW(pcFullPath, LONGPATH_PREFIX) != NULL) {
        *ppcpath = pcFullPath;
    } else {
        if (CDMtrLibUtilCheckPrefixW(pcFullPath, UNC_PREFIX) != NULL) {
            CDMtrLibUtilCreateStringW(ppcpath, LONGPATH_UNC_PREFIX, pcFullPath + wcslen(UNC_PREFIX), NULL);
        } else {
            CDMtrLibUtilCreateStringW(ppcpath, LONGPATH_PREFIX, pcFullPath, NULL);
        }
        CDMtrLibCmnFreeW((void **)&pcFullPath);
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief              œnꂽpXɁAVtH_ǉ
*/
static wchar_t *cdmtrLibFileAddPathNameW(wchar_t *pcPath, const wchar_t *pcAdd)
{
    wchar_t *pcRetPath = NULL;
    int iLen = 0;
    if (NULL == pcPath) {
        return NULL;
    }
    if (NULL == pcAdd) {
        iLen = wcslen(pcPath);
        pcRetPath = (wchar_t *)malloc(sizeof(wchar_t) * (iLen + 1));
        if (NULL == pcRetPath) {
            return NULL;
        }
        memset(pcRetPath, '\0', (iLen + 1) * sizeof(wchar_t));
        memcpy(pcRetPath, pcPath, iLen * sizeof(wchar_t));
        return pcRetPath;
    }
    iLen = wcslen(pcPath) + 1 + wcslen(pcAdd);
    pcRetPath = (wchar_t *)malloc(sizeof(wchar_t) * (iLen + 1));
    if (NULL == pcRetPath) {
        return NULL;
    }
    memset(pcRetPath, '\0', (iLen + 1) * sizeof(wchar_t));
    iLen = wcslen(pcPath);
    memcpy(pcRetPath, pcPath, iLen * sizeof(wchar_t));
    *(pcRetPath + iLen) = FILE_SEPALATE;
    memcpy(pcRetPath + iLen + 1, pcAdd, wcslen(pcAdd) * sizeof(wchar_t));
    return pcRetPath;
}


/*!
    @brief          w肵fBNg݂邩mF
*/
int CDMtrLibFileIsDirW(const wchar_t *pcdir)
{
    int iRet;
    DWORD attr = 0;
    iRet = RET_FALSE;
    if (pcdir == NULL) {
        goto FUNC_END;
    }
    attr = GetFileAttributesW(pcdir);
    if ((attr != -1) &&
            (attr & FILE_ATTRIBUTE_DIRECTORY) != 0) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}

/*!
    @brief          w肵t@C݂邩mF
*/
int CDMtrLibFileIsFileW( const wchar_t *pcfile )
{
    int iRet;
    DWORD attr = 0;
    iRet = RET_FALSE;
    if (pcfile == NULL) {
        goto FUNC_END;
    }
    attr = GetFileAttributesW(pcfile);
    if ((attr != -1) &&
            (attr & FILE_ATTRIBUTE_DIRECTORY) == 0) {
        iRet = RET_TRUE;
    }
FUNC_END:
    return iRet;
}


/*!
    @brief          wfBNgȉ̃t@Cꗗ擾
*/
int CDMtrLibFileGetFileListW(const wchar_t *pcdir, _FileList **ppflist)
{
    _FileList *pfiles;
    _FileTree tree_first;
    _FileTree *pftree;
    int iFileNum;
    int iRet, iFuncRet;
    int i;
    iRet = RET_NG;
    tree_first.next = NULL;
    pftree = &tree_first;
    if (pcdir == NULL || ppflist == NULL) {
        goto FUNC_END;
    }
    *ppflist = NULL;
    iFileNum = 0;
    if (wcslen(pcdir) <= 0) {
        goto FUNC_END;
    }
    if (CDMtrLibUtilCheckPrefixW(pcdir, LONGPATH_UNC_PREFIX) != NULL) {
    } else if (CDMtrLibFileIsFileW(pcdir)) {
        int iPathLen;
        pftree->next = (_FileTree *)malloc(sizeof(_FileTree));
        if (pftree->next == NULL) {
            goto FUNC_END;
        }
        memset(pftree->next, 0x00, sizeof(_FileTree));
        pftree = pftree->next;
        iPathLen = wcslen(pcdir);
        pftree->fname = (wchar_t *)malloc((iPathLen + 1) * sizeof(wchar_t));
        if (pftree->fname == NULL) {
            goto FUNC_END;
        }
        iFileNum++;
        wcsncpy(pftree->fname, pcdir, iPathLen);
        pftree->fname[iPathLen] = '\0';
    } else {
        iFuncRet = cdmtrLibFileGetDirW(pcdir, &iFileNum, &pftree);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    if (iFileNum > 0) {
        pfiles = (_FileList *)malloc(sizeof(_FileList) + (iFileNum - 1) * sizeof (wchar_t *));
    } else {
        pfiles = (_FileList *)malloc(sizeof(_FileList) + sizeof (wchar_t *));
    }
    if (pfiles == NULL) {
        goto FUNC_END;
    }
    memset(pfiles, 0x00, sizeof(_FileList) + (iFileNum - 1) * sizeof (wchar_t *));
    pfiles->count = iFileNum;
    pftree = tree_first.next;
    for (i = 0; i < iFileNum; i++) {
        pfiles->filename[i] = pftree->fname;
        pftree = pftree->next;
    }
    iRet = RET_OK;
    *ppflist = pfiles;
FUNC_END:
    for (pftree = tree_first.next; pftree != NULL; pftree = tree_first.next) {
        tree_first.next = pftree->next;
        free(pftree);
    }
    return iRet;
}

/*!
    @brief              t@CXg̈
*/
void CDMtrLibFileFreeFileListW(_FileList **ppflist)
{
    int iCount;
    if (ppflist == NULL) {
        goto FUNC_END;
    }
    if (*ppflist == NULL) {
        goto FUNC_END;
    }
    if ((*ppflist)->count > 0) {
        iCount = (*ppflist)->count - 1;
        for (; iCount >= 0 ; iCount--) {
            if ((*ppflist)->filename[iCount] != NULL) {
                free((*ppflist)->filename[iCount]);
            }
        }
    }
    free(*ppflist);
    *ppflist = NULL;
FUNC_END:
    return;
}

/*!
    @brief          fBNg̃t@CꗗXg擾
*/
int cdmtrLibFileGetDirW( const wchar_t *pcDirpath, int *piFilenum, _FileTree **ppftree )
{
    int iRet, iFuncRet;
    wchar_t *pcTargetPath;
    _FileTree *pFileTree;
    int iPathLen;
    wchar_t *pcTargetDir;
    wchar_t *pcSeachPath;
    WIN32_FIND_DATAW fd;
    HANDLE hFind;
    BOOL bFindRes;
    DWORD dwLastError = 0;
    bFindRes = RET_TRUE;
    iRet = RET_NG;
    pcTargetPath = NULL;
    pcTargetDir = NULL;
    pcSeachPath = NULL;
    hFind = INVALID_HANDLE_VALUE;
    pFileTree = *ppftree;
    if (CDMtrLibFileIsDirW(pcDirpath)) {
        pcTargetDir = CDMtrLibUtilCopyStringW(pcDirpath);
        if (pcTargetDir == NULL) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrLibFileSetDirFileNameW(pcDirpath, FILE_TARGET_TYPE_W, &pcSeachPath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    } else {
        pcSeachPath = CDMtrLibUtilCopyStringW(pcDirpath);
        if (pcSeachPath == NULL) {
            goto FUNC_END;
        }
        pcTargetDir = (wchar_t *)CDMtrLibCmnMallocW(_tcslen(pcSeachPath) + 1);
        if (pcTargetDir == NULL) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrLibFileGetDirNameW(pcSeachPath, pcTargetDir);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    hFind = FindFirstFileW(pcSeachPath, &fd);
    if (hFind == INVALID_HANDLE_VALUE) {
        dwLastError = GetLastError();
        if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
            if (dwLastError == ERROR_PATH_NOT_FOUND) {
                if (iDebug == DEBUG_ON) {
                    CDMtrLibCmnErrLogOutW( MSG_LOG_ID_C_00000539, dwLastError, pcSeachPath); //* MSG_LOG_ID_C_00000539
                } else {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000540, dwLastError, CDMtrLibFileRemoveLongPathMarkW(pcSeachPath)); //* MSG_LOG_ID_C_00000540
                }
                iNumOfUnrecognized++;
            } else if (dwLastError != ERROR_FILE_NOT_FOUND) {
                if (iDebug == DEBUG_ON) {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000542, dwLastError, pcTargetDir); //* MSG_LOG_ID_C_00000542
                } else {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000543, dwLastError, CDMtrLibFileRemoveLongPathMarkW(pcTargetDir)); //* MSG_LOG_ID_C_00000543
                }
                iNumOfUnrecognized++;
            }
        }
        iRet = RET_OK;
        goto FUNC_END;
    } else {
        do {
            if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0) {
            } else if (wcsstr(fd.cFileName, L"?") != NULL) {
                if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
                    if (iDebug == DEBUG_ON) {
                        CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000545, ERROR_PATH_NOT_FOUND, pcTargetDir, fd.cFileName); //* MSG_LOG_ID_C_00000545
                    } else {
                        CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000546, ERROR_PATH_NOT_FOUND, CDMtrLibFileRemoveLongPathMarkW(pcTargetDir), fd.cFileName); //* MSG_LOG_ID_C_00000546
                    }
                }
                iNumOfUnrecognized++;
            } else {
                CDMtrLibCmnFreeW((void **)&pcTargetPath);
                CDMtrLibFileSetDirFileNameW(pcTargetDir, fd.cFileName, &pcTargetPath);
                if (iFuncRet != RET_OK) {
                    goto FUNC_END;
                }
                if ( (fd.dwFileAttributes & 0x10) != 0 ) {
                    iFuncRet = cdmtrLibFileGetDirW(pcTargetPath, piFilenum, &pFileTree);
                    if (iFuncRet != RET_OK) {
                        goto FUNC_END;
                    }
                } else {
                    (*piFilenum)++;
                    pFileTree->next = (_FileTree *)malloc(sizeof(_FileTree));
                    if (pFileTree->next == NULL) {
                        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000571); //* MSG_LOG_ID_C_00000571
                        goto FUNC_END;
                    }
                    memset(pFileTree->next, 0x00, sizeof(_FileTree));
                    pFileTree = pFileTree->next;
                    iPathLen = wcslen(pcTargetPath);
                    pFileTree->fname = (wchar_t *)malloc((iPathLen + 1) * sizeof(wchar_t));
                    if (pFileTree->fname == NULL) {
                        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000572); //* MSG_LOG_ID_C_00000572
                        goto FUNC_END;
                    }
                    wcsncpy(pFileTree->fname, pcTargetPath, iPathLen);
                    pFileTree->fname[iPathLen] = '\0';
                }
            }
            while (!(bFindRes = FindNextFileW(hFind, &fd))) {
                dwLastError = GetLastError();
                if (ERROR_NO_MORE_FILES == dwLastError) {
                    break;
                } else {
                    if ( iShowDetails == DETAILS_SHOW || iDebug == DEBUG_ON || iNumOfUnrecognized == 0) {
                        if (iDebug == DEBUG_ON) {
                            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000548, dwLastError, pcTargetDir, fd.cFileName); //* MSG_LOG_ID_C_00000548
                        } else {
                            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000549, dwLastError, CDMtrLibFileRemoveLongPathMarkW(pcTargetDir), fd.cFileName); //* MSG_LOG_ID_C_00000549
                        }
                    }
                    iNumOfUnrecognized++;
                    bFindRes = RET_TRUE;
                }
            }
        } while (bFindRes);
    }
    iRet = RET_OK;
FUNC_END:
    if (pcTargetPath != NULL) {
        free(pcTargetPath);
    }
    if (pcTargetDir != NULL) {
        free(pcTargetDir);
    }
    if (pcSeachPath != NULL) {
        free(pcSeachPath);
    }
    if (hFind != INVALID_HANDLE_VALUE) {
        FindClose(hFind);
    }
    *ppftree = pFileTree;
    return iRet;
}

/*!
    @brief          fBNg쐬
*/
int CDMtrLibFileCreateDirW(const wchar_t *pcDirpath)
{
    int iRet;
    iRet = RET_NG;
    if (pcDirpath == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsDirW(pcDirpath)) {
        iRet = RET_OK;
        goto FUNC_END;
    }
    if (!CreateDirectoryW(pcDirpath, NULL)) {
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000573, pcDirpath); //* MSG_LOG_ID_C_00000573
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000574, errno); //* MSG_LOG_ID_C_00000574
        goto FUNC_END;
    }
    iRet = RET_OK;
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000575, pcDirpath); //* MSG_LOG_ID_C_00000575
FUNC_END:
    return iRet;
}

/*!
    @brief          PfBNg폜
*/
int CDMtrLibFileDeleteDirW(const wchar_t *pcDirpath)
{
    int iRet;
    iRet = RET_NG;
    if (pcDirpath == NULL) {
        goto FUNC_END;
    }
    if (!RemoveDirectoryW(pcDirpath)) {
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000576, pcDirpath); //* MSG_LOG_ID_C_00000576
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000577, errno); //* MSG_LOG_ID_C_00000577
        goto FUNC_END;
    } else {
        iRet = RET_OK;
    }
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000578, pcDirpath); //* MSG_LOG_ID_C_00000578
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNgẑׂẴt@C/fBNg폜
*/
int CDMtrLibFileDeleteDirAllW(const wchar_t *pcDirpath, int iDelFlg)
{
    int iRet, iFuncRet;
    iRet = RET_NG;
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000579, pcDirpath); //* MSG_LOG_ID_C_00000579
    iFuncRet = cdmtrLibFileDeleteSubDirW(pcDirpath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    if (iDelFlg != CDMTR_FALSE) {
        iFuncRet = CDMtrLibFileDeleteDirW(pcDirpath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000580); //* MSG_LOG_ID_C_00000580
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          fBNĝׂẴt@C/fBNg폜
*/
static int cdmtrLibFileDeleteSubDirW(const wchar_t *pcDirpath)
{
    int iRet;
    wchar_t *pcTargetPath;
    WIN32_FIND_DATA fd;
    HANDLE hFind;
    iRet = RET_NG;
    pcTargetPath = NULL;
    hFind = INVALID_HANDLE_VALUE;
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000581, pcDirpath); //* MSG_LOG_ID_C_00000581
    pcTargetPath = (wchar_t *)malloc((FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if (pcTargetPath == NULL) {
        goto FUNC_END;
    }
    memset(pcTargetPath, 0x00, (FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if (PathCombineW(pcTargetPath, pcDirpath, FILE_TARGET_TYPE_W) == NULL) {
        goto FUNC_END;
    }
    hFind = FindFirstFileW(pcTargetPath, &fd);
    if (hFind == INVALID_HANDLE_VALUE) {
        goto FUNC_END;
    } else {
        do {
            if (wcscmp(fd.cFileName, L".") == 0 || wcscmp(fd.cFileName, L"..") == 0) {
                continue;
            }
            memset(pcTargetPath, 0x00, (FILE_PATH_SIZE + 1) * sizeof(wchar_t));
            if (PathCombine(pcTargetPath, pcDirpath, fd.cFileName) == NULL) {
                goto FUNC_END;
            }
            if ( (fd.dwFileAttributes & 0x10) != 0 ) {
                iRet = cdmtrLibFileDeleteSubDirW(pcTargetPath);
                if (iRet != RET_OK) {
                    goto FUNC_END;
                }
                iRet = RemoveDirectoryW(pcTargetPath);
                if (iRet != RET_OK) {
                    goto FUNC_END;
                }
            } else {
                iRet = CDMtrLibFileDeleteW(pcTargetPath);
                if (iRet != RET_OK) {
                    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000582, pcTargetPath); //* MSG_LOG_ID_C_00000582
                    goto FUNC_END;
                }
            }
        } while (FindNextFileW(hFind, &fd));
    }
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000589, pcDirpath); //* MSG_LOG_ID_C_00000589
    iRet = RET_OK;
FUNC_END:
    if (pcTargetPath != NULL) {
        free(pcTargetPath);
    }
    if (hFind != INVALID_HANDLE_VALUE) {
        FindClose(hFind);
    }
    return iRet;
}


/*!
    @brief          Pt@C폜
*/
int CDMtrLibFileDeleteW(const wchar_t *pcTargetfile)
{
    int iRet;
    iRet = RET_NG;
    if (pcTargetfile == NULL) {
        goto FUNC_END;
    }
    if (!CDMtrLibFileIsFileW(pcTargetfile)) {
        iRet = RET_OK;
        goto FUNC_END;
    }
    if (!DeleteFileW(pcTargetfile)) {
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000590, pcTargetfile); //* MSG_LOG_ID_C_00000590
        CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000591, errno); //* MSG_LOG_ID_C_00000591
        goto FUNC_END;
    }
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000592, pcTargetfile); //* MSG_LOG_ID_C_00000592
    iRet = RET_OK;
FUNC_END:
    return iRet;
}


/*!
    @brief          fBNgAt@Cl[
*/
int CDMtrLibFileRenameW(const wchar_t *pcOldPath, const wchar_t *pcNewPath)
{
    int iRet;
    iRet = RET_NG;
    if (pcOldPath == NULL || pcNewPath == NULL) {
        goto FUNC_END;
    }
    if (MoveFileW(pcOldPath, pcNewPath) != 0) {
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000554, pcOldPath, pcNewPath); //* MSG_LOG_ID_C_00000554
        } else {
            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000555, CDMtrLibFileRemoveLongPathMarkW(pcOldPath), CDMtrLibFileRemoveLongPathMarkW(pcNewPath)); //* MSG_LOG_ID_C_00000555
        }
        CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000557, errno); //* MSG_LOG_ID_C_00000557
        goto FUNC_END;
    }
    if (iDebug == DEBUG_ON) {
        CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000558, pcOldPath, pcNewPath); //* MSG_LOG_ID_C_00000558
    } else {
        CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000559, CDMtrLibFileRemoveLongPathMarkW(pcOldPath), CDMtrLibFileRemoveLongPathMarkW(pcNewPath)); //* MSG_LOG_ID_C_00000559
    }
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          e|tH_Ƀe|t@C𐶐
*/
int CDMtrLibFileCreateTempFileW(wchar_t **ppcpath)
{
    int iRet;
    wchar_t *pcTempDir;
    wchar_t *pcTempPath;
    iRet = RET_NG;
    pcTempDir = NULL;
    pcTempPath = NULL;
    if (ppcpath == NULL) {
        goto FUNC_END;
    }
    *ppcpath = NULL;
    pcTempDir = (wchar_t *)malloc((FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if (pcTempDir == NULL) {
        goto FUNC_END;
    }
    memset(pcTempDir, 0x00, (FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if ((GetTempPathW(FILE_PATH_SIZE + 1, pcTempDir)) == 0) {
        goto FUNC_END;
    }
    pcTempPath = (wchar_t *)malloc((FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if (pcTempPath == NULL) {
        goto FUNC_END;
    }
    memset(pcTempPath, 0x00, (FILE_PATH_SIZE + 1) * sizeof(wchar_t));
    if (GetTempFileNameW(pcTempDir,
                         L"cdm",
                         0,
                         pcTempPath) == 0) {
        free(pcTempPath);
        goto FUNC_END;
    }
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000593, pcTempPath); //* MSG_LOG_ID_C_00000593
    *ppcpath = pcTempPath;
    iRet = RET_OK;
FUNC_END:
    if (pcTempDir != NULL) {
        free(pcTempDir);
    }
    return iRet;
}



/*!
    @brief          t@CRs[
*/
int CDMtrLibFileCopyW(wchar_t *pcDest, wchar_t *pcSrc)
{
    int iRet;
    const _TCHAR *pcSrcLogOut = NULL;
    const _TCHAR *pcDestLogOut = NULL;
    iRet = RET_NG;
    if ((pcDest == NULL) || (pcSrc == NULL)) {
        goto FUNC_END;
    }
    pcSrcLogOut = CDMtrLibFileRemoveLongPathMarkW(pcSrc);
    pcDestLogOut = CDMtrLibFileRemoveLongPathMarkW(pcDest);
    if (!CDMtrLibFileIsFileW(pcSrc)) {
        iRet = CDMtrLibFileDeleteW(pcDest);
        if (iRet != RET_OK) {
            CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000594, pcDestLogOut); //* MSG_LOG_ID_C_00000594
        }
        iRet = RET_OK;
        goto FUNC_END;
    }
    if (!CopyFile(pcSrc, pcDest, FALSE)) {
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000567, pcSrcLogOut); //* MSG_LOG_ID_C_00000567
        CDMtrLibCmnLogOut(MSG_LOG_ID_C_00000568, GetLastError()); //* MSG_LOG_ID_C_00000568
        goto FUNC_END;
    }
    SetFileAttributes (pcDest, FILE_ATTRIBUTE_NORMAL) ;
    CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000595, pcSrcLogOut, pcDestLogOut); //* MSG_LOG_ID_C_00000595
    iRet = RET_OK;
FUNC_END:
    return iRet;
}

/*!
    @brief          Ŏw肳ꂽpX̃V[gpXԂB
*/
int CDMtrLibFileGetShortPathNameW(const wchar_t *pwInPath, wchar_t **ppwPath)
{
    int iRet = RET_NG;
    DWORD dPathLength = 0;
    DWORD dRet = 0;
    wchar_t *wBuf = NULL;
    if (pwInPath == NULL || ppwPath == NULL) {
        return iRet;
    }
    dPathLength = GetShortPathNameW(pwInPath, NULL, 0);
    if (dPathLength == 0) {
        DWORD derrno = GetLastError();
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000561, derrno, pwInPath); //* MSG_LOG_ID_C_00000561
        }
    } else {
        wBuf = (wchar_t *)CDMtrLibCmnMallocW(dPathLength);
        if (wBuf != NULL) {
            dRet = GetShortPathNameW(pwInPath, wBuf, dPathLength);
            if (dRet == 0) {
                DWORD derrno = GetLastError();
                if (iDebug == DEBUG_ON) {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000562, derrno, pwInPath); //* MSG_LOG_ID_C_00000562
                }
                CDMtrLibCmnFreeW((void **)&wBuf);
            } else {
                CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000596, pwInPath, wBuf); //* MSG_LOG_ID_C_00000596
                *ppwPath = wBuf;
                iRet = RET_OK;
            }
        }
    }
    return iRet;
}

/*!
    @brief          Ŏw肳ꂽpX̃OpXԂB
*/
int CDMtrLibFileGetLongPathNameW(const wchar_t *pwInPath, wchar_t **ppwPath)
{
    int iRet = RET_NG;
    DWORD dPathLength = 0;
    DWORD dRet = 0;
    wchar_t *wBuf = NULL;
    if (pwInPath == NULL || ppwPath == NULL) {
        return iRet;
    }
    dPathLength = GetLongPathNameW(pwInPath, NULL, 0);
    if (dPathLength == 0) {
        DWORD derrno = GetLastError();
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000563, derrno, pwInPath); //* MSG_LOG_ID_C_00000563
        }
    } else {
        wBuf = (wchar_t *)CDMtrLibCmnMallocW(dPathLength);
        if (wBuf != NULL) {
            dRet = GetLongPathNameW(pwInPath, wBuf, dPathLength);
            if (dRet == 0) {
                DWORD derrno = GetLastError();
                if (iDebug == DEBUG_ON) {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000564, derrno, pwInPath); //* MSG_LOG_ID_C_00000564
                }
                CDMtrLibCmnFreeW((void **)&wBuf);
            } else {
                CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000597, pwInPath, wBuf); //* MSG_LOG_ID_C_00000597
                *ppwPath = wBuf;
                iRet = RET_OK;
            }
        }
    }
    return iRet;
}

/*!
    @brief          Ŏw肳ꂽpX̃tpXԂB
*/
int CDMtrLibFileGetFullPathNameW(const wchar_t *pwInPath, wchar_t **ppwPath)
{
    int iRet = RET_NG;
    DWORD dPathLength = 0;
    DWORD dRet = 0;
    wchar_t *wBuf = NULL;
    if (pwInPath == NULL || ppwPath == NULL) {
        return iRet;
    }
    dPathLength = GetFullPathNameW (pwInPath, 0, NULL, NULL);
    if (dPathLength == 0) {
        DWORD derrno = GetLastError();
        if (iDebug == DEBUG_ON) {
            CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000565, derrno, pwInPath); //* MSG_LOG_ID_C_00000565
        }
    } else {
        wBuf = (wchar_t *)CDMtrLibCmnMallocW(dPathLength);
        if (wBuf != NULL) {
            dRet = GetFullPathNameW (pwInPath, dPathLength, wBuf, NULL);
            if (dRet == 0) {
                DWORD derrno = GetLastError();
                if (iDebug == DEBUG_ON) {
                    CDMtrLibCmnErrLogOutW(MSG_LOG_ID_C_00000566, derrno, pwInPath); //* MSG_LOG_ID_C_00000566
                }
                CDMtrLibCmnFreeW((void **)&wBuf);
            } else {
                CDMtrLibCmnTrcLogOutW(MSG_LOG_ID_C_00000598, pwInPath, wBuf); //* MSG_LOG_ID_C_00000598
                *ppwPath = wBuf;
                iRet = RET_OK;
            }
        }
    }
    return iRet;
}

/*!
    @brief          Ŏw肳ꂽpX烍OpXLpXԋpB
*/
const wchar_t *CDMtrLibFileRemoveLongPathMarkW(const wchar_t *pwPath)
{
    const wchar_t *pRet = NULL;
    const wchar_t *pFuncRet = NULL;
    if (pwPath == NULL) {
        return NULL;
    }
    pFuncRet = CDMtrLibUtilCheckPrefixW(pwPath, LONGPATH_UNC_PREFIX);
    if (pFuncRet != NULL) {
        pRet = pwPath + LONGPATH_UNC_PREFIX_SIZE;
    } else {
        pFuncRet = CDMtrLibUtilCheckPrefixW(pwPath, LONGPATH_PREFIX);
        if (pFuncRet != NULL) {
            pRet = pwPath + LONGPATH_PREFIX_SIZE;
        } else {
            pRet = pwPath;
        }
    }
    return pRet;
}

/*!
    @brief      pXMAX_PATHߔ
*/
int CDMtrLibFileIsMaxPathW(const wchar_t *pwPath)
{
    int iRet = RET_FALSE;
    int ilen = 0;
    if (pwPath != NULL) {
        ilen = wcslen(pwPath);
        if (ilen >= MAX_PATH) {
            iRet = RET_TRUE;
        } else {
            iRet = RET_FALSE;
        }
    }
    return iRet;
}

/*!
    @brief          t@C̍s擾
    @param[in]      pcFileName    t@CpX
    @retval         1ȏ         t@C̍s
    @retval         0             s擾s
*/
int CDMtrLibFileGetLineOfFileW(const _TCHAR *pcFileName)
{
    int iFuncRet;
    FILE *fp = NULL;
    _TCHAR *cBuffer = NULL;
    _TCHAR *pcRet = NULL;
    int iCount = 0;
    if ((fp = _tfopen(pcFileName, _T("r"))) == NULL) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000392, pcFileName); //* MSG_LOG_ID_C_00000392
        goto FUNC_END;
    }
    iFuncRet = CDMtrLibUtilMallocW((void **)&cBuffer, BUFF_LINE_SIZE);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    }
    while (1) {
        pcRet = _fgetts(cBuffer, BUFF_LINE_LENGTH, fp);
        if (pcRet == NULL) {
            break;
        }
        iCount++;
    }
FUNC_END:
    if (fp) {
        fclose(fp);
        fp = NULL;
    }
    CDMtrLibUtilFreeW((void **)&cBuffer);
    return iCount;
}

/*!
    @brief          t@CTCY擾
*/
long CDMtrLibFileGetFileSizeW(_TCHAR *pcFileName)
{
    FILE *fp = NULL;
    long szSize = -1;
    if ((fp = _tfopen(pcFileName, _T("r"))) == NULL) {
        CDMtrLibCmnTrcLogOut(MSG_LOG_ID_C_00000393, pcFileName); //* MSG_LOG_ID_C_00000393
        goto FUNC_END;
    }
    if (fseek(fp, 0L, SEEK_END) != 0) {
        goto FUNC_END;
    }
    szSize = ftell(fp);
    fseek(fp, 0L, SEEK_SET);
FUNC_END:
    if (fp) {
        fclose(fp);
        fp = NULL;
    }
    return szSize;
}

/*!
    @brief          w肳ꂽpX𑽌܂܂ȂpXɕϊ
    @param[in]      psFilePath         ϊΏۂ̃pX("\\?\"t)
    @param[in]      psCopyFileName     Rs[̃t@C
    @param[in]      iShortpathFlg      V[gpXϊۃtO
    @param[out]     ppsOutFilePath     ϊ̃pX(܂܂ȂpX)
    @retval         RET_OK             I
    @retval         RET_NG             ُI
    @note           ߂l RET_OK ̏ꍇAppsOutFilePathɗ̈悪mۂĂB
                    ďoŉ邱ƁB
                    tO(iShortpathFlg)CDMTR_TRUEw肷ƁApsFilePathV[gpXɕϊ(ʏ험p)B
                    CDMTR_FALSEw肳ꂽꍇ́ApsFilePathOpXɕϊB
                    ϊ̃pXMAX_PATHȏ̏ꍇpsFilePathe|tH_zɃRs[A
                    ̃pXԋpBRs[̃t@CpsCopyFileNameŎw肷B
                    MAX_PATH̏ꍇ́Aϊ̃pXppsOutFilePathŕԋpB
                    psFilePathŎw肳ꂽt@C݂Ȃꍇ́ARET_NGԂB
                    V[gpX́AASCIIȊO܂܂\邪ÃVXe
                    @ftHgR[hŎwłȂ͊܂܂Ȃ߃ANZX\łB
*/
int CDMtrLibFileConvertAccessiblePathW(_TCHAR *psFilePath, _TCHAR *psCopyFileName, _TCHAR *psTempFolderName, int iShortpathFlg, _TCHAR **ppsOutFilePath)
{
    int iRet = RET_NG;
    int iFuncRet;
    _TCHAR *pcSourceFile = NULL;
    _TCHAR *pcPath = NULL;
    const _TCHAR *pcPathNoMark = NULL;
    if (psFilePath == NULL || psCopyFileName == NULL || ppsOutFilePath == NULL || psTempFolderName == NULL) {
        goto FUNC_END;
    }
    if (iShortpathFlg == CDMTR_FALSE) {
        iFuncRet = CDMtrLibFileGetLongPathName(psFilePath, &pcPath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    } else {
        iFuncRet = CDMtrLibFileGetShortPathName(psFilePath, &pcPath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
    }
    pcPathNoMark = CDMtrLibFileRemoveLongPathMarkW(pcPath);
    if (pcPathNoMark == NULL) {
        goto FUNC_END;
    }
    if (CDMtrLibFileIsMaxPathW(pcPathNoMark) == RET_TRUE) {
        iFuncRet = CDMtrLibUtilCreateStringW(&pcSourceFile, psTempFolderName, FOLDER_SEPALATE, psCopyFileName, NULL);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        iFuncRet = CDMtrLibFileCopy(pcSourceFile, psFilePath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        }
        *ppsOutFilePath = CDMtrLibUtilCopyStringW(pcSourceFile);
    } else {
        *ppsOutFilePath = CDMtrLibUtilCopyStringW(pcPathNoMark);
    }
    if (*ppsOutFilePath != NULL) {
        iRet = RET_OK;
    }
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcSourceFile);
    CDMtrLibUtilFreeW((void **)&pcPath);
    return iRet;
}

/*!
    @brief          w肳ꂽpX𑽌܂܂ȂpXɕϊ(CN[ht@Cp)
    @param[in]      pcSrcPath         ϊΏۂ̃pX
    @param[out]     ppcDistPath       ꕶ܂܂ȂpX
    @retval         RET_OK            pXϊ
    @retval         RET_NG            pXϊs
    @note           ߂l RET_OK ̏ꍇAppcDistPathɗ̈悪mۂĂB
                    ďoŉ邱ƁB
					RET_NG̏ꍇAppcDistPathɗ̈ɂNULLݒ肳ĂB
					w肳ꂽpXhCu܂ރpX̏ꍇA
					ShortPathϊpXppcDistPathŕԋpB
					ȉ̏ꍇ́ARET_NGԋpB
					@Ew肳ꂽpX̐KɎs
					  EShortPathϊɎs
					  EShortPathϊpXMAX_PATH𒴉߂ꍇ
					{APÍA-IIvVݒlAݒt@CSYSINCݒlɑ΂ėpB
*/
int CDMtrLibFileConvertIncludePathW(const _TCHAR *pcSrcPath, _TCHAR **ppcDistPath)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    _TCHAR *pcAbsolutePath = NULL;
    _TCHAR *pcShortPath = NULL;
    if ( NULL == pcSrcPath || NULL == ppcDistPath ) {
        return RET_NG;
    }
    *ppcDistPath = NULL;
    if (NULL == _tcschr(pcSrcPath, _T(':'))) {
        return RET_NG;
    } else {
        iFuncRet = CDMtrLibFileGetAbsolutePath(pcSrcPath, &pcAbsolutePath);
        if (iFuncRet != RET_OK) {
            goto FUNC_END;
        } else {
            iFuncRet = CDMtrLibFileGetShortPathWithinMaxpathW(pcAbsolutePath, &pcShortPath);
            if (iFuncRet != RET_OK) {
                goto FUNC_END;
            } else {
                *ppcDistPath = pcShortPath;
                pcShortPath = NULL;
                iRet = RET_OK;
            }
        }
    }
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcAbsolutePath);
    CDMtrLibUtilFreeW((void **)&pcShortPath);
    return iRet;
}

/*!
	@brief          w肳ꂽpXV[gpXɕϊ
    @param[in]      pcOrgPath         ϊΏۂ̃pX
    @param[out]     pcShortPath       ϊ̃pX(ShortPath)
    @retval         RET_OK            pXϊ
    @retval         RET_NG            pXϊs
    @note   ϊΏۂ̃pXMAX_PATH𒴉߂pXw肳ꂽꍇAREG_NGԋpB
            ԋpShortPath"\\?\"L͏̂ƂB
            ߂l RET_OK ̏ꍇApcShortPathɗ̈悪mۂĂB
             ďoŉ邱ƁB
			RET_NG̏ꍇApcShortPathɗ̈ɂNULLݒ肳ĂB
*/
int CDMtrLibFileGetShortPathWithinMaxpathW(const _TCHAR *pcOrgPath, _TCHAR **pcShortPath)
{
    int iRet = RET_NG;
    int iFuncRet = RET_NG;
    _TCHAR *pcTmpShortPath = NULL;
    if (NULL == pcOrgPath || NULL == pcShortPath) {
        return RET_NG;
    }
    *pcShortPath = NULL;
    iFuncRet = CDMtrLibFileGetShortPathName(pcOrgPath, &pcTmpShortPath);
    if (iFuncRet != RET_OK) {
        goto FUNC_END;
    } else {
        iFuncRet = CDMtrLibFileIsMaxPathW(CDMtrLibFileRemoveLongPathMarkW(pcTmpShortPath));
        if (iFuncRet == RET_FALSE) {
            *pcShortPath = CDMtrLibUtilCopyStringW(CDMtrLibFileRemoveLongPathMarkW(pcTmpShortPath));
            if (NULL != *pcShortPath) {
                iRet = RET_OK;
            }
        }
    }
FUNC_END:
    CDMtrLibUtilFreeW((void **)&pcTmpShortPath);
    return iRet;
}
