/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <stdio.h>

#include <tchar.h>

#include "CDMtrStmtLog.h"
#include "CDMtrStmtLex.h"
#include "CDMtrMessage.h"

int CDMtrStmtLangType = 0;

typedef enum {

    stmt_unknown = 0,

    stmt_func_header,
    stmt_struct_def,
    stmt_enum_def,
    stmt_def_with_init,
    stmt_statement,
    stmt_type_ident,
    stmt_type_initializer,
    stmt_anonymous_method,

    stmt_type_end
} type_statement;


/*!
    @brief      Xe[gg𐔂
*/
long CDMtrStmtReadOneStatement(int iGoalToken)
{
    long lStmt;
    long lSubStmt;
    int iCurToken;
    int iPrevToken;
    int iNumTokens;
    int iPrevNumToken;
    int iStmtType;
    int iStmtToken;
    int iFlgLoopEnd;
    static int iReCallLevel = 0;
    iReCallLevel++;
    iCurToken   = token_unknown;
    iPrevToken  = token_unknown;
    lStmt       = 0;
    lSubStmt    = 0;
    iNumTokens  = 0;
    iPrevNumToken = 0;
    iFlgLoopEnd = 1;
    iStmtType   = stmt_unknown;
    iStmtToken  = token_unknown;
    for ( ; ; ) {
        iPrevToken = iCurToken;
        iCurToken = CDMtrStmtGetToken();
        if (iCurToken < 0) {
            PASS_CHECK;
            lStmt = -1;
            break;
        } else if (iCurToken == token_end) {
            PASS_CHECK;
            break;
        }
        iNumTokens++;
        switch (iPrevToken) {
        case token_fixed:
        case token_using:
            if (iCurToken != token_lparent) {
                iStmtType = stmt_unknown;
            }
            break;
        default:
            break;
        }
        if (iCurToken == iGoalToken) {
            PASS_CHECK;
            switch (iGoalToken) {
            case token_semicolon:
                PASS_CHECK;
                if (iStmtType == stmt_statement) {
                    switch (iStmtToken) {
                    case token_for:
                    case token_foreach:
                    case token_while:
                    case token_switch:
                    case token_catch:
                    case token_if:
                    case token_fixed:
                    case token_lock:
                    case token_using:
                        PASS_CHECK;
                        if (iNumTokens <= 4) {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000725
                        } else {
                            PASS_CHECK;
                            lStmt += 2;
                            CDMTR_STMT_LOG_COUNT(2L);   //* MSG_LOG_ID_C_00000726
                        }
                        break;
                    case token_do:
                    case token_else:
                        PASS_CHECK;
                        if (iNumTokens <= 2) {
                            PASS_CHECK;
                            lStmt += 0;
                            CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000727
                        } else {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000728
                        }
                        break;
                    default:
                        PASS_CHECK;
                        if (iNumTokens <= 2) {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000729
                        } else {
                            PASS_CHECK;
                            lStmt += 2;
                            CDMTR_STMT_LOG_COUNT(2L);   //* MSG_LOG_ID_C_00000730
                        }
                        break;
                    }
                } else {
                    PASS_CHECK;
                    if (iNumTokens >= 2) {
                        PASS_CHECK;
                        lStmt += 1;
                        CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000731
                    } else {
                        PASS_CHECK;
                        lStmt  += 0;
                        CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000732
                    }
                }
                break;
            case token_rcbrace:
                break;
            case token_rparent:
            case token_rbracket:
                PASS_CHECK;
                lStmt = 0;
                CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000733
                break;
            default:
                PASS_CHECK;
                CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000719);); //* MSG_LOG_ID_C_00000719
                CDMtrStmtPrintToken(iGoalToken);
                lStmt = -1;
                break;
            }
            break;
        } else {
            PASS_CHECK;
            iFlgLoopEnd = 1;
            switch (iCurToken) {
            case token_lparent:
                PASS_CHECK;
                lSubStmt = CDMtrStmtReadOneStatement(token_rparent);
                if (lSubStmt < 0) {
                    PASS_CHECK;
                    lStmt = -1;
                    iFlgLoopEnd = 0;
                }
                CDMTR_STMT_LOG_COUNT(((-1) * lSubStmt));   //* MSG_LOG_ID_C_00000735
                iNumTokens++;
                switch (iStmtType) {
                case stmt_def_with_init:
                    switch (CDMtrStmtLangType) {
                    case CDMtrStmtLangModeCPP :
                    case CDMtrStmtLangModeCS :
                    case CDMtrStmtLangModeJAVA:
                        PASS_CHECK;
                        iStmtType = stmt_func_header;
                        iStmtToken = iCurToken;
                        break;
                    default:
                        break;
                    }
                    break;
                case stmt_unknown :
                case stmt_struct_def:
                case stmt_type_ident :
                    PASS_CHECK;
                    iStmtType = stmt_func_header;
                    iStmtToken = iCurToken;
                    break;
                default:
                    break;
                }
                break;
            case token_semicolon:
                PASS_CHECK;
                if (iStmtType == stmt_statement) {
                    switch (iStmtToken) {
                    case token_for:
                    case token_foreach:
                    case token_while:
                    case token_switch:
                    case token_catch:
                    case token_if:
                    case token_fixed:
                    case token_lock:
                    case token_using:
                        PASS_CHECK;
                        if (iNumTokens <= 4) {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000736
                        } else {
                            PASS_CHECK;
                            lStmt += 2;
                            CDMTR_STMT_LOG_COUNT(2L);   //* MSG_LOG_ID_C_00000737
                        }
                        break;
                    case token_do:
                    case token_else:
                        PASS_CHECK;
                        if (iNumTokens <= 2) {
                            PASS_CHECK;
                            lStmt += 0;
                            CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000738
                        } else {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000739
                        }
                        break;
                    default:
                        PASS_CHECK;
                        if (iNumTokens <= 2) {
                            PASS_CHECK;
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000740
                        } else {
                            PASS_CHECK;
                            lStmt += 2;
                            CDMTR_STMT_LOG_COUNT(2L);   //* MSG_LOG_ID_C_00000741
                        }
                        break;
                    }
                } else {
                    PASS_CHECK;
                    if (iNumTokens >= 2) {
                        PASS_CHECK;
                        lStmt += 1;
                        CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000742
                    } else {
                        PASS_CHECK;
                        lStmt  += 0;
                        CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000743
                    }
                }
                iNumTokens = 0;
                iStmtType  = stmt_unknown;
                iStmtToken = iCurToken;
                break;
            case token_colon:
                PASS_CHECK;
                if (iPrevToken == token_default) {
                    if (iStmtType == stmt_statement) {
                        lStmt += 2;
                        CDMTR_STMT_LOG_COUNT(2L);   //* MSG_LOG_ID_C_00000744
                        iNumTokens = 0;
                        iStmtType  = stmt_unknown;
                        iStmtToken = token_unknown;
                    } else {
                        lStmt += 1;
                        CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000745
                        iNumTokens = 0;
                        iStmtType  = stmt_unknown;
                        iStmtToken = token_unknown;
                    }
                } else if (iStmtType == stmt_unknown) {
                    PASS_CHECK;
                    if (iNumTokens == 2) {
                        PASS_CHECK;
                        lStmt += 0;
                        CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000746
                        iNumTokens = 0;
                        iStmtType  = stmt_unknown;
                        iStmtToken = token_unknown;
                    }
                } else if ( iStmtType == stmt_statement && iStmtToken == token_case ) {
                    PASS_CHECK;
                    lStmt += 1;
                    CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000747
                    iNumTokens = 0;
                    iStmtType  = stmt_unknown;
                    iStmtToken = token_unknown;
                } else {
                    PASS_CHECK;
                    ;
                }
                break;
            case token_if:
            case token_for:
            case token_foreach:
            case token_while:
            case token_switch:
            case token_try:
            case token_catch:
            case token_finally:
            case token_else:
            case token_do:
            case token_fixed:
            case token_lock:
            case token_using:
                PASS_CHECK;
                if (iStmtType == stmt_statement) {
                    PASS_CHECK;
                    if (iStmtToken != token_else) {
                        lStmt += 1;
                        CDMTR_STMT_LOG_COUNT(1L);     //* MSG_LOG_ID_C_00000748
                    }
                    iNumTokens = 1;
                }
                iStmtType   = stmt_statement;
                iStmtToken  = iCurToken;
                break;
            case token_struct:
            case token_union:
            case token_class:
            case token_interface:
                PASS_CHECK;
                if (iStmtType == stmt_statement) {
                    PASS_CHECK;
                    lStmt += 1;
                    CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000749
                    iNumTokens = 1;
                    iStmtType   = stmt_struct_def;
                    iStmtToken  = iCurToken;
                } else if (iStmtType == stmt_unknown || iStmtType == stmt_type_ident) {
                    PASS_CHECK;
                    iStmtType   = stmt_struct_def;
                    iStmtToken  = iCurToken;
                }
                break;
            case token_enum:
                PASS_CHECK;
                if (iStmtType == stmt_statement) {
                    PASS_CHECK;
                    lStmt += 1;
                    CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000750
                    iNumTokens = 1;
                    iStmtType   = stmt_enum_def;
                    iStmtToken  = iCurToken;
                } else if (iStmtType == stmt_unknown || iStmtType == stmt_type_ident) {
                    PASS_CHECK;
                    iStmtType   = stmt_enum_def;
                    iStmtToken  = iCurToken;
                }
                break;
            case token_eq:
                PASS_CHECK;
                if (iStmtType == stmt_unknown || iStmtType == stmt_func_header ||
                        iStmtType == stmt_struct_def || iStmtType == stmt_type_ident) {
                    PASS_CHECK;
                    iStmtType   = stmt_def_with_init;
                    iStmtToken  = iCurToken;
                }
                break;
            case token_lcbrace:
                PASS_CHECK;
                lSubStmt = CDMtrStmtReadOneStatement(token_rcbrace);
                if (lSubStmt < 0) {
                    PASS_CHECK;
                    lStmt = -1;
                    iFlgLoopEnd = 0;
                } else {
                    PASS_CHECK;
                    switch (iStmtType) {
                    case stmt_unknown :
                    case stmt_statement :
                    case stmt_func_header :
                    case stmt_type_initializer :
                    case stmt_anonymous_method :
                        if (iPrevToken == token_lbracket) {
                            CDMTR_STMT_LOG_COUNT(((-1) * lSubStmt));   //* MSG_LOG_ID_C_00000751
                            lSubStmt = 0;
                            iStmtType   = stmt_type_initializer;
                            iStmtToken  = iCurToken;
                        }
                        break;
                    case stmt_def_with_init :
                        PASS_CHECK;
                        CDMTR_STMT_LOG_COUNT(((-1) * lSubStmt));   //* MSG_LOG_ID_C_00000752
                        lSubStmt = 0;
                        break;
                    default:
                        break;
                    }
                    if (iStmtType == stmt_struct_def || iStmtType == stmt_def_with_init || iStmtType == stmt_type_ident) {
                        PASS_CHECK;
                        lStmt += lSubStmt;
                        if ((CDMtrStmtLangType == CDMtrStmtLangModeJAVA) ||  (CDMtrStmtLangType == CDMtrStmtLangModeCS)) {
                            if (iStmtType == stmt_struct_def) {
                                lStmt += 1;
                                CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000753
                                iPrevNumToken = iNumTokens;
                                iNumTokens = 0;
                                iStmtType   = stmt_unknown;
                                iStmtToken  = token_unknown;
                                CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000720, __LINE__, lSubStmt + 1);); //* MSG_LOG_ID_C_00000720
                            }
                        }
                    } else if (iStmtType == stmt_enum_def) {
                        PASS_CHECK;
                        lStmt += 0;
                        if ((CDMtrStmtLangType == CDMtrStmtLangModeJAVA) ||
                                (CDMtrStmtLangType == CDMtrStmtLangModeCS)) {
                            lStmt += 1;
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000754
                            iPrevNumToken = iNumTokens;
                            iNumTokens = 0;
                            iStmtType   = stmt_unknown;
                            iStmtToken  = token_unknown;
                            CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000721, __LINE__, 0L);); //* MSG_LOG_ID_C_00000721
                        }
                    } else if (iStmtType == stmt_statement) {
                        PASS_CHECK;
                        if (iStmtToken == token_else || iStmtToken == token_do) {
                            PASS_CHECK;
                            lStmt += lSubStmt;
                            CDMTR_STMT_LOG_COUNT(0L);   //* MSG_LOG_ID_C_00000755
                        } else {
                            PASS_CHECK;
                            lStmt += (lSubStmt + 1);
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000756
                        }
                        iPrevNumToken = iNumTokens;
                        iNumTokens = 0;
                        iStmtType   = stmt_unknown;
                        iStmtToken  = token_unknown;
                    } else if (iStmtType == stmt_func_header) {
                        PASS_CHECK;
                        lStmt += (lSubStmt + 1);
                        CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000757
                        iPrevNumToken = iNumTokens;
                        iNumTokens = 0;
                        iStmtType   = stmt_unknown;
                        iStmtToken  = token_unknown;
                        CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000722, __LINE__, lSubStmt + 1);); //* MSG_LOG_ID_C_00000722
                        if (iReCallLevel == 1) {
                            iFlgLoopEnd = 0;
                        }
                    } else if (iStmtType == stmt_type_initializer ||
                               iStmtType == stmt_anonymous_method) {
                        PASS_CHECK;
                        lStmt += lSubStmt;
                    } else {
                        PASS_CHECK;
                        if (iNumTokens > 1) {
                            lStmt += (lSubStmt + 1);
                            CDMTR_STMT_LOG_COUNT(1L);   //* MSG_LOG_ID_C_00000758
                            CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000723, __LINE__, lSubStmt + 1);); //* MSG_LOG_ID_C_00000723
                        } else {
                            lStmt += lSubStmt;
                            CDMTR_STMT_LOG_FUNC(CDMtrStmtLogFileOut(MSG_LOG_ID_C_00000724, __LINE__, lSubStmt);); //* MSG_LOG_ID_C_00000724
                        }
                        iPrevNumToken = iNumTokens;
                        iNumTokens = 0;
                        iStmtType   = stmt_unknown;
                        iStmtToken  = token_unknown;
                        if (iReCallLevel == 1) {
                            iFlgLoopEnd = 0;
                        }
                    }
                }
                break;
            case token_delegate:
                switch (iStmtType) {
                case stmt_unknown:
                case stmt_func_header:
                case stmt_def_with_init:
                case stmt_type_initializer:
                case stmt_anonymous_method:
                    iStmtType   = stmt_anonymous_method;
                    iStmtToken  = iCurToken;
                    break;
                default:
                    break;
                }
                break;
            case token_lbracket:
                if (iPrevToken == token_this) {
                    iStmtType = stmt_struct_def;
                }
                lSubStmt = CDMtrStmtReadOneStatement(token_rbracket);
                if (lSubStmt < 0) {
                    PASS_CHECK;
                    lStmt = -1;
                    iFlgLoopEnd = 0;
                }
                iNumTokens++;
                break;
            case token_case:
                if (iPrevToken != token_goto) {
                    if (iStmtType == stmt_statement) {
                        PASS_CHECK;
                        lStmt += 1;
                        CDMTR_STMT_LOG_COUNT(1L);     //* MSG_LOG_ID_C_00000761
                        iNumTokens = 1;
                    }
                    iStmtType   = stmt_statement;
                    iStmtToken  = iCurToken;
                }
                break;
            case token_default:
            default :
                PASS_CHECK;
                break;
            }
            if (iFlgLoopEnd != 1) {
                PASS_CHECK;
                break;
            }
        }
    }
    iReCallLevel--;
    return lStmt;
}
