/*
 *  source code metrics counter
 *  Copyright (C) 2015  NEC Corporation.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <ctype.h>

#include "CDMtrVBCmn.h"
#include "CDMtrLibCmn.h"
#include "CDMtrVBMessage.h"


#define MAX_FILE_NAME            (2048)

#define TABLE_EXTERN_NUM         (500)

#define FUNCTABLE_EXTERN_SIZE    (sizeof(FUNCTION_TABLE_T) * TABLE_EXTERN_NUM)

#define FOLDER_SEPALATE         "\\"

#define _IS_SEPARATECHAR(ch) ( !((ch >= '0') && (ch <= '9')) && \
                               !((ch >= 'a') && (ch <= 'z')) && \
                               !((ch >= 'A') && (ch <= 'Z')) && \
                               (ch != '_') )

#define FUNCINFO_NAME_DEBUG2   "_FuncInfo_dbg.csv"
#define LINEINFO_NAME_DEBUG2 "_LineInfo_dbg.csv"


static int cdmtrVBCmnSourceInfoInit();
static int cdmtrVBCmnTableInfoInit();
static int cdmtrVBCmnParsePhase_INIT();
static int cdmtrVBCmnParsePhase_N();
static int cdmtrVBCmnParsePhase_END();

static void cdmtrVBCmnSetSkip(int ist, int ied);
static void cdmtrVBCmnSearchSkip(int ist, int ied);
static int cdmtrVBCmnGetNextPrep(int iseffective, int ist, int ied, int *pitype, int *piepos);

static void cdmtrVBCmnPrintFunctionTable(FUNCTION_TABLE_T *ptable, FILE *fp, int isdebug);
static void cdmtrVBCmnPrintMetricTable(METRIC_TABLE_T *ptable, FILE *fp, int isdebug);
static void cdmtrVBCmnPrintValue(int count, FILE *fp);

static int eventSourceInput(char *buffer, int max_size);
static int eventGetFuncID();
static NAME_T *eventGetNameInfo(int itype);
static void eventEndNameInfo();

static ParsesEventFunc M_method = {
    eventSourceInput,
    eventGetFuncID,
    eventGetNameInfo,
    eventEndNameInfo,
    cdmtrVBCmnLog
};

static SOURCE_INFO_T strSourceInfo;

static PARSE_INFO_T strParseInfo;

static PREFNAME_INFO_T strPrefName;

ParsesEventFunc *p_method = &M_method;

PARAM_INFO_T     *pParamInfo     = NULL;
FUNCTION_TABLE_T *pFunctionTable = NULL;
METRIC_TABLE_T   *pMetricTable   = NULL;
SOURCE_INFO_T    *pSourceInfo    = &strSourceInfo;
PARSE_INFO_T     *pParseInfo     = &strParseInfo;

extern int yyparse();

jmp_buf g_cdmtrvbiface_jmp_buf;



/*!
    @brief      ʊ֐֐
*/
int cdmtrVBCmnInit(PARAM_INFO_T *pparaminfo)
{
    pParamInfo = pparaminfo;
    pFunctionTable = NULL;
    pMetricTable = NULL;
    memset(&strSourceInfo, 0x00, sizeof(SOURCE_INFO_T));
    memset(&strParseInfo, 0x00, sizeof(PARSE_INFO_T));
    memset(&strPrefName, 0x00, sizeof(PREFNAME_INFO_T));
    if (cdmtrVBCmnSourceInfoInit() != CDMTRVB_RET_OK) {
        goto ERR_LBL;
    }
    if (cdmtrVBCmnTableInfoInit() != CDMTRVB_RET_OK) {
        goto ERR_LBL;
    }
    cdmtrVBParsesInit(VBIF_PHASE_INIT);
    cdmtrVBLexerInit(VBIF_PHASE_INIT);
    return CDMTRVB_RET_OK;
ERR_LBL:
    cdmtrVBCmnFinal();
    return CDMTRVB_RET_NG;
}

/*!
    @brief  ʊ֐I֐
*/
void cdmtrVBCmnFinal()
{
    int i = 0;
    NAME_T *pname = NULL;
    NAME_T *pname_w = NULL;
    CDMtrLibCmnFree((void **)&pMetricTable);
    if (pFunctionTable != NULL) {
        for (i = 0; i < strSourceInfo.iMaxFuncid; i++) {
            CDMtrLibCmnFree((void **) & (pFunctionTable[i].param_name));
        }
        CDMtrLibCmnFree((void **)&pFunctionTable);
    }
    CDMtrLibCmnFree((void **) & (strSourceInfo.pcSource));
    memset(&strSourceInfo, 0x00, sizeof(SOURCE_INFO_T));
    memset(&strParseInfo, 0x00, sizeof(PARSE_INFO_T));
    pname = strPrefName.pTopName;
    while (pname != NULL) {
        pname_w = pname->next;
        CDMtrLibCmnFree((void **)&pname);
        pname = pname_w;
    }
    memset(&strPrefName, 0x00, sizeof(PREFNAME_INFO_T));
}

/*!
    @brief  \[X񏉊֐
*/
static int cdmtrVBCmnSourceInfoInit()
{
    FILE *fp = NULL;
    int isize = 0;
    int iret = 0;
    char *pfilepath = pParamInfo->pcTargetFile;
    if ((fp = fopen(pfilepath, "rb")) == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000762); //* MSG_LOG_ID_C_00000762
        return CDMTRVB_RET_NG;
    }
    if (fseek(fp, 0, SEEK_END) != 0) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000763); //* MSG_LOG_ID_C_00000763
        goto ERR_LBL;
    }
    isize = ftell(fp);
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000764, isize); //* MSG_LOG_ID_C_00000764
    if (isize < 0) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000765); //* MSG_LOG_ID_C_00000765
        goto ERR_LBL;
    }
    if (fseek(fp, 0, SEEK_SET) != 0) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000766); //* MSG_LOG_ID_C_00000766
        goto ERR_LBL;
    }
    strSourceInfo.pcSource = (char *)CDMtrLibCmnMalloc(sizeof(char) * (isize + 1));
    if (strSourceInfo.pcSource == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000767); //* MSG_LOG_ID_C_00000767
        goto ERR_LBL;
    }
    strSourceInfo.iSize = isize;
    iret = (int)fread(strSourceInfo.pcSource, sizeof(char), isize, fp);
    if (iret != isize) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000768, iret); //* MSG_LOG_ID_C_00000768
        goto ERR_LBL;
    }
    strSourceInfo.pcSource[strSourceInfo.iSize] = '\0';
    fclose(fp);
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000769); //* MSG_LOG_ID_C_00000769
    return CDMTRVB_RET_OK;
ERR_LBL:
    CDMtrLibCmnFree((void **) & (strSourceInfo.pcSource));
    fclose(fp);
    return CDMTRVB_RET_NG;
}

/*!
    @brief  ͌ʃe[u֐
*/
static int cdmtrVBCmnTableInfoInit()
{
    int ilinecnt = 0;
    int isemcnt  = 0;
    char *pstart = strSourceInfo.pcSource;
    char *pend   = strSourceInfo.pcSource + strSourceInfo.iSize - 1;
    char *pcur   = pstart;
    if (pcur == NULL) {
        return CDMTRVB_RET_NG;
    }
    ilinecnt = 1;
    while (pcur <= pend) {
        if (*pcur == '\t') {
            *pcur = ' ';
        }
        if (*pcur == ':') {
            isemcnt++;
        }
        if (*pcur == '\r') {
            ilinecnt++;
            if ((pend - pcur >= 1) && (*(pcur + 1) == '\n')) {
                pcur++;
            }
        } else {
            if (*pcur == '\n') {
                ilinecnt++;
            }
        }
        pcur++;
    }
    strSourceInfo.iMaxIndex = ilinecnt + isemcnt - 1;
    strSourceInfo.iMaxLineno = ilinecnt;
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000770, ilinecnt, strSourceInfo.iMaxIndex + 2); //* MSG_LOG_ID_C_00000770
    pMetricTable = (METRIC_TABLE_T *)CDMtrLibCmnMalloc(sizeof(METRIC_TABLE_T) * (strSourceInfo.iMaxIndex + 2));
    if (pMetricTable == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000771); //* MSG_LOG_ID_C_00000771
        return CDMTRVB_RET_NG;
    }
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000772); //* MSG_LOG_ID_C_00000772
    return CDMTRVB_RET_OK;
}

/*!
    @brief  p[Ts֐
*/
int cdmtrVBCmnParse(int iphase)
{
    int iretval = CDMTRVB_RET_NG ;
    switch (iphase) {
    case VBIF_PHASE_1:
        iretval = cdmtrVBCmnParsePhase_INIT();
        break;
    case VBIF_PHASE_2:
        iretval = cdmtrVBCmnParsePhase_N(iphase);
        if (iretval == CDMTRVB_RET_OK) {
            cdmtrVBCmnSearchSkip(0, strSourceInfo.iMaxIndex);
        }
        break;
    case VBIF_PHASE_3:
        iretval = cdmtrVBCmnParsePhase_N(iphase);
        break;
    case VBIF_PHASE_4:
        iretval = cdmtrVBCmnParsePhase_END();
        break;
    default:
        break;
    }
    return (iretval);
}

/*!
    @brief  p[TiPhase1)s֐
*/
static int cdmtrVBCmnParsePhase_INIT()
{
    int ilinecnt   = 0;
    int iindex     = 0;
    char chbefore  = ' ';
    char *pstart   = strSourceInfo.pcSource;
    char *pend     = strSourceInfo.pcSource + strSourceInfo.iSize - 1;
    char *pcur     = pstart;
    char *pwk      = NULL;
    int  iwk       = 0;
    int ischarcode = CDMTRVB_FALSE;
    int isCR       = CDMTRVB_FALSE;
    int iscomment  = CDMTRVB_FALSE;
    int iscomment_continuation = CDMTRVB_FALSE;
    int isnotblank = CDMTRVB_FALSE;
    int isbreakdirective = CDMTRVB_FALSE;
    int islabel = CDMTRVB_FALSE;
    int isaveindex = 0;
    int istatement = 0;
    pMetricTable[iindex].index = iindex;
    pMetricTable[iindex].offset = 0;
    pMetricTable[iindex].line_number = ilinecnt + 1;
    while (pcur <= pend) {
        if (*pcur == '\'') {
            iscomment = CDMTRVB_TRUE;
            cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000773, //* MSG_LOG_ID_C_00000773
                          pMetricTable[iindex].line_number,
                          (pcur - pstart) - pMetricTable[iindex].offset + 1);
            pcur++;
        } else {
            if ((_IS_SEPARATECHAR(chbefore)) &&
                    (toupper(*pcur) == 'R') &&
                    (pend - pcur >= 2) &&
                    (toupper(*(pcur + 1)) == 'E') &&
                    (toupper(*(pcur + 2)) == 'M')) {
                if ((pend - pcur == 2) ||
                        (_IS_SEPARATECHAR(*(pcur + 3))) ) {
                    iscomment = CDMTRVB_TRUE;
                    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000774, //* MSG_LOG_ID_C_00000774
                                  pMetricTable[iindex].line_number,
                                  (pcur - pstart) - pMetricTable[iindex].offset + 1);
                    *pcur = '\'';
                    *(pcur + 1) = ' ';
                    *(pcur + 2) = ' ';
                    pcur += 3;
                }
            }
        }
        if (iscomment) {
            isnotblank = CDMTRVB_TRUE;
            pwk = NULL;
            while (pcur <= pend) {
                if ((*pcur == '\r') || (*pcur == '\n')) {
                    break;
                }
                if (*pcur != ' ') {
                    if (*pcur == '_') {
                        pwk = pcur;
                    } else {
                        pwk = NULL;
                    }
                    *pcur = '@';
                }
                chbefore = *pcur;
                pcur++;
            }
            if ((pwk != NULL) && (*(pwk - 1) == ' ')) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000775, //* MSG_LOG_ID_C_00000775
                              pMetricTable[iindex].line_number);
                iscomment_continuation = CDMTRVB_TRUE;
            }
            if (pcur > pend) {
                break;
            }
        }
        if (*pcur == '"') {
            ischarcode = CDMTRVB_TRUE;
            isnotblank = CDMTRVB_TRUE;
            cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000776, //* MSG_LOG_ID_C_00000776
                          pMetricTable[iindex].line_number,
                          (pcur - pstart) - pMetricTable[iindex].offset + 1);
            *pcur = 'A';
            iwk = 0;
            while (pcur <= pend) {
                if ((*pcur == '\r') || (*pcur == '\n')) {
                    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000777, //* MSG_LOG_ID_C_00000777
                                  (pcur - 1 - pstart) - pMetricTable[iindex].offset + 1);
                    break;
                }
                if (*pcur == '"') {
                    if (iwk % 2 == 0) {
                        cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000778, //* MSG_LOG_ID_C_00000778
                                      (pcur - pstart) - pMetricTable[iindex].offset + 1);
                        *pcur = 'A';
                        break;
                    }
                }
                if (*pcur == '\\') {
                    iwk++;
                } else {
                    iwk = 0;
                }
                chbefore = '"';
                *pcur = 'A';
                pcur++;
            }
            continue;
        }
        if ((chbefore == ' ') &&
                (*pcur == '_')) {
            if ((pend - pcur == 0) ||
                    (*(pcur + 1) == ' ') ||
                    (*(pcur + 1) == '\r') ||
                    (*(pcur + 1) == '\n')) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000779, //* MSG_LOG_ID_C_00000779
                              pMetricTable[iindex].line_number,
                              (pcur - pstart) - pMetricTable[iindex].offset + 1);
                isbreakdirective = CDMTRVB_TRUE;
                isnotblank = CDMTRVB_TRUE;
                *pcur = ' ';
                while (pcur <= pend) {
                    if ((*pcur == '\r') || (*pcur == '\n')) {
                        break;
                    }
                    *pcur = ' ';
                    chbefore = *pcur;
                    pcur++;
                }
                if (pcur > pend) {
                    break;
                }
            }
        }
        if (*pcur == ':') {
            if ((chbefore == ' ') &&
                    ((pend - pcur == 0) ||
                     (*(pcur + 1) == ' ') ||
                     (*(pcur + 1) == '\r') ||
                     (*(pcur + 1) == '\n')) ) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000780, //* MSG_LOG_ID_C_00000780
                              pMetricTable[iindex].line_number,
                              (pcur - pstart) - pMetricTable[iindex].offset + 1);
                isnotblank = CDMTRVB_TRUE;
                if (ischarcode) {
                    pMetricTable[iindex].statements = 1;
                }
                pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
                pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
                iindex++;
                pMetricTable[iindex].index = iindex;
                pMetricTable[iindex].offset = (pcur - pstart) + 1;
                pMetricTable[iindex].line_number = ilinecnt + 1;
                *pcur = '\n';
                chbefore = ' ';
                ischarcode = CDMTRVB_FALSE;
                pcur++;
                continue;
            }
            if ((iindex == 0) ||
                    ((pMetricTable[iindex - 1].line_number != pMetricTable[iindex].line_number) &&
                     (!isbreakdirective))) {
                pwk = pstart + pMetricTable[iindex].offset;
                if (pwk < pcur) {
                    islabel = CDMTRVB_TRUE;
                    while (pwk < pcur) {
                        if (_IS_SEPARATECHAR(*pwk)) {
                            islabel = CDMTRVB_FALSE;
                            break;
                        }
                        pwk++;
                    }
                    if (islabel) {
                        cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000781, //* MSG_LOG_ID_C_00000781
                                      pMetricTable[iindex].line_number,
                                      (pcur - pstart) - pMetricTable[iindex].offset + 1);
                        isnotblank = CDMTRVB_TRUE;
                        pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
                        pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
                        chbefore = *pcur;
                        ischarcode = CDMTRVB_FALSE;
                        pcur++;
                        continue;
                    }
                }
            }
        }
        if (*pcur == '\n') {
            isCR = CDMTRVB_TRUE;
        } else {
            if (*pcur == '\r') {
                isCR = CDMTRVB_TRUE;
                if (pend - pcur >= 1) {
                    if (*(pcur + 1) == '\n') {
                        if (isbreakdirective) {
                            *pcur = ' ';
                        }
                        pcur++;
                    }
                }
            }
        }
        if (isCR) {
            if (ischarcode) {
                pMetricTable[iindex].statements = 1;
                pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
                pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
            }
            if (!isnotblank) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000782, //* MSG_LOG_ID_C_00000782
                              pMetricTable[iindex].line_number);
                pMetricTable[iindex].blank_line = LINEDTL_BLANK_LINE;
            }
            if (iscomment) {
                pMetricTable[iindex].comment_line = LINEDTL_COMMENT_LINE;
                pMetricTable[iindex].cloc_kind = LINEDTL_CLOC_KIND_NOMAL;
            }
            if (iscomment_continuation) {
                pMetricTable[iindex].comment_continuation = LINEDTL_COMMENT_CONTINUATION_TRUE;
            }
            if (isbreakdirective) {
                *pcur = ' ';
                pMetricTable[iindex].line_continuation = LINEDTL_CONTINUATION_LINE;
                pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
                pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
            }
            chbefore = ' ';
            isCR = CDMTRVB_FALSE;
            ischarcode = CDMTRVB_FALSE;
            iscomment = CDMTRVB_FALSE;
            iscomment_continuation = CDMTRVB_FALSE;
            isbreakdirective = CDMTRVB_FALSE;
            isnotblank = CDMTRVB_FALSE;
            ilinecnt++;
            iindex++;
            pMetricTable[iindex].index = iindex;
            pMetricTable[iindex].offset = (pcur - pstart) + 1;
            pMetricTable[iindex].line_number = ilinecnt + 1;
        } else {
            chbefore = *pcur;
            if (*pcur != ' ') {
                ischarcode = CDMTRVB_TRUE;
                isnotblank = CDMTRVB_TRUE;
            }
        }
        pcur++;
    }
    if (ischarcode) {
        pMetricTable[iindex].statements = 1;
        pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
        pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
    }
    if (!isnotblank) {
        if ((pend - pstart >= 1) &&
                (*(pend - 1) != '\r') &&
                (*(pend - 1) != '\n')) {
            cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000783, //* MSG_LOG_ID_C_00000783
                          pMetricTable[iindex].line_number);
            pMetricTable[iindex].blank_line = LINEDTL_BLANK_LINE;
        }
    }
    if (iscomment) {
        pMetricTable[iindex].comment_line = LINEDTL_COMMENT_LINE;
        pMetricTable[iindex].cloc_kind = LINEDTL_CLOC_KIND_NOMAL;
    }
    if (iscomment_continuation) {
        pMetricTable[iindex].comment_continuation = LINEDTL_COMMENT_CONTINUATION_TRUE;
    }
    if (isbreakdirective) {
        pMetricTable[iindex].line_continuation = LINEDTL_CONTINUATION_LINE;
        pMetricTable[iindex].effective_line = LINEDTL_EFFECTIVE_LINE;
        pMetricTable[iindex].eloc_kind = LINEDTL_ELOC_KIND_CODE;
    }
    pSourceInfo->iMaxIndex = iindex;
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000784, iindex); //* MSG_LOG_ID_C_00000784
    pMetricTable[strSourceInfo.iMaxIndex + 1].offset = strSourceInfo.iSize;
    for (iindex = 0; iindex <= strSourceInfo.iMaxIndex; iindex++) {
        if (pMetricTable[iindex].line_continuation == 1) {
            isaveindex = iindex;
            istatement = 0;
            while (iindex <= strSourceInfo.iMaxIndex) {
                if (pMetricTable[iindex].statements != 0) {
                    istatement = 1;
                }
                pMetricTable[iindex].statements = 0;
                if (pMetricTable[iindex].line_continuation == 0) {
                    break;
                }
                iindex++;
            }
            pMetricTable[isaveindex].statements = istatement;
        }
    }
    return CDMTRVB_RET_OK;
}

/*!
    @brief      p[TiPhase2,3)s֐
*/
static int cdmtrVBCmnParsePhase_N(int iphase)
{
    int iindex_st = 0;
    int iindex_ed = 0;
    int iretval = CDMTRVB_RET_NG;
    memset(&strParseInfo, 0x00, sizeof(PARSE_INFO_T));
    for (iindex_st = 0; iindex_st <= strSourceInfo.iMaxIndex; iindex_st++) {
        if (pMetricTable[iindex_st].effective_line == LINEDTL_EFFECTIVE_NONE) {
            continue;
        }
        if ((pMetricTable[iindex_st].eloc_kind != LINEDTL_ELOC_KIND_CODE) &&
                (pMetricTable[iindex_st].eloc_kind != LINEDTL_ELOC_KIND_SKIP)) {
            continue;
        }
        strParseInfo.pcStart = strSourceInfo.pcSource + pMetricTable[iindex_st].offset;
        strParseInfo.iCurIndex = iindex_st;
        strParseInfo.iCurPos = 0;
        strParseInfo.iSize = 0;
        for (iindex_ed = iindex_st; iindex_ed <= strSourceInfo.iMaxIndex; iindex_ed++) {
            if (pMetricTable[iindex_ed].line_continuation == 1) {
                continue;
            }
            strParseInfo.iSize = pMetricTable[iindex_ed + 1].offset - pMetricTable[iindex_st].offset;
            iindex_st = iindex_ed;
            break;
        }
        if (strParseInfo.iSize == 0) {
            cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000785, iphase); //* MSG_LOG_ID_C_00000785
            pMetricTable[iindex_st].status = 2;
            return CDMTRVB_RET_NG;
        }
        cdmtrVBLexerInit(iphase);
        cdmtrVBParsesInit(iphase);
        if (setjmp(g_cdmtrvbiface_jmp_buf) != 0) {
            cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000786, iphase); //* MSG_LOG_ID_C_00000786
            iretval = CDMTRVB_RET_NG;
            return CDMTRVB_RET_NG;
        }
        iretval = yyparse();
        if (iretval != CDMTRVB_RET_OK) {
            return CDMTRVB_RET_NG;
        }
    }
    return CDMTRVB_RET_OK;
}

/*!
    @brief  p[TiPhase4)s֐
*/
static int cdmtrVBCmnParsePhase_END()
{
    int i = 0;
    int j = 0;
    int ifuncid = 0;
    int inest = 0;
    for (i = 0; i < strSourceInfo.iMaxFuncid; i++) {
        ifuncid = pFunctionTable[i].func_id;
        inest   = 0;
        if (pFunctionTable[i].effective) {
            if (IS_DEBUG1) {
                NAME_T *pname = pFunctionTable[i].func_name;
                NAME_T dmy;
                if (pname == NULL) {
                    strcat(dmy.chName, "Unknown");
                    pname = &dmy;
                }
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000787, //* MSG_LOG_ID_C_00000787
                              pFunctionTable[i].func_startline, pFunctionTable[i].func_endline, pname->chName);
            }
            for (j = pFunctionTable[i].startindex; j <= pFunctionTable[i].endindex; j++) {
                if (pMetricTable[j].status != LINEDTL_STATUS_NORMAL) {
                    cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000788, //* MSG_LOG_ID_C_00000788
                                  pMetricTable[j].line_number, pMetricTable[j].status);
                    pFunctionTable[i].status = pMetricTable[j].status;
                    break;
                }
                pMetricTable[j].func_id = ifuncid;
                pMetricTable[j].func_line = LINEDTL_FUNC_LINE_IN;
                if (pMetricTable[j].nest_level != 0) {
                    if (pMetricTable[j].type == VBIF_TYPE_IF2) {
                        pMetricTable[j].nest_level = inest + pMetricTable[j].nest_level;
                    } else {
                        inest += pMetricTable[j].nest_level;
                        pMetricTable[j].nest_level = inest;
                    }
                }
            }
        } else {
            if (IS_DEBUG1) {
                NAME_T *pname = pFunctionTable[i].func_name;
                NAME_T dmy;
                if (pname == NULL) {
                    strcat(dmy.chName, "Unknown");
                    pname = &dmy;
                }
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000789, //* MSG_LOG_ID_C_00000789
                              pFunctionTable[i].func_startline, pFunctionTable[i].func_endline, pname->chName);
            }
        }
    }
    for (i = 0; i <= strSourceInfo.iMaxIndex; i++) {
        if (pMetricTable[i].func_line != LINEDTL_FUNC_LINE_IN) {
            pMetricTable[i].nest_level = 0;
        }
    }
    return CDMTRVB_RET_OK;
}

/*!
    @brief      SKIPݒ֐
*/
static void cdmtrVBCmnSearchSkip(int ist, int ied)
{
    int i;
    int inexttype = 0;
    int inextpos = 0;
    int iret = CDMTRVB_FALSE;
    int isFoundEffectivePrep = CDMTRVB_FALSE;
    int isEffective = CDMTRVB_FALSE;
    for (i = ist; i <= ied; i++) {
        switch (pMetricTable[i].type) {
        case VBIF_TYPE_PREP_IF:
            iret = cdmtrVBCmnGetNextPrep(CDMTRVB_TRUE, i + 1, ied, &inexttype, &inextpos);
            if (!iret) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000790, pMetricTable[i].line_number); //* MSG_LOG_ID_C_00000790
                return;
            }
            cdmtrVBCmnSearchSkip(i + 1, inextpos - 1);
            i = inextpos;
            while (inexttype != VBIF_TYPE_PREP_ENDIF) {
                iret = cdmtrVBCmnGetNextPrep(CDMTRVB_FALSE, i + 1, ied, &inexttype, &inextpos);
                if (!iret) {
                    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000791, pMetricTable[i].line_number); //* MSG_LOG_ID_C_00000791
                    cdmtrVBCmnSetSkip(i + 1, ied);
                    return;
                }
                cdmtrVBCmnSetSkip(i + 1, inextpos - 1);
                i = inextpos;
            }
            break;
        case VBIF_TYPE_PREP_IFFALSE:
            iret = cdmtrVBCmnGetNextPrep(CDMTRVB_FALSE, i + 1, ied, &inexttype, &inextpos);
            if (!iret) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000792, pMetricTable[i].line_number); //* MSG_LOG_ID_C_00000792
                cdmtrVBCmnSetSkip(i + 1, ied);
                return;
            }
            cdmtrVBCmnSetSkip(i + 1, inextpos - 1);
            i = inextpos;
            isFoundEffectivePrep = CDMTRVB_FALSE;
            while (inexttype != VBIF_TYPE_PREP_ENDIF) {
                if (!isFoundEffectivePrep) {
                    if (inexttype == VBIF_TYPE_PREP_ELSEIFFALSE) {
                        isEffective = CDMTRVB_FALSE;
                    } else {
                        isFoundEffectivePrep = CDMTRVB_TRUE;
                        isEffective = CDMTRVB_TRUE;
                    }
                } else {
                    isEffective = CDMTRVB_FALSE;
                }
                iret = cdmtrVBCmnGetNextPrep(isFoundEffectivePrep, i + 1, ied, &inexttype, &inextpos);
                if (!iret) {
                    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000793, pMetricTable[i].line_number); //* MSG_LOG_ID_C_00000793
                    if (isEffective) {
                        cdmtrVBCmnSearchSkip(i + 1, ied);
                    } else {
                        cdmtrVBCmnSetSkip(i + 1, ied);
                    }
                    return;
                }
                if (isEffective) {
                    cdmtrVBCmnSearchSkip(i + 1, inextpos - 1);
                } else {
                    cdmtrVBCmnSetSkip(i + 1, inextpos - 1);
                }
                i = inextpos;
            }
            break;
        default:
            break;
        }
    }
}

/*!
    @brief      SKIPݒ֐
*/
static void cdmtrVBCmnSetSkip(int ist, int ied)
{
    cdmtrVBCmnLog(VBIF_DEBUGMODE_1, MSG_LOG_ID_C_00000794, //* MSG_LOG_ID_C_00000794
                  pMetricTable[ist].line_number,
                  pMetricTable[ied].line_number);
    while (ist <= ied) {
        pMetricTable[ist].eloc_kind = LINEDTL_ELOC_KIND_SKIP;
        ist++;
    }
}

/*!
    @brief      PREPs֐
*/
static int cdmtrVBCmnGetNextPrep(int iseffective, int ist, int ied, int *pitype, int *piepos)
{
    int icnt = 0;
    int iret = CDMTRVB_FALSE;
    int icur = ist;
    while (icur <= ied) {
        if (pMetricTable[icur].eloc_kind != LINEDTL_ELOC_KIND_PREP) {
            icur++;
            continue;
        }
        if (!iseffective) {
            if ((icur - 1 >= ist) &&
                    (pMetricTable[icur - 1].comment_continuation == LINEDTL_COMMENT_CONTINUATION_TRUE)) {
                pMetricTable[icur].eloc_kind = LINEDTL_ELOC_KIND_OTHER;
                pMetricTable[icur].prep_kind = LINEDTL_PREP_NONE;
                icur++;
                continue;
            }
        }
        switch (pMetricTable[icur].type) {
        case VBIF_TYPE_PREP_IF:
        case VBIF_TYPE_PREP_IFFALSE:
            icnt++;
            break;
        case VBIF_TYPE_PREP_ELSE:
        case VBIF_TYPE_PREP_ELSEIF:
        case VBIF_TYPE_PREP_ELSEIFFALSE:
            if (icnt == 0) {
                iret = CDMTRVB_TRUE;
            }
            break;
        case VBIF_TYPE_PREP_ENDIF:
            if (icnt == 0) {
                iret = CDMTRVB_TRUE;
            }
            icnt--;
            break;
        default:
            break;
        }
        if (iret == CDMTRVB_TRUE) {
            *pitype = pMetricTable[icur].type;
            *piepos = icur;
            break;
        }
        icur++;
    }
    return iret;
}

/*!
    @brief  p[X(Phase2)\[Xǂݍ݊֐
*/
static int eventSourceInput(char *buffer, int max_size)
{
    int retval  =  0;
    int readsize = max_size;
    if ((strParseInfo.iSize - strParseInfo.iCurPos) < max_size) {
        readsize = strParseInfo.iSize - strParseInfo.iCurPos;
    }
    if (readsize > 0) {
        memcpy(buffer, (strParseInfo.pcStart + strParseInfo.iCurPos), readsize);
        strParseInfo.iCurPos += readsize;
        retval = readsize;
    }
    return retval;
}

/*!
    @brief  p[X(Phase2)֐ID擾֐
*/
static int eventGetFuncID()
{
    int ifuncid = strSourceInfo.iMaxFuncid + 1;
    if (ifuncid > strSourceInfo.iFuncCnt) {
        if (pFunctionTable == NULL) {
            pFunctionTable = (FUNCTION_TABLE_T *)CDMtrLibCmnMalloc(FUNCTABLE_EXTERN_SIZE);
            if (pFunctionTable == NULL) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000795); //* MSG_LOG_ID_C_00000795
                return 0;
            }
        } else {
            pFunctionTable = realloc(pFunctionTable, (sizeof(FUNCTION_TABLE_T) * (strSourceInfo.iFuncCnt + TABLE_EXTERN_NUM)));
            if (pFunctionTable == NULL) {
                cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000796); //* MSG_LOG_ID_C_00000796
                return 0;
            }
            memset(pFunctionTable + strSourceInfo.iFuncCnt, 0x00, FUNCTABLE_EXTERN_SIZE);
        }
        strSourceInfo.iFuncCnt += TABLE_EXTERN_NUM;
    }
    pFunctionTable[ifuncid - 1].func_id = ifuncid;
    strSourceInfo.iMaxFuncid = ifuncid;
    return ifuncid;
}

/*!
    @brief      p[X(Phase2) NameSpaceȂǂ̏Cq̕ۑ̈擾֐
*/
static NAME_T *eventGetNameInfo(int itype)
{
    NAME_T *pname = NULL;
    pname = (NAME_T *)CDMtrLibCmnMalloc(sizeof(NAME_T));
    if (pname == NULL) {
        return NULL;
    }
    strPrefName.iMaxid++;
    pname->iType = itype;
    pname->iId = strPrefName.iMaxid;
    pname->parent = strPrefName.pCurName;
    strPrefName.pCurName = pname;
    if (strPrefName.iMaxid == 1) {
        strPrefName.pTopName = pname;
    } else {
        strPrefName.pLastName->next = pname;
    }
    strPrefName.pLastName = pname;
    return pname;
}

/*!
    @brief      p[X(Phase2) NameSpaceȂǂ̏Cq̕ۑ̈̉
*/
static void eventEndNameInfo(int itype)
{
    if (strPrefName.pCurName != NULL) {
        strPrefName.pCurName = strPrefName.pCurName->parent;
    }
}
/*!
    @brief  Functione[uo͊֐
*/
int cdmtrVBCmnOutputFunctionTable()
{
    int i;
    FILE *fp;
    char cfilepath[MAX_FILE_NAME + 1];
    int isout = CDMTRVB_FALSE;
    if ((strlen(pParamInfo->pcMetricFilePath) +
            strlen(pParamInfo->pcSourceID) +
            strlen(FUNCINFO_NAME) + 1) > MAX_FILE_NAME) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000797); //* MSG_LOG_ID_C_00000797
        return CDMTRVB_RET_NG;
    }
    sprintf(cfilepath,
            "%s%s%s%s",
            pParamInfo->pcMetricFilePath,
            FOLDER_SEPALATE,
            pParamInfo->pcSourceID,
            FUNCINFO_NAME);
    if ((fp = fopen(cfilepath, "wb")) == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000798); //* MSG_LOG_ID_C_00000798
        return CDMTRVB_RET_NG;
    }
    cdmtrVBCmnPrintFunctionTable(NULL, fp, CDMTRVB_FALSE);
    if (pFunctionTable == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000799); //* MSG_LOG_ID_C_00000799
        goto FUNC_END;
    }
    for (i = 0; i < strSourceInfo.iMaxFuncid; i++) {
        if (pFunctionTable[i].effective) {
            cdmtrVBCmnPrintFunctionTable(&pFunctionTable[i], fp, CDMTRVB_FALSE);
            isout = CDMTRVB_TRUE;
        }
    }
    if (!isout) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000800); //* MSG_LOG_ID_C_00000800
    }
FUNC_END:
    fclose(fp);
    return CDMTRVB_RET_OK;
}

/*!
    @brief  Functione[uo͊֐
*/
static void cdmtrVBCmnPrintFunctionTable(FUNCTION_TABLE_T *ptable, FILE *fp, int isdebug)
{
    NAME_T *pname_cur = NULL;
    NAME_T *pname_last = NULL;
    if (ptable == NULL) {
        fprintf(fp, "\"Func ID\",");
        fprintf(fp, "\"Func Name\",");
        fprintf(fp, "\"Func_StartLine\",");
        fprintf(fp, "\"Func EndLine\",");
        fprintf(fp, "\"PARMS\",");
        fprintf(fp, "\"Param Name\",");
        fprintf(fp, "\"Internal\",");
        fprintf(fp, "\"Status\"");
        if (isdebug) {
            fprintf(fp, ",\"type\"");
            fprintf(fp, ",\"startindex\"");
            fprintf(fp, ",\"endindex\"");
            fprintf(fp, ",\"effective\"");
        }
        fprintf(fp, "\n");
    } else {
        fprintf(fp, "%d,", ptable->func_id);
        pname_last = NULL;
        pname_cur = ptable->func_name;
        if ((pname_cur != NULL) &&
                (pname_cur->chName[0] != '\0')) {
            fprintf(fp, "\"");
            pname_last = NULL;
            while (pname_last != (ptable->func_name)->parent) {
                pname_cur = (ptable->func_name)->parent;
                while (pname_cur->parent != pname_last) {
                    pname_cur = pname_cur->parent;
                }
                fprintf(fp, "%s.", pname_cur->chName);
                pname_last = pname_cur;
            }
            fprintf(fp, "%s\",", (ptable->func_name)->chName);
        } else {
            fprintf(fp, ",");
        }
        fprintf(fp, "%d,", ptable->func_startline);
        fprintf(fp, "%d,", ptable->func_endline);
        fprintf(fp, "-,");
        if (ptable->param_name != NULL) {
            fprintf(fp, "\"%s\",", ptable->param_name);
        } else {
            fprintf(fp, ",");
        }
        fprintf(fp, "0,");
        fprintf(fp, "%d", ptable->status);
        if (isdebug) {
            fprintf(fp, ",%d", ptable->type);
            fprintf(fp, ",%d", ptable->startindex);
            fprintf(fp, ",%d", ptable->endindex);
            fprintf(fp, ",%d", ptable->effective);
        }
        fprintf(fp, "\n");
    }
}

/*!
    @brief  Metrice[uo͊֐
*/
int cdmtrVBCmnOutputMetricTable()
{
    int i;
    int ibeforeline = 0;
    int iafterline = 0;
    int icurline = 0;
    FILE *fp;
    METRIC_TABLE_T *ptable = pMetricTable;
    METRIC_TABLE_T strmtrtbl;
    char cfilepath[MAX_FILE_NAME + 1];
    if (pParamInfo->pcMetricFilePath == NULL) {
        return CDMTRVB_RET_NG;
    }
    if ((strlen(pParamInfo->pcMetricFilePath) +
            strlen(pParamInfo->pcSourceID) +
            strlen(LINEINFO_NAME) + 1) > MAX_FILE_NAME) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000801); //* MSG_LOG_ID_C_00000801
        return CDMTRVB_RET_NG;
    }
    sprintf(cfilepath,
            "%s%s%s%s",
            pParamInfo->pcMetricFilePath,
            FOLDER_SEPALATE,
            pParamInfo->pcSourceID,
            LINEINFO_NAME);
    if ((fp = fopen(cfilepath, "wb")) == NULL) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000802); //* MSG_LOG_ID_C_00000802
        return CDMTRVB_RET_NG;
    }
    cdmtrVBCmnPrintMetricTable(NULL, fp, CDMTRVB_FALSE);
    if (strSourceInfo.iMaxLineno == 0) {
        cdmtrVBCmnLog(VBIF_DEBUGMODE_NONE, MSG_LOG_ID_C_00000803); //* MSG_LOG_ID_C_00000803
        goto FUNC_END;
    }
    ibeforeline = 0;
    memset(&strmtrtbl, 0x00, sizeof(METRIC_TABLE_T));
    for (i = 0; i <= pSourceInfo->iMaxIndex; i++) {
        icurline = ptable[i].line_number;
        iafterline = ptable[i + 1].line_number;
        if (icurline != ibeforeline) {
            if (icurline != iafterline) {
                cdmtrVBCmnPrintMetricTable(&ptable[i], fp, CDMTRVB_FALSE);
            } else {
                memcpy(&strmtrtbl, &ptable[i], sizeof(METRIC_TABLE_T));
            }
        } else {
            if (strmtrtbl.status == LINEDTL_STATUS_NORMAL) {
                strmtrtbl.status = ptable[i].status;
            }
            strmtrtbl.namespace_delim = (strmtrtbl.namespace_delim | ptable[i].namespace_delim);
            strmtrtbl.class_delim = (strmtrtbl.class_delim | ptable[i].class_delim);
            strmtrtbl.func_delim = (strmtrtbl.func_delim | ptable[i].func_delim);
            if (strmtrtbl.func_id == 0) {
                strmtrtbl.func_id = ptable[i].func_id;
            }
            if (strmtrtbl.func_line == LINEDTL_FUNC_LINE_OUT) {
                strmtrtbl.func_line = ptable[i].func_line;
            }
            strmtrtbl.effective_line = LINEDTL_EFFECTIVE_LINE;
            if (strmtrtbl.comment_line == LINEDTL_COMMENT_NONE) {
                strmtrtbl.comment_line = ptable[i].comment_line;
            }
            if (strmtrtbl.cloc_kind == LINEDTL_CLOC_KIND_NONE) {
                strmtrtbl.cloc_kind = ptable[i].cloc_kind;
            }
            strmtrtbl.blank_line = LINEDTL_BLANK_NONE;
            if (strmtrtbl.eloc_kind == LINEDTL_ELOC_KIND_OTHER) {
                strmtrtbl.eloc_kind = ptable[i].eloc_kind;
            }
            if (strmtrtbl.prep_kind == LINEDTL_PREP_NONE) {
                strmtrtbl.prep_kind = ptable[i].prep_kind;
            }
            strmtrtbl.statements += ptable[i].statements;
            strmtrtbl.control_statements += ptable[i].control_statements;
            strmtrtbl.cyclowc += ptable[i].cyclowc;
            strmtrtbl.branches += ptable[i].branches;
            strmtrtbl.cndops += ptable[i].cndops;
            if (strmtrtbl.nest_level < ptable[i].nest_level) {
                strmtrtbl.nest_level = ptable[i].nest_level;
            }
            if (strmtrtbl.line_continuation == LINEDTL_CONTINUATION_NONE) {
                strmtrtbl.line_continuation = ptable[i].line_continuation;
            }
            if (icurline != iafterline) {
                cdmtrVBCmnPrintMetricTable(&strmtrtbl, fp, CDMTRVB_FALSE);
            }
        }
        ibeforeline = icurline;
    }
FUNC_END:
    fclose(fp);
    return CDMTRVB_RET_OK;
}

/*!
    @brief  lo͊֐

*/
static void cdmtrVBCmnPrintValue(int count, FILE *fp)
{
    if (count == 0) {
        fprintf(fp, ",");
    } else {
        fprintf(fp, "%d,", count);
    }
}

/*!
    @brief  Metrice[uo͊֐
*/
static void cdmtrVBCmnPrintMetricTable(METRIC_TABLE_T *ptable, FILE *fp, int isdebug)
{
    if (ptable == NULL) {
        fprintf(fp, "\"Line Number\",");
        fprintf(fp, "\"Status\",");
        fprintf(fp, "\"namespace Delim\",");
        fprintf(fp, "\"Class Delim\",");
        fprintf(fp, "\"Func Delim\",");
        fprintf(fp, "\"Func ID\",");
        fprintf(fp, "\"Func Line\",");
        fprintf(fp, "\"Effective Line\",");
        fprintf(fp, "\"Comment Line\",");
        fprintf(fp, "\"Blank Line\",");
        fprintf(fp, "\"ELOC Kind\",");
        fprintf(fp, "\"PREP Kind\",");
        fprintf(fp, "\"CLOC Kind\",");
        fprintf(fp, "\"Statements\",");
        fprintf(fp, "\"Control Statement\",");
        fprintf(fp, "\"CYCLOWC\",");
        fprintf(fp, "\"Control Kind\",");
        fprintf(fp, "\"BRANCHS\",");
        fprintf(fp, "\"CNDOPS\",");
        fprintf(fp, "\"Nest Level\",");
        fprintf(fp, "\"Line Continuation\",");
        fprintf(fp, "\"Comment Continuation\",");
        fprintf(fp, "\"Attribute Line\",");
        fprintf(fp, "\"EVALS\",");
        fprintf(fp, "\"EVALR\",");
        fprintf(fp, "\"AVALS\"");
        if (isdebug) {
            fprintf(fp, ",");
            fprintf(fp, "\"index\",");
            fprintf(fp, "\"offset\",");
            fprintf(fp, "\"nameid\",");
            fprintf(fp, "\"source\"");
        }
        fprintf(fp, "\n");
    } else {
        cdmtrVBCmnPrintValue(ptable->line_number, fp);
        cdmtrVBCmnPrintValue(ptable->status, fp);
        cdmtrVBCmnPrintValue(ptable->namespace_delim, fp);
        cdmtrVBCmnPrintValue(ptable->class_delim, fp);
        cdmtrVBCmnPrintValue(ptable->func_delim, fp);
        cdmtrVBCmnPrintValue(ptable->func_id, fp);
        cdmtrVBCmnPrintValue(ptable->func_line, fp);
        cdmtrVBCmnPrintValue(ptable->effective_line, fp);
        cdmtrVBCmnPrintValue(ptable->comment_line, fp);
        cdmtrVBCmnPrintValue(ptable->blank_line, fp);
        cdmtrVBCmnPrintValue(ptable->eloc_kind, fp);
        cdmtrVBCmnPrintValue(ptable->prep_kind, fp);
        cdmtrVBCmnPrintValue(ptable->cloc_kind, fp);
        cdmtrVBCmnPrintValue(ptable->statements, fp);
        cdmtrVBCmnPrintValue(ptable->control_statements, fp);
        cdmtrVBCmnPrintValue(ptable->cyclowc, fp);
        cdmtrVBCmnPrintValue(ptable->type, fp);
        cdmtrVBCmnPrintValue(ptable->branches, fp);
        cdmtrVBCmnPrintValue(ptable->cndops, fp);
        cdmtrVBCmnPrintValue(ptable->nest_level, fp);
        cdmtrVBCmnPrintValue(ptable->nest_level, fp);
        cdmtrVBCmnPrintValue(ptable->nest_level, fp);
        fprintf(fp, ",");
        fprintf(fp, ",");
        fprintf(fp, ",");
        fprintf(fp, "");
        if (isdebug) {
            int j = 0;
            fprintf(fp, ",");
            fprintf(fp, "%d,", ptable->index);
            fprintf(fp, "%d,", ptable->offset);
            fprintf(fp, "%d,", ptable->nameid);
            j = ptable->offset;
            while (j < (ptable + 1)->offset) {
                if ((strSourceInfo.pcSource[j] == '\r') ||
                        (strSourceInfo.pcSource[j] == '\n')) {
                    break;
                }
                j++;
            }
            if (j < (ptable + 1)->offset) {
                j = j - ptable->offset;
            } else {
                j = (ptable + 1)->offset - ptable->offset;
            }
            if (j > 0) {
                fprintf(fp, "\"");
                fwrite(strSourceInfo.pcSource + ptable->offset, 1, j, fp);
                fprintf(fp, "\"");
            }
        }
        fprintf(fp, "\n");
    }
}


/*!
    @brief  Functione[uo͊֐iDebugpj
*/
int cdmtrVBCmnOutputFunctionTable_debug2()
{
    int i;
    FUNCTION_TABLE_T *ptable = pFunctionTable;
    FILE *fp;
    char cfilepath[MAX_FILE_NAME + 1];
    if ((strlen(pParamInfo->pcMetricFilePath) +
            strlen(pParamInfo->pcSourceID) +
            strlen(FUNCINFO_NAME_DEBUG2) + 1) > MAX_FILE_NAME) {
        return CDMTRVB_RET_NG;
    }
    sprintf(cfilepath,
            "%s%s%s%s",
            pParamInfo->pcMetricFilePath,
            FOLDER_SEPALATE,
            pParamInfo->pcSourceID,
            FUNCINFO_NAME_DEBUG2);
    if ((fp = fopen(cfilepath, "wb")) == NULL) {
        return CDMTRVB_RET_NG;
    }
    cdmtrVBCmnPrintFunctionTable(NULL, fp, CDMTRVB_TRUE);
    if (ptable == NULL) {
        goto FUNC_END;
    }
    for (i = 0; i < pSourceInfo->iMaxFuncid; i++) {
        cdmtrVBCmnPrintFunctionTable(&ptable[i], fp, CDMTRVB_TRUE);
    }
FUNC_END:
    fclose(fp);
    return CDMTRVB_RET_OK;
}

/*!
    @brief  Metrice[uo͊֐iDebugpj
*/
int cdmtrVBCmnOutputMetricTable_debug2()
{
    int i;
    METRIC_TABLE_T *ptable = pMetricTable;
    FILE *fp;
    char cfilepath[MAX_FILE_NAME + 1];
    if ((strlen(pParamInfo->pcMetricFilePath) +
            strlen(pParamInfo->pcSourceID) +
            strlen(LINEINFO_NAME_DEBUG2) + 1) > MAX_FILE_NAME) {
        return CDMTRVB_RET_NG;
    }
    sprintf(cfilepath,
            "%s%s%s%s",
            pParamInfo->pcMetricFilePath,
            FOLDER_SEPALATE,
            pParamInfo->pcSourceID,
            LINEINFO_NAME_DEBUG2);
    if ((fp = fopen(cfilepath, "wb")) == NULL) {
        return CDMTRVB_RET_NG;
    }
    cdmtrVBCmnPrintMetricTable(NULL, fp, CDMTRVB_TRUE);
    if (ptable == NULL) {
        goto FUNC_END;
    }
    for (i = 0; i <= pSourceInfo->iMaxIndex; i++) {
        cdmtrVBCmnPrintMetricTable(&ptable[i], fp, CDMTRVB_TRUE);
    }
FUNC_END:
    fclose(fp);
    return CDMTRVB_RET_OK;
}

/*!
    @brief  \[Xo͊֐iDebugpj
*/
int cdmtrVBCmnOutputSource_debug2()
{
    FILE *fp = NULL;
    char cfilepath[MAX_FILE_NAME + 1];
    if ((strlen(pParamInfo->pcMetricFilePath) +
            strlen(pParamInfo->pcSourceID) +
            strlen("s_src.txt") + 1) > MAX_FILE_NAME) {
        return CDMTRVB_RET_NG;
    }
    sprintf(cfilepath,
            "%s%s%s_src.txt",
            pParamInfo->pcMetricFilePath,
            FOLDER_SEPALATE,
            pParamInfo->pcSourceID);
    if ((fp = fopen(cfilepath, "wb")) == NULL) {
        return CDMTRVB_RET_NG;
    }
    fwrite(strSourceInfo.pcSource, 1, strSourceInfo.iSize, fp);
    fclose(fp);
    return CDMTRVB_RET_OK;
}

/*!
    @brief      \[Xo͊֐iDebugpj
*/
void cdmtrVBCmnLog(int idebugmode, char *pFormat, ...)
{
    va_list ap;
    if (pFormat == NULL) {
        return;
    }
    if (idebugmode != VBIF_DEBUGMODE_NONE) {
        if ((idebugmode & pParamInfo->iDebugMode) == 0) {
            return;
        }
    }
    if (pParamInfo->errfile != NULL) {
        va_start(ap, pFormat);
        vfprintf(pParamInfo->errfile, pFormat, ap);
        va_end(ap);
    }
}
